@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -21,6 +21,7 @@ import org.eclipse.swt.widgets.*;
 import org.eclipse.swt.graphics.Image;
 import org.eclipse.swt.custom.BusyIndicator;
 
+import org.eclipse.core.resources.IFile;
 import org.eclipse.core.runtime.*;
 import org.eclipse.ui.IPersistableElement;
 import org.eclipse.ui.IEditorInput;
@@ -39,8 +40,8 @@ import org.eclipse.compare.structuremergeviewer.*;
 
 /**
  * A compare operation which can present its results in a special editor.
- * Running the compare operation and presentating the results in a compare editor
- * are combined in one interface because it allows a client to keep the implementation
+ * Running the compare operation and presenting the results in a compare editor
+ * are combined in one class because it allows a client to keep the implementation
  * all in one place while separating it from the innards of a specific UI implementation of compare/merge.
  * <p> 
  * A <code>CompareEditorInput</code> defines methods for the following sequence steps:
@@ -98,7 +99,7 @@ public abstract class CompareEditorInput implements IEditorInput, IPropertyChang
 	 */
 	public static final String DIRTY_STATE= "DIRTY_STATE"; //$NON-NLS-1$
 		
-	private static final String COMPARE_EDITOR_IMAGE_NAME= "cview16/compare_view.gif"; //$NON-NLS-1$
+	private static final String COMPARE_EDITOR_IMAGE_NAME= "eview16/compare_view.gif"; //$NON-NLS-1$
 	private static Image fgTitleImage;
 	
 	private Splitter fComposite;
@@ -121,6 +122,7 @@ public abstract class CompareEditorInput implements IEditorInput, IPropertyChang
 	private ShowPseudoConflicts fShowPseudoConflicts;
 	
 	boolean fStructureCompareOnSingleClick= true;
+	boolean fUseOutlineView= false;
 
 	/**
 	 * Creates a <code>CompareEditorInput</code> which is initialized with the given
@@ -133,10 +135,14 @@ public abstract class CompareEditorInput implements IEditorInput, IPropertyChang
 		fCompareConfiguration= configuration;
 		Assert.isNotNull(configuration);
 		
-		ResourceBundle bundle= CompareUIPlugin.getResourceBundle();
+		Object object= fCompareConfiguration.getProperty(CompareConfiguration.USE_OUTLINE_VIEW);
+		if (object instanceof Boolean)
+			fUseOutlineView= ((Boolean) object).booleanValue();
+
+		ResourceBundle bundle= CompareUI.getResourceBundle();
 		fIgnoreWhitespace= new IgnoreWhiteSpaceAction(bundle, configuration);
 		fShowPseudoConflicts= new ShowPseudoConflicts(bundle, configuration);
-		
+
 		fDirtyStateListener= new IPropertyChangeListener() {
 			public void propertyChange(PropertyChangeEvent e) {
 				String propertyName= e.getProperty();
@@ -163,7 +169,7 @@ public abstract class CompareEditorInput implements IEditorInput, IPropertyChang
 	 * see IAdaptable.getAdapter
 	 */
 	public Object getAdapter(Class adapter) {
-		if (CompareNavigator.class.equals(adapter)) {
+		if (ICompareNavigator.class.equals(adapter) || CompareNavigator.class.equals(adapter)) {
 			if (fNavigator == null)
 				fNavigator= new CompareNavigator(
 					new CompareViewerSwitchingPane[] {
@@ -175,6 +181,22 @@ public abstract class CompareEditorInput implements IEditorInput, IPropertyChang
 				);
 			return fNavigator;
 		}
+		if (IFile.class.equals(adapter)) {
+		    IProgressMonitor pm= new NullProgressMonitor();
+			// flush changes in any dirty viewer
+			try {
+	            flushViewer(fStructureInputPane, pm);
+	            flushViewer(fStructurePane1, pm);
+	            flushViewer(fStructurePane2, pm);
+	            flushViewer(fContentInputPane, pm);
+	        } catch (CoreException e) {
+	            CompareUIPlugin.log(e);
+	        }
+		    IFile[] files= (IFile[]) getAdapter(IFile[].class);
+		    if (files != null && files.length > 0)
+		        return files[0];	// can only return one: limitation on IDE.saveAllEditors; see #64617
+		    return null;
+		}
 		return null;
 	}
 	
@@ -363,8 +385,56 @@ public abstract class CompareEditorInput implements IEditorInput, IPropertyChang
 
 		fComposite= new Splitter(parent, SWT.VERTICAL);
 		fComposite.setData(this);
-			
-		final Splitter h= new Splitter(fComposite, SWT.HORIZONTAL);
+				
+		Control outline= null;
+		if (!fUseOutlineView)
+			outline= createOutlineContents(fComposite, SWT.HORIZONTAL);
+					
+		fContentInputPane= new CompareViewerSwitchingPane(fComposite, SWT.BORDER | SWT.FLAT) {
+			protected Viewer getViewer(Viewer oldViewer, Object input) {
+				if (input instanceof ICompareInput)
+					return findContentViewer(oldViewer, (ICompareInput)input, this);
+				return null;
+			}
+		};
+		if (fFocusPane == null)
+			fFocusPane= fContentInputPane;
+		if (outline != null)
+			fComposite.setVisible(outline, false);
+		fComposite.setVisible(fContentInputPane, true);
+		
+		if (fStructureInputPane != null)
+			fComposite.setWeights(new int[] { 30, 70 });
+		
+		fComposite.layout();
+
+		if (fStructureInputPane != null && fInput instanceof ICompareInput) {
+			fStructureInputPane.setInput(fInput);
+			ISelection sel= fStructureInputPane.getSelection();
+			if (sel == null || sel.isEmpty())
+				feed1(sel);	// we only feed downstream viewers if the top left pane is empty
+		}
+		
+		fComposite.setData("Nav", //$NON-NLS-1$
+			new CompareViewerSwitchingPane[] {
+				fStructureInputPane,
+				fStructurePane1,
+				fStructurePane2,
+				fContentInputPane
+			}
+		);
+	
+		return fComposite;
+	}
+	
+	/**
+	 * @param parent the parent control under which the control must be created
+	 * @param direction the layout direction of the contents, either </code>SWT.HORIZONTAL<code> or </code>SWT.VERTICAL<code> 
+	 * @return the SWT control hierarchy for the outline part of the compare editor
+	 * @since 3.0
+	 */
+	public Control createOutlineContents(Composite parent, int direction) {
+		final Splitter h= new Splitter(parent, direction);
 
 		fStructureInputPane= new CompareViewerSwitchingPane(h, SWT.BORDER | SWT.FLAT, true) {
 			protected Viewer getViewer(Viewer oldViewer, Object input) {
@@ -397,21 +467,7 @@ public abstract class CompareEditorInput implements IEditorInput, IPropertyChang
 			}
 		};
 		h.setVisible(fStructurePane2, false);
-				
-		fContentInputPane= new CompareViewerSwitchingPane(fComposite, SWT.BORDER | SWT.FLAT) {
-			protected Viewer getViewer(Viewer oldViewer, Object input) {
-				if (input instanceof ICompareInput)
-					return findContentViewer(oldViewer, (ICompareInput)input, this);
-				return null;
-			}
-		};
-		fComposite.setVisible(h, false);
-		fComposite.setVisible(fContentInputPane, true);
 		
-		fComposite.setWeights(new int[] { 30, 70 });
-		
-		fComposite.layout();
-
 		// setup the wiring for top left pane
 		fStructureInputPane.addOpenListener(
 			new IOpenListener() {
@@ -453,25 +509,27 @@ public abstract class CompareEditorInput implements IEditorInput, IPropertyChang
 			}
 		);		
 
-		if (fInput instanceof ICompareInput) {
-			fStructureInputPane.setInput((ICompareInput) fInput);
-			ISelection sel= fStructureInputPane.getSelection();
-			if (sel == null || sel.isEmpty())
-				feed1(sel);	// we only feed downstream viewers if the top left pane is empty
-		}
-		
-		fComposite.setData("Nav", //$NON-NLS-1$
-			new CompareViewerSwitchingPane[] {
-				fStructureInputPane,
-				fStructurePane1,
-				fStructurePane2,
-				fContentInputPane
+		if (fUseOutlineView) {
+			if (fInput instanceof ICompareInput) {
+				fStructureInputPane.setInput(fInput);
+				ISelection sel= fStructureInputPane.getSelection();
+				if (sel == null || sel.isEmpty())
+					feed1(sel);	// we only feed downstream viewers if the top left pane is empty
 			}
-		);
-	
-		return fComposite;
+			
+			fComposite.setData("Nav", //$NON-NLS-1$
+				new CompareViewerSwitchingPane[] {
+					fStructureInputPane,
+					fStructurePane1,
+					fStructurePane2,
+					fContentInputPane
+				}
+			);
+		}
+
+		return h;
 	}
-	
+
 	private void feed1(final ISelection selection) {
 		BusyIndicator.showWhile(fComposite.getDisplay(),
 			new Runnable() {
@@ -600,11 +658,13 @@ public abstract class CompareEditorInput implements IEditorInput, IPropertyChang
 	 * <p>
 	 * Subclasses may override to implement a different strategy.
 	 * </p>
-	 *
+	 * @param oldViewer a new viewer is only created if this old viewer cannot show the given input
+	 * @param input the input object for which to find a structure viewer
+	 * @param parent the SWT parent composite under which the new viewer is created
 	 * @return a compare viewer which is suitable for the given input object or <code>null</code>
 	 */
 	public Viewer findStructureViewer(Viewer oldViewer, ICompareInput input, Composite parent) {
-		return CompareUIPlugin.findStructureViewer(oldViewer, input, parent, fCompareConfiguration);
+		return CompareUI.findStructureViewer(oldViewer, input, parent, fCompareConfiguration);
 	}
 
 	/**
@@ -618,12 +678,14 @@ public abstract class CompareEditorInput implements IEditorInput, IPropertyChang
 	 * <p>
 	 * Subclasses may override to implement a different strategy.
 	 * </p>
-	 *
+	 * @param oldViewer a new viewer is only created if this old viewer cannot show the given input
+	 * @param input the input object for which to find a structure viewer
+	 * @param parent the SWT parent composite under which the new viewer is created
 	 * @return a compare viewer which is suitable for the given input object or <code>null</code>
 	 */
 	public Viewer findContentViewer(Viewer oldViewer, ICompareInput input, Composite parent) {
 
-		Viewer newViewer= CompareUIPlugin.findContentViewer(oldViewer, input, parent, fCompareConfiguration);
+		Viewer newViewer= CompareUI.findContentViewer(oldViewer, input, parent, fCompareConfiguration);
 		
 		boolean isNewViewer= newViewer != oldViewer;
 		if (DEBUG) System.out.println("CompareEditorInput.findContentViewer: " + isNewViewer); //$NON-NLS-1$
@@ -651,7 +713,7 @@ public abstract class CompareEditorInput implements IEditorInput, IPropertyChang
 	 
 	 * Returns <code>true</code> if this input has unsaved changes,
 	 * that is if <code>setDirty(true)</code> has been called.
-	 * Subclasses don't have to override if the functionality provided by <doce>setDirty</code>
+	 * Subclasses don't have to override if the functionality provided by <code>setDirty</code>
 	 * is sufficient.
 	 *
 	 * @return <code>true</code> if there are changes that need to be saved
@@ -687,7 +749,7 @@ public abstract class CompareEditorInput implements IEditorInput, IPropertyChang
 			fDirtyViewers.add(source);
 		else
 			fDirtyViewers.remove(source);
-		boolean newDirty= fDirtyViewers.size() > 0;
+		boolean newDirty= fDirty || fDirtyViewers.size() > 0;
 		if (DEBUG) System.out.println("setDirty("+source+", "+dirty+"): " + newDirty); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
 		if (oldDirty != newDirty)
 			Utilities.firePropertyChange(fListenerList, this, DIRTY_STATE, new Boolean(oldDirty), new Boolean(newDirty));
@@ -697,14 +759,16 @@ public abstract class CompareEditorInput implements IEditorInput, IPropertyChang
 	 * see IPropertyChangeNotifier.addListener
 	 */
 	public void addPropertyChangeListener(IPropertyChangeListener listener) {
-		fListenerList.add(listener);
+		if (listener != null)
+			fListenerList.add(listener);
 	}
 
 	/* (non Javadoc)
 	 * see IPropertyChangeNotifier.removeListener
 	 */
 	public void removePropertyChangeListener(IPropertyChangeListener listener) {
-		fListenerList.remove(listener);
+		if (listener != null)
+			fListenerList.remove(listener);
 	}
 
 	/**
@@ -712,10 +776,11 @@ public abstract class CompareEditorInput implements IEditorInput, IPropertyChang
 	 * Empty implementation.
 	 * Subclasses must override to save any changes.
 	 *
-	 * @param progressMonitor an <code>IProgressMonitor</code> that the implementation of save may use to show progress
+	 * @param pm an <code>IProgressMonitor</code> that the implementation of save may use to show progress
 	 * @deprecated Override method saveChanges instead.
 	 */
 	public void save(IProgressMonitor pm) {
+		// empty default implementation
 	}
 	
 	/**
@@ -724,7 +789,8 @@ public abstract class CompareEditorInput implements IEditorInput, IPropertyChang
 	 * This implementation tries to flush changes in all viewers by
 	 * calling <code>ISavable.save</code> on them.
 	 *
-	 * @param progressMonitor an <code>IProgressMonitor</code> that the implementation of save may use to show progress
+	 * @param pm an <code>IProgressMonitor</code> that the implementation of save may use to show progress
+	 * @throws CoreException
 	 * @since 2.0
 	 */
 	public void saveChanges(IProgressMonitor pm) throws CoreException {
@@ -737,7 +803,7 @@ public abstract class CompareEditorInput implements IEditorInput, IPropertyChang
 
 		save(pm);
 	}
-	
+		
 	private static void flushViewer(CompareViewerSwitchingPane pane, IProgressMonitor pm) throws CoreException {
 		if (pane != null) {
 			Viewer v= pane.getViewer();
