@@ -1,9 +1,9 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
+ * http://www.eclipse.org/legal/epl-v10.html
  * 
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -20,10 +20,11 @@ import java.util.Map;
 import javax.xml.parsers.DocumentBuilder;
 import javax.xml.parsers.DocumentBuilderFactory;
 import javax.xml.parsers.ParserConfigurationException;
+import javax.xml.transform.TransformerException;
 
-import org.apache.xerces.dom.DocumentImpl;
 import org.eclipse.core.resources.IContainer;
 import org.eclipse.core.resources.IFile;
+import org.eclipse.core.resources.IncrementalProjectBuilder;
 import org.eclipse.core.resources.ResourcesPlugin;
 import org.eclipse.core.runtime.CoreException;
 import org.eclipse.core.runtime.IPath;
@@ -33,6 +34,7 @@ import org.eclipse.core.runtime.NullProgressMonitor;
 import org.eclipse.core.runtime.Path;
 import org.eclipse.core.runtime.PlatformObject;
 import org.eclipse.core.runtime.Status;
+import org.eclipse.core.runtime.SubProgressMonitor;
 import org.eclipse.debug.core.DebugException;
 import org.eclipse.debug.core.DebugPlugin;
 import org.eclipse.debug.core.ILaunch;
@@ -41,11 +43,14 @@ import org.eclipse.debug.core.ILaunchConfigurationType;
 import org.eclipse.debug.core.ILaunchConfigurationWorkingCopy;
 import org.eclipse.debug.core.Launch;
 import org.eclipse.debug.core.model.ILaunchConfigurationDelegate;
+import org.eclipse.debug.core.model.ILaunchConfigurationDelegate2;
 import org.eclipse.debug.core.model.IPersistableSourceLocator;
+import org.eclipse.debug.core.sourcelookup.IPersistableSourceLocator2;
 import org.w3c.dom.Document;
 import org.w3c.dom.Element;
 import org.xml.sax.InputSource;
 import org.xml.sax.SAXException;
+import org.xml.sax.helpers.DefaultHandler;
 
 /**
  * Launch configuration handle.
@@ -76,7 +81,7 @@ public class LaunchConfiguration extends PlatformObject implements ILaunchConfig
 	 * 
 	 * @param memento launch configuration memento
 	 * @exception CoreException if the memento is invalid or
-	 * 	an exception occurrs reading the memento
+	 * 	an exception occurs reading the memento
 	 */
 	protected LaunchConfiguration(String memento) throws CoreException {
 		Exception ex = null;
@@ -84,6 +89,7 @@ public class LaunchConfiguration extends PlatformObject implements ILaunchConfig
 			Element root = null;
 			DocumentBuilder parser =
 				DocumentBuilderFactory.newInstance().newDocumentBuilder();
+			parser.setErrorHandler(new DefaultHandler());
 			StringReader reader = new StringReader(memento);
 			InputSource source = new InputSource(reader);
 			root = parser.parse(source).getDocumentElement();
@@ -93,9 +99,9 @@ public class LaunchConfiguration extends PlatformObject implements ILaunchConfig
 
 			String message = null;				
 			if (path == null) {
-				message = DebugCoreMessages.getString("LaunchConfiguration.Invalid_launch_configuration_memento__missing_path_attribute_3"); //$NON-NLS-1$
+				message = DebugCoreMessages.LaunchConfiguration_Invalid_launch_configuration_memento__missing_path_attribute_3; //$NON-NLS-1$
 			} else if (localString == null) {
-				message = DebugCoreMessages.getString("LaunchConfiguration.Invalid_launch_configuration_memento__missing_local_attribute_4"); //$NON-NLS-1$
+				message = DebugCoreMessages.LaunchConfiguration_Invalid_launch_configuration_memento__missing_local_attribute_4; //$NON-NLS-1$
 			}
 			if (message != null) {
 				IStatus s = newStatus(message, DebugException.INTERNAL_ERROR, null);
@@ -111,7 +117,7 @@ public class LaunchConfiguration extends PlatformObject implements ILaunchConfig
 			}
 			setLocation(location);
 			if (location == null) {
-				IStatus s = newStatus(MessageFormat.format(DebugCoreMessages.getString("LaunchConfiguration.Unable_to_restore_location_for_launch_configuration_from_memento__{0}_1"), new String[]{path}), DebugPlugin.INTERNAL_ERROR, null); //$NON-NLS-1$
+				IStatus s = newStatus(MessageFormat.format(DebugCoreMessages.LaunchConfiguration_Unable_to_restore_location_for_launch_configuration_from_memento___0__1, new String[]{path}), DebugPlugin.INTERNAL_ERROR, null); //$NON-NLS-1$
 				throw new CoreException(s);
 			}
 			return;
@@ -122,7 +128,7 @@ public class LaunchConfiguration extends PlatformObject implements ILaunchConfig
 		} catch (IOException e) {
 			ex = e;
 		}
-		IStatus s = newStatus(DebugCoreMessages.getString("LaunchConfiguration.Exception_occurred_parsing_memento_5"), DebugException.INTERNAL_ERROR, ex); //$NON-NLS-1$
+		IStatus s = newStatus(DebugCoreMessages.LaunchConfiguration_Exception_occurred_parsing_memento_5, DebugException.INTERNAL_ERROR, ex); //$NON-NLS-1$
 		throw new CoreException(s);
 	}
 	
@@ -143,28 +149,7 @@ public class LaunchConfiguration extends PlatformObject implements ILaunchConfig
 	 * @see ILaunchConfiguration#launch(String, IProgressMonitor)
 	 */
 	public ILaunch launch(String mode, IProgressMonitor monitor) throws CoreException {
-		// bug 28245 - force the delegate to load in case it is interested in launch notifications
-		ILaunchConfigurationDelegate delegate= getDelegate();
-		
-		ILaunch launch = new Launch(this, mode, null);
-		getLaunchManager().addLaunch(launch);
-		if (monitor == null) {
-			monitor= new NullProgressMonitor();
-		}
-		try {
-			initializeSourceLocator(launch);
-			delegate.launch(this, mode, launch, monitor);
-		} catch (CoreException e) {
-			// if there was an exception, and the launch is empty, remove it
-			if (!launch.hasChildren()) {
-				getLaunchManager().removeLaunch(launch);
-			}
-			throw e;
-		}
-		if (monitor.isCanceled()) {
-			getLaunchManager().removeLaunch(launch);
-		}
-		return launch;
+		return launch(mode, monitor, false);
 	}
 	
 	/**
@@ -176,13 +161,19 @@ public class LaunchConfiguration extends PlatformObject implements ILaunchConfig
 	protected void initializeSourceLocator(ILaunch launch) throws CoreException {
 		if (launch.getSourceLocator() == null) {
 			String type = getAttribute(ATTR_SOURCE_LOCATOR_ID, (String)null);
+			if (type == null) {
+				type = getType().getSourceLocatorId();
+			}
 			if (type != null) {
 				IPersistableSourceLocator locator = getLaunchManager().newSourceLocator(type);
 				String memento = getAttribute(ATTR_SOURCE_LOCATOR_MEMENTO, (String)null);
 				if (memento == null) {
 					locator.initializeDefaults(this);
 				} else {
-					locator.initializeFromMemento(memento);
+					if(locator instanceof IPersistableSourceLocator2)
+						((IPersistableSourceLocator2)locator).initializeFromMemento(memento, this);
+					else
+						locator.initializeFromMemento(memento);
 				}
 				launch.setSourceLocator(locator);
 			}
@@ -208,7 +199,9 @@ public class LaunchConfiguration extends PlatformObject implements ILaunchConfig
 	
 	private String getLastLocationSegment() {
 		String name = getLocation().lastSegment();
-		name = name.substring(0, name.length() - (LAUNCH_CONFIGURATION_FILE_EXTENSION.length() + 1));
+		if (name.length() > LAUNCH_CONFIGURATION_FILE_EXTENSION.length()) {
+			name = name.substring(0, name.length() - (LAUNCH_CONFIGURATION_FILE_EXTENSION.length() + 1));
+		}
 		return name;
 	}
 
@@ -317,8 +310,8 @@ public class LaunchConfiguration extends PlatformObject implements ILaunchConfig
 			if (isLocal()) {
 				if (!(getLocation().toFile().delete())) {
 					throw new DebugException(
-						new Status(Status.ERROR, DebugPlugin.getUniqueIdentifier(),
-						 DebugException.REQUEST_FAILED, DebugCoreMessages.getString("LaunchConfiguration.Failed_to_delete_launch_configuration._1"), null) //$NON-NLS-1$
+						new Status(IStatus.ERROR, DebugPlugin.getUniqueIdentifier(),
+						 DebugException.REQUEST_FAILED, DebugCoreMessages.LaunchConfiguration_Failed_to_delete_launch_configuration__1, null) //$NON-NLS-1$
 					);
 				}
 				// manually update the launch manager cache since there
@@ -358,14 +351,15 @@ public class LaunchConfiguration extends PlatformObject implements ILaunchConfig
 	
 	/**
 	 * Returns the launch configuration delegate for this
-	 * launch configuration.
+	 * launch configuration, for the specified launch mode.
 	 * 
+	 * @param mode launch mode
 	 * @return launch configuration delegate
 	 * @exception CoreException if the delegate was unable
 	 *  to be created
 	 */
-	protected ILaunchConfigurationDelegate getDelegate() throws CoreException {
-		return ((LaunchConfigurationType)getType()).getDelegate();
+	protected ILaunchConfigurationDelegate getDelegate(String mode) throws CoreException {
+		return getType().getDelegate(mode);
 	}
 	
 	/**
@@ -391,24 +385,33 @@ public class LaunchConfiguration extends PlatformObject implements ILaunchConfig
 			IFile file = getFile();
 			if (file == null) {
 				// cannot generate memento - missing file
-				IStatus status = newStatus(MessageFormat.format(DebugCoreMessages.getString("LaunchConfiguration.Unable_to_generate_memento_for_{0},_shared_file_does_not_exist._1"), new String[]{getName()}), DebugException.INTERNAL_ERROR, null); //$NON-NLS-1$
+				IStatus status = newStatus(MessageFormat.format(DebugCoreMessages.LaunchConfiguration_Unable_to_generate_memento_for__0___shared_file_does_not_exist__1, new String[]{getName()}), DebugException.INTERNAL_ERROR, null); //$NON-NLS-1$
 				throw new CoreException(status); 
 			}
 			relativePath = getFile().getFullPath();
 		}
-		
-		Document doc = new DocumentImpl();
-		Element node = doc.createElement("launchConfiguration"); //$NON-NLS-1$
-		doc.appendChild(node);
-		node.setAttribute("local", (new Boolean(isLocal())).toString()); //$NON-NLS-1$
-		node.setAttribute("path", relativePath.toString()); //$NON-NLS-1$
-		
+		Exception e= null;
 		try {
+			Document doc = LaunchManager.getDocument();
+			Element node = doc.createElement("launchConfiguration"); //$NON-NLS-1$
+			doc.appendChild(node);
+			node.setAttribute("local", (new Boolean(isLocal())).toString()); //$NON-NLS-1$
+			node.setAttribute("path", relativePath.toString()); //$NON-NLS-1$
 			return LaunchManager.serializeDocument(doc);
-		} catch (IOException e) {
-			IStatus status = newStatus(DebugCoreMessages.getString("LaunchConfiguration.Exception_occurred_creating_launch_configuration_memento_9"), DebugException.INTERNAL_ERROR,  e); //$NON-NLS-1$
+		} catch (IOException ioe) {
+			e= ioe;
+		} catch (ParserConfigurationException pce) {
+			e= pce;
+		} catch (TransformerException te) {
+			e= te;
+		}
+		if (e != null) {
+			IStatus status = newStatus(DebugCoreMessages.LaunchConfiguration_Exception_occurred_creating_launch_configuration_memento_9, DebugException.INTERNAL_ERROR,  e); //$NON-NLS-1$
 			throw new CoreException(status);
 		}
+		//execution will never reach here
+		return null;
+		
 	}
 
 	/**
@@ -418,7 +421,11 @@ public class LaunchConfiguration extends PlatformObject implements ILaunchConfig
 		if (isLocal()) {
 			return null;
 		}
-		return ResourcesPlugin.getWorkspace().getRoot().getFileForLocation(getLocation());
+		IFile[] files= ResourcesPlugin.getWorkspace().getRoot().findFilesForLocation(getLocation());
+		if (files.length > 0) {
+			return files[0];
+		}
+		return null;
 	}
 
 	/**
@@ -501,5 +508,97 @@ public class LaunchConfiguration extends PlatformObject implements ILaunchConfig
 		return info.getAttributes();
 	}
 
+	/* (non-Javadoc)
+	 * @see org.eclipse.debug.core.ILaunchConfiguration#launch(java.lang.String, org.eclipse.core.runtime.IProgressMonitor, boolean)
+	 */
+	public ILaunch launch(String mode, IProgressMonitor monitor, boolean build) throws CoreException {
+	    return launch(mode, monitor, build, true);
+	}
+	
+	
+    /* (non-Javadoc)
+     * @see org.eclipse.debug.core.ILaunchConfiguration#launch(java.lang.String, org.eclipse.core.runtime.IProgressMonitor, boolean, boolean)
+     */
+    public ILaunch launch(String mode, IProgressMonitor monitor, boolean build, boolean register) throws CoreException {
+		// bug 28245 - force the delegate to load in case it is interested in launch notifications
+		ILaunchConfigurationDelegate delegate= getDelegate(mode);
+		ILaunchConfigurationDelegate2 delegate2 = null;
+		if (delegate instanceof ILaunchConfigurationDelegate2) {
+			delegate2 = (ILaunchConfigurationDelegate2) delegate;
+		}
+		// allow the delegate to provide a launch implementation
+		ILaunch launch = null;
+		if (delegate2 != null) {
+			launch = delegate2.getLaunch(this, mode);
+		}
+		if (launch == null) {
+			launch = new Launch(this, mode, null);
+		} else {
+			// ensure the launch mode is valid
+			if (!mode.equals(launch.getLaunchMode())) {
+				IStatus status = new Status(IStatus.ERROR, DebugPlugin.getUniqueIdentifier(), DebugPlugin.INTERNAL_ERROR, 
+						MessageFormat.format(DebugCoreMessages.LaunchConfiguration_13, new String[]{mode, launch.getLaunchMode()}), null); //$NON-NLS-1$
+				throw new CoreException(status);
+			}
+		}
+		
+		boolean captureOutput = getAttribute(DebugPlugin.ATTR_CAPTURE_OUTPUT, true);
+		if(!captureOutput) {
+		    launch.setAttribute(DebugPlugin.ATTR_CAPTURE_OUTPUT, "false"); //$NON-NLS-1$
+		} else {
+		    launch.setAttribute(DebugPlugin.ATTR_CAPTURE_OUTPUT, null);
+		}
+		
+		
+		if (monitor == null) {
+			monitor= new NullProgressMonitor();
+		}		
+		// perform initial pre-launch sanity checks
+		if (delegate2 != null) {
+			if (!(delegate2.preLaunchCheck(this, mode, monitor))) {
+				// canceled
+				monitor.setCanceled(true);
+				return launch;
+			}
+		}
+		// preform pre-launch build
+		IProgressMonitor subMonitor = monitor;
+		if (build) {
+			subMonitor = new SubProgressMonitor(monitor, 100);
+			if (delegate2 != null) {
+				build = delegate2.buildForLaunch(this, mode, subMonitor);
+			}
+			if (build) {
+				ResourcesPlugin.getWorkspace().build(IncrementalProjectBuilder.INCREMENTAL_BUILD, subMonitor);				
+			}
+			subMonitor = new SubProgressMonitor(monitor, 100);
+		}
+		// final validation
+		if (delegate2 != null) {
+			if (!(delegate2.finalLaunchCheck(this, mode, subMonitor))) {
+				// canceled
+				monitor.setCanceled(true);
+				return launch;
+			}
+		}
+		
+		if (register) {
+		    getLaunchManager().addLaunch(launch);
+		}
+		try {
+			initializeSourceLocator(launch);
+			delegate.launch(this, mode, launch, subMonitor);
+		} catch (CoreException e) {
+			// if there was an exception, and the launch is empty, remove it
+			if (!launch.hasChildren()) {
+				getLaunchManager().removeLaunch(launch);
+			}
+			throw e;
+		}
+		if (monitor.isCanceled()) {
+			getLaunchManager().removeLaunch(launch);
+		}
+		return launch;
+    }
 }
 
