@@ -1,9 +1,9 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
+ * http://www.eclipse.org/legal/epl-v10.html
  * 
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -21,15 +21,14 @@ import org.eclipse.debug.core.DebugException;
 import org.eclipse.debug.core.DebugPlugin;
 import org.eclipse.debug.core.ILaunch;
 import org.eclipse.debug.core.ILaunchManager;
-import org.eclipse.debug.core.ILaunchesListener;
+import org.eclipse.debug.core.ILaunchesListener2;
+import org.eclipse.debug.core.model.IDebugElement;
 import org.eclipse.debug.core.model.IDebugTarget;
 import org.eclipse.debug.core.model.IProcess;
 import org.eclipse.debug.core.model.IStackFrame;
 import org.eclipse.debug.core.model.IThread;
 import org.eclipse.debug.internal.ui.DebugUIPlugin;
-import org.eclipse.debug.internal.ui.InstructionPointerManager;
 import org.eclipse.debug.internal.ui.views.AbstractDebugEventHandler;
-import org.eclipse.jface.viewers.IBasicPropertyConstants;
 import org.eclipse.jface.viewers.IStructuredSelection;
 import org.eclipse.jface.viewers.ITreeContentProvider;
 import org.eclipse.jface.viewers.StructuredSelection;
@@ -37,7 +36,7 @@ import org.eclipse.jface.viewers.StructuredSelection;
 /**
  * Handles debug events, updating the launch view and viewer.
  */
-public class LaunchViewEventHandler extends AbstractDebugEventHandler implements ILaunchesListener {
+public class LaunchViewEventHandler extends AbstractDebugEventHandler implements ILaunchesListener2 {
 	/**
 	 * The timer used to time step and evaluation events. The timer allows
 	 * the UI to not refresh during fast evaluations and steps.
@@ -61,53 +60,99 @@ public class LaunchViewEventHandler extends AbstractDebugEventHandler implements
 	}
 	
 
+    /* (non-Javadoc)
+     * @see org.eclipse.debug.internal.ui.views.AbstractDebugEventHandler#filterEvents(org.eclipse.debug.core.DebugEvent[])
+     */
+    protected DebugEvent[] filterEvents(DebugEvent[] events) {
+        if (events.length > 0) {
+            DebugEvent event = events[0];
+            Object source = event.getSource();
+            ILaunch launch = null;
+            if (source instanceof IDebugElement) {
+                launch = ((IDebugElement)source).getLaunch();
+            } else if (source instanceof IProcess) {
+                launch = ((IProcess)source).getLaunch();
+            }
+            // we only need to consider the first event, as all events in an event set come
+            // from the same program
+            if (launch != null && DebugPlugin.getDefault().getLaunchManager().isRegistered(launch)) {
+                return events;
+            }
+            return EMPTY_EVENT_SET;
+        }
+        return events;
+    }
+	
+	/**
+	 * Returns the parent for the given element.
+	 * 
+	 * @param element
+	 * @return parent
+	 */
+	private Object getParent(Object element) {
+		return ((ITreeContentProvider)getTreeViewer().getContentProvider()).getParent(element);
+	}
+    
 	/**
 	 * @see AbstractDebugEventHandler#doHandleDebugEvents(DebugEvent[])
 	 */
-	protected void doHandleDebugEvents(DebugEvent[] events) {
+	protected void doHandleDebugEvents(DebugEvent[] events, Object data) {
 		fThreadTimer.handleDebugEvents(events);
 		Object suspendee = null;
 		for (int i = 0; i < events.length; i++) {
 			DebugEvent event = events[i];
 			Object source= event.getSource();
-			if (!(source instanceof IStackFrame || source instanceof IThread || source instanceof IDebugTarget || source instanceof IProcess)) {
-				// the launch view is not interested in any other types of elements
-				return;
-			}
 			switch (event.getKind()) {
 				case DebugEvent.CREATE :
-					insert(source);
-					if (source instanceof IDebugTarget) {
-						ILaunch launch= ((IDebugTarget)source).getLaunch();
-						getLaunchView().autoExpand(launch, true, true);
+					if (source instanceof IThread) {
+						insert(source);
+					} else {
+						Object parent = getParent(source);
+						if (parent != null) {
+							refresh(parent);
+						}
+						if (source instanceof IDebugTarget | source instanceof IProcess) {
+							getLaunchView().autoExpand(source, true);
+						}
 					}
 					break;
 				case DebugEvent.TERMINATE :
-					removeInstructionPointerAnnotations(source);
+					clearSourceSelection(source);
 					if (source instanceof IThread) {
-						clearSourceSelection((IThread)source);
 						fThreadTimer.getTimedOutThreads().remove(source);
 						remove(source);
 					} else {
-						clearSourceSelection(null);
-						Object parent = ((ITreeContentProvider)getTreeViewer().getContentProvider()).getParent(source);
-						refresh(parent);
+					    Object parent = getParent(source);
+						if (parent != null) {
+						    refresh(parent);
+						}
 					}
 					break;
 				case DebugEvent.RESUME :
-					doHandleResumeEvent(event, source);
+					doHandleResumeEvent(event, source, data);
 					break;
 				case DebugEvent.SUSPEND :
 					if (suspendee == null || !suspendee.equals(source)) {
-						doHandleSuspendEvent(source, event);
+						doHandleSuspendEvent(source, event, data);
 						suspendee = source;
 					}
 					break;
 				case DebugEvent.CHANGE :
+                    Object element = null;
+                    IStructuredSelection selection = getLaunchViewer().getDeferredSelection();
+                    if (selection == null) {
+                        selection = (IStructuredSelection) getLaunchViewer().getSelection();
+                    } 
+                    
+                    element = selection.getFirstElement();
+                    IStackFrame lastFrame = null;
+                    if (element instanceof IStackFrame) {
+                        lastFrame = (IStackFrame) element;
+                    }
 					if (source instanceof IStackFrame) {
-						IStackFrame lastFrame= getLaunchView().getStackFrame();
 						if (source.equals(lastFrame)) {
 							getLaunchView().setStackFrame(null);
+							getLaunchView().autoExpand(lastFrame, true);
 						}
 					}
 					if (event.getDetail() == DebugEvent.STATE) {
@@ -116,6 +161,11 @@ public class LaunchViewEventHandler extends AbstractDebugEventHandler implements
 						//structural change
 						refresh(source);
 					}
+					if (lastFrame != null && source instanceof IThread) {
+					    if (lastFrame.getThread().equals(source)) {
+					        getLaunchView().autoExpand(lastFrame, true);
+					    }
+					}
 					break;
 			}
 		}
@@ -124,9 +174,9 @@ public class LaunchViewEventHandler extends AbstractDebugEventHandler implements
 	/**
 	 * Handles the given resume event with the given source.
 	 */
-	protected void doHandleResumeEvent(DebugEvent event, Object source) {
+	protected void doHandleResumeEvent(DebugEvent event, Object source, Object data) {
 		if (!event.isEvaluation()) {
-			removeInstructionPointerAnnotations(source);
+			clearSourceSelection(source);
 		}
 		if (event.isEvaluation() || event.isStepStart()) {
 			// Do not update for step starts and evaluation
@@ -136,14 +186,15 @@ public class LaunchViewEventHandler extends AbstractDebugEventHandler implements
 				fThreadTimer.startTimer(thread);
 			}
 			return;
-		}	
-		clearSourceSelection(null);
+		}
 		refresh(source);
 		if (source instanceof IThread) {
-			selectAndReveal(source);
-			return;
-		}	
-		labelChanged(source);
+		    if (data instanceof IStackFrame) {
+				selectAndReveal(data);
+				return;
+			}
+            selectAndReveal(source);
+		}
 	}
 	
 	/**
@@ -154,98 +205,87 @@ public class LaunchViewEventHandler extends AbstractDebugEventHandler implements
 	 */
 	protected void updateRunningThread(IThread thread) {
 		labelChanged(thread);
-		getLaunchViewer().updateStackFrameIcons(thread);
-		getLaunchView().clearSourceSelection();
-		// fire selection change to update actions
-		getLaunchViewer().setSelection(getLaunchViewer().getSelection());
+		getLaunchViewer().updateStackFrameImages(thread);
+		clearSourceSelection(thread);
 	}
 
-	protected void doHandleSuspendEvent(Object element, DebugEvent event) {
+	protected void doHandleSuspendEvent(Object element, DebugEvent event, Object data) {
 		IThread thread= getThread(element);
 		if (thread != null) {
-			fThreadTimer.stopTimer((IThread)element);
+			fThreadTimer.stopTimer(thread);
 		}
 		
 		boolean wasTimedOut= fThreadTimer.getTimedOutThreads().remove(thread);
 		if (event.isEvaluation() && ((event.getDetail() & DebugEvent.EVALUATION_IMPLICIT) != 0)) {
-			if (thread != null && wasTimedOut) {
-				// Refresh the thread label when a timed out evaluation finishes.
-				// This is necessary because the timeout updates
-				// the label when it occurs
-				updateRunningThread(thread);
+			if (thread != null && !wasTimedOut) {
+				// No refresh required for implicit evaluations that complete on time
+				return;
 			}
-			// Don't refresh fully for evaluation completion.
-			return;
 		}
 		if (element instanceof IThread) {
-			doHandleSuspendThreadEvent((IThread)element, event, wasTimedOut);
+			doHandleSuspendThreadEvent((IThread)element, event, wasTimedOut, data);
 			return;
 		}
 		refresh(element);
 	}
 	
-	/**
-	 * Remove all instruction pointer annotations associated with the specified element.
-	 */
-	protected void removeInstructionPointerAnnotations(Object element) {
-		if (element instanceof IThread) {
-			InstructionPointerManager.getDefault().removeAnnotations((IThread)element);
-		} else if (element instanceof IDebugTarget) {
-			InstructionPointerManager.getDefault().removeAnnotations((IDebugTarget)element);			
-		}
-	}
-	
 	/**
 	 * Updates the given thread for the given suspend event.
 	 */
-	protected void doHandleSuspendThreadEvent(IThread thread, DebugEvent event, boolean wasTimedOut) {
+	protected void doHandleSuspendThreadEvent(IThread thread, DebugEvent event, boolean wasTimedOut, Object data) {
 		// if the thread has already resumed, do nothing
-		if (!thread.isSuspended()) {
+		if (!thread.isSuspended() || !isAvailable()) {
 			return;
 		}
 
 		// do not update source selection for evaluation events
 		boolean evaluationEvent = event.isEvaluation();
 		
+		// get the top frame
+		IStackFrame frame = null;
+		if (data instanceof IStackFrame) {
+		    frame = (IStackFrame) data;
+		}
+	    
 		// if the top frame is the same, only update labels and images, and re-select
 		// the frame to display source
-		try {
-			IStackFrame frame = thread.getTopStackFrame();
-			if (frame != null && frame.equals(fLastStackFrame)) {
-				Object[] objectsToUpdate= new Object[]{thread, frame};
-				if (wasTimedOut) {
-					getLaunchViewer().updateStackFrameIcons(thread);
-				}
-				getLaunchViewer().update(objectsToUpdate, new String[] {IBasicPropertyConstants.P_IMAGE, IBasicPropertyConstants.P_TEXT});
-				if (!evaluationEvent) {
-					getLaunchViewer().setSelection(new StructuredSelection(frame));
-				}
-				return;
+		if (frame != null && frame.equals(fLastStackFrame)) {
+			if (wasTimedOut) {
+				getLaunchViewer().updateStackFrameImages(thread);
 			}
-		} catch (DebugException e) {
+			getLaunchViewer().update(new Object[] {thread, frame}, null);
+			if (!evaluationEvent) {
+			    getLaunchViewer().deferExpansion(thread);
+				getLaunchViewer().deferSelection(new StructuredSelection(frame));
+			} else if (wasTimedOut) {
+				getLaunchView().showEditorForCurrentSelection();
+			}
+			return;
 		}
 		
-		// Auto-expand the thread. Only select the thread if this wasn't the end
-		// of an evaluation		
-		
-		getLaunchView().autoExpand(thread, true, !evaluationEvent);
-		
-		try {
-			fLastStackFrame = thread.getTopStackFrame();
-		} catch (DebugException e) {
-			fLastStackFrame = null;
+		if (frame == null) {
+			// suspend event, but no frames in the thead
+			fLastStackFrame = null;			
+			refresh(thread);
+			getLaunchView().autoExpand(thread, !evaluationEvent);
+		} else {
+		    fLastStackFrame = frame;
+			// Auto-expand the thread. Only select the thread if this wasn't the end
+			// of an evaluation
+		    refresh(thread);
+			getLaunchView().autoExpand(frame, !evaluationEvent);
 		}
 	}
 	
 	/**
 	 * @see AbstractDebugEventHandler#updateForDebugEvents(DebugEvent[])
 	 */
-	protected void updateForDebugEvents(DebugEvent[] events) {
-		super.updateForDebugEvents(events);
+	protected void updateForDebugEvents(DebugEvent[] events, Object data) {
+		super.updateForDebugEvents(events, data);
 		if (isViewVisible()) {
 			return;
 		}
-		doHandleDebugEvents(events);
+		doHandleDebugEvents(events, data);
 	}
 	
 	/**
@@ -261,27 +301,11 @@ public class LaunchViewEventHandler extends AbstractDebugEventHandler implements
 	/**
 	 * Clear the selection in the editor - must be called in UI thread
 	 */
-	private void clearSourceSelection(IThread thread) {
+	private void clearSourceSelection(Object source) {
 		if (getViewer() != null) {
-			if (thread != null) {
-				IStructuredSelection selection= (IStructuredSelection)getLaunchViewer().getSelection();
-				Object element= selection.getFirstElement();
-				if (element instanceof IStackFrame) {
-					IStackFrame stackFrame = (IStackFrame) element;
-					if (!stackFrame.getThread().equals(thread)) {
-						//do not clear the source selection
-						//a thread has terminated that is not the
-						//parent of the currently selected stack frame
-						return;
-					}
-				} else {
-					return;
-				}
-			}
-		
-			getLaunchView().clearSourceSelection();
+			getLaunchView().clearSourceSelection(source);
 		}
-	}
+	}	
 
 	/**
 	 * Returns this event handler's launch viewer
@@ -318,6 +342,16 @@ public class LaunchViewEventHandler extends AbstractDebugEventHandler implements
 		 * The time allotted before a thread will be updated
 		 */
 		private long TIMEOUT= 500;
+		/**
+		 * Time in milliseconds that the thread timer started
+		 * running with no timers.
+		 */
+		private long timeEmpty= 0;
+		/**
+		 * The maximum time in milliseconds that the thread
+		 * will continue running with no timers.
+		 */
+		private long MAX_TIME_EMPTY= 3000;
 		private boolean fStopped= false;
 		private Object fLock= new Object();
 		
@@ -400,6 +434,7 @@ public class LaunchViewEventHandler extends AbstractDebugEventHandler implements
 					
 				}
 			}, "Thread timer"); //$NON-NLS-1$
+			fThread.setDaemon(true);
 			fThread.start();
 		}
 		
@@ -407,6 +442,16 @@ public class LaunchViewEventHandler extends AbstractDebugEventHandler implements
 			long timeToWait= TIMEOUT;
 			Map.Entry[] entries;
 			synchronized (fLock) {
+				if (fStopTimes.size() == 0) {
+					if (timeEmpty == 0) {
+						timeEmpty= System.currentTimeMillis();
+					} else 	if (System.currentTimeMillis() - timeEmpty > MAX_TIME_EMPTY) {
+						stop();
+						return;
+					}
+				} else {
+					timeEmpty= 0;
+				}
 				entries= (Map.Entry[])fStopTimes.entrySet().toArray(new Map.Entry[0]);
 			}
 			long stopTime, currentTime= System.currentTimeMillis();
@@ -418,7 +463,7 @@ public class LaunchViewEventHandler extends AbstractDebugEventHandler implements
 				if (entryValue == null) {
 					continue;
 				}
-				stopTime= ((Long)entryValue).longValue();
+				stopTime= entryValue.longValue();
 				if (stopTime <= currentTime) {
 					// The timer has expired for this thread.
 					// Refresh the UI to show that the thread
@@ -456,7 +501,7 @@ public class LaunchViewEventHandler extends AbstractDebugEventHandler implements
 					}
 					for (int i = 0; i < launches.length; i++) {
 						if (launches[i].hasChildren()) {
-							getLaunchView().autoExpand(launches[i], false, i == (launches.length - 1));
+							getLaunchView().autoExpand(launches[i], false);
 						}
 					}					
 
@@ -481,14 +526,14 @@ public class LaunchViewEventHandler extends AbstractDebugEventHandler implements
 					}
 					for (int i = 0; i < launches.length; i++) {
 						if (launches[i].hasChildren()) {
-							getLaunchView().autoExpand(launches[i], false, i == (launches.length - 1));
+							getLaunchView().autoExpand(launches[i], false);
 						}						
 					}
 				}
 			}
 		};
-
-		getView().syncExec(r);				
+		
+		getView().asyncExec(r);				
 	}
 
 	/**
@@ -503,6 +548,9 @@ public class LaunchViewEventHandler extends AbstractDebugEventHandler implements
 					} else {
 						refresh();
 					}
+					
+					getLaunchView().cleanupLaunches(launches);
+					
 					ILaunchManager lm= DebugPlugin.getDefault().getLaunchManager();
 					IDebugTarget[] targets= lm.getDebugTargets();
 					if (targets.length > 0) {
@@ -511,7 +559,10 @@ public class LaunchViewEventHandler extends AbstractDebugEventHandler implements
 							IThread[] threads= target.getThreads();
 							for (int i=0; i < threads.length; i++) {
 								if (threads[i].isSuspended()) {
-									getLaunchView().autoExpand(threads[i], false, true);
+									IStackFrame topStackFrame = threads[i].getTopStackFrame();
+									if (topStackFrame != null) {
+									    getLaunchView().autoExpand(topStackFrame, true);
+									}
 									return;
 								}
 							}						
@@ -519,7 +570,7 @@ public class LaunchViewEventHandler extends AbstractDebugEventHandler implements
 							DebugUIPlugin.log(de);
 						}
 						
-						getLaunchView().autoExpand(target.getLaunch(), false, true);
+						getLaunchView().autoExpand(target.getLaunch(), true);
 					}
 				}
 			}
@@ -528,4 +579,66 @@ public class LaunchViewEventHandler extends AbstractDebugEventHandler implements
 		getView().asyncExec(r);		
 	}
 
+
+	/* (non-Javadoc)
+	 * @see org.eclipse.debug.core.ILaunchesListener2#launchesTerminated(org.eclipse.debug.core.ILaunch[])
+	 */
+	public void launchesTerminated(final ILaunch[] launches) {
+		if (fLastStackFrame != null) {
+			ILaunch launch= fLastStackFrame.getLaunch();
+			for (int i = 0; i < launches.length; i++) {
+				ILaunch terminatedLaunch = launches[i];
+				if (terminatedLaunch.equals(launch)) {
+					fLastStackFrame= null;
+				}
+			}
+		}
+		Runnable r= new Runnable() {
+			public void run() {
+				getLaunchView().cleanupLaunches(launches);
+			}
+		};
+		getView().asyncExec(r);
+	}
+
+    /* (non-Javadoc)
+     * @see org.eclipse.debug.internal.ui.views.AbstractDebugEventHandler#doPreprocessEvents(org.eclipse.debug.core.DebugEvent[])
+     */
+    protected DebugEvent[] doPreprocessEvents(DebugEvent[] events) {
+        for (int i = 0; i < events.length; i++) {
+            DebugEvent event = events[i];
+            Object source = event.getSource();
+            switch (event.getKind()) {
+            	case DebugEvent.SUSPEND:
+            	    if (source instanceof IThread) {
+            	        IThread thread = (IThread)source;
+            		    try {
+            		        IStackFrame frame = thread.getTopStackFrame();
+            		        queueData(frame);
+            		    } catch (DebugException e) {
+            		    }
+            	    }
+            	    break;
+            	case DebugEvent.RESUME:
+            		if (source instanceof IThread && event.getDetail() == DebugEvent.CLIENT_REQUEST) {
+            			// When a thread resumes, try to select another suspended thread
+            			// in the same target.
+            			try {
+            			    IDebugTarget target = ((IThread) source).getDebugTarget();
+            				IThread[] threads= target.getThreads();
+            				for (int j = 0; j < threads.length; j++) {
+            					IStackFrame frame = threads[j].getTopStackFrame();
+            					if (frame != null) {
+            						queueData(frame);
+            						break;
+            					}
+            				}
+            			} catch (DebugException e) {
+            			}
+            		}
+            		break;
+            }
+        }
+        return events;
+    }
 }
