@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2004 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -24,6 +24,9 @@ import org.eclipse.core.runtime.IPath;
 import org.eclipse.core.runtime.IProgressMonitor;
 import org.eclipse.jdt.core.*;
 import org.eclipse.jdt.core.compiler.CharOperation;
+import org.eclipse.jdt.internal.core.util.MementoTokenizer;
+import org.eclipse.jdt.internal.core.util.Messages;
+import org.eclipse.jdt.internal.core.util.Util;
 
 /**
  * @see IPackageFragmentRoot
@@ -54,8 +57,8 @@ public class PackageFragmentRoot extends Openable implements IPackageFragmentRoo
  * Constructs a package fragment root which is the root of the java package
  * directory hierarchy.
  */
-protected PackageFragmentRoot(IResource resource, IJavaProject project, String name) {
-	super(PACKAGE_FRAGMENT_ROOT, project, name);
+protected PackageFragmentRoot(IResource resource, JavaProject project) {
+	super(project);
 	this.resource = resource;
 }
 
@@ -66,7 +69,7 @@ public void attachSource(IPath sourcePath, IPath rootPath, IProgressMonitor moni
 	try {
 		verifyAttachSource(sourcePath);
 		if (monitor != null) {
-			monitor.beginTask(Util.bind("element.attachingSource"), 2); //$NON-NLS-1$
+			monitor.beginTask(Messages.element_attachingSource, 2); 
 		}
 		SourceMapper oldMapper= getSourceMapper();
 		IWorkspace workspace = ResourcesPlugin.getWorkspace();
@@ -157,11 +160,25 @@ public void attachSource(IPath sourcePath, IPath rootPath, IProgressMonitor moni
 	}
 }
 
+/**
+ * @see Openable
+ */
+protected boolean buildStructure(OpenableElementInfo info, IProgressMonitor pm, Map newElements, IResource underlyingResource) throws JavaModelException {
+	
+	// check whether this pkg fragment root can be opened
+	IStatus status = validateOnClasspath();
+	if (!status.isOK()) throw newJavaModelException(status);
+	if (!resourceExists()) throw newNotPresentException();
+
+	((PackageFragmentRootInfo) info).setRootKind(determineKind(underlyingResource));
+	return computeChildren(info, newElements);
+}
+
 SourceMapper createSourceMapper(IPath sourcePath, IPath rootPath) {
 	SourceMapper mapper = new SourceMapper(
 		sourcePath, 
 		rootPath == null ? null : rootPath.toOSString(), 
-		this.isExternal() ? JavaCore.getOptions() : this.getJavaProject().getOptions(true)); // only project options if associated with resource
+		getJavaProject().getOptions(true)); // cannot use workspace options if external jar is 1.5 jar and workspace options are 1.4 options
 	return mapper;
 }
 /*
@@ -174,33 +191,27 @@ public void delete(
 	throws JavaModelException {
 
 	DeletePackageFragmentRootOperation op = new DeletePackageFragmentRootOperation(this, updateResourceFlags, updateModelFlags);
-	runOperation(op, monitor);
+	op.runOperation(monitor);
 }
 
-/**
- * This root is being closed.  If this root has an associated source attachment, 
- * close it too.
- *
- * @see JavaElement
- */
-//protected void closing(Object info) throws JavaModelException { TODO remove after 2.1
-//	((PackageFragmentRootInfo) info).sourceMapper = null;
-//	super.closing(info);
-//}
 /**
  * Compute the package fragment children of this package fragment root.
  * 
  * @exception JavaModelException  The resource associated with this package fragment root does not exist
  */
-protected boolean computeChildren(OpenableElementInfo info) throws JavaModelException {
+protected boolean computeChildren(OpenableElementInfo info, Map newElements) throws JavaModelException {
+	// Note the children are not opened (so not added to newElements) for a regular package fragment root
+	// Howver they are opened for a Jar package fragment root (see JarPackageFragmentRoot#computeChildren)
 	try {
 		// the underlying resource may be a folder or a project (in the case that the project folder
 		// is actually the package fragment root)
-		IResource resource = getResource();
-		if (resource.getType() == IResource.FOLDER || resource.getType() == IResource.PROJECT) {
+		IResource underlyingResource = getResource();
+		if (underlyingResource.getType() == IResource.FOLDER || underlyingResource.getType() == IResource.PROJECT) {
 			ArrayList vChildren = new ArrayList(5);
+			IContainer rootFolder = (IContainer) underlyingResource;
+			char[][] inclusionPatterns = fullInclusionPatternChars();
 			char[][] exclusionPatterns = fullExclusionPatternChars();
-			computeFolderChildren((IContainer) resource, "", vChildren, exclusionPatterns); //$NON-NLS-1$
+			computeFolderChildren(rootFolder, !Util.isExcluded(rootFolder, inclusionPatterns, exclusionPatterns), CharOperation.NO_STRINGS, vChildren, inclusionPatterns, exclusionPatterns); //$NON-NLS-1$
 			IJavaElement[] children = new IJavaElement[vChildren.size()];
 			vChildren.toArray(children);
 			info.setChildren(children);
@@ -219,29 +230,46 @@ protected boolean computeChildren(OpenableElementInfo info) throws JavaModelExce
  * 
  * @exception JavaModelException  The resource associated with this package fragment does not exist
  */
-protected void computeFolderChildren(IContainer folder, String prefix, ArrayList vChildren, char[][] exclusionPatterns) throws JavaModelException {
-	IPackageFragment pkg = getPackageFragment(prefix);
-	vChildren.add(pkg);
+protected void computeFolderChildren(IContainer folder, boolean isIncluded, String[] pkgName, ArrayList vChildren, char[][] inclusionPatterns, char[][] exclusionPatterns) throws JavaModelException {
+
+	if (isIncluded) {
+	    IPackageFragment pkg = getPackageFragment(pkgName);
+		vChildren.add(pkg);
+	}
 	try {
 		JavaProject javaProject = (JavaProject)getJavaProject();
+		JavaModelManager manager = JavaModelManager.getJavaModelManager();
 		IResource[] members = folder.members();
+		boolean hasIncluded = isIncluded;
 		for (int i = 0, max = members.length; i < max; i++) {
 			IResource member = members[i];
 			String memberName = member.getName();
-			if (member.getType() == IResource.FOLDER 
-				&& Util.isValidFolderNameForPackage(memberName)
-				&& !Util.isExcluded(member, exclusionPatterns)) {
-					
-				// eliminate binary output only if nested inside direct subfolders
-				if (javaProject.contains(member)) {
-					String newPrefix;
-					if (prefix.length() == 0) {
-						newPrefix = memberName;
-					} else {
-						newPrefix = prefix + "." + memberName; //$NON-NLS-1$
+			
+			switch(member.getType()) {
+			    
+			    case IResource.FOLDER:
+					if (Util.isValidFolderNameForPackage(memberName)) {
+					    boolean isMemberIncluded = !Util.isExcluded(member, inclusionPatterns, exclusionPatterns);
+						// keep looking inside as long as included already, or may have child included due to inclusion patterns
+					    if (isMemberIncluded || inclusionPatterns != null) { 
+							// eliminate binary output only if nested inside direct subfolders
+							if (javaProject.contains(member)) {
+								String[] newNames = Util.arrayConcat(pkgName, manager.intern(memberName));
+								computeFolderChildren((IFolder) member, isMemberIncluded, newNames, vChildren, inclusionPatterns, exclusionPatterns);
+							}
+						}
 					}
-					computeFolderChildren((IFolder) member, newPrefix, vChildren, exclusionPatterns);
-				}
+			    	break;
+			    case IResource.FILE:
+			        // inclusion filter may only include files, in which case we still want to include the immediate parent package (lazily)
+					if (!hasIncluded
+								&& Util.isValidCompilationUnitName(memberName)
+								&& !Util.isExcluded(member, inclusionPatterns, exclusionPatterns)) {
+						hasIncluded = true;
+					    IPackageFragment pkg = getPackageFragment(pkgName);
+					    vChildren.add(pkg); 
+					}
+			        break;
 			}
 		}
 	} catch(IllegalArgumentException e){
@@ -250,26 +278,7 @@ protected void computeFolderChildren(IContainer folder, String prefix, ArrayList
 		throw new JavaModelException(e);
 	}
 }
-/*
- * Computes and returns the source attachment root path for the given source attachment path.
- * Returns <code>null</code> if none could be found.
- * 
- * @param sourceAttachmentPath the given absolute path to the source archive or folder
- * @return the computed source attachment root path or <code>null</cde> if none could be found
- * @throws JavaModelException
- */
-public IPath computeSourceAttachmentRootPath(IPath sourceAttachmentPath) throws JavaModelException {
-	IPath sourcePath = this.getSourceAttachmentPath();
-	if (sourcePath == null) return null;
-	SourceMapper mapper = 
-		new SourceMapper(
-		sourcePath, 
-		null, // detect root path
-		this.isExternal() ? JavaCore.getOptions() : this.getJavaProject().getOptions(true) // only project options if associated with resource
-	);
-	if (mapper.rootPath == null) return null;
-	return new Path(mapper.rootPath);
-}
+
 /*
  * @see org.eclipse.jdt.core.IPackageFragmentRoot#copy
  */
@@ -283,36 +292,34 @@ public void copy(
 		
 	CopyPackageFragmentRootOperation op = 
 		new CopyPackageFragmentRootOperation(this, destination, updateResourceFlags, updateModelFlags, sibling);
-	runOperation(op, monitor);
+	op.runOperation(monitor);
 }
 
-
-
 /**
  * Returns a new element info for this element.
  */
-protected OpenableElementInfo createElementInfo() {
+protected Object createElementInfo() {
 	return new PackageFragmentRootInfo();
 }
 
 /**
  * @see IPackageFragmentRoot
  */
-public IPackageFragment createPackageFragment(String name, boolean force, IProgressMonitor monitor) throws JavaModelException {
-	CreatePackageFragmentOperation op = new CreatePackageFragmentOperation(this, name, force);
-	runOperation(op, monitor);
-	return getPackageFragment(name);
+public IPackageFragment createPackageFragment(String pkgName, boolean force, IProgressMonitor monitor) throws JavaModelException {
+	CreatePackageFragmentOperation op = new CreatePackageFragmentOperation(this, pkgName, force);
+	op.runOperation(monitor);
+	return getPackageFragment(op.pkgName);
 }
 
 /**
  * Returns the root's kind - K_SOURCE or K_BINARY, defaults
  * to K_SOURCE if it is not on the classpath.
  *
- * @exception NotPresentException if the project and root do
+ * @exception JavaModelException if the project and root do
  * 		not exist.
  */
 protected int determineKind(IResource underlyingResource) throws JavaModelException {
-	IClasspathEntry[] entries= ((JavaProject)getJavaProject()).getExpandedClasspath(true);
+	IClasspathEntry[] entries= ((JavaProject)getJavaProject()).getResolvedClasspath(true);
 	for (int i= 0; i < entries.length; i++) {
 		IClasspathEntry entry= entries[i];
 		if (entry.getPath().equals(underlyingResource.getFullPath())) {
@@ -325,7 +332,7 @@ protected int determineKind(IResource underlyingResource) throws JavaModelExcept
 /**
  * Compares two objects for equality;
  * for <code>PackageFragmentRoot</code>s, equality is having the
- * same <code>JavaModel</code>, same resources, and occurrence count.
+ * same parent, same resources, and occurrence count.
  *
  */
 public boolean equals(Object o) {
@@ -334,17 +341,15 @@ public boolean equals(Object o) {
 	if (!(o instanceof PackageFragmentRoot))
 		return false;
 	PackageFragmentRoot other = (PackageFragmentRoot) o;
-	return getJavaModel().equals(other.getJavaModel()) && 
-			this.resource.equals(other.resource) &&
-			fOccurrenceCount == other.fOccurrenceCount;
+	return this.resource.equals(other.resource) && 
+			this.parent.equals(other.parent);
 }
 
 /**
  * @see IJavaElement
  */
 public boolean exists() {
-	return super.exists() 
-				&& isOnClasspath();
+	return super.exists() && validateOnClasspath().isOK();
 }
 
 public IClasspathEntry findSourceAttachmentRecommendation() {
@@ -359,18 +364,19 @@ public IClasspathEntry findSourceAttachmentRecommendation() {
 			entry = parentProject.getClasspathEntryFor(rootPath);
 			if (entry != null){
 				Object target = JavaModel.getTarget(workspaceRoot, entry.getSourceAttachmentPath(), true);
-				if (target instanceof IFile){
-					IFile file = (IFile) target;
-					if (Util.isArchiveFileName(file.getName())){
+				if (target instanceof IResource) {
+					if (target instanceof IFile) {
+						IFile file = (IFile) target;
+						if (org.eclipse.jdt.internal.compiler.util.Util.isArchiveFileName(file.getName())){
+							return entry;
+						}
+					} else if (target instanceof IContainer) {
 						return entry;
 					}
-				} else if (target instanceof IFolder) {
-					return entry;
-				}
-				if (target instanceof java.io.File){
-					java.io.File file = (java.io.File) target;
-					if (file.isFile()) {
-						if (Util.isArchiveFileName(file.getName())){
+				} else if (target instanceof java.io.File){
+					java.io.File file = JavaModel.getFile(target);
+					if (file != null) {
+						if (org.eclipse.jdt.internal.compiler.util.Util.isArchiveFileName(file.getName())){
 							return entry;
 						}
 					} else {
@@ -380,6 +386,7 @@ public IClasspathEntry findSourceAttachmentRecommendation() {
 				}
 			}
 		} catch(JavaModelException e){
+			// ignore
 		}
 		
 		// iterate over all projects
@@ -392,18 +399,19 @@ public IClasspathEntry findSourceAttachmentRecommendation() {
 				entry = jProject.getClasspathEntryFor(rootPath);
 				if (entry != null){
 					Object target = JavaModel.getTarget(workspaceRoot, entry.getSourceAttachmentPath(), true);
-					if (target instanceof IFile){
-						IFile file = (IFile) target;
-						if (Util.isArchiveFileName(file.getName())){
+					if (target instanceof IResource) {
+						if (target instanceof IFile){
+							IFile file = (IFile) target;
+							if (org.eclipse.jdt.internal.compiler.util.Util.isArchiveFileName(file.getName())){
+								return entry;
+							}
+						} else if (target instanceof IContainer) {
 							return entry;
 						}
-					} else if (target instanceof IFolder) {
-						return entry;
-					}
-					if (target instanceof java.io.File){
+					} else if (target instanceof java.io.File){
 						java.io.File file = (java.io.File) target;
 						if (file.isFile()) {
-							if (Util.isArchiveFileName(file.getName())){
+							if (org.eclipse.jdt.internal.compiler.util.Util.isArchiveFileName(file.getName())){
 								return entry;
 							}
 						} else {
@@ -413,9 +421,11 @@ public IClasspathEntry findSourceAttachmentRecommendation() {
 					}
 				}
 			} catch(JavaModelException e){
+				// ignore
 			}
 		}
 	} catch(JavaModelException e){
+		// ignore
 	}
 
 	return null;
@@ -424,7 +434,7 @@ public IClasspathEntry findSourceAttachmentRecommendation() {
 /*
  * Returns the exclusion patterns from the classpath entry associated with this root.
  */
-char[][] fullExclusionPatternChars() {
+public char[][] fullExclusionPatternChars() {
 	try {
 		if (this.isOpen() && this.getKind() != IPackageFragmentRoot.K_SOURCE) return null;
 		ClasspathEntry entry = (ClasspathEntry)getRawClasspathEntry();
@@ -438,42 +448,88 @@ char[][] fullExclusionPatternChars() {
 	}
 }		
 
-/**
- * @see Openable
+/*
+ * Returns the inclusion patterns from the classpath entry associated with this root.
  */
-protected boolean generateInfos(OpenableElementInfo info, IProgressMonitor pm, Map newElements, IResource underlyingResource) throws JavaModelException {
-	
-	((PackageFragmentRootInfo) info).setRootKind(determineKind(underlyingResource));
-	return computeChildren(info);
+public char[][] fullInclusionPatternChars() {
+	try {
+		if (this.isOpen() && this.getKind() != IPackageFragmentRoot.K_SOURCE) return null;
+		ClasspathEntry entry = (ClasspathEntry)getRawClasspathEntry();
+		if (entry == null) {
+			return null;
+		} else {
+			return entry.fullInclusionPatternChars();
+		}
+	} catch (JavaModelException e) { 
+		return null;
+	}
+}		
+public String getElementName() {
+	if (this.resource instanceof IFolder)
+		return ((IFolder) this.resource).getName();
+	return ""; //$NON-NLS-1$
+}
+/**
+ * @see IJavaElement
+ */
+public int getElementType() {
+	return PACKAGE_FRAGMENT_ROOT;
 }
-
 /**
  * @see JavaElement#getHandleMemento()
  */
 protected char getHandleMementoDelimiter() {
 	return JavaElement.JEM_PACKAGEFRAGMENTROOT;
 }
-/**
- * @see JavaElement#getHandleMemento()
+/*
+ * @see JavaElement
  */
-public String getHandleMemento(){
+public IJavaElement getHandleFromMemento(String token, MementoTokenizer memento, WorkingCopyOwner owner) {
+	switch (token.charAt(0)) {
+		case JEM_PACKAGEFRAGMENT:
+			String pkgName;
+			if (memento.hasMoreTokens()) {
+				pkgName = memento.nextToken();
+				char firstChar = pkgName.charAt(0);
+				if (firstChar == JEM_CLASSFILE || firstChar == JEM_COMPILATIONUNIT || firstChar == JEM_COUNT) {
+					token = pkgName;
+					pkgName = IPackageFragment.DEFAULT_PACKAGE_NAME;
+				} else {
+					token = null;
+				}
+			} else {
+				pkgName = IPackageFragment.DEFAULT_PACKAGE_NAME;
+				token = null;
+			}
+			JavaElement pkg = (JavaElement)getPackageFragment(pkgName);
+			if (token == null) {
+				return pkg.getHandleFromMemento(memento, owner);
+			} else {
+				return pkg.getHandleFromMemento(token, memento, owner);
+			}
+	}
+	return null;
+}
+/**
+ * @see JavaElement#getHandleMemento(StringBuffer)
+ */
+protected void getHandleMemento(StringBuffer buff) {
 	IPath path;
-	IResource resource = getResource();
-	if (resource != null) {
+	IResource underlyingResource = getResource();
+	if (underlyingResource != null) {
 		// internal jar or regular root
 		if (getResource().getProject().equals(getJavaProject().getProject())) {
-			path = resource.getProjectRelativePath();
+			path = underlyingResource.getProjectRelativePath();
 		} else {
-			path = resource.getFullPath();
+			path = underlyingResource.getFullPath();
 		}
 	} else {
 		// external jar
 		path = getPath();
 	}
-	StringBuffer buff= new StringBuffer(((JavaElement)getParent()).getHandleMemento());
+	((JavaElement)getParent()).getHandleMemento(buff);
 	buff.append(getHandleMementoDelimiter());
-	buff.append(path.toString()); 
-	return buff.toString();
+	escapeMementoName(buff, path.toString()); 
 }
 /**
  * @see IPackageFragmentRoot
@@ -493,37 +549,30 @@ public Object[] getNonJavaResources() throws JavaModelException {
  * @see IPackageFragmentRoot
  */
 public IPackageFragment getPackageFragment(String packageName) {
-	if (packageName.indexOf(' ') != -1) { // tolerate package names with spaces (e.g. 'x . y') (http://bugs.eclipse.org/bugs/show_bug.cgi?id=21957)
-		char[][] compoundName = Util.toCompoundChars(packageName);
-		StringBuffer buffer = new StringBuffer(packageName.length());
-		for (int i = 0, length = compoundName.length; i < length; i++) {
-			buffer.append(CharOperation.trim(compoundName[i]));
-			if (i != length-1) {
-				buffer.append('.');
-			}
-		}
-		packageName = buffer.toString();
-	}
-	return new PackageFragment(this, packageName);
+	// tolerate package names with spaces (e.g. 'x . y') (http://bugs.eclipse.org/bugs/show_bug.cgi?id=21957)
+	String[] pkgName = Util.getTrimmedSimpleNames(packageName); 
+	return getPackageFragment(pkgName);
+}
+public PackageFragment getPackageFragment(String[] pkgName) {
+	return new PackageFragment(this, pkgName);
 }
-
 /**
  * Returns the package name for the given folder
  * (which is a decendent of this root).
  */
-protected String getPackageName(IFolder folder) throws JavaModelException {
+protected String getPackageName(IFolder folder) {
 	IPath myPath= getPath();
 	IPath pkgPath= folder.getFullPath();
 	int mySegmentCount= myPath.segmentCount();
 	int pkgSegmentCount= pkgPath.segmentCount();
-	StringBuffer name = new StringBuffer(IPackageFragment.DEFAULT_PACKAGE_NAME);
+	StringBuffer pkgName = new StringBuffer(IPackageFragment.DEFAULT_PACKAGE_NAME);
 	for (int i= mySegmentCount; i < pkgSegmentCount; i++) {
 		if (i > mySegmentCount) {
-			name.append('.');
+			pkgName.append('.');
 		}
-		name.append(pkgPath.segment(i));
+		pkgName.append(pkgPath.segment(i));
 	}
-	return name.toString();
+	return pkgName.toString();
 }
 
 /**
@@ -539,10 +588,9 @@ public IPath getPath() {
 public IClasspathEntry getRawClasspathEntry() throws JavaModelException {
 
 	IClasspathEntry rawEntry = null;
-	IJavaProject project = this.getJavaProject();
-	project.getResolvedClasspath(true); // force the reverse rawEntry cache to be populated
-	JavaModelManager.PerProjectInfo perProjectInfo = 
-		JavaModelManager.getJavaModelManager().getPerProjectInfoCheckExistence(project.getProject());
+	JavaProject project = (JavaProject)this.getJavaProject();
+	project.getResolvedClasspath(true/*ignoreUnresolvedEntry*/, false/*don't generateMarkerOnError*/, false/*don't returnResolutionInProgress*/); // force the reverse rawEntry cache to be populated
+	JavaModelManager.PerProjectInfo perProjectInfo = project.getPerProjectInfo();
 	if (perProjectInfo != null && perProjectInfo.resolvedPathToRawEntries != null) {
 		rawEntry = (IClasspathEntry) perProjectInfo.resolvedPathToRawEntries.get(this.getPath());
 	}
@@ -615,7 +663,7 @@ protected String getSourceAttachmentProperty() throws JavaModelException {
  * Returns the qualified name for the source attachment property
  * of this root.
  */
-protected QualifiedName getSourceAttachmentPropertyName() throws JavaModelException {
+protected QualifiedName getSourceAttachmentPropertyName() {
 	return new QualifiedName(JavaCore.PLUGIN_ID, "sourceattachment: " + this.getPath().toOSString()); //$NON-NLS-1$
 }
 
@@ -623,6 +671,7 @@ public void setSourceAttachmentProperty(String property) {
 	try {
 		ResourcesPlugin.getWorkspace().getRoot().setPersistentProperty(this.getSourceAttachmentPropertyName(), property);
 	} catch (CoreException ce) {
+		// ignore
 	}
 }
 
@@ -704,6 +753,14 @@ public IResource getUnderlyingResource() throws JavaModelException {
 	return getResource();
 }
 
+/**
+ * @see IParent 
+ */
+public boolean hasChildren() throws JavaModelException {
+	// a package fragment root always has the default package as a child
+	return true;
+}
+
 public int hashCode() {
 	return this.resource.hashCode();
 }
@@ -723,28 +780,26 @@ public boolean isExternal() {
 }
 
 /*
- * Returns whether this package fragment root is on the classpath of its project.
+ * Validate whether this package fragment root is on the classpath of its project.
  */
-protected boolean isOnClasspath() {
-	if (this.getElementType() == IJavaElement.JAVA_PROJECT){
-		return true;
-	}
+protected IStatus validateOnClasspath() {
 	
 	IPath path = this.getPath();
 	try {
 		// check package fragment root on classpath of its project
-		IJavaProject project = this.getJavaProject();
-		IClasspathEntry[] classpath = project.getResolvedClasspath(true);	
+		JavaProject project = (JavaProject) getJavaProject();
+		IClasspathEntry[] classpath = project.getResolvedClasspath(true/*ignoreUnresolvedEntry*/, false/*don't generateMarkerOnError*/, false/*don't returnResolutionInProgress*/);	
 		for (int i = 0, length = classpath.length; i < length; i++) {
 			IClasspathEntry entry = classpath[i];
 			if (entry.getPath().equals(path)) {
-				return true;
+				return Status.OK_STATUS;
 			}
 		}
 	} catch(JavaModelException e){
 		// could not read classpath, then assume it is outside
+		return e.getJavaModelStatus();
 	}
-	return false;
+	return new JavaModelStatus(IJavaModelStatusConstants.ELEMENT_NOT_ON_CLASSPATH, this);
 }
 /*
  * @see org.eclipse.jdt.core.IPackageFragmentRoot#move
@@ -759,56 +814,27 @@ public void move(
 
 	MovePackageFragmentRootOperation op = 
 		new MovePackageFragmentRootOperation(this, destination, updateResourceFlags, updateModelFlags, sibling);
-	runOperation(op, monitor);
-}
-
-
-protected void openWhenClosed(IProgressMonitor pm) throws JavaModelException {
-	if (!this.resourceExists() 
-			|| !this.isOnClasspath()) {
-		throw newNotPresentException();
-	}
-	super.openWhenClosed(pm);
-}
-
-/**
- * Recomputes the children of this element, based on the current state
- * of the workbench.
- */
-public void refreshChildren() {
-	try {
-		OpenableElementInfo info= (OpenableElementInfo)getElementInfo();
-		computeChildren(info);
-	} catch (JavaModelException e) {
-		// do nothing.
-	}
-}
-
-/*
- * @see JavaElement#rootedAt(IJavaProject)
- */
-public IJavaElement rootedAt(IJavaProject project) {
-	return
-		new PackageFragmentRoot(
-			getResource(),
-			project, 
-			fName);
+	op.runOperation(monitor);
 }
 
 /**
  * @private Debugging purposes
  */
-protected void toStringInfo(int tab, StringBuffer buffer, Object info) {
+protected void toStringInfo(int tab, StringBuffer buffer, Object info, boolean showResolvedInfo) {
 	buffer.append(this.tabString(tab));
-	if (getElementName().length() == 0) {
-		buffer.append("[project root]"); //$NON-NLS-1$
-	} else {
-		IPath path = getPath();
-		if (getJavaProject().getElementName().equals(path.segment(0))) {
+	IPath path = getPath();
+	if (getJavaProject().getElementName().equals(path.segment(0))) {
+	    if (path.segmentCount() == 1) {
+			buffer.append("<project root>"); //$NON-NLS-1$
+	    } else {
 			buffer.append(path.removeFirstSegments(1).makeRelative());
-		} else {
+	    }
+	} else {
+	    if (isExternal()) {
+			buffer.append(path.toOSString());
+	    } else {
 			buffer.append(path);
-		}
+	    }
 	}
 	if (info == null) {
 		buffer.append(" (not open)"); //$NON-NLS-1$
