@@ -1,9 +1,9 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
+ * http://www.eclipse.org/legal/epl-v10.html
  * 
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -17,6 +17,9 @@ import java.util.List;
 import org.eclipse.jdt.core.Flags;
 import org.eclipse.jdt.core.dom.ASTNode;
 import org.eclipse.jdt.core.dom.ASTVisitor;
+import org.eclipse.jdt.core.dom.AbstractTypeDeclaration;
+import org.eclipse.jdt.core.dom.AnnotationTypeDeclaration;
+import org.eclipse.jdt.core.dom.AnnotationTypeMemberDeclaration;
 import org.eclipse.jdt.core.dom.AnonymousClassDeclaration;
 import org.eclipse.jdt.core.dom.ArrayAccess;
 import org.eclipse.jdt.core.dom.ArrayCreation;
@@ -25,6 +28,7 @@ import org.eclipse.jdt.core.dom.ArrayType;
 import org.eclipse.jdt.core.dom.AssertStatement;
 import org.eclipse.jdt.core.dom.Assignment;
 import org.eclipse.jdt.core.dom.Block;
+import org.eclipse.jdt.core.dom.BlockComment;
 import org.eclipse.jdt.core.dom.BodyDeclaration;
 import org.eclipse.jdt.core.dom.BooleanLiteral;
 import org.eclipse.jdt.core.dom.BreakStatement;
@@ -38,6 +42,9 @@ import org.eclipse.jdt.core.dom.ConstructorInvocation;
 import org.eclipse.jdt.core.dom.ContinueStatement;
 import org.eclipse.jdt.core.dom.DoStatement;
 import org.eclipse.jdt.core.dom.EmptyStatement;
+import org.eclipse.jdt.core.dom.EnhancedForStatement;
+import org.eclipse.jdt.core.dom.EnumConstantDeclaration;
+import org.eclipse.jdt.core.dom.EnumDeclaration;
 import org.eclipse.jdt.core.dom.ExpressionStatement;
 import org.eclipse.jdt.core.dom.FieldAccess;
 import org.eclipse.jdt.core.dom.FieldDeclaration;
@@ -46,22 +53,34 @@ import org.eclipse.jdt.core.dom.IfStatement;
 import org.eclipse.jdt.core.dom.ImportDeclaration;
 import org.eclipse.jdt.core.dom.InfixExpression;
 import org.eclipse.jdt.core.dom.Initializer;
+import org.eclipse.jdt.core.dom.InstanceofExpression;
 import org.eclipse.jdt.core.dom.Javadoc;
 import org.eclipse.jdt.core.dom.LabeledStatement;
+import org.eclipse.jdt.core.dom.LineComment;
+import org.eclipse.jdt.core.dom.MarkerAnnotation;
+import org.eclipse.jdt.core.dom.MemberRef;
+import org.eclipse.jdt.core.dom.MemberValuePair;
 import org.eclipse.jdt.core.dom.MethodDeclaration;
 import org.eclipse.jdt.core.dom.MethodInvocation;
+import org.eclipse.jdt.core.dom.MethodRef;
+import org.eclipse.jdt.core.dom.MethodRefParameter;
+import org.eclipse.jdt.core.dom.Modifier;
 import org.eclipse.jdt.core.dom.Name;
+import org.eclipse.jdt.core.dom.NormalAnnotation;
 import org.eclipse.jdt.core.dom.NullLiteral;
 import org.eclipse.jdt.core.dom.NumberLiteral;
 import org.eclipse.jdt.core.dom.PackageDeclaration;
+import org.eclipse.jdt.core.dom.ParameterizedType;
 import org.eclipse.jdt.core.dom.ParenthesizedExpression;
 import org.eclipse.jdt.core.dom.PostfixExpression;
 import org.eclipse.jdt.core.dom.PrefixExpression;
 import org.eclipse.jdt.core.dom.PrimitiveType;
 import org.eclipse.jdt.core.dom.QualifiedName;
+import org.eclipse.jdt.core.dom.QualifiedType;
 import org.eclipse.jdt.core.dom.ReturnStatement;
 import org.eclipse.jdt.core.dom.SimpleName;
 import org.eclipse.jdt.core.dom.SimpleType;
+import org.eclipse.jdt.core.dom.SingleMemberAnnotation;
 import org.eclipse.jdt.core.dom.SingleVariableDeclaration;
 import org.eclipse.jdt.core.dom.StringLiteral;
 import org.eclipse.jdt.core.dom.SuperConstructorInvocation;
@@ -70,6 +89,8 @@ import org.eclipse.jdt.core.dom.SuperMethodInvocation;
 import org.eclipse.jdt.core.dom.SwitchCase;
 import org.eclipse.jdt.core.dom.SwitchStatement;
 import org.eclipse.jdt.core.dom.SynchronizedStatement;
+import org.eclipse.jdt.core.dom.TagElement;
+import org.eclipse.jdt.core.dom.TextElement;
 import org.eclipse.jdt.core.dom.ThisExpression;
 import org.eclipse.jdt.core.dom.ThrowStatement;
 import org.eclipse.jdt.core.dom.TryStatement;
@@ -77,10 +98,12 @@ import org.eclipse.jdt.core.dom.Type;
 import org.eclipse.jdt.core.dom.TypeDeclaration;
 import org.eclipse.jdt.core.dom.TypeDeclarationStatement;
 import org.eclipse.jdt.core.dom.TypeLiteral;
+import org.eclipse.jdt.core.dom.TypeParameter;
 import org.eclipse.jdt.core.dom.VariableDeclarationExpression;
 import org.eclipse.jdt.core.dom.VariableDeclarationFragment;
 import org.eclipse.jdt.core.dom.VariableDeclarationStatement;
 import org.eclipse.jdt.core.dom.WhileStatement;
+import org.eclipse.jdt.core.dom.WildcardType;
 
 public class SourceBasedSourceGenerator extends ASTVisitor  {
 	
@@ -94,9 +117,7 @@ public class SourceBasedSourceGenerator extends ASTVisitor  {
 		
 	private boolean fRightTypeFound;
 	
-	private boolean fIsInAStaticMethod;
-	
-	private boolean fCreateInAnInstanceMethod;
+	private boolean fCreateInAStaticMethod;
 	
 	private boolean fEvaluateNextEndTypeDeclaration;
 	
@@ -104,9 +125,9 @@ public class SourceBasedSourceGenerator extends ASTVisitor  {
 	
 	private CompilationUnit fUnit;
 	
-	private int fPosition;
+	private String fTypeName;
 	
-	private boolean fIsLineNumber;
+	private int fPosition;
 	
 	private StringBuffer fSource;
 	
@@ -123,16 +144,15 @@ public class SourceBasedSourceGenerator extends ASTVisitor  {
 	 * which contains the code snippet is an no-static method, even if <code>position</code>
 	 * is in a static method.
 	 */
-	public SourceBasedSourceGenerator(CompilationUnit unit, int position, boolean isLineNumber, boolean createInAnInstanceMethod, String[] localTypesNames, String[] localVariables, String codeSnippet) {
+	public SourceBasedSourceGenerator(CompilationUnit unit, String typeName, int position, boolean createInAStaticMethod, String[] localTypesNames, String[] localVariables, String codeSnippet) {
 		fRightTypeFound= false;
 		fUnit= unit;
+		fTypeName= typeName;
 		fPosition= position;
 		fLocalVariableTypeNames= localTypesNames;
 		fLocalVariableNames= localVariables;
 		fCodeSnippet= codeSnippet;
-		fIsInAStaticMethod= false;
-		fIsLineNumber= isLineNumber;
-		fCreateInAnInstanceMethod= createInAnInstanceMethod;
+		fCreateInAStaticMethod= createInAStaticMethod;
 	}
 	
 	/**
@@ -191,18 +211,10 @@ public class SourceBasedSourceGenerator extends ASTVisitor  {
 		return fError;
 	}
 	
-	private boolean isInAStaticMethod() {
-		return fIsInAStaticMethod && ! fCreateInAnInstanceMethod;
-	}
-	
-	private void setIsInAStaticMethod(boolean value) {
-		fIsInAStaticMethod= value;
-	}
-	
 	private StringBuffer buildRunMethod(List bodyDeclarations) {
 		StringBuffer buffer = new StringBuffer();
 
-		if (isInAStaticMethod()) {
+		if (fCreateInAStaticMethod) {
 			buffer.append("static "); //$NON-NLS-1$
 		}
 
@@ -233,23 +245,125 @@ public class SourceBasedSourceGenerator extends ASTVisitor  {
 	private String getDotName(String typeName) {
 		return typeName.replace('$', '.');
 	}
-	
-	private boolean containsLine(ASTNode node) {
+
+	private boolean isRightType(ASTNode node) {
 		int position= getPosition();
-		if (fIsLineNumber) {
-			int startLineNumber= getCorrespondingLineNumber(node.getStartPosition());
-			int endLineNumber= getCorrespondingLineNumber(node.getStartPosition() + node.getLength() - 1);
-			return startLineNumber <= position && position <= endLineNumber;
-		} else {
-			int startPosition= node.getStartPosition();
-			return startPosition <= position && position <= startPosition + node.getLength();
+		int startLineNumber= getCorrespondingLineNumber(node.getStartPosition());
+		int endLineNumber= getCorrespondingLineNumber(node.getStartPosition() + node.getLength() - 1);
+		if (startLineNumber <= position && position <= endLineNumber) {
+			// check the typeName
+			String typeName= fTypeName;
+			while (node != null) {
+				if (node instanceof TypeDeclaration || node instanceof EnumDeclaration) {
+					AbstractTypeDeclaration abstractTypeDeclaration= (AbstractTypeDeclaration) node;
+					String name= abstractTypeDeclaration.getName().getIdentifier();
+					if (abstractTypeDeclaration.isLocalTypeDeclaration()) {
+						if (! typeName.endsWith('$' + name)) {
+							return false;
+						}
+						typeName= typeName.substring(0, typeName.length() - name.length() - 1);
+						int index= typeName.lastIndexOf('$');
+						if (index < 0) {
+							return false;
+						}
+						for (int i= typeName.length() - 1; i > index; i--) {
+							if (!Character.isDigit(typeName.charAt(i))) {
+								return false;
+							}
+						}
+						typeName= typeName.substring(0, index);
+						ASTNode parent= node.getParent();
+						while (!(parent instanceof CompilationUnit)) {
+							node= parent;
+							parent= node.getParent();
+						}
+					} else {
+						if (abstractTypeDeclaration.isPackageMemberTypeDeclaration()) {
+							PackageDeclaration packageDeclaration= ((CompilationUnit) node.getParent()).getPackage();
+							if (packageDeclaration == null) {
+								return typeName.equals(name);
+							}
+							return typeName.equals(getQualifiedIdentifier(packageDeclaration.getName()) + '.' + name);
+						}
+						if (!typeName.endsWith('$' + name)) {
+							return false;
+						}
+						typeName= typeName.substring(0, typeName.length() - name.length() - 1);
+						node= node.getParent();
+					}
+				} else if (node instanceof ClassInstanceCreation) {
+					int index= typeName.lastIndexOf('$');
+					if (index < 0) {
+						return false;
+					}
+					for (int i= typeName.length() - 1; i > index; i--) {
+						if (!Character.isDigit(typeName.charAt(i))) {
+							return false;
+						}
+					}
+					typeName= typeName.substring(0, index);
+					ASTNode parent= node.getParent();
+					while (!(parent instanceof CompilationUnit)) {
+						node= parent;
+						parent= node.getParent();
+					}
+				}
+			}
 		}
+		return false;
 	}
 	
 	private StringBuffer buildTypeBody(StringBuffer buffer, List list) {
 		StringBuffer source = new StringBuffer();
 		
 		source.append('{').append('\n');
+		
+		if (buffer != null) {
+			fSnippetStartPosition+= source.length();
+		}
+		
+		source.append(buildBody(buffer, list));
+		source.append('}').append('\n');
+		
+		return source;
+	}
+	
+	private StringBuffer buildEnumBody(StringBuffer buffer, List constantDeclarations, List bodyDeclarations) {
+		StringBuffer source = new StringBuffer();
+		
+		source.append('{').append('\n');
+		if (constantDeclarations.isEmpty()) {
+			source.append(';').append('\n');
+		} else {
+			for (Iterator iter= constantDeclarations.iterator(); iter.hasNext();) {
+				source.append(((EnumConstantDeclaration) iter.next()).getName().getIdentifier());
+				if (iter.hasNext()) {
+					source.append(',');
+				} else {
+					source.append(';');
+				}
+				source.append('\n');
+			}
+		}
+		
+		if (buffer != null) {
+			fSnippetStartPosition+= source.length();
+		}
+		
+		source.append(buildBody(buffer, bodyDeclarations));
+		source.append('}').append('\n');
+		
+		return source;
+		
+	}
+
+	/**
+	 * @param buffer
+	 * @param list
+	 * @param source
+	 */
+	private StringBuffer buildBody(StringBuffer buffer, List list) {
+		StringBuffer source= new StringBuffer();
 		if (buffer != null) {
 			fSnippetStartPosition += source.length();
 			source.append(buffer.toString());
@@ -263,12 +377,15 @@ public class SourceBasedSourceGenerator extends ASTVisitor  {
 			} else if (bodyDeclaration instanceof TypeDeclaration) {
 				TypeDeclaration typeDeclaration = (TypeDeclaration) bodyDeclaration;
 				if (!typeDeclaration.getName().getIdentifier().equals(fLastTypeName)) {
-					source.append(buildTypeDeclaration(null, (TypeDeclaration) bodyDeclaration));
+					source.append(buildTypeDeclaration(null, typeDeclaration));
+				}
+			} else if (bodyDeclaration instanceof EnumDeclaration) {
+				EnumDeclaration enumDeclaration= (EnumDeclaration) bodyDeclaration;
+				if (!enumDeclaration.getName().getIdentifier().equals(fLastTypeName)) {
+					source.append(buildEnumDeclaration(null, enumDeclaration));
 				}
 			}
 		}
-		source.append('}').append('\n');
-		
 		return source;
 	}
 	
@@ -308,7 +425,7 @@ public class SourceBasedSourceGenerator extends ASTVisitor  {
 		boolean isConstructor= methodDeclaration.isConstructor();
 		
 		if (!isConstructor) {
-			source.append(getDotName(getTypeName(methodDeclaration.getReturnType())));
+			source.append(getDotName(getTypeName(methodDeclaration.getReturnType2())));
 			source.append(' ');
 		}
 		
@@ -324,6 +441,9 @@ public class SourceBasedSourceGenerator extends ASTVisitor  {
 				source.append(',');
 			}
 			source.append(getDotName(getTypeName(singleVariableDeclaration.getType())));
+			if (singleVariableDeclaration.isVarargs()) {
+				source.append("..."); //$NON-NLS-1$
+			}
 			source.append(' ');
 			source.append(singleVariableDeclaration.getName().getIdentifier());
 			appendExtraDimensions(source, singleVariableDeclaration.getExtraDimensions());
@@ -350,7 +470,9 @@ public class SourceBasedSourceGenerator extends ASTVisitor  {
 			source.append(";\n"); //$NON-NLS-1$
 		} else {
 			source.append('{').append('\n');
-			source.append(getReturnExpression(methodDeclaration.getReturnType())); 
+			if (!isConstructor) {
+				source.append(getReturnExpression(methodDeclaration.getReturnType2())); 
+			}
 			source.append('}').append('\n');
 		}
 		
@@ -365,7 +487,33 @@ public class SourceBasedSourceGenerator extends ASTVisitor  {
 			}
 		}
 	}
-	
+
+	private StringBuffer buildEnumDeclaration(StringBuffer buffer, EnumDeclaration enumDeclaration) {
+		StringBuffer source = new StringBuffer();
+		source.append(Flags.toString(enumDeclaration.getModifiers()));
+		source.append(" enum "); //$NON-NLS-1$
+		
+		source.append(enumDeclaration.getName().getIdentifier());
+		
+		Iterator iterator= enumDeclaration.superInterfaceTypes().iterator();
+		if (iterator.hasNext()) {
+			source.append(" implements "); //$NON-NLS-1$
+			source.append(getTypeName((Type) iterator.next()));
+			while (iterator.hasNext()) {
+				source.append(',');
+				source.append(getTypeName((Type) iterator.next()));
+			}
+		}
+
+		if (buffer != null) {
+			fSnippetStartPosition+= source.length();
+		}
+		source.append(buildEnumBody(buffer, enumDeclaration.enumConstants(), enumDeclaration.bodyDeclarations()));
+		
+		return source;
+	}
+		
+
 	private StringBuffer buildTypeDeclaration(StringBuffer buffer, TypeDeclaration typeDeclaration) {
 		
 		StringBuffer source = new StringBuffer();
@@ -375,24 +523,61 @@ public class SourceBasedSourceGenerator extends ASTVisitor  {
 		} else {
 			source.append(" class "); //$NON-NLS-1$
 		}
-		source.append(typeDeclaration.getName().getIdentifier());
 		
-		Name superClass = typeDeclaration.getSuperclass();
+		source.append(typeDeclaration.getName().getIdentifier());
+
+		List typeParameters= typeDeclaration.typeParameters();
+		if (!typeParameters.isEmpty()) {
+			source.append('<');
+			Iterator iter= typeParameters.iterator();
+			TypeParameter typeParameter= (TypeParameter) iter.next();
+			source.append(typeParameter.getName().getIdentifier());
+			List typeBounds= typeParameter.typeBounds();
+			if (!typeBounds.isEmpty()) {
+				source.append(" extends "); //$NON-NLS-1$
+				Iterator iter2= typeBounds.iterator();
+				source.append(getTypeName((Type) iter2.next()));
+				while (iter.hasNext()) {
+					source.append('&');
+					source.append(getTypeName((Type) iter2.next()));
+				}
+			}
+			while (iter.hasNext()) {
+				source.append(',');
+				typeParameter= (TypeParameter) iter.next();
+				source.append(typeParameter.getName().getIdentifier());
+				typeBounds= typeParameter.typeBounds();
+				if (!typeBounds.isEmpty()) {
+					source.append(" extends "); //$NON-NLS-1$
+					Iterator iter2= typeBounds.iterator();
+					source.append(getTypeName((Type) iter2.next()));
+					while (iter.hasNext()) {
+						source.append('&');
+						source.append(getTypeName((Type) iter2.next()));
+					}
+				}
+			}
+			source.append('>');
+		}
+
+		Type superClass = typeDeclaration.getSuperclassType();
 		if (superClass != null) {
 			source.append(" extends "); //$NON-NLS-1$
-			source.append(getQualifiedIdentifier(superClass));
+			source.append(getTypeName(superClass));
 		}
-		
-		boolean first = true;
-		for (Iterator iterator = typeDeclaration.superInterfaces().iterator(); iterator.hasNext();) {
-			Name name = (Name) iterator.next();
-			if (first) {
-				first = false;
-				source.append(" implements "); //$NON-NLS-1$
+
+		Iterator iter= typeDeclaration.superInterfaceTypes().iterator();
+		if (iter.hasNext()) {
+			if (typeDeclaration.isInterface()) {
+				source.append(" extends "); //$NON-NLS-1$
 			} else {
+				source.append(" implements "); //$NON-NLS-1$
+			}
+			source.append(getTypeName((Type) iter.next()));
+			while (iter.hasNext()) {
 				source.append(',');
+				source.append(getTypeName((Type) iter.next()));
 			}
-			source.append(getQualifiedIdentifier(name));
 		}
 		
 		if (buffer != null) {
@@ -416,6 +601,9 @@ public class SourceBasedSourceGenerator extends ASTVisitor  {
 		for (Iterator iterator = compilationUnit.imports().iterator(); iterator.hasNext();) {
 			ImportDeclaration importDeclaration = (ImportDeclaration) iterator.next();
 			source.append("import "); //$NON-NLS-1$
+            if (importDeclaration.isStatic()) {
+                source.append("static "); //$NON-NLS-1$
+            }
 			source.append(getQualifiedIdentifier(importDeclaration.getName()));
 			if (importDeclaration.isOnDemand()) {
 				source.append(".*"); //$NON-NLS-1$
@@ -427,12 +615,16 @@ public class SourceBasedSourceGenerator extends ASTVisitor  {
 		source.append(buffer);
 		
 		for (Iterator iterator = compilationUnit.types().iterator(); iterator.hasNext();) {
-			TypeDeclaration typeDeclaration = (TypeDeclaration) iterator.next();
+			AbstractTypeDeclaration typeDeclaration = (AbstractTypeDeclaration) iterator.next();
 			if (Flags.isPublic(typeDeclaration.getModifiers())) {
 				fCompilationUnitName = typeDeclaration.getName().getIdentifier();
 			}
 			if (!fLastTypeName.equals(typeDeclaration.getName().getIdentifier())) {
-				source.append(buildTypeDeclaration(null,typeDeclaration));
+				if (typeDeclaration instanceof TypeDeclaration) {
+					source.append(buildTypeDeclaration(null, (TypeDeclaration)typeDeclaration));
+				} else if (typeDeclaration instanceof EnumDeclaration) {
+					source.append(buildEnumDeclaration(null, (EnumDeclaration)typeDeclaration));
+				}
 			}
 		}
 		if (fCompilationUnitName == null) {		
@@ -508,39 +700,56 @@ public class SourceBasedSourceGenerator extends ASTVisitor  {
 	
 	public String getTypeName(Type type) {
 		if (type.isSimpleType()) {
-			SimpleType simpleType= (SimpleType) type;
-			return getQualifiedIdentifier(simpleType.getName());
+			return getQualifiedIdentifier(((SimpleType) type).getName());
 		} else if (type.isArrayType()) {
-			ArrayType arrayType= (ArrayType) type;
-			String res = getTypeName(arrayType.getElementType());
-			for (int i = 0, dim= arrayType.getDimensions(); i < dim; i++) {
-				res += "[]"; //$NON-NLS-1$
-			}
-			return res;
+			return getTypeName(((ArrayType) type).getComponentType()) + "[]"; //$NON-NLS-1$
 		} else if (type.isPrimitiveType()) {
-			PrimitiveType primitiveType = (PrimitiveType) type;
-			return primitiveType.getPrimitiveTypeCode().toString();
+			return ((PrimitiveType) type).getPrimitiveTypeCode().toString();
+		} else if (type.isQualifiedType()) {
+			QualifiedType qualifiedType= (QualifiedType) type;
+			return getTypeName(qualifiedType.getQualifier()) + '.' + qualifiedType.getName().getIdentifier();
+		} else if (type.isParameterizedType()) {
+			ParameterizedType parameterizedType= (ParameterizedType)type;
+			StringBuffer buff= new StringBuffer(getTypeName(parameterizedType.getType()));
+			Iterator iter= parameterizedType.typeArguments().iterator();
+			if (iter.hasNext()) {
+				buff.append('<');
+				buff.append(getTypeName((Type)iter.next()));
+				while (iter.hasNext()) {
+					buff.append(',');
+					buff.append(getTypeName((Type)iter.next()));
+				}
+				buff.append('>');
+			}
+			return buff.toString();
+		} else if (type.isWildcardType()) {
+			WildcardType wildcardType= (WildcardType)type;
+			StringBuffer buff= new StringBuffer("?"); //$NON-NLS-1$
+			Type bound= wildcardType.getBound();
+			if (bound != null) {
+				buff.append(wildcardType.isUpperBound() ? " extends " : " super "); //$NON-NLS-1$ //$NON-NLS-2$
+				buff.append(getTypeName(bound));
+			}
+			return buff.toString();
 		}
 		return null;
 		
 	}
 	
 	public String getReturnExpression(Type type) {
-		if (type.isSimpleType() || type.isArrayType()) {
+		if (type.isSimpleType() || type.isArrayType() || type.isQualifiedType() || type.isWildcardType() || type.isParameterizedType()) {
 			return "return null;"; //$NON-NLS-1$
 		} else if (type.isPrimitiveType()) {
 			String typeName= ((PrimitiveType) type).getPrimitiveTypeCode().toString();
 			char char0 = typeName.charAt(0);
 			if (char0 == 'v') {
 				return ""; //$NON-NLS-1$
-			} else {
-				char char1 = typeName.charAt(1);
-				if (char0 == 'b' && char1 == 'o') {
-					return "return false;"; //$NON-NLS-1$
-				} else {
-					return "return 0;"; //$NON-NLS-1$
-				}
 			}
+			char char1 = typeName.charAt(1);
+			if (char0 == 'b' && char1 == 'o') {
+				return "return false;"; //$NON-NLS-1$
+			}
+			return "return 0;"; //$NON-NLS-1$
 		}
 		return null;
 	}
@@ -557,7 +766,7 @@ public class SourceBasedSourceGenerator extends ASTVisitor  {
 		}
 		AnonymousClassDeclaration anonymousClassDeclaration = node.getAnonymousClassDeclaration();
 		if (anonymousClassDeclaration != null) {
-			if (!rightTypeFound() && containsLine(node)) {
+			if (!rightTypeFound() && isRightType(node)) {
 				setRightTypeFound(true);
 				
 				fSource= buildRunMethod(anonymousClassDeclaration.bodyDeclarations());
@@ -566,11 +775,6 @@ public class SourceBasedSourceGenerator extends ASTVisitor  {
 		
 			if (rightTypeFound()) {
 				
-				if (!node.arguments().isEmpty()) {
-					setError(EvaluationEngineMessages.getString("SourceBasedSourceGenerator.Cannot_evaluate_an_expression_in_the_context_of_anonymous_type_with_no_default_constructor._1")); //$NON-NLS-1$
-					return;
-				}
-				
 				List bodyDeclarations= anonymousClassDeclaration.bodyDeclarations();
 				
 				StringBuffer source = buildTypeBody(fSource, bodyDeclarations);
@@ -593,7 +797,7 @@ public class SourceBasedSourceGenerator extends ASTVisitor  {
 					fSource.append(getUniqueMethodName(EVAL_METHOD_NAME, bodyDeclarations));
 					fSource.append("() {\n"); //$NON-NLS-1$
 					fSource.append("new "); //$NON-NLS-1$
-					fSource.append(getQualifiedIdentifier(node.getName()));
+					fSource.append(getTypeName(node.getType())); 
 					fSource.append("()"); //$NON-NLS-1$
 					
 					fSnippetStartPosition+= fSource.length();
@@ -616,7 +820,7 @@ public class SourceBasedSourceGenerator extends ASTVisitor  {
 					fSource.append(' ');
 					fSource.append(getUniqueFieldName(EVAL_FIELD_NAME, bodyDeclarations));
 					fSource.append(" = new "); //$NON-NLS-1$
-					fSource.append(getQualifiedIdentifier(node.getName()));
+					fSource.append(getTypeName(node.getType()));
 					fSource.append("()"); //$NON-NLS-1$
 					
 					fSnippetStartPosition+= fSource.length();
@@ -643,27 +847,56 @@ public class SourceBasedSourceGenerator extends ASTVisitor  {
 		fSource = buildCompilationUnit(fSource, node);
 	}
 
-	/**
-	 * @see ASTVisitor#endVisit(Initializer)
+	/* (non-Javadoc)
+	 * @see org.eclipse.jdt.core.dom.ASTVisitor#endVisit(org.eclipse.jdt.core.dom.EnumDeclaration)
 	 */
-	public void endVisit(Initializer node) {
+	public void endVisit(EnumDeclaration node) {
+		
 		if (hasError()) {
 			return;
 		}
-		if (!rightTypeFound() && containsLine(node)) {
-			setIsInAStaticMethod(Flags.isStatic(node.getModifiers()));
+		
+		if (!rightTypeFound() && isRightType(node)) {
+			setRightTypeFound(true);
+			
+			fSource= buildRunMethod(node.bodyDeclarations());
+			fEvaluateNextEndTypeDeclaration = true;
 		}
-	}
-
-	/**
-	 * @see ASTVisitor#endVisit(MethodDeclaration)
-	 */
-	public void endVisit(MethodDeclaration node) {
-		if (hasError()) {
+		
+		if (!fEvaluateNextEndTypeDeclaration) {
+			fEvaluateNextEndTypeDeclaration = true;
 			return;
 		}
-		if (!rightTypeFound() && containsLine(node)) {
-			setIsInAStaticMethod(Flags.isStatic(node.getModifiers()));
+		
+		if (rightTypeFound()) {
+			
+			StringBuffer source = buildEnumDeclaration(fSource, node);
+			
+			if (node.isLocalTypeDeclaration()) {
+				// enclose in a method if nessecary
+				
+				ASTNode parent = node.getParent();
+				while (!(parent instanceof MethodDeclaration)) {
+					parent= parent.getParent();
+				}
+				MethodDeclaration enclosingMethodDeclaration = (MethodDeclaration) parent;
+				
+				fSource = new StringBuffer();
+				
+				if (Flags.isStatic(enclosingMethodDeclaration.getModifiers())) {
+					fSource.append("static "); //$NON-NLS-1$
+				}
+				
+				fSource.append("void ___eval() {\n"); //$NON-NLS-1$
+				fSnippetStartPosition+= fSource.length();
+				fSource.append(source);
+				fSource.append("}\n"); //$NON-NLS-1$
+				
+				fLastTypeName = ""; //$NON-NLS-1$
+			} else {
+				fSource = source;
+				fLastTypeName = node.getName().getIdentifier();
+			}
 		}
 	}
 
@@ -676,7 +909,7 @@ public class SourceBasedSourceGenerator extends ASTVisitor  {
 			return;
 		}
 		
-		if (!rightTypeFound() && containsLine(node)) {
+		if (!rightTypeFound() && isRightType(node)) {
 			setRightTypeFound(true);
 			
 			fSource= buildRunMethod(node.bodyDeclarations());
@@ -720,6 +953,20 @@ public class SourceBasedSourceGenerator extends ASTVisitor  {
 		}
 	}
 
+	/* (non-Javadoc)
+	 * @see org.eclipse.jdt.core.dom.ASTVisitor#visit(org.eclipse.jdt.core.dom.AnnotationTypeDeclaration)
+	 */
+	public boolean visit(AnnotationTypeDeclaration node) {
+		return false;
+	}
+	
+	/* (non-Javadoc)
+	 * @see org.eclipse.jdt.core.dom.ASTVisitor#visit(org.eclipse.jdt.core.dom.AnnotationTypeMemberDeclaration)
+	 */
+	public boolean visit(AnnotationTypeMemberDeclaration node) {
+		return false;
+	}
+	
 	/**
 	 * @see ASTVisitor#visit(AnonymousClassDeclaration)
 	 */
@@ -800,6 +1047,13 @@ public class SourceBasedSourceGenerator extends ASTVisitor  {
 		return true;
 	}
 
+	/* (non-Javadoc)
+	 * @see org.eclipse.jdt.core.dom.ASTVisitor#visit(org.eclipse.jdt.core.dom.BlockComment)
+	 */
+	public boolean visit(BlockComment node) {
+		return false;
+	}
+
 	/**
 	 * @see ASTVisitor#visit(BooleanLiteral)
 	 */
@@ -863,7 +1117,7 @@ public class SourceBasedSourceGenerator extends ASTVisitor  {
 	/**
 	 * @see ASTVisitor#visit(CompilationUnit)
 	 */
-	public boolean visit(CompilationUnit node) {
+ 	public boolean visit(CompilationUnit node) {
 		if (rightTypeFound()) {
 			return false;
 		}
@@ -920,6 +1174,36 @@ public class SourceBasedSourceGenerator extends ASTVisitor  {
 		return true;
 	}
 
+	/* (non-Javadoc)
+	 * @see org.eclipse.jdt.core.dom.ASTVisitor#visit(org.eclipse.jdt.core.dom.EnhancedForStatement)
+	 */
+	public boolean visit(EnhancedForStatement node) {
+		if (rightTypeFound()) {
+			return false;
+		}
+		return true;
+	}
+
+	/* (non-Javadoc)
+	 * @see org.eclipse.jdt.core.dom.ASTVisitor#visit(org.eclipse.jdt.core.dom.EnumConstantDeclaration)
+	 */
+	public boolean visit(EnumConstantDeclaration node) {
+		if (rightTypeFound()) {
+			return false;
+		}
+		return true;
+	}
+	
+	/* (non-Javadoc)
+	 * @see org.eclipse.jdt.core.dom.ASTVisitor#visit(org.eclipse.jdt.core.dom.EnumDeclaration)
+	 */
+	public boolean visit(EnumDeclaration node) {
+		if (rightTypeFound()) {
+			return false;
+		}
+		return true;
+	}
+	
 	/**
 	 * @see ASTVisitor#visit(ExpressionStatement)
 	 */
@@ -1000,6 +1284,15 @@ public class SourceBasedSourceGenerator extends ASTVisitor  {
 		return true;
 	}
 
+	/* (non-Javadoc)
+	 * @see org.eclipse.jdt.core.dom.ASTVisitor#visit(org.eclipse.jdt.core.dom.InstanceofExpression)
+	 */
+	public boolean visit(InstanceofExpression node) {
+		if (rightTypeFound()) {
+			return false;
+		}
+		return true;
+	}
 	/**
 	 * @see ASTVisitor#visit(Javadoc)
 	 */
@@ -1020,6 +1313,33 @@ public class SourceBasedSourceGenerator extends ASTVisitor  {
 		return true;
 	}
 
+	/* (non-Javadoc)
+	 * @see org.eclipse.jdt.core.dom.ASTVisitor#visit(org.eclipse.jdt.core.dom.LineComment)
+	 */
+	public boolean visit(LineComment node) {
+		return false;
+	}
+	
+	/* (non-Javadoc)
+	 * @see org.eclipse.jdt.core.dom.ASTVisitor#visit(org.eclipse.jdt.core.dom.MarkerAnnotation)
+	 */
+	public boolean visit(MarkerAnnotation node) {
+		return false;
+	}
+	
+	/* (non-Javadoc)
+	 * @see org.eclipse.jdt.core.dom.ASTVisitor#visit(org.eclipse.jdt.core.dom.MemberRef)
+	 */
+	public boolean visit(MemberRef node) {
+		return false;
+	}
+	
+	/* (non-Javadoc)
+	 * @see org.eclipse.jdt.core.dom.ASTVisitor#visit(org.eclipse.jdt.core.dom.MemberValuePair)
+	 */
+	public boolean visit(MemberValuePair node) {
+		return false;
+	}
 	/**
 	 * @see ASTVisitor#visit(MethodDeclaration)
 	 */
@@ -1040,6 +1360,34 @@ public class SourceBasedSourceGenerator extends ASTVisitor  {
 		return true;
 	}
 
+	/* (non-Javadoc)
+	 * @see org.eclipse.jdt.core.dom.ASTVisitor#visit(org.eclipse.jdt.core.dom.MethodRef)
+	 */
+	public boolean visit(MethodRef node) {
+		return false;
+	}
+	
+	/* (non-Javadoc)
+	 * @see org.eclipse.jdt.core.dom.ASTVisitor#visit(org.eclipse.jdt.core.dom.MethodRefParameter)
+	 */
+	public boolean visit(MethodRefParameter node) {
+		return false;
+	}
+
+	/* (non-Javadoc)
+	 * @see org.eclipse.jdt.core.dom.ASTVisitor#visit(org.eclipse.jdt.core.dom.Modifier)
+	 */
+	public boolean visit(Modifier node) {
+		return false;
+	}
+	
+	/* (non-Javadoc)
+	 * @see org.eclipse.jdt.core.dom.ASTVisitor#visit(org.eclipse.jdt.core.dom.NormalAnnotation)
+	 */
+	public boolean visit(NormalAnnotation node) {
+		return false;
+	}
+	
 	/**
 	 * @see ASTVisitor#visit(NullLiteral)
 	 */
@@ -1070,6 +1418,16 @@ public class SourceBasedSourceGenerator extends ASTVisitor  {
 		return true;
 	}
 
+	/* (non-Javadoc)
+	 * @see org.eclipse.jdt.core.dom.ASTVisitor#visit(org.eclipse.jdt.core.dom.ParameterizedType)
+	 */
+	public boolean visit(ParameterizedType node) {
+		if (rightTypeFound()) {
+			return false;
+		}
+		return true;
+	}
+
 	/**
 	 * @see ASTVisitor#visit(ParenthesizedExpression)
 	 */
@@ -1120,6 +1478,12 @@ public class SourceBasedSourceGenerator extends ASTVisitor  {
 		return true;
 	}
 
+	/* (non-Javadoc)
+	 * @see org.eclipse.jdt.core.dom.ASTVisitor#visit(org.eclipse.jdt.core.dom.QualifiedType)
+	 */
+	public boolean visit(QualifiedType node) {
+		return false;
+	}
 	/**
 	 * @see ASTVisitor#visit(ReturnStatement)
 	 */
@@ -1150,6 +1514,13 @@ public class SourceBasedSourceGenerator extends ASTVisitor  {
 		return true;
 	}
 
+	/* (non-Javadoc)
+	 * @see org.eclipse.jdt.core.dom.ASTVisitor#visit(org.eclipse.jdt.core.dom.SingleMemberAnnotation)
+	 */
+	public boolean visit(SingleMemberAnnotation node) {
+		return false;
+	}
+	
 	/**
 	 * @see ASTVisitor#visit(SingleVariableDeclaration)
 	 */
@@ -1230,6 +1601,20 @@ public class SourceBasedSourceGenerator extends ASTVisitor  {
 		return true;
 	}
 
+	/* (non-Javadoc)
+	 * @see org.eclipse.jdt.core.dom.ASTVisitor#visit(org.eclipse.jdt.core.dom.TagElement)
+	 */
+	public boolean visit(TagElement node) {
+		return false;
+	}
+	
+	/* (non-Javadoc)
+	 * @see org.eclipse.jdt.core.dom.ASTVisitor#visit(org.eclipse.jdt.core.dom.TextElement)
+	 */
+	public boolean visit(TextElement node) {
+		return false;
+	}
+
 	/**
 	 * @see ASTVisitor#visit(ThisExpression)
 	 */
@@ -1291,6 +1676,13 @@ public class SourceBasedSourceGenerator extends ASTVisitor  {
 		return true;
 	}
 
+	/* (non-Javadoc)
+	 * @see org.eclipse.jdt.core.dom.ASTVisitor#visit(org.eclipse.jdt.core.dom.TypeParameter)
+	 */
+	public boolean visit(TypeParameter node) {
+		return false;
+	}
+
 	/**
 	 * @see ASTVisitor#visit(VariableDeclarationExpression)
 	 */
@@ -1330,4 +1722,12 @@ public class SourceBasedSourceGenerator extends ASTVisitor  {
 		}
 		return true;
 	}
+	
+	/* (non-Javadoc)
+	 * @see org.eclipse.jdt.core.dom.ASTVisitor#visit(org.eclipse.jdt.core.dom.WildcardType)
+	 */
+	public boolean visit(WildcardType node) {
+		return false;
+	}
+
 }
