@@ -1,9 +1,9 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
+ * http://www.eclipse.org/legal/epl-v10.html
  * 
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -18,29 +18,37 @@ import java.util.Comparator;
 import java.util.Iterator;
 import java.util.List;
 
+import org.eclipse.core.runtime.IStatus;
+import org.eclipse.core.runtime.Status;
 import org.eclipse.debug.core.DebugEvent;
 import org.eclipse.debug.core.DebugException;
 import org.eclipse.debug.core.model.IRegisterGroup;
 import org.eclipse.debug.core.model.IStackFrame;
 import org.eclipse.debug.core.model.IThread;
 import org.eclipse.debug.core.model.IVariable;
+import org.eclipse.jdt.core.Signature;
 import org.eclipse.jdt.debug.core.IJavaClassType;
 import org.eclipse.jdt.debug.core.IJavaModifiers;
 import org.eclipse.jdt.debug.core.IJavaObject;
+import org.eclipse.jdt.debug.core.IJavaReferenceType;
 import org.eclipse.jdt.debug.core.IJavaStackFrame;
 import org.eclipse.jdt.debug.core.IJavaThread;
 import org.eclipse.jdt.debug.core.IJavaVariable;
 import org.eclipse.jdt.internal.debug.core.JDIDebugPlugin;
 
 import com.sun.jdi.AbsentInformationException;
+import com.sun.jdi.ClassType;
 import com.sun.jdi.Field;
+import com.sun.jdi.IncompatibleThreadStateException;
 import com.sun.jdi.LocalVariable;
 import com.sun.jdi.Location;
 import com.sun.jdi.Method;
 import com.sun.jdi.NativeMethodException;
 import com.sun.jdi.ObjectReference;
+import com.sun.jdi.ReferenceType;
 import com.sun.jdi.StackFrame;
 import com.sun.jdi.Type;
+import com.sun.jdi.VirtualMachine;
 
 /**
  * Proxy to a stack frame on the target.
@@ -48,14 +56,18 @@ import com.sun.jdi.Type;
 
 public class JDIStackFrame extends JDIDebugElement implements IJavaStackFrame {
 
+	/**
+	 * This frame's depth in the call stack (0 == bottom of stack).
+	 * A new frame is indicated by -2.
+	 * An invalid frame is indicated by -1.
+	 */
+	private int fDepth = -2;
+	
 	/**
 	 * Underlying JDI stack frame.
 	 */
 	private StackFrame fStackFrame;
-	/**
-	 * The last (previous) underlying method.
-	 */
-	private Method fLastMethod;
+
 	/**
 	 * Containing thread.
 	 */
@@ -64,21 +76,13 @@ public class JDIStackFrame extends JDIDebugElement implements IJavaStackFrame {
 	 * Visible variables.
 	 */
 	private List fVariables;
-	/**
-	 * The method this stack frame is associated with.
-	 * Cached lazily on first access.
-	 */
-	private Method fMethod= null;
+
 	/**
 	 * The underlying Object associated with this stack frame.  
 	 * Cached lazily on first access.
 	 */
 	private ObjectReference fThisObject;
-	/**
-	 * The name of the type in which the method for this stack frame was
-	 * declared (implemented).  Cached lazily on first access.
-	 */
-	private String fDeclaringTypeName;
+
 	/**
 	 * The name of the type of the object that received the method call associated
 	 * with this stack frame.  Cached lazily on first access.
@@ -94,26 +98,70 @@ public class JDIStackFrame extends JDIDebugElement implements IJavaStackFrame {
 	 * calculating its out of synch state.
 	 */
 	private boolean fIsOutOfSynch= false;
-	/**
-	 * The source name debug attribute. Cached lazily on first access.
-	 */
-	private String fSourceName;
 	
 	/**
 	 * Whether local variable information was available
 	 */
 	private boolean fLocalsAvailable = true;
+	
+	/**
+	 * Location of this stack frame
+	 */
+	private Location fLocation;
 
 	/**
 	 * Creates a new stack frame in the given thread.
 	 * 
 	 * @param thread The parent JDI thread
-	 * @param stackFrame The underlying stack frame
+	 * @param frame underlying frame
+	 * @param depth depth on the stack (0 is bottom)
 	 */
-	public JDIStackFrame(JDIThread thread, StackFrame stackFrame) {
+	public JDIStackFrame(JDIThread thread, StackFrame frame, int depth) {
 		super((JDIDebugTarget)thread.getDebugTarget());
-		setUnderlyingStackFrame(stackFrame);
 		setThread(thread);
+		bind(frame, depth);
+	}
+	
+	/**
+	 * Binds this frame to the given underlying frame
+	 * on the target VM or returns a new frame representing the
+	 * given frame. A frame can only be re-bound to an underlying frame
+	 * if it refers to the same depth on the stack in the same method.
+	 * 
+	 * @param frame underlying frame, or <code>null</code>
+	 * @param depth depth in the call stack, or -1 to indicate the
+	 *  frame should become invalid
+	 *  @param return a frame to refer to the given frame or <code>null</code>
+	 */
+	protected JDIStackFrame bind(StackFrame frame, int depth) {
+		synchronized (fThread) {
+			if (fDepth == -2) {
+				// first initialization
+				fStackFrame = frame;
+				fDepth = depth;
+				fLocation = frame.location();
+				return this;
+			} else if (depth == -1) {
+				// mark as invalid
+				fDepth = -1;
+				fStackFrame = null;
+				return null;
+			} else if (fDepth == depth) {
+				Location location = frame.location();
+				if (location.method().equals(fLocation.method())) {
+					// TODO: what about receiving type being the same?
+					fStackFrame = frame;
+					fLocation = location;
+					clearCachedData();
+					return this;
+				}
+			}
+			// invalidate this franme
+			bind(null, -1);
+			// return a new frame
+			return new JDIStackFrame(fThread, frame, depth);
+		}
+
 	}
 	
 	/**
@@ -153,12 +201,7 @@ public class JDIStackFrame extends JDIDebugElement implements IJavaStackFrame {
 	 * @see IStep#canStepOver()
 	 */
 	public boolean canStepOver() {
-		try {
-			return exists() && !isObsolete() && getThread().canStepOver();
-		} catch (DebugException e) {
-			logError(e);
-			return false;
-		}
+		return exists() && !isObsolete() && getThread().canStepOver();
 	}
 
 	/**
@@ -191,15 +234,10 @@ public class JDIStackFrame extends JDIDebugElement implements IJavaStackFrame {
 	 * Returns the underlying method associated with this stack frame,
 	 * retreiving the method is necessary.
 	 */
-	public Method getUnderlyingMethod() throws DebugException {
-		if (fStackFrame == null || fMethod == null) {
-			try {
-				fMethod= getUnderlyingStackFrame().location().method();
-			} catch (RuntimeException e) {
-				targetRequestFailed(MessageFormat.format(JDIDebugModelMessages.getString("JDIStackFrame.exception_retrieving_method"), new String[] {e.toString()}), e); //$NON-NLS-1$
-			}
+	public Method getUnderlyingMethod() {
+		synchronized (fThread) {
+			return fLocation.method();
 		}
-		return fMethod;
 	}
 
 	/**
@@ -210,68 +248,71 @@ public class JDIStackFrame extends JDIDebugElement implements IJavaStackFrame {
 		return (IVariable[])list.toArray(new IVariable[list.size()]);
 	}
 	
-	protected synchronized List getVariables0() throws DebugException {
-		if (fVariables == null) {
-			
-			// throw exception if native method, so variable view will update
-			// with information message
-			if (isNative()) {
-				requestFailed(JDIDebugModelMessages.getString("JDIStackFrame.Variable_information_unavailable_for_native_methods"), null); //$NON-NLS-1$
-			}
-			
-			Method method= getUnderlyingMethod();
-			fVariables= new ArrayList();
-			// #isStatic() does not claim to throw any exceptions - so it is not try/catch coded
-			if (method.isStatic()) {
-				// add statics
-				List allFields= null;
-				try {
-					allFields= method.declaringType().allFields();
-				} catch (RuntimeException e) {
-					targetRequestFailed(MessageFormat.format(JDIDebugModelMessages.getString("JDIStackFrame.exception_retrieving_fields"),new String[] {e.toString()}), e); //$NON-NLS-1$
-					// execution will not reach this line, as 
-					// #targetRequestFailed will throw an exception					
-					return Collections.EMPTY_LIST;
+	protected List getVariables0() throws DebugException {
+		synchronized (fThread) {
+			if (fVariables == null) {
+				
+				// throw exception if native method, so variable view will update
+				// with information message
+				if (isNative()) {
+					requestFailed(JDIDebugModelMessages.JDIStackFrame_Variable_information_unavailable_for_native_methods, null); //$NON-NLS-1$
 				}
-				if (allFields != null) {
-					Iterator fields= allFields.iterator();
-					while (fields.hasNext()) {
-						Field field= (Field) fields.next();
-						if (field.isStatic()) {
-							fVariables.add(new JDIFieldVariable((JDIDebugTarget)getDebugTarget(), field, null));
-						}
+				
+				Method method= getUnderlyingMethod();
+				fVariables= new ArrayList();
+				// #isStatic() does not claim to throw any exceptions - so it is not try/catch coded
+				if (method.isStatic()) {
+					// add statics
+					List allFields= null;
+					ReferenceType declaringType = method.declaringType();
+					try {
+						allFields= declaringType.allFields();
+					} catch (RuntimeException e) {
+						targetRequestFailed(MessageFormat.format(JDIDebugModelMessages.JDIStackFrame_exception_retrieving_fields,new String[] {e.toString()}), e); //$NON-NLS-1$
+						// execution will not reach this line, as 
+						// #targetRequestFailed will throw an exception					
+						return Collections.EMPTY_LIST;
 					}
-					Collections.sort(fVariables, new Comparator() {
-						public int compare(Object a, Object b) {
-							JDIFieldVariable v1= (JDIFieldVariable)a;
-							JDIFieldVariable v2= (JDIFieldVariable)b;
-							try {
-								return v1.getName().compareToIgnoreCase(v2.getName());
-							} catch (DebugException de) {
-								logError(de);
-								return -1;
+					if (allFields != null) {
+						Iterator fields= allFields.iterator();
+						while (fields.hasNext()) {
+							Field field= (Field) fields.next();
+							if (field.isStatic()) {
+								fVariables.add(new JDIFieldVariable((JDIDebugTarget)getDebugTarget(), field, declaringType));
 							}
 						}
-					});
+						Collections.sort(fVariables, new Comparator() {
+							public int compare(Object a, Object b) {
+								JDIFieldVariable v1= (JDIFieldVariable)a;
+								JDIFieldVariable v2= (JDIFieldVariable)b;
+								try {
+									return v1.getName().compareToIgnoreCase(v2.getName());
+								} catch (DebugException de) {
+									logError(de);
+									return -1;
+								}
+							}
+						});
+					}
+				} else {
+					// add "this"
+					ObjectReference t= getUnderlyingThisObject();
+					if (t != null) {
+						fVariables.add(new JDIThisVariable((JDIDebugTarget)getDebugTarget(), t));
+					}
 				}
-			} else {
-				// add "this"
-				ObjectReference t= getUnderlyingThisObject();
-				if (t != null) {
-					fVariables.add(new JDIThisVariable((JDIDebugTarget)getDebugTarget(), t));
+				// add locals
+				Iterator variables= getUnderlyingVisibleVariables().iterator();
+				while (variables.hasNext()) {
+					LocalVariable var= (LocalVariable) variables.next();
+					fVariables.add(new JDILocalVariable(this, var));
 				}
+			} else if (fRefreshVariables) {
+				updateVariables();
 			}
-			// add locals
-			Iterator variables= getUnderlyingVisibleVariables().iterator();
-			while (variables.hasNext()) {
-				LocalVariable var= (LocalVariable) variables.next();
-				fVariables.add(new JDILocalVariable(this, var));
-			}
-		} else if (fRefreshVariables) {
-			updateVariables();
+			fRefreshVariables = false;
+			return fVariables;
 		}
-		fRefreshVariables = false;
-		return fVariables;
 	}
 
 	/**
@@ -286,9 +327,21 @@ public class JDIStackFrame extends JDIDebugElement implements IJavaStackFrame {
 	 */
 	public List getArgumentTypeNames() throws DebugException {
 		try {
-			return getUnderlyingMethod().argumentTypeNames();
+			Method underlyingMethod= getUnderlyingMethod();
+			String genericSignature= underlyingMethod.genericSignature();
+			if (genericSignature == null) {
+				// no generic signature
+				return underlyingMethod.argumentTypeNames();
+			}
+			// generic signature
+			String[] parameterTypes= Signature.getParameterTypes(genericSignature);
+			List argumentTypeNames= new ArrayList();
+			for (int i= 0; i < parameterTypes.length; i++) {
+				argumentTypeNames.add(Signature.toString(parameterTypes[i]).replace('/', '.'));
+			}
+			return argumentTypeNames;
 		} catch (RuntimeException e) {
-			targetRequestFailed(MessageFormat.format(JDIDebugModelMessages.getString("JDIStackFrame.exception_retrieving_argument_type_names"), new String[] {e.toString()}), e); //$NON-NLS-1$
+			targetRequestFailed(MessageFormat.format(JDIDebugModelMessages.JDIStackFrame_exception_retrieving_argument_type_names, new String[] {e.toString()}), e); //$NON-NLS-1$
 			// execution will never reach this line, as
 			// #targetRequestFailed will throw an exception			
 			return null;
@@ -299,15 +352,12 @@ public class JDIStackFrame extends JDIDebugElement implements IJavaStackFrame {
 	 * @see IStackFrame#getLineNumber()
 	 */
 	public int getLineNumber() throws DebugException {
-		if (isSuspended()) {
+		synchronized (fThread) {
 			try {
-				Location location= getUnderlyingStackFrame().location();
-				if (location != null) {
-					return location.lineNumber();
-				}
+				return fLocation.lineNumber();
 			} catch (RuntimeException e) {
 				if (getThread().isSuspended()) {
-					targetRequestFailed(MessageFormat.format(JDIDebugModelMessages.getString("JDIStackFrame.exception_retrieving_line_number"), new String[] {e.toString()}), e); //$NON-NLS-1$
+					targetRequestFailed(MessageFormat.format(JDIDebugModelMessages.JDIStackFrame_exception_retrieving_line_number, new String[] {e.toString()}), e); //$NON-NLS-1$
 				}
 			}
 		}
@@ -444,7 +494,7 @@ public class JDIStackFrame extends JDIDebugElement implements IJavaStackFrame {
 		} catch (NativeMethodException e) {
 			locals= Collections.EMPTY_LIST;
 		} catch (RuntimeException e) {
-			targetRequestFailed(MessageFormat.format(JDIDebugModelMessages.getString("JDIStackFrame.exception_retrieving_visible_variables"),new String[] {e.toString()}), e); //$NON-NLS-1$
+			targetRequestFailed(MessageFormat.format(JDIDebugModelMessages.JDIStackFrame_exception_retrieving_visible_variables,new String[] {e.toString()}), e); //$NON-NLS-1$
 			// execution will not reach this line, as 
 			// #targetRequestFailed will throw an exception			
 			return;
@@ -477,23 +527,34 @@ public class JDIStackFrame extends JDIDebugElement implements IJavaStackFrame {
 			fVariables.add(local);
 		}
 	}
+    
+    /**
+     * @see org.eclipse.debug.core.model.IDropToFrame#canDropToFrame()
+     */
+    public boolean canDropToFrame() {
+        return supportsDropToFrame();
+    }
 
 	/**
 	 * @see IJavaStackFrame#supportsDropToFrame()
 	 */
 	public boolean supportsDropToFrame() {
-		//FIXME 1GH3XDA: ITPDUI:ALL - Drop to frame hangs if after invoke
 		JDIThread thread= (JDIThread) getThread();
 		JDIDebugTarget target= (JDIDebugTarget)thread.getDebugTarget();
 		try {
-			if (!target.isAvailable() || !thread.isSuspended() || thread.isTerminated()) {
+			if (!target.isAvailable() || !thread.isSuspended() || thread.isTerminated() ||
+					thread.isInvokingMethod()) {
 				return false;
 			} 
 			boolean j9Support= false;
 			boolean jdkSupport= target.canPopFrames();
+			VirtualMachine vm = getVM();
+			if (vm == null) {
+				return false;
+			}
 			try {
 				j9Support= (thread.getUnderlyingThread() instanceof org.eclipse.jdi.hcr.ThreadReference) &&
-						((org.eclipse.jdi.hcr.VirtualMachine) ((JDIDebugTarget) getDebugTarget()).getVM()).canDoReturn();
+						((org.eclipse.jdi.hcr.VirtualMachine)vm).canDoReturn();
 			} catch (UnsupportedOperationException uoe) {
 				j9Support= false;
 			}
@@ -505,14 +566,15 @@ public class JDIStackFrame extends JDIDebugElement implements IJavaStackFrame {
 				if (jdkSupport) {
 					// JDK 1.4 VMs are currently unable to pop the bottom
 					// stack frame.
-					if (frames.get(frames.size() - 1) == this) {
+					if ((frames.size() > 0) && frames.get(frames.size() - 1) == this) {
 						return false;
 					}
 				}
-				Iterator iter= frames.iterator();
+				int index = 0;
 				JDIStackFrame frame= null;
-				while (iter.hasNext()) {
-					frame= (JDIStackFrame) iter.next();
+				while (index < frames.size()) {
+					frame= (JDIStackFrame) frames.get(index);
+					index++;
 					if (frame.isNative()) {
 						return false;
 					}
@@ -520,7 +582,7 @@ public class JDIStackFrame extends JDIDebugElement implements IJavaStackFrame {
 						if (jdkSupport) {
 							// JDK 1.4 VMs are currently unable to pop the
 							// frame directly above a native frame
-							if (iter.hasNext() && ((JDIStackFrame)iter.next()).isNative()) {
+							if (index < frames.size() && ((JDIStackFrame)frames.get(index)).isNative()) {
 								return false;
 							}
 						}
@@ -530,6 +592,10 @@ public class JDIStackFrame extends JDIDebugElement implements IJavaStackFrame {
 			}
 			return false;
 		} catch (DebugException e) {
+			if (e.getStatus().getException() instanceof IncompatibleThreadStateException || e.getStatus().getCode() == IJavaThread.ERR_THREAD_NOT_SUSPENDED) {
+				// if the thread has since resumed, drop is not supported
+				return false;
+			}
 			logError(e);
 		} catch (UnsupportedOperationException e) {
 			// drop to frame not supported - this is an expected
@@ -548,7 +614,7 @@ public class JDIStackFrame extends JDIDebugElement implements IJavaStackFrame {
 		if (supportsDropToFrame()) {
 			((JDIThread) getThread()).dropToFrame(this);
 		} else {
-			notSupported(JDIDebugModelMessages.getString("JDIStackFrame.Drop_to_frame_not_supported")); //$NON-NLS-1$
+			notSupported(JDIDebugModelMessages.JDIStackFrame_Drop_to_frame_not_supported); //$NON-NLS-1$
 		}
 	}
 
@@ -556,7 +622,7 @@ public class JDIStackFrame extends JDIDebugElement implements IJavaStackFrame {
 		if (supportsDropToFrame()) {
 			((JDIThread) getThread()).popFrame(this);
 		} else {
-			notSupported(JDIDebugModelMessages.getString("JDIStackFrame.pop_frame_not_supported")); //$NON-NLS-1$
+			notSupported(JDIDebugModelMessages.JDIStackFrame_pop_frame_not_supported); //$NON-NLS-1$
 		}
 	}
 
@@ -576,7 +642,7 @@ public class JDIStackFrame extends JDIDebugElement implements IJavaStackFrame {
 			}
 			if (var instanceof JDIThisVariable) {
 				// save for later - check for instance and static vars
-				thisVariable= (IJavaVariable)var;
+				thisVariable= var;
 			}
 		}
 
@@ -602,18 +668,19 @@ public class JDIStackFrame extends JDIDebugElement implements IJavaStackFrame {
 	 * @see JDIDebugElement#targetRequestFailed(String, RuntimeException)
 	 */
 	protected List getUnderlyingVisibleVariables() throws DebugException {
-		List variables= Collections.EMPTY_LIST;
-		try {
-			variables= getUnderlyingStackFrame().visibleVariables();
-		} catch (AbsentInformationException e) {
-			setLocalsAvailable(false);
-		} catch (NativeMethodException e) {
-			setLocalsAvailable(false);
-		} catch (RuntimeException e) {
-			targetRequestFailed(MessageFormat.format(JDIDebugModelMessages.getString("JDIStackFrame.exception_retrieving_visible_variables_2"),new String[] {e.toString()}), e); //$NON-NLS-1$
+		synchronized (fThread) {
+			List variables= Collections.EMPTY_LIST;
+			try {
+				variables= getUnderlyingStackFrame().visibleVariables();
+			} catch (AbsentInformationException e) {
+				setLocalsAvailable(false);
+			} catch (NativeMethodException e) {
+				setLocalsAvailable(false);
+			} catch (RuntimeException e) {
+				targetRequestFailed(MessageFormat.format(JDIDebugModelMessages.JDIStackFrame_exception_retrieving_visible_variables_2,new String[] {e.toString()}), e); //$NON-NLS-1$
+			}
+			return variables;
 		}
-
-		return variables;
 	}
 
 	/**
@@ -623,17 +690,19 @@ public class JDIStackFrame extends JDIDebugElement implements IJavaStackFrame {
 	 * @see JDIDebugElement#targetRequestFailed(String, RuntimeException)
 	 */
 	protected ObjectReference getUnderlyingThisObject() throws DebugException {
-		if (fStackFrame == null || fThisObject == null && !isStatic()) {
-			try {
-				fThisObject = getUnderlyingStackFrame().thisObject();
-			} catch (RuntimeException e) {
-				targetRequestFailed(MessageFormat.format(JDIDebugModelMessages.getString("JDIStackFrame.exception_retrieving_this"),new String[] {e.toString()}), e); //$NON-NLS-1$
-				// execution will not reach this line, as 
-				// #targetRequestFailed will throw an exception			
-				return null;
+		synchronized (fThread) {
+			if ((fStackFrame == null || fThisObject == null) && !isStatic()) {
+				try {
+					fThisObject = getUnderlyingStackFrame().thisObject();
+				} catch (RuntimeException e) {
+					targetRequestFailed(MessageFormat.format(JDIDebugModelMessages.JDIStackFrame_exception_retrieving_this,new String[] {e.toString()}), e); //$NON-NLS-1$
+					// execution will not reach this line, as 
+					// #targetRequestFailed will throw an exception			
+					return null;
+				}
 			}
+			return fThisObject;
 		}
-		return fThisObject;
 	}
 	
 	/**
@@ -653,7 +722,7 @@ public class JDIStackFrame extends JDIDebugElement implements IJavaStackFrame {
 		try {
 			return getUnderlyingMethod().signature();
 		} catch (RuntimeException e) {
-			targetRequestFailed(MessageFormat.format(JDIDebugModelMessages.getString("JDIStackFrame.exception_retrieving_method_signature"), new String[] {e.toString()}), e); //$NON-NLS-1$
+			targetRequestFailed(MessageFormat.format(JDIDebugModelMessages.JDIStackFrame_exception_retrieving_method_signature, new String[] {e.toString()}), e); //$NON-NLS-1$
 			// execution will not reach this line, as 
 			// #targetRequestFailed will throw an exception			
 			return null;
@@ -664,21 +733,19 @@ public class JDIStackFrame extends JDIDebugElement implements IJavaStackFrame {
 	 * @see IJavaStackFrame#getDeclaringTypeName()
 	 */
 	public String getDeclaringTypeName() throws DebugException {
-		if (fStackFrame == null || fDeclaringTypeName == null) {
+		synchronized (fThread) {
 			try {
 				if (isObsolete()) {
-					fDeclaringTypeName=  JDIDebugModelMessages.getString("JDIStackFrame.<unknown_declaring_type>_1"); //$NON-NLS-1$
-				} else {
-					fDeclaringTypeName= getUnderlyingMethod().declaringType().name();
+					return  JDIDebugModelMessages.JDIStackFrame__unknown_declaring_type__1; //$NON-NLS-1$
 				}
+				return JDIReferenceType.getGenericName(getUnderlyingMethod().declaringType());
 			} catch (RuntimeException e) {
 				if (getThread().isSuspended()) {
-					targetRequestFailed(MessageFormat.format(JDIDebugModelMessages.getString("JDIStackFrame.exception_retrieving_declaring_type"), new String[] {e.toString()}), e); //$NON-NLS-1$
+					targetRequestFailed(MessageFormat.format(JDIDebugModelMessages.JDIStackFrame_exception_retrieving_declaring_type, new String[] {e.toString()}), e); //$NON-NLS-1$
 				}
-				return JDIDebugModelMessages.getString("JDIStackFrame.<unknown_declaring_type>_1"); //$NON-NLS-1$
+				return JDIDebugModelMessages.JDIStackFrame__unknown_declaring_type__1; //$NON-NLS-1$
 			}
 		}
-		return fDeclaringTypeName;
 	}
 	
 	/**
@@ -688,20 +755,20 @@ public class JDIStackFrame extends JDIDebugElement implements IJavaStackFrame {
 		if (fStackFrame == null || fReceivingTypeName == null) {
 			try {
 				if (isObsolete()) {
-					fReceivingTypeName=JDIDebugModelMessages.getString("JDIStackFrame.<unknown_receiving_type>_2"); //$NON-NLS-1$
+					fReceivingTypeName=JDIDebugModelMessages.JDIStackFrame__unknown_receiving_type__2; //$NON-NLS-1$
 				} else {
 					ObjectReference thisObject = getUnderlyingThisObject();
 					if (thisObject == null) {
 						fReceivingTypeName = getDeclaringTypeName();
 					} else {
-						fReceivingTypeName = thisObject.referenceType().name();
+						fReceivingTypeName = JDIReferenceType.getGenericName(thisObject.referenceType());
 					}
 				}
 			} catch (RuntimeException e) {
 				if (getThread().isSuspended()) {
-					targetRequestFailed(MessageFormat.format(JDIDebugModelMessages.getString("JDIStackFrame.exception_retrieving_receiving_type"), new String[] {e.toString()}), e); //$NON-NLS-1$
+					targetRequestFailed(MessageFormat.format(JDIDebugModelMessages.JDIStackFrame_exception_retrieving_receiving_type, new String[] {e.toString()}), e); //$NON-NLS-1$
 				}
-				return JDIDebugModelMessages.getString("JDIStackFrame.<unknown_receiving_type>_2"); //$NON-NLS-1$
+				return JDIDebugModelMessages.JDIStackFrame__unknown_receiving_type__2; //$NON-NLS-1$
 			}
 		}
 		return fReceivingTypeName;
@@ -715,9 +782,9 @@ public class JDIStackFrame extends JDIDebugElement implements IJavaStackFrame {
 			return getUnderlyingMethod().name();	
 		} catch (RuntimeException e) {
 			if (getThread().isSuspended()) {
-				targetRequestFailed(MessageFormat.format(JDIDebugModelMessages.getString("JDIStackFrame.exception_retrieving_method_name"), new String[] {e.toString()}), e); //$NON-NLS-1$
+				targetRequestFailed(MessageFormat.format(JDIDebugModelMessages.JDIStackFrame_exception_retrieving_method_name, new String[] {e.toString()}), e); //$NON-NLS-1$
 			}
-			return JDIDebugModelMessages.getString("JDIStackFrame.<unknown_method>_1"); //$NON-NLS-1$
+			return JDIDebugModelMessages.JDIStackFrame__unknown_method__1; //$NON-NLS-1$
 		}
 	}
 	
@@ -802,25 +869,18 @@ public class JDIStackFrame extends JDIDebugElement implements IJavaStackFrame {
 	 * @see IJavaStackFrame#getSourceName()
 	 */
 	public String getSourceName() throws DebugException {
-		if (fSourceName == null) {
+		synchronized (fThread) {
 			try {
-				Location l = getUnderlyingMethod().location();
-				if (l != null) {
-					fSourceName = l.sourceName();
-				}
+				return fLocation.sourceName();
 			} catch (AbsentInformationException e) {
-				fSourceName = ""; //$NON-NLS-1$
+				return null;
 			} catch (NativeMethodException e) {
-				fSourceName = ""; //$NON-NLS-1$
+				return null;
 			} catch (RuntimeException e) {
-				targetRequestFailed(MessageFormat.format(JDIDebugModelMessages.getString("JDIStackFrame.exception_retrieving_source_name"), new String[] {e.toString()}), e); //$NON-NLS-1$
+				targetRequestFailed(MessageFormat.format(JDIDebugModelMessages.JDIStackFrame_exception_retrieving_source_name, new String[] {e.toString()}), e); //$NON-NLS-1$
 			}
 		}
-		if (fSourceName == null || fSourceName.length() == 0) {
-			return null;
-		} else {
-			return fSourceName;
-		}
+		return null;
 	}
 	
 	protected boolean isTopStackFrame() throws DebugException {
@@ -858,7 +918,7 @@ public class JDIStackFrame extends JDIDebugElement implements IJavaStackFrame {
 	/**
 	 * @see IJavaStackFrame#isObsolete()
 	 */
-	public boolean isObsolete() throws DebugException {
+	public boolean isObsolete() {
 		if (!JDIDebugPlugin.isJdiVersionGreaterThanOrEqual(new int[] {1,4}) || !((JDIDebugTarget)getDebugTarget()).hasHCROccurred()) {
 			// If no hot code replace has occurred, this frame
 			// cannot be obsolete.
@@ -866,34 +926,25 @@ public class JDIStackFrame extends JDIDebugElement implements IJavaStackFrame {
 		}
 		// if this frame's thread is not suspended, the obsolete status cannot
 		// change until it suspends again
-		if (getThread().isSuspended()) {
-			try {
+		synchronized (fThread) {
+			if (getThread().isSuspended()) {
 				return getUnderlyingMethod().isObsolete();
-			} catch (RuntimeException re) {
-				targetRequestFailed(MessageFormat.format(JDIDebugModelMessages.getString("JDIStackFrame.Exception_occurred_determining_if_stack_frame_is_obsolete_1"), new String[] {re.toString()}), re); //$NON-NLS-1$
-				// execution will not reach this line, as 
-				// #targetRequestFailed will throw an exception			
-				return true;
 			}
+			return false;
 		}
-		return false;
 	}
 	
-	protected boolean exists() throws DebugException {
-		return ((JDIThread)getThread()).computeStackFrames().indexOf(this) != -1;
+	protected boolean exists() {
+		synchronized (fThread) {
+			return fDepth != -1;
+		}
 	}
 	
 	/**
 	 * @see ITerminate#canTerminate()
 	 */
 	public boolean canTerminate() {
-		boolean exists= false;
-		try {
-			exists= exists();
-		} catch (DebugException e) {
-			logError(e);
-		}
-		return exists && getThread().canTerminate() || getDebugTarget().canTerminate();
+		return exists() && getThread().canTerminate() || getDebugTarget().canTerminate();
 	}
 
 	/**
@@ -923,15 +974,24 @@ public class JDIStackFrame extends JDIDebugElement implements IJavaStackFrame {
 	 *  resumed, and is not yet suspended).
 	 */
 	protected StackFrame getUnderlyingStackFrame() throws DebugException {
-		if (fStackFrame == null) {
-			// In the case of preserved stack frames, the underlying stack frame
-			// could have been set via a call to JDIThread#updateStackFrames
-			((JDIThread)getThread()).computeStackFrames();
+		synchronized (fThread) {
 			if (fStackFrame == null) {
-				requestFailed(JDIDebugModelMessages.getString("JDIStackFrame.Thread_not_suspended,_stack_frame_unavailable._3"), null, IJavaThread.ERR_THREAD_NOT_SUSPENDED); //$NON-NLS-1$
+				if (fDepth == -1) {
+					throw new DebugException(new Status(IStatus.ERROR, JDIDebugPlugin.getUniqueIdentifier(), IJavaStackFrame.ERR_INVALID_STACK_FRAME, JDIDebugModelMessages.JDIStackFrame_25, null)); //$NON-NLS-1$
+				}
+				if (fThread.isSuspended()) {
+					// re-index stack frames - See Bug 47198
+					fThread.computeStackFrames();
+					if (fDepth == -1) {
+						// If depth is -1, then this is an invalid frame
+						throw new DebugException(new Status(IStatus.ERROR, JDIDebugPlugin.getUniqueIdentifier(), IJavaStackFrame.ERR_INVALID_STACK_FRAME, JDIDebugModelMessages.JDIStackFrame_25, null)); //$NON-NLS-1$
+					}
+				} else {
+					throw new DebugException(new Status(IStatus.ERROR, JDIDebugPlugin.getUniqueIdentifier(), IJavaThread.ERR_THREAD_NOT_SUSPENDED, JDIDebugModelMessages.JDIStackFrame_25, null)); //$NON-NLS-1$
+				}
 			}
+			return fStackFrame;
 		}
-		return fStackFrame;
 	}
 	
 	/**
@@ -941,36 +1001,14 @@ public class JDIStackFrame extends JDIDebugElement implements IJavaStackFrame {
 	 * @param frame The underlying stack frame
 	 */
 	protected void setUnderlyingStackFrame(StackFrame frame) {
-		if (frame != null) {
-			fMethod= frame.location().method();
-			if (fLastMethod != null && !fLastMethod.equals(fMethod)) {
-				clearCachedData();
-			}
-			fLastMethod= fMethod;
-		} else {
-			if (fMethod != null) {
-				// only set the last method if the current method is not null:
-				//     * last method should never be null
-				fLastMethod = fMethod;
+		synchronized (fThread) {
+			fStackFrame = frame;
+			if (frame == null) {
+				fRefreshVariables = true;
 			}
 		}
-		fStackFrame = frame;
-		fRefreshVariables = true;
-		// always clear 'this' and receiver type name, as we could be in a
-		// different receiver next time, but still in the same location (method)
-		fThisObject = null;		
-		fReceivingTypeName = null;
 	}
 	
-	/**
-	 * The underlying method that existed before the current underlying
-	 * method.  Used only so that equality can be checked on stack frame
-	 * after the new one has been set.
-	 */
-	protected Method getLastMethod() {
-		return fLastMethod;
-	}
-
 	protected void setThread(JDIThread thread) {
 		fThread = thread;
 	}
@@ -1010,7 +1048,7 @@ public class JDIStackFrame extends JDIDebugElement implements IJavaStackFrame {
 	 * 
 	 * @see IStackFrame#getRegisterGroups()
 	 */
-	public IRegisterGroup[] getRegisterGroups() throws DebugException {
+	public IRegisterGroup[] getRegisterGroups() {
 		return new IRegisterGroup[0];
 	}
 
@@ -1021,12 +1059,29 @@ public class JDIStackFrame extends JDIDebugElement implements IJavaStackFrame {
 		Method method = getUnderlyingMethod();
 		try {
 			Type type = method.declaringType();
-			return (IJavaClassType)JDIType.createType((JDIDebugTarget)getDebugTarget(), type);
+			if (type instanceof ClassType) {
+				return (IJavaClassType)JDIType.createType((JDIDebugTarget)getDebugTarget(), type);
+			}
+			targetRequestFailed(JDIDebugModelMessages.JDIStackFrame_0, null);
 		} catch (RuntimeException e) {
-			targetRequestFailed(MessageFormat.format(JDIDebugModelMessages.getString("JDIStackFrame.exception_retreiving_declaring_type"), new String[] {e.toString()}), e); //$NON-NLS-1$
+			targetRequestFailed(MessageFormat.format(JDIDebugModelMessages.JDIStackFrame_exception_retreiving_declaring_type, new String[] {e.toString()}), e); //$NON-NLS-1$
 		}
 		return null;
 	}
+	
+	/* (non-Javadoc)
+	 * @see org.eclipse.jdt.debug.core.IJavaStackFrame#getReferenceType()
+	 */
+	public IJavaReferenceType getReferenceType() throws DebugException {
+		Method method = getUnderlyingMethod();
+		try {
+			Type type = method.declaringType();
+			return (IJavaReferenceType)JDIType.createType((JDIDebugTarget)getDebugTarget(), type);
+		} catch (RuntimeException e) {
+			targetRequestFailed(MessageFormat.format(JDIDebugModelMessages.JDIStackFrame_exception_retreiving_declaring_type, new String[] {e.toString()}), e); //$NON-NLS-1$
+		}
+		return null;
+	}	
 
 	/**
 	 * Expression level stepping not supported.
@@ -1052,11 +1107,8 @@ public class JDIStackFrame extends JDIDebugElement implements IJavaStackFrame {
 	 * that the cached data may not be valid.
 	 */
 	private void clearCachedData() {
-		fMethod= null;
 		fThisObject= null;
-		fDeclaringTypeName= null;
 		fReceivingTypeName= null;	
-		fSourceName= null;
 	}
 	
 	/**
@@ -1083,7 +1135,7 @@ public class JDIStackFrame extends JDIDebugElement implements IJavaStackFrame {
 	/**
 	 * @see IStackFrame#hasRegisterGroups()
 	 */
-	public boolean hasRegisterGroups() throws DebugException {
+	public boolean hasRegisterGroups() {
 		return false;
 	}
 
@@ -1112,4 +1164,73 @@ public class JDIStackFrame extends JDIDebugElement implements IJavaStackFrame {
 		((IJavaThread)getThread()).stepWithFilters();
 	}
 
+	/**
+	 * @see org.eclipse.jdt.debug.core.IJavaStackFrame#getSourcePath(java.lang.String)
+	 */
+	public String getSourcePath(String stratum) throws DebugException {
+		synchronized (fThread) {
+			try {
+				return fLocation.sourcePath(stratum);
+			} catch (AbsentInformationException e) {
+			} catch (RuntimeException e) {
+				targetRequestFailed(MessageFormat.format(JDIDebugModelMessages.JDIStackFrame_exception_retrieving_source_path, new String[] {e.toString()}), e); //$NON-NLS-1$
+			}
+		}
+		return null;
+	}
+	
+	/**
+	 * @see org.eclipse.jdt.debug.core.IJavaStackFrame#getSourcePath()
+	 */
+	public String getSourcePath() throws DebugException {
+		synchronized (fThread) {
+			try {
+				return fLocation.sourcePath();
+			} catch (AbsentInformationException e) {
+			} catch (RuntimeException e) {
+				targetRequestFailed(MessageFormat.format(JDIDebugModelMessages.JDIStackFrame_exception_retrieving_source_path, new String[] {e.toString()}), e); //$NON-NLS-1$
+			}
+		}
+		return null;
+	}	
+
+	/*
+	 * @see org.eclipse.jdt.debug.core.IJavaStackFrame#getLineNumber(java.lang.String)
+	 */
+	public int getLineNumber(String stratum) throws DebugException {
+		synchronized (fThread) {
+			try {
+				return fLocation.lineNumber(stratum);
+			} catch (RuntimeException e) {
+				if (getThread().isSuspended()) {
+					targetRequestFailed(MessageFormat.format(JDIDebugModelMessages.JDIStackFrame_exception_retrieving_line_number, new String[] {e.toString()}), e); //$NON-NLS-1$
+				}
+			}
+		}
+		return -1;
+	}
+
+	/*
+	 * @see org.eclipse.jdt.debug.core.IJavaStackFrame#getSourceName(java.lang.String)
+	 */
+	public String getSourceName(String stratum) throws DebugException {
+		synchronized (fThread) {
+			try {
+				return fLocation.sourceName(stratum);
+			} catch (AbsentInformationException e) {
+			} catch (NativeMethodException e) {
+			} catch (RuntimeException e) {
+				targetRequestFailed(MessageFormat.format(JDIDebugModelMessages.JDIStackFrame_exception_retrieving_source_name, new String[] {e.toString()}), e); //$NON-NLS-1$
+			}
+		}
+		return null;
+	}
+
+	/* (non-Javadoc)
+	 * @see org.eclipse.jdt.debug.core.IJavaStackFrame#isVarargs()
+	 */
+	public boolean isVarArgs() throws DebugException {
+		return getUnderlyingMethod().isVarArgs();
+	}
+
 }
