@@ -1,9 +1,9 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
+ * http://www.eclipse.org/legal/epl-v10.html
  * 
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -18,6 +18,7 @@ import java.io.FileOutputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.net.URL;
+import java.text.MessageFormat;
 import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.Hashtable;
@@ -27,10 +28,10 @@ import java.util.Map;
 
 import javax.xml.parsers.DocumentBuilder;
 import javax.xml.parsers.DocumentBuilderFactory;
+import javax.xml.parsers.FactoryConfigurationError;
 import javax.xml.parsers.ParserConfigurationException;
+import javax.xml.transform.TransformerException;
 
-import org.apache.xerces.dom.DocumentImpl;
-import org.eclipse.core.internal.events.ResourceChangeEvent;
 import org.eclipse.core.resources.IResourceChangeEvent;
 import org.eclipse.core.resources.IResourceChangeListener;
 import org.eclipse.core.resources.IWorkspaceRunnable;
@@ -39,17 +40,16 @@ import org.eclipse.core.runtime.CoreException;
 import org.eclipse.core.runtime.IConfigurationElement;
 import org.eclipse.core.runtime.IExtensionPoint;
 import org.eclipse.core.runtime.IPath;
-import org.eclipse.core.runtime.IPluginDescriptor;
 import org.eclipse.core.runtime.IProgressMonitor;
 import org.eclipse.core.runtime.IStatus;
 import org.eclipse.core.runtime.MultiStatus;
-import org.eclipse.core.runtime.NullProgressMonitor;
 import org.eclipse.core.runtime.Path;
 import org.eclipse.core.runtime.Platform;
 import org.eclipse.core.runtime.Plugin;
 import org.eclipse.core.runtime.Preferences;
 import org.eclipse.core.runtime.Status;
 import org.eclipse.core.runtime.Preferences.PropertyChangeEvent;
+import org.eclipse.core.runtime.jobs.Job;
 import org.eclipse.debug.core.DebugEvent;
 import org.eclipse.debug.core.DebugPlugin;
 import org.eclipse.debug.core.IDebugEventSetListener;
@@ -61,29 +61,47 @@ import org.eclipse.debug.core.model.IProcess;
 import org.eclipse.jdt.core.IClasspathEntry;
 import org.eclipse.jdt.core.IJavaProject;
 import org.eclipse.jdt.core.JavaCore;
+import org.eclipse.jdt.launching.IRuntimeClasspathEntry2;
 import org.eclipse.jdt.launching.IVMConnector;
 import org.eclipse.jdt.launching.IVMInstall;
 import org.eclipse.jdt.launching.IVMInstallChangedListener;
 import org.eclipse.jdt.launching.JavaRuntime;
 import org.eclipse.jdt.launching.VMStandin;
 import org.eclipse.jdt.launching.sourcelookup.ArchiveSourceLocation;
+import org.osgi.framework.BundleContext;
 import org.w3c.dom.Document;
 import org.w3c.dom.Element;
 import org.w3c.dom.Node;
 import org.w3c.dom.NodeList;
 import org.xml.sax.InputSource;
 import org.xml.sax.SAXException;
+import org.xml.sax.helpers.DefaultHandler;
 
 public class LaunchingPlugin extends Plugin implements Preferences.IPropertyChangeListener, IVMInstallChangedListener, IResourceChangeListener, ILaunchesListener, IDebugEventSetListener {
 	
+	/**
+	 * The id of the JDT launching plug-in (value <code>"org.eclipse.jdt.launching"</code>).
+	 */	
+	public static final String ID_PLUGIN= "org.eclipse.jdt.launching"; //$NON-NLS-1$
+	
 	/**
 	 * Identifier for 'vmConnectors' extension point
 	 */
-	public static final String ID_EXTENSION_POINT_VM_CONNECTORS = getUniqueIdentifier() + ".vmConnectors"; //$NON-NLS-1$
+	public static final String ID_EXTENSION_POINT_VM_CONNECTORS = "vmConnectors"; //$NON-NLS-1$
+	
+	/**
+	 * Identifier for 'runtimeClasspathEntries' extension point
+	 */
+	public static final String ID_EXTENSION_POINT_RUNTIME_CLASSPATH_ENTRIES = "runtimeClasspathEntries"; //$NON-NLS-1$	
 	
 	private static LaunchingPlugin fgLaunchingPlugin;
 	
 	private HashMap fVMConnectors = null;
+	
+	/**
+	 * Runtime classpath extensions
+	 */
+	private HashMap fClasspathEntryExtensions = null;
 
 	private String fOldVMPrefString = EMPTY_STRING;
 	
@@ -103,6 +121,11 @@ public class LaunchingPlugin extends Plugin implements Preferences.IPropertyChan
 	 */
 	private boolean fBatchingChanges = false;
 	
+	/**
+	 * Shared XML parser
+	 */
+	private static DocumentBuilder fgXMLParser = null;
+	
 	/**
 	 * Stores VM changes resulting from a JRE preference change.
 	 */
@@ -177,21 +200,28 @@ public class LaunchingPlugin extends Plugin implements Preferences.IPropertyChan
 		 * changes.
 		 */
 		public void process() throws CoreException {
+			JREUpdateJob job = new JREUpdateJob(this);
+			job.schedule();
+		}
+		
+		protected void doit(IProgressMonitor monitor) throws CoreException {
 			IWorkspaceRunnable runnable = new IWorkspaceRunnable() {
-				public void run(IProgressMonitor monitor) throws CoreException {
-					rebind();
+				public void run(IProgressMonitor monitor1) throws CoreException {
+					IJavaProject[] projects = JavaCore.create(ResourcesPlugin.getWorkspace().getRoot()).getJavaProjects();
+					monitor1.beginTask(LaunchingMessages.LaunchingPlugin_0, projects.length + 1); //$NON-NLS-1$
+					rebind(monitor1, projects);
+					monitor1.done();
 				}
 			};
-			ResourcesPlugin.getWorkspace().run(runnable, new NullProgressMonitor());
+			JavaCore.run(runnable, null, monitor);
 		}
 				
 		/**
 		 * Re-bind classpath variables and containers affected by the JRE
 		 * changes.
+		 * @param monitor
 		 */
-		private void rebind() throws CoreException {
-			
-			IJavaProject[] projects = JavaCore.create(ResourcesPlugin.getWorkspace().getRoot()).getJavaProjects();
+		private void rebind(IProgressMonitor monitor, IJavaProject[] projects) throws CoreException {
 			 
 			if (fDefaultChanged) {
 				// re-bind JRELIB if the default VM changed
@@ -200,8 +230,9 @@ public class LaunchingPlugin extends Plugin implements Preferences.IPropertyChan
 				initializer.initialize(JavaRuntime.JRESRC_VARIABLE);
 				initializer.initialize(JavaRuntime.JRESRCROOT_VARIABLE);
 			}
+			monitor.worked(1);
 														
-			// re-bind all container entries, noting which project need to be re-built
+			// re-bind all container entries
 			for (int i = 0; i < projects.length; i++) {
 				IJavaProject project = projects[i];
 				IClasspathEntry[] entries = project.getRawClasspath();
@@ -241,16 +272,38 @@ public class LaunchingPlugin extends Plugin implements Preferences.IPropertyChan
 				if (replace) {
 					project.setRawClasspath(entries, null);
 				}
+				monitor.worked(1);
 			}
 
 		}
 
 	}
 	
+	class JREUpdateJob extends Job {
+		private VMChanges fChanges;
+		
+		public JREUpdateJob(VMChanges changes) {
+			super(LaunchingMessages.LaunchingPlugin_1); //$NON-NLS-1$
+			fChanges = changes;
+			setSystem(true);
+		}
+
+		/* (non-Javadoc)
+		 * @see org.eclipse.core.runtime.jobs.Job#run(org.eclipse.core.runtime.IProgressMonitor)
+		 */
+		protected IStatus run(IProgressMonitor monitor) {
+			try {
+				fChanges.doit(monitor);
+			} catch (CoreException e) {
+				return e.getStatus();
+			}
+			return Status.OK_STATUS;
+		}
+		
+	}
 	
-	
-	public LaunchingPlugin(IPluginDescriptor descriptor) {
-		super(descriptor);
+	public LaunchingPlugin() {
+		super();
 		fgLaunchingPlugin= this;
 	}
 	
@@ -294,7 +347,7 @@ public class LaunchingPlugin extends Plugin implements Preferences.IPropertyChan
 	public static File getFileInPlugin(IPath path) {
 		try {
 			URL installURL =
-				new URL(getDefault().getDescriptor().getInstallURL(), path.toString());
+				new URL(getDefault().getBundle().getEntry("/"), path.toString()); //$NON-NLS-1$
 			URL localURL = Platform.asLocalURL(installURL);
 			return new File(localURL.getFile());
 		} catch (IOException ioe) {
@@ -306,13 +359,7 @@ public class LaunchingPlugin extends Plugin implements Preferences.IPropertyChan
 	 * Convenience method which returns the unique identifier of this plugin.
 	 */
 	public static String getUniqueIdentifier() {
-		if (getDefault() == null) {
-			// If the default instance is not yet initialized,
-			// return a static identifier. This identifier must
-			// match the plugin id defined in plugin.xml
-			return "org.eclipse.jdt.launching"; //$NON-NLS-1$
-		}
-		return getDefault().getDescriptor().getUniqueIdentifier();
+		return ID_PLUGIN;
 	}
 
 	public static LaunchingPlugin getDefault() {
@@ -335,25 +382,29 @@ public class LaunchingPlugin extends Plugin implements Preferences.IPropertyChan
 	 * Clears zip file cache.
 	 * Shutdown the launch config helper.
 	 * 
-	 * @see Plugin#shutdown()
+	 * @see Plugin#stop(BundleContext)
 	 */
-	public void shutdown() throws CoreException {
-		DebugPlugin.getDefault().getLaunchManager().removeLaunchListener(this);
-		DebugPlugin.getDefault().removeDebugEventListener(this);
-		ResourcesPlugin.getWorkspace().removeResourceChangeListener(this);
-		ArchiveSourceLocation.closeArchives();
-		getPluginPreferences().removePropertyChangeListener(this);
-		JavaRuntime.removeVMInstallChangedListener(this);
-		JavaRuntime.saveVMConfiguration();
-		savePluginPreferences();
-		super.shutdown();
+	public void stop(BundleContext context) throws Exception {
+		try {
+			DebugPlugin.getDefault().getLaunchManager().removeLaunchListener(this);
+			DebugPlugin.getDefault().removeDebugEventListener(this);
+			ResourcesPlugin.getWorkspace().removeResourceChangeListener(this);
+			ArchiveSourceLocation.closeArchives();
+			getPluginPreferences().removePropertyChangeListener(this);
+			JavaRuntime.removeVMInstallChangedListener(this);
+			JavaRuntime.saveVMConfiguration();
+			savePluginPreferences();
+			fgXMLParser = null;
+		} finally {
+			super.stop(context);
+		}
 	}
 		
 	/**
-	 * @see Plugin#startup()
+	 * @see Plugin#start(BundleContext)
 	 */
-	public void startup() throws CoreException {
-		super.startup();
+	public void start(BundleContext context) throws Exception {
+		super.start(context);
 		
 		// Exclude launch configurations from being copied to the output directory
 		String launchFilter = "*." + ILaunchConfiguration.LAUNCH_CONFIGURATION_FILE_EXTENSION; //$NON-NLS-1$
@@ -378,7 +429,7 @@ public class LaunchingPlugin extends Plugin implements Preferences.IPropertyChan
 		getPluginPreferences().addPropertyChangeListener(this);
 
 		JavaRuntime.addVMInstallChangedListener(this);
-		ResourcesPlugin.getWorkspace().addResourceChangeListener(this, ResourceChangeEvent.PRE_DELETE | ResourceChangeEvent.PRE_CLOSE);
+		ResourcesPlugin.getWorkspace().addResourceChangeListener(this, IResourceChangeEvent.PRE_DELETE | IResourceChangeEvent.PRE_CLOSE);
 		DebugPlugin.getDefault().getLaunchManager().addLaunchListener(this);
 		DebugPlugin.getDefault().addDebugEventListener(this);
 	}
@@ -413,9 +464,9 @@ public class LaunchingPlugin extends Plugin implements Preferences.IPropertyChan
 	 * Loads VM connector extensions
 	 */
 	private void initializeVMConnectors() {
-		IExtensionPoint extensionPoint= Platform.getPluginRegistry().getExtensionPoint(ID_EXTENSION_POINT_VM_CONNECTORS);
+		IExtensionPoint extensionPoint= Platform.getExtensionRegistry().getExtensionPoint(ID_PLUGIN, ID_EXTENSION_POINT_VM_CONNECTORS);
 		IConfigurationElement[] configs= extensionPoint.getConfigurationElements(); 
-		MultiStatus status= new MultiStatus(getUniqueIdentifier(), IStatus.OK, LaunchingMessages.getString("LaunchingPlugin.Exception_occurred_reading_vmConnectors_extensions_1"), null); //$NON-NLS-1$
+		MultiStatus status= new MultiStatus(getUniqueIdentifier(), IStatus.OK, LaunchingMessages.LaunchingPlugin_Exception_occurred_reading_vmConnectors_extensions_1, null); //$NON-NLS-1$
 		fVMConnectors = new HashMap(configs.length);
 		for (int i= 0; i < configs.length; i++) {
 			try {
@@ -430,11 +481,41 @@ public class LaunchingPlugin extends Plugin implements Preferences.IPropertyChan
 		}			
 	}
 	
+	/**
+	 * Returns a new runtime classpath entry of the specified type.
+	 * 
+	 * @param id extension type id
+	 * @return new uninitialized runtime classpath entry
+	 * @throws CoreException if unable to create an entry
+	 */
+	public IRuntimeClasspathEntry2 newRuntimeClasspathEntry(String id) throws CoreException {
+		if (fClasspathEntryExtensions == null) {
+			initializeRuntimeClasspathExtensions();
+		}
+		IConfigurationElement config = (IConfigurationElement) fClasspathEntryExtensions.get(id);
+		if (config == null) {
+			abort(MessageFormat.format(LaunchingMessages.LaunchingPlugin_32, new String[]{id}), null); //$NON-NLS-1$
+		}
+		return (IRuntimeClasspathEntry2) config.createExecutableExtension("class"); //$NON-NLS-1$
+	}
+	
+	/**
+	 * Loads runtime classpath extensions
+	 */
+	private void initializeRuntimeClasspathExtensions() {
+		IExtensionPoint extensionPoint= Platform.getExtensionRegistry().getExtensionPoint(LaunchingPlugin.ID_PLUGIN, ID_EXTENSION_POINT_RUNTIME_CLASSPATH_ENTRIES);
+		IConfigurationElement[] configs= extensionPoint.getConfigurationElements(); 
+		fClasspathEntryExtensions = new HashMap(configs.length);
+		for (int i= 0; i < configs.length; i++) {
+			fClasspathEntryExtensions.put(configs[i].getAttribute("id"), configs[i]); //$NON-NLS-1$
+		}
+	}	
+	
 	/**
 	 * Save preferences whenever the connect timeout changes.
 	 * Process changes to the list of installed JREs.
 	 * 
-	 * @see IPropertyChangeListener#propertyChange(PropertyChangeEvent)
+	 * @see org.eclipse.core.runtime.Preferences.IPropertyChangeListener#propertyChange(PropertyChangeEvent)
 	 */
 	public void propertyChange(PropertyChangeEvent event) {
 		String property = event.getProperty();
@@ -546,7 +627,7 @@ public class LaunchingPlugin extends Plugin implements Preferences.IPropertyChan
 	
 	/**
 	 * Parse the given xml into a VM definitions container, returning an empty
-	 * container if an exception occurrs.
+	 * container if an exception occurs.
 	 * 
 	 * @param xml
 	 * @return VMDefinitionsContainer
@@ -579,14 +660,14 @@ public class LaunchingPlugin extends Plugin implements Preferences.IPropertyChan
 		}
 	}
 
-	/**
-	 * @see IVMInstallChangedListener#vmAdded(IVMInstall)
+	/* (non-Javadoc)
+	 * @see org.eclipse.jdt.launching.IVMInstallChangedListener#vmAdded(org.eclipse.jdt.launching.IVMInstall)
 	 */
 	public void vmAdded(IVMInstall vm) {
 	}
 
-	/**
-	 * @see IVMInstallChangedListener#vmChanged(PropertyChangeEvent)
+	/* (non-Javadoc)
+	 * @see org.eclipse.jdt.launching.IVMInstallChangedListener#vmChanged(org.eclipse.jdt.launching.PropertyChangeEvent)
 	 */
 	public void vmChanged(org.eclipse.jdt.launching.PropertyChangeEvent event) {
 		if (!fBatchingChanges) {
@@ -600,8 +681,8 @@ public class LaunchingPlugin extends Plugin implements Preferences.IPropertyChan
 		}		
 	}
 
-	/**
-	 * @see IVMInstallChangedListener#vmRemoved(IVMInstall)
+	/* (non-Javadoc)
+	 * @see org.eclipse.jdt.launching.IVMInstallChangedListener#vmRemoved(org.eclipse.jdt.launching.IVMInstall)
 	 */
 	public void vmRemoved(IVMInstall vm) {
 		if (!fBatchingChanges) {
@@ -643,10 +724,9 @@ public class LaunchingPlugin extends Plugin implements Preferences.IPropertyChan
 	 * <li>serialization of the XML document failed</li>
 	 * </ul>
 	 */
-	private static String getLibraryInfoAsXML() throws IOException{
+	private static String getLibraryInfoAsXML() throws ParserConfigurationException, IOException, TransformerException{
 		
-		// Create the Document and the top-level node
-		Document doc = new DocumentImpl();
+		Document doc = getDocument();
 		Element config = doc.createElement("libraryInfos");    //$NON-NLS-1$
 		doc.appendChild(config);
 						
@@ -664,6 +744,18 @@ public class LaunchingPlugin extends Plugin implements Preferences.IPropertyChan
 		return JavaLaunchConfigurationUtils.serializeDocument(doc);
 	}	
 	
+	/**
+	 * Returns a Document that can be used to build a DOM tree
+	 * @return the Document
+	 * @throws ParserConfigurationException if an exception occurs creating the document builder
+	 */
+	public static Document getDocument() throws ParserConfigurationException {
+		DocumentBuilderFactory dfactory= DocumentBuilderFactory.newInstance();
+		DocumentBuilder docBuilder= dfactory.newDocumentBuilder();
+		Document doc= docBuilder.newDocument();
+		return doc;
+	}
+
 	/**
 	 * Creates an XML element for the given info.
 	 * 
@@ -706,6 +798,7 @@ public class LaunchingPlugin extends Plugin implements Preferences.IPropertyChan
 	 * Saves the library info in a local workspace state location 
 	 */
 	private static void saveLibraryInfo() {
+		FileOutputStream stream= null;
 		try {
 			String xml = getLibraryInfoAsXML();
 			IPath libPath = getDefault().getStateLocation();
@@ -714,12 +807,22 @@ public class LaunchingPlugin extends Plugin implements Preferences.IPropertyChan
 			if (!file.exists()) {
 				file.createNewFile();
 			}
-			FileOutputStream stream = new FileOutputStream(file);
+			stream = new FileOutputStream(file);
 			stream.write(xml.getBytes("UTF8")); //$NON-NLS-1$
-			stream.close();
 		} catch (IOException e) {
 			log(e);
-		}	
+		} catch (ParserConfigurationException e) {
+			log(e);
+		} catch (TransformerException e) {
+			log(e);
+		} finally {
+			if (stream != null) {
+				try {
+					stream.close();
+				} catch (IOException e1) {
+				}
+			}
+		}
 	}
 	
 	/**
@@ -734,6 +837,7 @@ public class LaunchingPlugin extends Plugin implements Preferences.IPropertyChan
 			try {
 				InputStream stream = new FileInputStream(file);
 				DocumentBuilder parser = DocumentBuilderFactory.newInstance().newDocumentBuilder();
+				parser.setErrorHandler(new DefaultHandler());
 				Element root = parser.parse(new InputSource(stream)).getDocumentElement();
 				if(!root.getNodeName().equals("libraryInfos")) { //$NON-NLS-1$
 					return;
@@ -842,6 +946,39 @@ public class LaunchingPlugin extends Plugin implements Preferences.IPropertyChan
 			}
 		}
 	}
+	
+	/**
+	 * Returns a shared XML parser.
+	 * 
+	 * @return an XML parser 
+	 * @throws CoreException if unable to create a parser
+	 * @since 3.0
+	 */
+	public static DocumentBuilder getParser() throws CoreException {
+		if (fgXMLParser == null) {
+			try {
+				fgXMLParser = DocumentBuilderFactory.newInstance().newDocumentBuilder();
+				fgXMLParser.setErrorHandler(new DefaultHandler());
+			} catch (ParserConfigurationException e) {
+				abort(LaunchingMessages.LaunchingPlugin_33, e); //$NON-NLS-1$
+			} catch (FactoryConfigurationError e) {
+				abort(LaunchingMessages.LaunchingPlugin_34, e); //$NON-NLS-1$
+			}
+		}
+		return fgXMLParser;
+	}	
+	
+	/**
+	 * Throws an exception with the given message and underlying exception.
+	 * 
+	 * @param message error message
+	 * @param exception underlying exception or <code>null</code> if none
+	 * @throws CoreException
+	 */
+	protected static void abort(String message, Throwable exception) throws CoreException {
+		IStatus status = new Status(IStatus.ERROR, LaunchingPlugin.getUniqueIdentifier(), 0, message, exception);
+		throw new CoreException(status);
+	}	
 }
 
  
