@@ -1,9 +1,9 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
+ * http://www.eclipse.org/legal/epl-v10.html
  * 
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -23,8 +23,8 @@ import java.util.List;
 import javax.xml.parsers.DocumentBuilder;
 import javax.xml.parsers.DocumentBuilderFactory;
 import javax.xml.parsers.ParserConfigurationException;
+import javax.xml.transform.TransformerException;
 
-import org.apache.xerces.dom.DocumentImpl;
 import org.eclipse.core.resources.IProject;
 import org.eclipse.core.resources.IResource;
 import org.eclipse.core.resources.ResourcesPlugin;
@@ -59,6 +59,7 @@ import org.w3c.dom.Node;
 import org.w3c.dom.NodeList;
 import org.xml.sax.InputSource;
 import org.xml.sax.SAXException;
+import org.xml.sax.helpers.DefaultHandler;
 
 
 /**
@@ -69,6 +70,28 @@ import org.xml.sax.SAXException;
  * </p>
  * @see org.eclipse.debug.core.model.ISourceLocator
  * @since 2.0
+ * @deprecated In 3.0, the debug platform provides source lookup facilities that
+ *  should be used in place of the Java source lookup support provided in 2.0.
+ *  The new facilities provide a source lookup director that coordinates source
+ *  lookup among a set of participants, searching a set of source containers.
+ *  See the following packages: <code>org.eclipse.debug.core.sourcelookup</code>
+ *  and <code>org.eclipse.debug.core.sourcelookup.containers</code>. This class
+ *  has been replaced by a Java source lookup director and Java source lookup
+ *  participant. To migrate to the new source lookup support clients should
+ *  add two new attributes to their launch configuration type extensions:<ul>
+ *  <li>sourceLocatorId="org.eclipse.jdt.launching.sourceLocator.JavaSourceLookupDirector"</li>
+ *  <li>sourcePathComputerId="org.eclipse.jdt.launching.sourceLookup.javaSourcePathComputer"</li>
+ *  </ul>
+ *  The source locator id attribute specifies to use the Java source lookup director
+ *  for launch configurations of the associated type, and the source path computer id
+ *  attribute specifies the class to use when computing a default source lookup
+ *  path for a launch configuration. The path computer referenced/provided (by the
+ *  above id), computes a default source lookup path based on the support provided in
+ *  the 2.0 release - i.e. a configuration's <code>ATTR_SOURCE_PATH_PROVIDER</code>
+ *  attribute (if present), or a default source lookup path based on a configuration's
+ *  runtime classpath. This class has been replaced by the Java source lookup
+ *  director which is an internal class, but can be used via the
+ *  <code>sourceLocatorId</code> attribute on a launch configuration type extension.  
  */
 public class JavaSourceLocator implements IPersistableSourceLocator {
 	
@@ -134,9 +157,8 @@ public class JavaSourceLocator implements IPersistableSourceLocator {
 			} catch (CoreException e) {
 				if (e instanceof JavaModelException) {
 					throw (JavaModelException)e;
-				} else {
-					throw new JavaModelException(e);
-				}
+				} 
+				throw new JavaModelException(e);
 			}
 		}
 		IJavaSourceLocation[] locations = (IJavaSourceLocation[])list.toArray(new IJavaSourceLocation[list.size()]);
@@ -231,8 +253,8 @@ public class JavaSourceLocator implements IPersistableSourceLocator {
 		return null;
 	}	
 			
-	/**
-	 * @see org.eclipse.debug.core.model.ISourceLocator#getSourceElement(IStackFrame)
+	/* (non-Javadoc)
+	 * @see org.eclipse.debug.core.model.ISourceLocator#getSourceElement(org.eclipse.debug.core.model.IStackFrame)
 	 */
 	public Object getSourceElement(IStackFrame stackFrame) {
 		if (stackFrame instanceof IJavaStackFrame) {
@@ -352,33 +374,37 @@ public class JavaSourceLocator implements IPersistableSourceLocator {
 		return locator.getSourceLocations();
 	}
 	
-	/**
-	 * @see IPersistableSourceLocator#getMemento()
+	/* (non-Javadoc)
+	 * @see org.eclipse.debug.core.model.IPersistableSourceLocator#getMemento()
 	 */
 	public String getMemento() throws CoreException {
-		Document doc = new DocumentImpl();
-		Element node = doc.createElement("javaSourceLocator"); //$NON-NLS-1$
-		doc.appendChild(node);
-		
-		IJavaSourceLocation[] locations = getSourceLocations();
-		for (int i = 0; i < locations.length; i++) {
-			Element child = doc.createElement("javaSourceLocation"); //$NON-NLS-1$
-			child.setAttribute("class", locations[i].getClass().getName()); //$NON-NLS-1$
-			child.setAttribute("memento", locations[i].getMemento()); //$NON-NLS-1$
-			node.appendChild(child);
-		}
-		
 		try {
+			Document doc = LaunchingPlugin.getDocument();
+			Element node = doc.createElement("javaSourceLocator"); //$NON-NLS-1$
+			doc.appendChild(node);
+			
+			IJavaSourceLocation[] locations = getSourceLocations();
+			for (int i = 0; i < locations.length; i++) {
+				Element child = doc.createElement("javaSourceLocation"); //$NON-NLS-1$
+				child.setAttribute("class", locations[i].getClass().getName()); //$NON-NLS-1$
+				child.setAttribute("memento", locations[i].getMemento()); //$NON-NLS-1$
+				node.appendChild(child);
+			}
+		
 			return JavaLaunchConfigurationUtils.serializeDocument(doc);
 		} catch (IOException e) {
-			abort(LaunchingMessages.getString("JavaSourceLocator.Unable_to_create_memento_for_Java_source_locator._4"), e); //$NON-NLS-1$
-		}
+			abort(LaunchingMessages.JavaSourceLocator_Unable_to_create_memento_for_Java_source_locator__4, e); //$NON-NLS-1$
+		} catch (ParserConfigurationException e) {
+			abort(LaunchingMessages.JavaSourceLocator_Unable_to_create_memento_for_Java_source_locator__4, e); //$NON-NLS-1$
+		} catch (TransformerException e) {
+			abort(LaunchingMessages.JavaSourceLocator_Unable_to_create_memento_for_Java_source_locator__4, e); //$NON-NLS-1$
+		} 
 		// execution will not reach here
 		return null;
 	}
 
-	/**
-	 * @see IPersistableSourceLocator#initializeDefaults(ILaunchConfiguration)
+	/* (non-Javadoc)
+	 * @see org.eclipse.debug.core.model.IPersistableSourceLocator#initializeDefaults(org.eclipse.debug.core.ILaunchConfiguration)
 	 */
 	public void initializeDefaults(ILaunchConfiguration configuration) throws CoreException {
 		IRuntimeClasspathEntry[] entries = JavaRuntime.computeUnresolvedSourceLookupPath(configuration);
@@ -386,8 +412,8 @@ public class JavaSourceLocator implements IPersistableSourceLocator {
 		setSourceLocations(getSourceLocations(resolved));
 	}
 
-	/**
-	 * @see IPersistableSourceLocator#initializeFromMemento(String)
+	/* (non-Javadoc)
+	 * @see org.eclipse.debug.core.model.IPersistableSourceLocator#initializeFromMemento(java.lang.String)
 	 */
 	public void initializeFromMemento(String memento) throws CoreException {
 		Exception ex = null;
@@ -395,12 +421,13 @@ public class JavaSourceLocator implements IPersistableSourceLocator {
 			Element root = null;
 			DocumentBuilder parser =
 				DocumentBuilderFactory.newInstance().newDocumentBuilder();
+			parser.setErrorHandler(new DefaultHandler());
 			StringReader reader = new StringReader(memento);
 			InputSource source = new InputSource(reader);
 			root = parser.parse(source).getDocumentElement();
 												
 			if (!root.getNodeName().equalsIgnoreCase("javaSourceLocator")) {  //$NON-NLS-1$
-				abort(LaunchingMessages.getString("JavaSourceLocator.Unable_to_restore_Java_source_locator_-_invalid_format._6"), null); //$NON-NLS-1$
+				abort(LaunchingMessages.JavaSourceLocator_Unable_to_restore_Java_source_locator___invalid_format__6, null); //$NON-NLS-1$
 			}
 	
 			List sourceLocations = new ArrayList();
@@ -417,27 +444,27 @@ public class JavaSourceLocator implements IPersistableSourceLocator {
 						String className = entry.getAttribute("class"); //$NON-NLS-1$
 						String data = entry.getAttribute("memento"); //$NON-NLS-1$
 						if (isEmpty(className)) {
-							abort(LaunchingMessages.getString("JavaSourceLocator.Unable_to_restore_Java_source_locator_-_invalid_format._10"), null); //$NON-NLS-1$
+							abort(LaunchingMessages.JavaSourceLocator_Unable_to_restore_Java_source_locator___invalid_format__10, null); //$NON-NLS-1$
 						}
 						Class clazz  = null;
 						try {
 							clazz = classLoader.loadClass(className);
 						} catch (ClassNotFoundException e) {
-							abort(MessageFormat.format(LaunchingMessages.getString("JavaSourceLocator.Unable_to_restore_source_location_-_class_not_found__{0}_11"), new String[] {className}), e); //$NON-NLS-1$
+							abort(MessageFormat.format(LaunchingMessages.JavaSourceLocator_Unable_to_restore_source_location___class_not_found___0__11, new String[] {className}), e); //$NON-NLS-1$
 						}
 						
 						IJavaSourceLocation location = null;
 						try {
 							location = (IJavaSourceLocation)clazz.newInstance();
 						} catch (IllegalAccessException e) {
-							abort(LaunchingMessages.getString("JavaSourceLocator.Unable_to_restore_source_location._12"), e); //$NON-NLS-1$
+							abort(LaunchingMessages.JavaSourceLocator_Unable_to_restore_source_location__12, e); //$NON-NLS-1$
 						} catch (InstantiationException e) {
-							abort(LaunchingMessages.getString("JavaSourceLocator.Unable_to_restore_source_location._12"), e); //$NON-NLS-1$
+							abort(LaunchingMessages.JavaSourceLocator_Unable_to_restore_source_location__12, e); //$NON-NLS-1$
 						}
 						location.initializeFrom(data);
 						sourceLocations.add(location);
 					} else {
-						abort(LaunchingMessages.getString("JavaSourceLocator.Unable_to_restore_Java_source_locator_-_invalid_format._14"), null); //$NON-NLS-1$
+						abort(LaunchingMessages.JavaSourceLocator_Unable_to_restore_Java_source_locator___invalid_format__14, null); //$NON-NLS-1$
 					}
 				}
 			}
@@ -450,7 +477,7 @@ public class JavaSourceLocator implements IPersistableSourceLocator {
 		} catch (IOException e) {
 			ex = e;
 		}
-		abort(LaunchingMessages.getString("JavaSourceLocator.Exception_occurred_initializing_source_locator._15"), ex); //$NON-NLS-1$
+		abort(LaunchingMessages.JavaSourceLocator_Exception_occurred_initializing_source_locator__15, ex); //$NON-NLS-1$
 	}
 	
 	/**
@@ -497,7 +524,7 @@ public class JavaSourceLocator implements IPersistableSourceLocator {
 					}
 					break;
 				case IRuntimeClasspathEntry.CONTAINER:
-					throw new IllegalArgumentException(LaunchingMessages.getString("JavaSourceLocator.Illegal_to_have_a_container_resolved_to_a_container_1")); //$NON-NLS-1$
+					throw new IllegalArgumentException(LaunchingMessages.JavaSourceLocator_Illegal_to_have_a_container_resolved_to_a_container_1); //$NON-NLS-1$
 			}
 			if (location != null) {
 				locations.add(location);
@@ -589,7 +616,7 @@ public class JavaSourceLocator implements IPersistableSourceLocator {
 			IProject project = resource.getProject();
 			IJavaProject jp = JavaCore.create(project);
 			try {
-				if (jp.exists()) {
+				if (jp != null && jp.exists()) {
 					IPackageFragmentRoot root = jp.getPackageFragmentRoot(resource);
 					IPackageFragmentRoot[] allRoots = jp.getPackageFragmentRoots();
 					for (int j = 0; j < allRoots.length; j++) {
