@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -13,21 +13,26 @@ package org.eclipse.jdt.internal.corext.refactoring;
 import java.util.ArrayList;
 import java.util.List;
 
-import org.eclipse.core.resources.IContainer;
-import org.eclipse.core.resources.IFile;
-import org.eclipse.core.resources.IMarker;
-import org.eclipse.core.resources.IProject;
-import org.eclipse.core.resources.IResource;
-import org.eclipse.core.resources.ResourcesPlugin;
 import org.eclipse.core.runtime.CoreException;
 import org.eclipse.core.runtime.IPath;
 import org.eclipse.core.runtime.IProgressMonitor;
 import org.eclipse.core.runtime.IStatus;
 
+import org.eclipse.core.resources.IContainer;
+import org.eclipse.core.resources.IFile;
+import org.eclipse.core.resources.IMarker;
+import org.eclipse.core.resources.IProject;
+import org.eclipse.core.resources.IResource;
+import org.eclipse.core.resources.ResourceAttributes;
+import org.eclipse.core.resources.ResourcesPlugin;
+
+import org.eclipse.ltk.core.refactoring.RefactoringStatus;
+
 import org.eclipse.jdt.core.ICompilationUnit;
 import org.eclipse.jdt.core.IJavaElement;
 import org.eclipse.jdt.core.IJavaModelMarker;
 import org.eclipse.jdt.core.IJavaProject;
+import org.eclipse.jdt.core.ILocalVariable;
 import org.eclipse.jdt.core.IMember;
 import org.eclipse.jdt.core.IMethod;
 import org.eclipse.jdt.core.IPackageFragment;
@@ -40,21 +45,24 @@ import org.eclipse.jdt.core.Signature;
 import org.eclipse.jdt.core.dom.ASTNode;
 import org.eclipse.jdt.core.dom.AnonymousClassDeclaration;
 import org.eclipse.jdt.core.dom.Expression;
+import org.eclipse.jdt.core.dom.IBinding;
 import org.eclipse.jdt.core.dom.IMethodBinding;
 import org.eclipse.jdt.core.dom.ITypeBinding;
 import org.eclipse.jdt.core.dom.IVariableBinding;
 import org.eclipse.jdt.core.dom.Name;
+import org.eclipse.jdt.core.dom.SwitchCase;
 import org.eclipse.jdt.core.dom.VariableDeclaration;
 
 import org.eclipse.jdt.internal.corext.Assert;
 import org.eclipse.jdt.internal.corext.dom.ASTNodes;
-import org.eclipse.jdt.internal.corext.refactoring.base.JavaSourceContext;
-import org.eclipse.jdt.internal.corext.refactoring.base.RefactoringStatus;
+import org.eclipse.jdt.internal.corext.dom.Bindings;
+import org.eclipse.jdt.internal.corext.refactoring.base.JavaStatusContext;
 import org.eclipse.jdt.internal.corext.refactoring.changes.RenameResourceChange;
 import org.eclipse.jdt.internal.corext.refactoring.util.JavaElementUtil;
 import org.eclipse.jdt.internal.corext.refactoring.util.ResourceUtil;
 import org.eclipse.jdt.internal.corext.util.JavaModelUtil;
 import org.eclipse.jdt.internal.corext.util.JdtFlags;
+import org.eclipse.jdt.internal.corext.util.Messages;
 import org.eclipse.jdt.internal.corext.util.Resources;
 
 /**
@@ -75,7 +83,9 @@ public class Checks {
 
 	/**
 	 * Checks if method will have a constructor name after renaming.
-	 * 
+	 * @param method
+	 * @param newMethodName
+	 * @param newTypeName 
 	 * @return <code>RefactoringStatus</code> with <code>WARNING</code> severity if 
 	 * the give method will have a constructor name after renaming
 	 * <code>null</code> otherwise.
@@ -85,14 +95,14 @@ public class Checks {
 			return null;
 		else
 			return RefactoringStatus.createWarningStatus(
-				RefactoringCoreMessages.getFormattedString("Checks.constructor_name",  //$NON-NLS-1$
+				Messages.format(RefactoringCoreMessages.Checks_constructor_name,  
 				new Object[] {JavaElementUtil.createMethodSignature(method), JavaModelUtil.getFullyQualifiedName(method.getDeclaringType()) } ));
 	}
 		
 	/**
 	 * Checks if the given name is a valid Java field name.
 	 *
-	 * @param the java field name.
+	 * @param name the java field name.
 	 * @return a refactoring status containing the error message if the
 	 *  name is not a valid java field name.
 	 */
@@ -100,10 +110,21 @@ public class Checks {
 		return checkName(name, JavaConventions.validateFieldName(name));
 	}
 
+	/**
+	 * Checks if the given name is a valid Java type parameter name.
+	 *
+	 * @param name the java type parameter name.
+	 * @return a refactoring status containing the error message if the
+	 *  name is not a valid java type parameter name.
+	 */
+	public static RefactoringStatus checkTypeParameterName(String name) {
+		return checkName(name, JavaConventions.validateTypeVariableName(name));
+	}
+
 	/**
 	 * Checks if the given name is a valid Java identifier.
 	 *
-	 * @param the java identifier.
+	 * @param name the java identifier.
 	 * @return a refactoring status containing the error message if the
 	 *  name is not a valid java identifier.
 	 */
@@ -114,14 +135,14 @@ public class Checks {
 	/**
 	 * Checks if the given name is a valid Java method name.
 	 *
-	 * @param the java method name.
+	 * @param name the java method name.
 	 * @return a refactoring status containing the error message if the
 	 *  name is not a valid java method name.
 	 */
 	public static RefactoringStatus checkMethodName(String name) {
 		RefactoringStatus status= checkName(name, JavaConventions.validateMethodName(name));
 		if (status.isOK() && startsWithUpperCase(name))
-			return RefactoringStatus.createWarningStatus(RefactoringCoreMessages.getString("Checks.method_names_lowercase")); //$NON-NLS-1$
+			return RefactoringStatus.createWarningStatus(RefactoringCoreMessages.Checks_method_names_lowercase); 
 		else	
 			return status;
 	}
@@ -129,14 +150,14 @@ public class Checks {
 	/**
 	 * Checks if the given name is a valid Java type name.
 	 *
-	 * @param the java method name.
+	 * @param name the java method name.
 	 * @return a refactoring status containing the error message if the
 	 *  name is not a valid java type name.
 	 */
 	public static RefactoringStatus checkTypeName(String name) {
 		//fix for: 1GF5Z0Z: ITPJUI:WINNT - assertion failed after renameType refactoring
 		if (name.indexOf(".") != -1) //$NON-NLS-1$
-			return RefactoringStatus.createFatalErrorStatus(RefactoringCoreMessages.getString("Checks.no_dot"));//$NON-NLS-1$
+			return RefactoringStatus.createFatalErrorStatus(RefactoringCoreMessages.Checks_no_dot);
 		else	
 			return checkName(name, JavaConventions.validateJavaTypeName(name));
 	}
@@ -144,7 +165,7 @@ public class Checks {
 	/**
 	 * Checks if the given name is a valid Java package name.
 	 *
-	 * @param the java package name.
+	 * @param name the java package name.
 	 * @return a refactoring status containing the error message if the
 	 *  name is not a valid java package name.
 	 */
@@ -155,7 +176,7 @@ public class Checks {
 	/**
 	 * Checks if the given name is a valid compilation unit name.
 	 *
-	 * @param the compilation unit name.
+	 * @param name the compilation unit name.
 	 * @return a refactoring status containing the error message if the
 	 *  name is not a valid compilation unit name.
 	 */
@@ -165,11 +186,13 @@ public class Checks {
 
 	/**
 	 * Returns ok status if the new name is ok. This is when no other file with that name exists.
+	 * @param cu
 	 * @param newName 
+	 * @return the status
 	 */
-	public static RefactoringStatus checkCompilationUnitNewName(ICompilationUnit cu, String newName) throws JavaModelException{
+	public static RefactoringStatus checkCompilationUnitNewName(ICompilationUnit cu, String newName) {
 		if (resourceExists(RenameResourceChange.renamedResourcePath(ResourceUtil.getResource(cu).getFullPath(), newName + ".java"))) //$NON-NLS-1$
-			return RefactoringStatus.createFatalErrorStatus(RefactoringCoreMessages.getFormattedString("Checks.cu_name_used", newName));//$NON-NLS-1$
+			return RefactoringStatus.createFatalErrorStatus(Messages.format(RefactoringCoreMessages.Checks_cu_name_used, newName));
 		else
 			return new RefactoringStatus();
 	}
@@ -198,16 +221,27 @@ public class Checks {
 		return ResourcesPlugin.getWorkspace().getRoot().findMember(resourcePath) != null;
 	}
 	
-	/**
-	 * Returns <code>true</code> if the parameter is a top-level type, <code>false</code> otherwise.
-	 * 
-	 * @param type
-	 * @return <code>true</code> if the parameter is a top-level type, <code>false</code> otherwise.
-	 */
 	public static boolean isTopLevel(IType type){
 		return type.getDeclaringType() == null;
 	}
 
+	public static boolean isAnonymous(IType type) throws JavaModelException {
+		return type.isAnonymous();
+	}
+
+	public static boolean isTopLevelType(IMember member){
+		return  member.getElementType() == IJavaElement.TYPE && isTopLevel((IType) member);
+	}
+	
+	public static boolean isInsideLocalType(IType type) throws JavaModelException {
+		while (type != null) {
+			if (type.isLocal())
+				return true;
+			type= type.getDeclaringType();
+		}
+		return false;
+	}
+
 	public static boolean isAlreadyNamed(IJavaElement element, String name){
 		return name.equals(element.getElementName());
 	}
@@ -242,9 +276,9 @@ public class Checks {
 		RefactoringStatus result= new RefactoringStatus();
 		for (int i= 0; i < methods.length; i++) {
 			if (JdtFlags.isNative(methods[i])){
-				String msg= RefactoringCoreMessages.getFormattedString("Checks.method_native",  //$NON-NLS-1$
+				String msg= Messages.format(RefactoringCoreMessages.Checks_method_native,  
 								new String[]{JavaModelUtil.getFullyQualifiedName(methods[i].getDeclaringType()), methods[i].getElementName(), "UnsatisfiedLinkError"});//$NON-NLS-1$
-				result.addError(msg, JavaSourceContext.create(methods[i])); 
+				result.addError(msg, JavaStatusContext.create(methods[i])); 
 			}				
 		}
 		return result;
@@ -254,16 +288,21 @@ public class Checks {
 	
 	/**
 	 * Checks if the new method is already used in the given type.
+	 * @param type
+	 * @param methodName
+	 * @param parameters
+	 * @param scope
+	 * @return the status
 	 */
 	public static RefactoringStatus checkMethodInType(ITypeBinding type, String methodName, ITypeBinding[] parameters, IJavaProject scope) {
 		RefactoringStatus result= new RefactoringStatus();
 		if (methodName.equals(type.getName()))
-			result.addWarning(RefactoringCoreMessages.getString("Checks.methodName.constructor")); //$NON-NLS-1$
+			result.addWarning(RefactoringCoreMessages.Checks_methodName_constructor); 
 		IMethodBinding method= org.eclipse.jdt.internal.corext.dom.Bindings.findMethodInType(type, methodName, parameters);
 		if (method != null) 
-			result.addError(RefactoringCoreMessages.getFormattedString("Checks.methodName.exists",  //$NON-NLS-1$
+			result.addError(Messages.format(RefactoringCoreMessages.Checks_methodName_exists,  
 				new Object[] {methodName, type.getName()}),
-				JavaSourceContext.create(method, scope));
+				JavaStatusContext.create(method, scope));
 		return result;
 	}
 	
@@ -274,10 +313,16 @@ public class Checks {
 	 *   <li> if the new method overrides a method defined in the given type or in one of its
 	 * 		super classes. </li>
 	 * </ul>
+	 * @param type
+	 * @param methodName
+	 * @param returnType
+	 * @param parameters
+	 * @param scope
+	 * @return the status
 	 */
 	public static RefactoringStatus checkMethodInHierarchy(ITypeBinding type, String methodName, ITypeBinding returnType, ITypeBinding[] parameters, IJavaProject scope) {
 		RefactoringStatus result= new RefactoringStatus();
-		IMethodBinding method= org.eclipse.jdt.internal.corext.dom.Bindings.findMethodInHierarchy(type, methodName, parameters);
+		IMethodBinding method= Bindings.findMethodInHierarchy(type, methodName, parameters);
 		if (method != null) {
 			boolean returnTypeClash= false;
 			ITypeBinding methodReturnType= method.getReturnType();
@@ -292,13 +337,13 @@ public class Checks {
 			}
 			ITypeBinding dc= method.getDeclaringClass();
 			if (returnTypeClash) {
-				result.addError(RefactoringCoreMessages.getFormattedString("Checks.methodName.returnTypeClash", //$NON-NLS-1$
+				result.addError(Messages.format(RefactoringCoreMessages.Checks_methodName_returnTypeClash, 
 					new Object[] {methodName, dc.getName()}),
-					JavaSourceContext.create(method, scope));
+					JavaStatusContext.create(method, scope));
 			} else {
-				result.addError(RefactoringCoreMessages.getFormattedString("Checks.methodName.overrides", //$NON-NLS-1$
+				result.addError(Messages.format(RefactoringCoreMessages.Checks_methodName_overrides, 
 					new Object[] {methodName, dc.getName()}),
-					JavaSourceContext.create(method, scope));
+					JavaStatusContext.create(method, scope));
 			}
 		}
 		return result;
@@ -312,7 +357,7 @@ public class Checks {
 		 */
 		RefactoringStatus result= new RefactoringStatus();
 		if (JavaModelUtil.hasMainMethod(type))
-			result.addWarning(RefactoringCoreMessages.getFormattedString("Checks.has_main", JavaModelUtil.getFullyQualifiedName(type))); //$NON-NLS-1$
+			result.addWarning(Messages.format(RefactoringCoreMessages.Checks_has_main, JavaModelUtil.getFullyQualifiedName(type))); 
 		result.merge(checkForMainMethods(type.getTypes()));	
 		return result;
 	}
@@ -330,12 +375,58 @@ public class Checks {
 		return checkForMainMethods(cu.getTypes());
 	}
 	
+	//---- Selection checks --------------------------------------------------------------------
+
+	public static boolean isExtractableExpression(ASTNode[] selectedNodes, ASTNode coveringNode) {
+		ASTNode node= coveringNode;
+		if (isEnumCase(node))
+			return false;
+		if (selectedNodes != null && selectedNodes.length == 1)
+			node= selectedNodes[0];
+		return isExtractableExpression(node);
+	}
+
+	public static boolean isEnumCase(ASTNode node) {
+		if (node instanceof SwitchCase) {
+			final SwitchCase caze= (SwitchCase) node;
+			final Expression expression= caze.getExpression();
+			if (expression instanceof Name) {
+				final Name name= (Name) expression;
+				final IBinding binding= name.resolveBinding();
+				if (binding instanceof IVariableBinding) {
+					IVariableBinding variableBinding= (IVariableBinding) binding;
+					return variableBinding.isEnumConstant();
+				}
+			}
+		}
+		return false;
+	}
+
+	public static boolean isExtractableExpression(ASTNode node) {
+		if (!(node instanceof Expression))
+			return false;
+		if (node instanceof Name) {
+			IBinding binding= ((Name) node).resolveBinding();
+			return !(binding instanceof ITypeBinding);
+		}
+		return true;
+	}
+
+	public static boolean isInsideJavadoc(ASTNode node) {
+		do {
+			if (node.getNodeType() == ASTNode.JAVADOC)
+				return true;
+			node= node.getParent();
+		} while (node != null);
+		return false;
+	}
+
 	//---- Private helpers ----------------------------------------------------------------------
 	
 	private static RefactoringStatus checkName(String name, IStatus status) {
 		RefactoringStatus result= new RefactoringStatus();
 		if ("".equals(name)) //$NON-NLS-1$
-			return RefactoringStatus.createFatalErrorStatus(RefactoringCoreMessages.getString("Checks.Choose_name")); //$NON-NLS-1$
+			return RefactoringStatus.createFatalErrorStatus(RefactoringCoreMessages.Checks_Choose_name); 
 
 		if (status.isOK())
 			return result;
@@ -356,27 +447,38 @@ public class Checks {
 	 * Finds a method in a type
 	 * This searches for a method with the same name and signature. Parameter types are only
 	 * compared by the simple name, no resolving for the fully qualified type name is done
+	 * @param name
+	 * @param parameterCount
+	 * @param isConstructor
+	 * @param type
 	 * @return The first found method or null, if nothing found
+	 * @throws JavaModelException
 	 */
 	public static IMethod findMethod(String name, int parameterCount, boolean isConstructor, IType type) throws JavaModelException {
 		return findMethod(name, parameterCount, isConstructor, type.getMethods());
 	}
 	
 	/**
-	 * Finds a method in a type
-	 * This searches for a method with the same name and signature. Parameter types are only
-	 * compared by the simple name, no resolving for the fully qualified type name is done
+	 * Finds a method in a type.
+	 * Searches for a method with the same name and the same parameter count.
+	 * Parameter types are <b>not</b> compared.
+	 * @param method
+	 * @param type
 	 * @return The first found method or null, if nothing found
+	 * @throws JavaModelException
 	 */
 	public static IMethod findMethod(IMethod method, IType type) throws JavaModelException {
 		return findMethod(method.getElementName(), method.getParameterTypes().length, method.isConstructor(), type.getMethods());
 	}
-	
+
 	/**
-	 * Finds a method in an array of methods
-	 * This searches for a method with the same name and signature. Parameter types are only
-	 * compared by the simple name, no resolving for the fully qualified type name is done
+	 * Finds a method in an array of methods.
+	 * Searches for a method with the same name and the same parameter count.
+	 * Parameter types are <b>not</b> compared.
+	 * @param method
+	 * @param methods
 	 * @return The first found method or null, if nothing found
+	 * @throws JavaModelException
 	 */
 	public static IMethod findMethod(IMethod method, IMethod[] methods) throws JavaModelException {
 		return findMethod(method.getElementName(), method.getParameterTypes().length, method.isConstructor(), methods);
@@ -395,7 +497,39 @@ public class Checks {
 		}
 		return null;
 	}
-	
+
+	/**
+	 * Finds a method in a type.
+	 * This searches for a method with the same name and signature. Parameter types are only
+	 * compared by the simple name, no resolving for the fully qualified type name is done
+	 * @param method
+	 * @param type
+	 * @return The first found method or null, if nothing found
+	 * @throws JavaModelException
+	 */
+	public static IMethod findSimilarMethod(IMethod method, IType type) throws JavaModelException {
+		return findSimilarMethod(method, type.getMethods());
+	}
+
+	/**
+	 * Finds a method in an array of methods.
+	 * This searches for a method with the same name and signature. Parameter types are only
+	 * compared by the simple name, no resolving for the fully qualified type name is done
+	 * @param method
+	 * @param methods
+	 * @return The first found method or null, if nothing found
+	 * @throws JavaModelException
+	 */
+	public static IMethod findSimilarMethod(IMethod method, IMethod[] methods) throws JavaModelException {
+		boolean isConstructor= method.isConstructor();
+		for (int i= 0; i < methods.length; i++) {
+			IMethod otherMethod= methods[i];
+			if (otherMethod.isConstructor() == isConstructor && method.isSimilar(otherMethod))
+				return otherMethod;
+		}
+		return null;
+	}
+				
 	/*
 	 * Compare two parameter signatures
 	 */
@@ -420,9 +554,9 @@ public class Checks {
 	public static RefactoringStatus checkIfCuBroken(IMember member) throws JavaModelException{
 		ICompilationUnit cu= (ICompilationUnit)JavaCore.create(ResourceUtil.getResource(member));
 		if (cu == null)
-			return RefactoringStatus.createFatalErrorStatus(RefactoringCoreMessages.getString("Checks.cu_not_created"));	 //$NON-NLS-1$
+			return RefactoringStatus.createFatalErrorStatus(RefactoringCoreMessages.Checks_cu_not_created);	 
 		else if (! cu.isStructureKnown())
-			return RefactoringStatus.createFatalErrorStatus(RefactoringCoreMessages.getString("Checks.cu_not_parsed"));	 //$NON-NLS-1$
+			return RefactoringStatus.createFatalErrorStatus(RefactoringCoreMessages.Checks_cu_not_parsed);	 
 		return new RefactoringStatus();
 	}
 	
@@ -430,7 +564,11 @@ public class Checks {
 	 * From SearchResultGroup[] passed as the parameter
 	 * this method removes all those that correspond to a non-parsable ICompilationUnit
 	 * and returns it as a result.
-	 * Status object collect the result of checking.
+	 * @param grouped the array of search result groups from which non parsable compilation
+	 *  units are to be removed.
+	 * @param status a refactoring status to collect errors and problems
+	 * @return the array of search result groups 
+	 * @throws JavaModelException
 	 */	
 	public static SearchResultGroup[] excludeCompilationUnits(SearchResultGroup[] grouped, RefactoringStatus status) throws JavaModelException{
 		List result= new ArrayList();
@@ -444,14 +582,14 @@ public class Checks {
 			ICompilationUnit cu= (ICompilationUnit)JavaCore.create(resource);
 			if (! cu.isStructureKnown()){
 				String path= Checks.getFullPath(cu);
-				status.addError(RefactoringCoreMessages.getFormattedString("Checks.cannot_be_parsed", path)); //$NON-NLS-1$
+				status.addError(Messages.format(RefactoringCoreMessages.Checks_cannot_be_parsed, path)); 
 				continue; //removed, go to the next one
 			}
 			result.add(grouped[i]);	
 		}
 		
 		if ((!wasEmpty) && result.isEmpty())
-			status.addFatalError(RefactoringCoreMessages.getString("Checks.all_excluded")); //$NON-NLS-1$
+			status.addFatalError(RefactoringCoreMessages.Checks_all_excluded); 
 		
 		return (SearchResultGroup[])result.toArray(new SearchResultGroup[result.size()]);
 	}
@@ -464,11 +602,26 @@ public class Checks {
 	
 	public static RefactoringStatus checkCompileErrorsInAffectedFiles(SearchResultGroup[] grouped) throws JavaModelException {
 		RefactoringStatus result= new RefactoringStatus();
-		for (int i= 0; i < grouped.length; i++){
-			IResource resource= grouped[i].getResource();
-			if (hasCompileErrors(resource))
-				result.addWarning(RefactoringCoreMessages.getFormattedString("Checks.cu_has_compile_errors", resource.getFullPath().makeRelative())); //$NON-NLS-1$
+		for (int i= 0; i < grouped.length; i++)
+			checkCompileErrorsInAffectedFile(result, grouped[i].getResource());
+		return result;
+	}
+	
+	public static void checkCompileErrorsInAffectedFile(RefactoringStatus result, IResource resource) throws JavaModelException {
+		if (hasCompileErrors(resource))
+			result.addWarning(Messages.format(RefactoringCoreMessages.Checks_cu_has_compile_errors, resource.getFullPath().makeRelative())); 
+	}
+	
+	public static RefactoringStatus checkCompileErrorsInAffectedFiles(SearchResultGroup[] references, IResource declaring) throws JavaModelException {
+		RefactoringStatus result= new RefactoringStatus();
+		for (int i= 0; i < references.length; i++){
+			IResource resource= references[i].getResource();
+			if (resource.equals(declaring))
+				declaring= null;
+			checkCompileErrorsInAffectedFile(result, resource);
 		}
+		if (declaring != null)
+			checkCompileErrorsInAffectedFile(result, declaring);
 		return result;
 	}
 	
@@ -502,8 +655,9 @@ public class Checks {
 		return false;
 	}
 	
-	public static boolean isReadOnly(IResource res) throws JavaModelException{
-		if (res.isReadOnly()) 
+	public static boolean isReadOnly(IResource res) throws JavaModelException {
+		ResourceAttributes attributes= res.getResourceAttributes();
+		if (attributes != null && attributes.isReadOnly())
 			return true;
 		
 		if (! (res instanceof IContainer))	
@@ -524,27 +678,52 @@ public class Checks {
 		}
 	}
 	
-	public static boolean isClasspathDelete(IPackageFragmentRoot pkgRoot) throws JavaModelException {
+	public static boolean isClasspathDelete(IPackageFragmentRoot pkgRoot) {
 		IResource res= pkgRoot.getResource();
 		if (res == null)
 			return true;
 		IProject definingProject= res.getProject();
+		if (res.getParent() != null && pkgRoot.isArchive() && ! res.getParent().equals(definingProject))
+			return true;
+		
 		IProject occurringProject= pkgRoot.getJavaProject().getProject();
 		return !definingProject.equals(occurringProject);
 	}
 	
 	//-------- validateEdit checks ----
 	
-	public static RefactoringStatus validateModifiesFiles(IFile[] filesToModify) {
+	public static RefactoringStatus validateModifiesFiles(IFile[] filesToModify, Object context) {
 		RefactoringStatus result= new RefactoringStatus();
 		IStatus status= Resources.checkInSync(filesToModify);
 		if (!status.isOK())
 			result.merge(RefactoringStatus.create(status));
-		status= Resources.makeCommittable(filesToModify, null);
-		if (!status.isOK())
+		status= Resources.makeCommittable(filesToModify, context);
+		if (!status.isOK()) {
 			result.merge(RefactoringStatus.create(status));
+			if (!result.hasFatalError()) {
+				result.addFatalError(RefactoringCoreMessages.Checks_validateEdit); 
+			}			
+		}
 		return result;
 	}
+	
+	public static RefactoringStatus validateEdit(ICompilationUnit unit, Object context) {
+		IResource resource= JavaModelUtil.toOriginal(unit).getResource();
+		RefactoringStatus result= new RefactoringStatus();
+		if (resource == null)
+			return result;
+		IStatus status= Resources.checkInSync(resource);
+		if (!status.isOK())
+			result.merge(RefactoringStatus.create(status));
+		status= Resources.makeCommittable(resource, context);
+		if (!status.isOK()) {
+			result.merge(RefactoringStatus.create(status));
+			if (!result.hasFatalError()) {
+				result.addFatalError(RefactoringCoreMessages.Checks_validateEdit); 
+			}			
+		}
+		return result;
+	}	
 
 	/**
 	 * Checks whether it is possible to modify the given <code>IJavaElement</code>.
@@ -552,27 +731,47 @@ public class Checks {
 	 * Moreover, if it is a <code>IMember</code> it must not be binary.
 	 * The returned <code>RefactoringStatus</code> has <code>ERROR</code> severity if
 	 * it is not possible to modify the element.
+	 * @param javaElement
+	 * @return the status
+	 * @throws JavaModelException
 	 *
 	 * @see IJavaElement#exists
 	 * @see IJavaElement#isReadOnly
 	 * @see IMember#isBinary
 	 * @see RefactoringStatus
-	 *
 	 */ 
 	public static RefactoringStatus checkAvailability(IJavaElement javaElement) throws JavaModelException{
 		RefactoringStatus result= new RefactoringStatus();
 		if (! javaElement.exists())
-			result.addFatalError(RefactoringCoreMessages.getFormattedString("Refactoring.not_in_model", javaElement.getElementName())); //$NON-NLS-1$
+			result.addFatalError(Messages.format(RefactoringCoreMessages.Refactoring_not_in_model, javaElement.getElementName())); 
 		if (javaElement.isReadOnly())
-			result.addFatalError(RefactoringCoreMessages.getFormattedString("Refactoring.read_only", javaElement.getElementName()));	 //$NON-NLS-1$
+			result.addFatalError(Messages.format(RefactoringCoreMessages.Refactoring_read_only, javaElement.getElementName()));	 
 		if (javaElement.exists() && !javaElement.isStructureKnown())
-			result.addFatalError(RefactoringCoreMessages.getFormattedString("Refactoring.unknown_structure", javaElement.getElementName()));	 //$NON-NLS-1$
+			result.addFatalError(Messages.format(RefactoringCoreMessages.Refactoring_unknown_structure, javaElement.getElementName()));	 
 		if (javaElement instanceof IMember && ((IMember)javaElement).isBinary())
-			result.addFatalError(RefactoringCoreMessages.getFormattedString("Refactoring.binary", javaElement.getElementName())); //$NON-NLS-1$
+			result.addFatalError(Messages.format(RefactoringCoreMessages.Refactoring_binary, javaElement.getElementName())); 
 		return result;
 	}
+	
+	public static boolean isAvailable(IJavaElement javaElement) throws JavaModelException {
+		if (javaElement == null)
+			return false;
+		if (! javaElement.exists())
+			return false;
+		if (javaElement.isReadOnly())
+			return false;
+		// work around for https://bugs.eclipse.org/bugs/show_bug.cgi?id=48422
+		// the Java project is now cheating regarding its children so we shouldn't
+		// call isStructureKnown if the project isn't open.
+		// see bug https://bugs.eclipse.org/bugs/show_bug.cgi?id=52474
+		if (!(javaElement instanceof IJavaProject) && !(javaElement instanceof ILocalVariable) && !javaElement.isStructureKnown())
+			return false;
+		if (javaElement instanceof IMember && ((IMember)javaElement).isBinary())
+			return false;
+		return true;
+	}
 
-	public static IType findTypeInPackage(IPackageFragment pack, String name) throws JavaModelException{
+	public static IType findTypeInPackage(IPackageFragment pack, String name) throws JavaModelException {
 		Assert.isTrue(pack.exists());
 		Assert.isTrue(!pack.isReadOnly());
 		
@@ -592,27 +791,41 @@ public class Checks {
 		if (result.hasFatalError())
 			return result;
 		if (! Checks.startsWithLowerCase(newName))
-			result.addWarning(RefactoringCoreMessages.getString("ExtractTempRefactoring.convention")); //$NON-NLS-1$
+			result.addWarning(RefactoringCoreMessages.ExtractTempRefactoring_convention); 
 		return result;		
 	}
-	
-	public static RefactoringStatus checkConstantName(String newName) {
+
+	public static RefactoringStatus checkEnumConstantName(String newName) {
 		RefactoringStatus result= Checks.checkFieldName(newName);
-		if(result.hasFatalError())
+		if (result.hasFatalError())
 			return result;
-		for(int i= 0; i < newName.length(); i++) {
+		for (int i= 0; i < newName.length(); i++) {
 			char c= newName.charAt(i);
-			if(Character.isLetter(c) && !Character.isUpperCase(c)) {
-				result.addWarning(RefactoringCoreMessages.getString("ExtractConstantRefactoring.convention")); //$NON-NLS-1$	
+			if (Character.isLetter(c) && !Character.isUpperCase(c)) {
+				result.addWarning(RefactoringCoreMessages.RenameEnumConstRefactoring_convention); 
 				break;
-			}	
+			}
 		}
 		return result;
 	}
-	
+
+	public static RefactoringStatus checkConstantName(String newName) {
+		RefactoringStatus result= Checks.checkFieldName(newName);
+		if (result.hasFatalError())
+			return result;
+		for (int i= 0; i < newName.length(); i++) {
+			char c= newName.charAt(i);
+			if (Character.isLetter(c) && !Character.isUpperCase(c)) {
+				result.addWarning(RefactoringCoreMessages.ExtractConstantRefactoring_convention); 
+				break;
+			}
+		}
+		return result;
+	}
+
 	public static boolean isException(IType iType, IProgressMonitor pm) throws JavaModelException {
 		try{
-			if (iType.isInterface())
+			if (! iType.isClass())
 				return false;
 			IType[] superTypes= iType.newSupertypeHierarchy(pm).getAllSupertypes(iType);
 			for (int i= 0; i < superTypes.length; i++) {
@@ -624,12 +837,7 @@ public class Checks {
 			pm.done();
 		}	
 	}
-	
-	public static boolean isRValue(Expression e) {
-		return checkExpressionIsRValue(e) == IS_RVALUE;
-	}
-	
-	
+		
 	/**
 	 * @param e
 	 * @return int
@@ -660,6 +868,7 @@ public class Checks {
 		ASTNode anonymous= ASTNodes.getParent(tempDeclaration, AnonymousClassDeclaration.class);	
 		if (anonymous == null)
 			return true;
+		// stupid code. Is to find out if the variable declaration isn't a field.
 		if (ASTNodes.isParent(anonymous, initializer))
 			return false;
 		return true;	
