@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -17,15 +17,13 @@ import java.util.Iterator;
 import java.util.List;
 
 import org.eclipse.swt.SWT;
-import org.eclipse.swt.custom.TableCursor;
-import org.eclipse.swt.events.FocusAdapter;
-import org.eclipse.swt.events.FocusEvent;
 import org.eclipse.swt.events.KeyAdapter;
 import org.eclipse.swt.events.KeyEvent;
-import org.eclipse.swt.events.MouseAdapter;
-import org.eclipse.swt.events.MouseEvent;
 import org.eclipse.swt.events.SelectionAdapter;
 import org.eclipse.swt.events.SelectionEvent;
+import org.eclipse.swt.events.TraverseEvent;
+import org.eclipse.swt.events.TraverseListener;
+import org.eclipse.swt.graphics.Font;
 import org.eclipse.swt.graphics.Image;
 import org.eclipse.swt.layout.GridData;
 import org.eclipse.swt.layout.GridLayout;
@@ -35,27 +33,34 @@ import org.eclipse.swt.widgets.Label;
 import org.eclipse.swt.widgets.Table;
 import org.eclipse.swt.widgets.TableColumn;
 import org.eclipse.swt.widgets.TableItem;
-import org.eclipse.swt.widgets.Widget;
+import org.eclipse.swt.widgets.Text;
 
-import org.eclipse.jface.dialogs.IDialogConstants;
-import org.eclipse.jface.viewers.CellEditor;
+import org.eclipse.jface.contentassist.SubjectControlContentAssistant;
+import org.eclipse.jface.resource.JFaceResources;
 import org.eclipse.jface.viewers.ColumnWeightData;
 import org.eclipse.jface.viewers.ICellModifier;
 import org.eclipse.jface.viewers.ISelection;
 import org.eclipse.jface.viewers.ISelectionChangedListener;
 import org.eclipse.jface.viewers.IStructuredContentProvider;
 import org.eclipse.jface.viewers.IStructuredSelection;
+import org.eclipse.jface.viewers.ITableFontProvider;
 import org.eclipse.jface.viewers.ITableLabelProvider;
 import org.eclipse.jface.viewers.LabelProvider;
 import org.eclipse.jface.viewers.SelectionChangedEvent;
 import org.eclipse.jface.viewers.StructuredSelection;
 import org.eclipse.jface.viewers.TableViewer;
-import org.eclipse.jface.viewers.TextCellEditor;
 import org.eclipse.jface.viewers.Viewer;
 
+import org.eclipse.ui.contentassist.ContentAssistHandler;
+
 import org.eclipse.jdt.internal.corext.Assert;
 import org.eclipse.jdt.internal.corext.refactoring.ParameterInfo;
+import org.eclipse.jdt.internal.corext.refactoring.StubTypeContext;
 
+import org.eclipse.jdt.internal.ui.dialogs.TableTextCellEditor;
+import org.eclipse.jdt.internal.ui.refactoring.contentassist.ControlContentAssistHelper;
+import org.eclipse.jdt.internal.ui.refactoring.contentassist.JavaTypeCompletionProcessor;
+import org.eclipse.jdt.internal.ui.refactoring.contentassist.VariableNamesProcessor;
 import org.eclipse.jdt.internal.ui.util.SWTUtil;
 import org.eclipse.jdt.internal.ui.util.TableLayoutComposite;
 
@@ -63,7 +68,29 @@ import org.eclipse.jdt.internal.ui.util.TableLayoutComposite;
  * A special control to edit and reorder method parameters.
  */
 public class ChangeParametersControl extends Composite {
-
+	
+	public static class Mode {
+		private final String fName;
+		private Mode(String name) {
+			fName= name;
+		}
+		public static final Mode EXTRACT_METHOD= new Mode("EXTRACT_METHOD"); //$NON-NLS-1$
+		public static final Mode CHANGE_METHOD_SIGNATURE= new Mode("CHANGE_METHOD_SIGNATURE"); //$NON-NLS-1$
+		public static final Mode INTRODUCE_PARAMETER= new Mode("INTRODUCE_PARAMETER"); //$NON-NLS-1$
+		public String toString() {
+			return fName;
+		}
+		public boolean canChangeTypes() {
+			return this == CHANGE_METHOD_SIGNATURE;
+		}
+		public boolean canAddParameters() {
+			return this == Mode.CHANGE_METHOD_SIGNATURE;
+		}
+		public boolean canChangeDefault() {
+			return this == Mode.CHANGE_METHOD_SIGNATURE;
+		}
+	}
+	
 	private static class ParameterInfoContentProvider implements IStructuredContentProvider {
 		public Object[] getElements(Object inputElement) {
 			return removeMarkedAsDeleted((List) inputElement);
@@ -85,30 +112,42 @@ public class ChangeParametersControl extends Composite {
 		}
 	}
 
-	private static class ParameterInfoLabelProvider extends LabelProvider implements ITableLabelProvider {
+	private static class ParameterInfoLabelProvider extends LabelProvider implements ITableLabelProvider, ITableFontProvider {
 		public Image getColumnImage(Object element, int columnIndex) {
 			return null;
 		}
 		public String getColumnText(Object element, int columnIndex) {
 			ParameterInfo info= (ParameterInfo) element;
-			if (columnIndex == TYPE_PROP)
-				return info.getNewTypeName();
-			if (columnIndex == NEWNAME_PROP)
-				return info.getNewName();
-			if (columnIndex == DEFAULT_PROP)
-				return info.getDefaultValue();
-			Assert.isTrue(false);
-			return ""; //$NON-NLS-1$
+			switch (columnIndex) {
+				case TYPE_PROP:
+					return info.getNewTypeName();
+				case NEWNAME_PROP:
+					return info.getNewName();
+				case DEFAULT_PROP:
+				    if (info.isAdded())
+				        return info.getDefaultValue();
+				    else
+				        return "-"; //$NON-NLS-1$
+				default:
+					throw new IllegalArgumentException(columnIndex + ": " + element); //$NON-NLS-1$
+			}
 		}
+		public Font getFont(Object element, int columnIndex) {
+			ParameterInfo info= (ParameterInfo) element;
+			if (info.isAdded())
+				return JFaceResources.getFontRegistry().getBold(JFaceResources.DIALOG_FONT);
+			else
+				return null;
+		}		
 	}
 
 	private class ParametersCellModifier implements ICellModifier {
 		public boolean canModify(Object element, String property) {
 			Assert.isTrue(element instanceof ParameterInfo);
 			if (property.equals(PROPERTIES[TYPE_PROP]))
-				return ChangeParametersControl.this.fCanChangeTypesOfOldParameters;
+				return fMode.canChangeTypes();
 			else if (property.equals(PROPERTIES[NEWNAME_PROP]))
-				return ChangeParametersControl.this.fCanChangeParameterNames;
+				return true;
 			else if (property.equals(PROPERTIES[DEFAULT_PROP]))
 				return (((ParameterInfo)element).isAdded());
 			Assert.isTrue(false);
@@ -126,86 +165,117 @@ public class ChangeParametersControl extends Composite {
 			return null;
 		}
 		public void modify(Object element, String property, Object value) {
-			if (!(element instanceof TableItem))
+			if (element instanceof TableItem)
+				element= ((TableItem) element).getData();
+			if (!(element instanceof ParameterInfo))
 				return;
-			Object data= ((TableItem) element).getData();
-			if (!(data instanceof ParameterInfo))
-				return;
-			ParameterInfo parameterInfo= (ParameterInfo) data;
-			if (property.equals(PROPERTIES[NEWNAME_PROP])) 
+			boolean unchanged;
+			ParameterInfo parameterInfo= (ParameterInfo) element;
+			if (property.equals(PROPERTIES[NEWNAME_PROP])) {
+				unchanged= parameterInfo.getNewName().equals(value);
 				parameterInfo.setNewName((String) value);
-			else if (property.equals(PROPERTIES[DEFAULT_PROP]))
+			} else if (property.equals(PROPERTIES[DEFAULT_PROP])) {
+				unchanged= parameterInfo.getDefaultValue().equals(value);
 				parameterInfo.setDefaultValue((String) value);
-			else if (property.equals(PROPERTIES[TYPE_PROP]))
+			} else if (property.equals(PROPERTIES[TYPE_PROP])) {
+				unchanged= parameterInfo.getNewTypeName().equals(value);
 				parameterInfo.setNewTypeName((String) value);
- 			else 
- 				Assert.isTrue(false);
-			ChangeParametersControl.this.fListener.parameterChanged(parameterInfo);
-			ChangeParametersControl.this.fTableViewer.update(parameterInfo, new String[] { property });
+			} else {
+				throw new IllegalStateException();
+			}
+			if (! unchanged) {
+				ChangeParametersControl.this.fListener.parameterChanged(parameterInfo);
+				ChangeParametersControl.this.fTableViewer.update(parameterInfo, new String[] { property });
+			}
 		}
-	};
+	}
 
 	private static final String[] PROPERTIES= { "type", "new", "default" }; //$NON-NLS-2$ //$NON-NLS-1$ //$NON-NLS-3$
 	private static final int TYPE_PROP= 0;
 	private static final int NEWNAME_PROP= 1;
 	private static final int DEFAULT_PROP= 2;
 
-	private static final int ROW_COUNT= 10;
+	private static final int ROW_COUNT= 7;
 
-	private final boolean fCanChangeParameterNames;
-	private final boolean fCanChangeTypesOfOldParameters;
-	private final boolean fCanAddParameters;
+	private final Mode fMode;
 	private final IParameterListChangeListener fListener;
+	private List fParameterInfos;
+	private final StubTypeContext fTypeContext;
+	private final String[] fParamNameProposals;
+	private ContentAssistHandler fNameContentAssistHandler;
 
+	private TableViewer fTableViewer;
 	private Button fUpButton;
 	private Button fDownButton;
-	private TableViewer fTableViewer;
 	private Button fEditButton;
 	private Button fAddButton;
 	private Button fRemoveButton;
-	private List fParameterInfos;
-	private TableCursor fTableCursor; 
 
-	public ChangeParametersControl(Composite parent, int style, String label, IParameterListChangeListener listener, boolean canChangeParameterNames, boolean canChangeTypesOfOldParameters, boolean canAddParameters) {
+	public ChangeParametersControl(Composite parent, int style, String label, IParameterListChangeListener listener, Mode mode, StubTypeContext typeContext) {
+		this(parent, style, label, listener, mode, typeContext, new String[0]);
+	}
+	
+	public ChangeParametersControl(Composite parent, int style, String label, IParameterListChangeListener listener, Mode mode) {
+		this(parent, style, label, listener, mode, null, new String[0]);
+	}
+	
+	public ChangeParametersControl(Composite parent, int style, String label, IParameterListChangeListener listener, Mode mode, String[] paramNameProposals) {
+		this(parent, style, label, listener, mode, null, paramNameProposals);
+	}
+	
+	/**
+	 * @param label the label before the table or <code>null</code>
+	 * @param typeContext the package in which to complete types
+	 */
+	private ChangeParametersControl(Composite parent, int style, String label, IParameterListChangeListener listener, Mode mode, StubTypeContext typeContext, String[] paramNameProposals) {
 		super(parent, style);
 		Assert.isNotNull(listener);
 		fListener= listener;
-		fCanChangeParameterNames= canChangeParameterNames;
-		fCanChangeTypesOfOldParameters= canChangeTypesOfOldParameters;
-		fCanAddParameters= canAddParameters;
+		fMode= mode;
+		fTypeContext= typeContext;
+		fParamNameProposals= paramNameProposals;
+
 		GridLayout layout= new GridLayout();
 		layout.numColumns= 2;
 		layout.marginWidth= 0;
 		layout.marginHeight= 0;
 		setLayout(layout);
 
-		Label tableLabel= new Label(this, SWT.NONE);
-		GridData labelGd= new GridData();
-		labelGd.horizontalSpan= 2;
-		tableLabel.setLayoutData(labelGd);
-		tableLabel.setText(label);
+		if (label != null) {
+			Label tableLabel= new Label(this, SWT.NONE);
+			GridData labelGd= new GridData();
+			labelGd.horizontalSpan= 2;
+			tableLabel.setLayoutData(labelGd);
+			tableLabel.setText(label);
+		}
 
 		createParameterList(this);
 		createButtonComposite(this);
 	}
 
+
 	public void setInput(List parameterInfos) {
 		Assert.isNotNull(parameterInfos);
 		fParameterInfos= parameterInfos;
 		fTableViewer.setInput(fParameterInfos);
 		if (fParameterInfos.size() > 0)
 			fTableViewer.setSelection(new StructuredSelection(fParameterInfos.get(0)));
-			
-		//XXX workaround for bug 24798	
-		if (canEditTableCells() && getTableItemCount() != 0) {
-			showTableCursor(true);
-		}	
+	}
+	
+	public void editParameter(ParameterInfo info) {
+		fTableViewer.getControl().setFocus();
+		if (! info.isDeleted()) {
+			fTableViewer.setSelection(new StructuredSelection(info), true);
+			updateButtonsEnabledState();
+			editColumnOrNextPossible(NEWNAME_PROP);
+			return;	
+		}
 	}
 
 	// ---- Parameter table -----------------------------------------------------------------------------------
 
 	private void createParameterList(Composite parent) {
-		TableLayoutComposite layouter= new TableLayoutComposite(parent, SWT.NULL);
+		TableLayoutComposite layouter= new TableLayoutComposite(parent, SWT.NONE);
 		addColumnLayoutData(layouter);
 		
 		final Table table= new Table(layouter, SWT.MULTI | SWT.BORDER | SWT.FULL_SELECTION);
@@ -214,23 +284,21 @@ public class ChangeParametersControl extends Composite {
 		TableColumn tc;
 		tc= new TableColumn(table, SWT.NONE, TYPE_PROP);
 		tc.setResizable(true);
-		tc.setText(RefactoringMessages.getString("ChangeParametersControl.table.type")); //$NON-NLS-1$
+		tc.setText(RefactoringMessages.ChangeParametersControl_table_type); 
 		
 		tc= new TableColumn(table, SWT.NONE, NEWNAME_PROP);
 		tc.setResizable(true);
-		tc.setText(RefactoringMessages.getString("ChangeParametersControl.table.name")); //$NON-NLS-1$
+		tc.setText(RefactoringMessages.ChangeParametersControl_table_name); 
 
-		if (fCanAddParameters){
+		if (fMode.canChangeDefault()){
 			tc= new TableColumn(table, SWT.NONE, DEFAULT_PROP);
 			tc.setResizable(true);
-			tc.setText(RefactoringMessages.getString("ChangeParametersControl.table.defaultValue")); //$NON-NLS-1$
+			tc.setText(RefactoringMessages.ChangeParametersControl_table_defaultValue); 
 		}	
 		
-		GridData gd= new GridData(GridData.FILL_HORIZONTAL | GridData.GRAB_HORIZONTAL);
+		GridData gd= new GridData(GridData.FILL_BOTH);
 		gd.heightHint= SWTUtil.getTableHeightHint(table, ROW_COUNT);
 		gd.widthHint= 40;
-		gd.verticalAlignment= GridData.BEGINNING;
-		gd.grabExcessVerticalSpace= true;
 		layouter.setLayoutData(gd);
 
 		fTableViewer= new TableViewer(table);
@@ -243,74 +311,62 @@ public class ChangeParametersControl extends Composite {
 			}
 		});
 
-		if (canEditTableCells()){
-			addCellEditors();
-		}	
-	}
-
-    private boolean canEditTableCells() {
-        return fCanChangeParameterNames || fCanChangeTypesOfOldParameters;
-    }
-
-	private void addTableCursor(final Table table) {
-		fTableCursor = new TableCursor(table, SWT.NONE);
-		fTableCursor.addSelectionListener(new SelectionAdapter() {
-			public void widgetSelected(SelectionEvent e) {
-				fTableViewer.setSelection(new StructuredSelection(fTableCursor.getRow().getData()));
-			}
-			public void widgetDefaultSelected(SelectionEvent e){
-				ChangeParametersControl.this.editFirstSelected();
-			}
-		});	
-		fTableCursor.addMouseListener(new MouseAdapter(){
-			public void mouseDoubleClick(MouseEvent e) {
-				ChangeParametersControl.this.editFirstSelected();
-			}
-			public void mouseDown(MouseEvent e) {
-				ChangeParametersControl.this.editFirstSelected();
-			}
-		});
-		// Hide the TableCursor when the user hits the "CTRL" or "SHIFT" key.
-		// This alows the user to select multiple items in the table.
-		fTableCursor.addKeyListener(new KeyAdapter() {
-			public void keyPressed(KeyEvent e) {
-				if (	e.keyCode == SWT.MOD1 || 
-				    	e.keyCode == SWT.MOD2 || 
-				    (e.stateMask & SWT.MOD1) != 0 || 
-				    (e.stateMask & SWT.MOD2) != 0) {
-						setTableCursorVisible(false);
+		table.addTraverseListener(new TraverseListener() {
+			public void keyTraversed(TraverseEvent e) {
+				if (e.detail == SWT.TRAVERSE_RETURN && e.stateMask == SWT.NONE) {
+					editColumnOrNextPossible(0);
+					e.detail= SWT.TRAVERSE_NONE;
 				}
 			}
 		});
-		// Show the TableCursor when the user releases the "SHIFT" or "CTRL" key.
-		// This signals the end of the multiple selection task.
 		table.addKeyListener(new KeyAdapter() {
-			public void keyReleased(KeyEvent e) {
-				if (e.keyCode == SWT.MOD1 	&& (e.stateMask & SWT.MOD2) 	!= 0) return;
-				if (e.keyCode == SWT.MOD2 	&& (e.stateMask & SWT.MOD1) 	!= 0) return;
-				if (e.keyCode != SWT.MOD1 	&& (e.stateMask & SWT.MOD1) 	!= 0) return;
-				if (e.keyCode != SWT.MOD2 	&& (e.stateMask & SWT.MOD2) 	!= 0) return;
-				if (table.getSelectionCount() != 1) return;
-				
-				TableItem[] selection = table.getSelection();
-				TableItem row = (selection.length == 0) ? table.getItem(table.getTopIndex()) : selection[0];
-				table.showItem(row);
-				fTableCursor.setSelection(row, 0);
-				setTableCursorVisible(true);
-				fTableCursor.setFocus();
+			public void keyPressed(KeyEvent e) {
+				if (e.keyCode == SWT.F2 && e.stateMask == SWT.NONE) {
+					editColumnOrNextPossible(0);
+					e.doit= false;
+				}
 			}
 		});
-	}
-	
-	private void editFirstSelected(){
-		ParameterInfo[]	selected= getSelectedItems();
-		if (selected.length != 1)
-			return;
-		fTableViewer.editElement(selected[0], fTableCursor.getColumn());
+
+		addCellEditors();
 	}
 
+	private void editColumnOrNextPossible(int column){
+		ParameterInfo[]	selected= getSelectedElements();
+		if (selected.length != 1)
+			return;
+		int nextColumn= column;
+		do {
+			fTableViewer.editElement(selected[0], nextColumn);
+			if (fTableViewer.isCellEditorActive())
+				return;
+			nextColumn= nextColumn(nextColumn);
+		} while (nextColumn != column);
+	}
+	
+	private void editColumnOrPrevPossible(int column){
+		ParameterInfo[]	selected= getSelectedElements();
+		if (selected.length != 1)
+			return;
+		int prevColumn= column;
+		do {
+			fTableViewer.editElement(selected[0], prevColumn);
+			if (fTableViewer.isCellEditorActive())
+			    return;
+			prevColumn= prevColumn(prevColumn);
+		} while (prevColumn != column);
+	}
+	
+	private int nextColumn(int column) {
+		return (column >= getTable().getColumnCount() - 1) ? 0 : column + 1;
+	}
+	
+	private int prevColumn(int column) {
+		return (column <= 0) ? getTable().getColumnCount() - 1 : column - 1;
+	}
+	
 	private void addColumnLayoutData(TableLayoutComposite layouter) {
-		if (fCanAddParameters){
+		if (fMode.canChangeDefault()){
 			layouter.addColumnData(new ColumnWeightData(33, true));
 			layouter.addColumnData(new ColumnWeightData(33, true));
 			layouter.addColumnData(new ColumnWeightData(34, true));
@@ -320,7 +376,7 @@ public class ChangeParametersControl extends Composite {
 		}	
 	}
 
-	private ParameterInfo[] getSelectedItems() {
+	private ParameterInfo[] getSelectedElements() {
 		ISelection selection= fTableViewer.getSelection();
 		if (selection == null)
 			return new ParameterInfo[0];
@@ -342,22 +398,20 @@ public class ChangeParametersControl extends Composite {
 		gl.marginWidth= 0;
 		buttonComposite.setLayout(gl);
 
-		if(fCanAddParameters)
+		if (fMode.canAddParameters())
 			fAddButton= createAddButton(buttonComposite);	
 
-		if (fCanChangeParameterNames)
-			fEditButton= createEditButton(buttonComposite);
+		fEditButton= createEditButton(buttonComposite);
+		
+		if (fMode.canAddParameters())
+			fRemoveButton= createRemoveButton(buttonComposite);	
 		
 		if (buttonComposite.getChildren().length != 0)
 			addSpacer(buttonComposite);
 
-		fUpButton= createButton(buttonComposite, RefactoringMessages.getString("ChangeParametersControl.buttons.move_up"), true); //$NON-NLS-1$
-		fDownButton= createButton(buttonComposite, RefactoringMessages.getString("ChangeParametersControl.buttons.move_down"), false); //$NON-NLS-1$
+		fUpButton= createButton(buttonComposite, RefactoringMessages.ChangeParametersControl_buttons_move_up, true); 
+		fDownButton= createButton(buttonComposite, RefactoringMessages.ChangeParametersControl_buttons_move_down, false); 
 
-		if(fCanAddParameters){
-			addSpacer(buttonComposite);
-			fRemoveButton= createRemoveButton(buttonComposite);
-		}
 		updateButtonsEnabledState();
 	}
 
@@ -393,57 +447,50 @@ public class ChangeParametersControl extends Composite {
 	
 	private Button createEditButton(Composite buttonComposite) {
 		Button button= new Button(buttonComposite, SWT.PUSH);
-		button.setText(RefactoringMessages.getString("ChangeParametersControl.buttons.edit")); //$NON-NLS-1$
+		button.setText(RefactoringMessages.ChangeParametersControl_buttons_edit); 
 		button.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
 		SWTUtil.setButtonDimensionHint(button);
 		button.addSelectionListener(new SelectionAdapter() {
 			public void widgetSelected(SelectionEvent e) {
-				ISelection selection= fTableViewer.getSelection();
 				try {
-					ParameterInfo[] selected= getSelectedItems();
+					ParameterInfo[] selected= getSelectedElements();
 					Assert.isTrue(selected.length == 1);
 					ParameterInfo parameterInfo= selected[0];
-					int column= getTableCursorColumn();
-					int row= getTableCursorRow();
-					showTableCursor(false);
-					ParameterEditDialog dialog= new ParameterEditDialog(getShell(), parameterInfo, fCanChangeTypesOfOldParameters);
-					if (dialog.open() == IDialogConstants.CANCEL_ID) {
-						fTableViewer.setSelection(selection);
-						return;
-					}
+					ParameterEditDialog dialog= new ParameterEditDialog(getShell(), parameterInfo, fMode.canChangeTypes(), fMode.canChangeDefault(), fTypeContext);
+					dialog.open();
 					fListener.parameterChanged(parameterInfo);
 					fTableViewer.update(parameterInfo, PROPERTIES);
-					if (column >= 0 && row >= 0) {
-						showTableCursor(true);
-						setTableCursorSelection(row, column);
-					}
 				} finally {
-					fTableViewer.refresh();
 					fTableViewer.getControl().setFocus();
-					fTableViewer.setSelection(selection);
 				}
 			}
 		});
 		return button;
 	}
-
+	
 	private Button createAddButton(Composite buttonComposite) {
 		Button button= new Button(buttonComposite, SWT.PUSH);
-		button.setText(RefactoringMessages.getString("ChangeParametersControl.buttons.add")); //$NON-NLS-1$
+		button.setText(RefactoringMessages.ChangeParametersControl_buttons_add); 
 		button.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
 		SWTUtil.setButtonDimensionHint(button);
 		button.addSelectionListener(new SelectionAdapter() {
 			public void widgetSelected(SelectionEvent e) {
 				ParameterInfo newInfo= ParameterInfo.createInfoForAddedParameter();
-				fParameterInfos.add(newInfo);
+				int insertIndex= fParameterInfos.size();
+				for (int i= fParameterInfos.size() - 1;  i >= 0; i--) {
+					ParameterInfo info= (ParameterInfo) fParameterInfos.get(i);
+					if (info.isNewVarargs()) {
+						insertIndex= i;
+						break;
+					}
+				}
+				fParameterInfos.add(insertIndex, newInfo);
 				fListener.parameterAdded(newInfo);
 				fTableViewer.refresh();
 				fTableViewer.getControl().setFocus();
-				int row= getTableItemCount() - 1;
-				getTable().setSelection(row);
+				fTableViewer.setSelection(new StructuredSelection(newInfo), true);
 				updateButtonsEnabledState();
-				showTableCursor(true);
-				setTableCursorSelection(row, 0);
+				editColumnOrNextPossible(0);
 			}
 		});	
 		return button;
@@ -451,13 +498,13 @@ public class ChangeParametersControl extends Composite {
 
 	private Button createRemoveButton(Composite buttonComposite) {
 		final Button button= new Button(buttonComposite, SWT.PUSH);
-		button.setText(RefactoringMessages.getString("ChangeParametersControl.buttons.remove")); //$NON-NLS-1$
+		button.setText(RefactoringMessages.ChangeParametersControl_buttons_remove); 
 		button.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
 		SWTUtil.setButtonDimensionHint(button);
 		button.addSelectionListener(new SelectionAdapter() {
 			public void widgetSelected(SelectionEvent e) {
 				int index= getTable().getSelectionIndices()[0];
-				ParameterInfo[] selected= getSelectedItems();
+				ParameterInfo[] selected= getSelectedElements();
 				for (int i= 0; i < selected.length; i++) {
 					if (selected[i].isAdded())
 						fParameterInfos.remove(selected[i]);
@@ -470,14 +517,9 @@ public class ChangeParametersControl extends Composite {
 				fTableViewer.refresh();
 				fTableViewer.getControl().setFocus();
 				int itemCount= getTableItemCount();
-				if (itemCount != 0) {
-					if (index >= itemCount)
-						index= itemCount - 1;
+				if (itemCount != 0 && index >= itemCount) {
+					index= itemCount - 1;
 					getTable().setSelection(index);
-					setTableCursorSelection(index, 0);
-					showTableCursor(true);
-				} else {
-					showTableCursor(false); 
 				}
 				fListener.parameterListChanged();
 				updateButtonsEnabledState();
@@ -496,16 +538,9 @@ public class ChangeParametersControl extends Composite {
 				ISelection savedSelection= fTableViewer.getSelection();
 				if (savedSelection == null)
 					return;
-				ParameterInfo[] selection= getSelectedItems();
+				ParameterInfo[] selection= getSelectedElements();
 				if (selection.length == 0)
 					return;
-
-				int column= getTableCursorColumn();
-				Object element= null;
-				int r=  getTableCursorRow();
-				if (r >= 0) {
-					element= fTableViewer.getElementAt(r);
-				}
 					
 				if (up) {
 					moveUp(selection);
@@ -513,99 +548,89 @@ public class ChangeParametersControl extends Composite {
 					moveDown(selection);
 				}
 				fTableViewer.refresh();
-				fTableViewer.getControl().setFocus();
 				fTableViewer.setSelection(savedSelection);
 				fListener.parameterListChanged();
-				Widget item= fTableViewer.testFindItem(element);
-				int row= 0;
-				if (item instanceof TableItem) {
-					row= getTable().indexOf((TableItem)item);
-					if (row < 0)
-						row= 0;
-				}
-				setTableCursorSelection(row, column);
+				fTableViewer.getControl().setFocus();
 			}
 		});
 		return button;
 	}
 	
-	private void showTableCursor(boolean show) {
-		if (show) {
-			if (fTableCursor == null || fTableCursor.isDisposed())
-				addTableCursor(fTableViewer.getTable());
-			fTableCursor.setVisible(true);
-		} else {
-			if (fTableCursor != null && !fTableCursor.isDisposed()) {
-				fTableCursor.setVisible(false);
-				fTableCursor.dispose();
-			}
-			fTableCursor= null;
-		}
-	}
-	
-	private void setTableCursorSelection(int row, int column) {
-		if (fTableCursor != null && !fTableCursor.isDisposed())
-			fTableCursor.setSelection(row, column);
-	}
-	
-	private void setTableCursorVisible(boolean visible) {
-		if (fTableCursor != null && !fTableCursor.isDisposed())
-			fTableCursor.setVisible(visible);
-	}
-	
-	private int getTableCursorColumn() {
-		if (fTableCursor != null && !fTableCursor.isDisposed())
-			return fTableCursor.getColumn();
-		return -1;
-	}
-	
-	private int getTableCursorRow() {
-		if (fTableCursor != null && !fTableCursor.isDisposed()) {
-			TableItem item= fTableCursor.getRow();
-			return getTable().indexOf(item);
-		}
-		return -1;
-	}
-	
 	//---- editing -----------------------------------------------------------------------------------------------
 
 	private void addCellEditors() {
-		final CellEditor editors[]= new CellEditor[PROPERTIES.length];
-
-		class AutoApplyTextCellEditor extends TextCellEditor {
-			public AutoApplyTextCellEditor(Composite parent) {
-				super(parent);
-			}
-			public void fireApplyEditorValue() {
-				super.fireApplyEditorValue();
-			}
-		};
-
-		editors[TYPE_PROP]= new AutoApplyTextCellEditor(getTable());
-		editors[TYPE_PROP].getControl().addFocusListener(new FocusAdapter() {
-			public void focusLost(FocusEvent e) {
-				((AutoApplyTextCellEditor) editors[TYPE_PROP]).fireApplyEditorValue();
-			}
-		});
-
-		editors[NEWNAME_PROP]= new AutoApplyTextCellEditor(getTable());
-		editors[NEWNAME_PROP].getControl().addFocusListener(new FocusAdapter() {
-			public void focusLost(FocusEvent e) {
-				((AutoApplyTextCellEditor) editors[NEWNAME_PROP]).fireApplyEditorValue();
-			}
-		});
-		editors[DEFAULT_PROP]= new AutoApplyTextCellEditor(getTable());
-		editors[DEFAULT_PROP].getControl().addFocusListener(new FocusAdapter() {
-			public void focusLost(FocusEvent e) {
-				((AutoApplyTextCellEditor) editors[DEFAULT_PROP]).fireApplyEditorValue();
-			}
-		});
-
-		fTableViewer.setCellEditors(editors);
 		fTableViewer.setColumnProperties(PROPERTIES);
+		
+		final TableTextCellEditor editors[]= new TableTextCellEditor[PROPERTIES.length];
+
+		editors[TYPE_PROP]= new TableTextCellEditor(fTableViewer, TYPE_PROP);
+		editors[NEWNAME_PROP]= new TableTextCellEditor(fTableViewer, NEWNAME_PROP);
+		editors[DEFAULT_PROP]= new TableTextCellEditor(fTableViewer, DEFAULT_PROP);
+		
+		if (fMode.canChangeTypes()) {
+			SubjectControlContentAssistant assistant= installParameterTypeContentAssist(editors[TYPE_PROP].getText());
+			editors[TYPE_PROP].setContentAssistant(assistant);
+		}
+		if (fParamNameProposals.length > 0) {
+			SubjectControlContentAssistant assistant= installParameterNameContentAssist(editors[NEWNAME_PROP].getText());
+			editors[NEWNAME_PROP].setContentAssistant(assistant);
+		}
+		
+		for (int i = 0; i < editors.length; i++) {
+			final int editorColumn= i;
+			final TableTextCellEditor editor = editors[i];
+			// support tabbing between columns while editing:
+			editor.getText().addTraverseListener(new TraverseListener() {
+				public void keyTraversed(TraverseEvent e) {
+					switch (e.detail) {
+						case SWT.TRAVERSE_TAB_NEXT :
+							editColumnOrNextPossible(nextColumn(editorColumn));
+							e.detail= SWT.TRAVERSE_NONE;
+							break;
+
+						case SWT.TRAVERSE_TAB_PREVIOUS :
+							editColumnOrPrevPossible(prevColumn(editorColumn));
+							e.detail= SWT.TRAVERSE_NONE;
+							break;
+						
+						default :
+							break;
+					}
+				}
+			});
+		}
+		
+		editors[NEWNAME_PROP].setActivationListener(new TableTextCellEditor.IActivationListener(){
+			public void activate() {
+				ParameterInfo[] selected= getSelectedElements();
+				if (selected.length == 1 && fNameContentAssistHandler != null) {
+					fNameContentAssistHandler.setEnabled(selected[0].isAdded());
+				}
+			}
+		});
+		
+		fTableViewer.setCellEditors(editors);
 		fTableViewer.setCellModifier(new ParametersCellModifier());
 	}
 
+	private SubjectControlContentAssistant installParameterTypeContentAssist(Text text) {
+		JavaTypeCompletionProcessor processor= new JavaTypeCompletionProcessor(true, false);
+		if (fTypeContext == null)
+			processor.setCompletionContext(null, null, null);
+		else
+			processor.setCompletionContext(fTypeContext.getCuHandle(), fTypeContext.getBeforeString(), fTypeContext.getAfterString());
+		SubjectControlContentAssistant contentAssistant= ControlContentAssistHelper.createJavaContentAssistant(processor);
+		ContentAssistHandler.createHandlerForText(text, contentAssistant);
+		return contentAssistant;
+	}
+
+	private SubjectControlContentAssistant installParameterNameContentAssist(Text text) {
+		VariableNamesProcessor processor= new VariableNamesProcessor(fParamNameProposals);
+		SubjectControlContentAssistant contentAssistant= ControlContentAssistHelper.createJavaContentAssistant(processor);
+		fNameContentAssistHandler= ContentAssistHandler.createHandlerForText(text, contentAssistant);
+		return contentAssistant;
+	}
+
 	//---- change order ----------------------------------------------------------------------------------------
 
 	private void moveUp(ParameterInfo[] selection) {
@@ -620,11 +645,14 @@ public class ChangeParametersControl extends Composite {
 
 	private static void moveUp(List elements, List move) {
 		List res= new ArrayList(elements.size());
+		List deleted= new ArrayList();
 		Object floating= null;
 		for (Iterator iter= elements.iterator(); iter.hasNext();) {
 			Object curr= iter.next();
 			if (move.contains(curr)) {
 				res.add(curr);
+			} else if (((ParameterInfo) curr).isDeleted()) {
+				deleted.add(curr);
 			} else {
 				if (floating != null)
 					res.add(floating);
@@ -634,6 +662,7 @@ public class ChangeParametersControl extends Composite {
 		if (floating != null) {
 			res.add(floating);
 		}
+		res.addAll(deleted);
 		elements.clear();
 		for (Iterator iter= res.iterator(); iter.hasNext();) {
 			elements.add(iter.next());
@@ -641,13 +670,13 @@ public class ChangeParametersControl extends Composite {
 	}
 
 	private boolean canMove(boolean up) {
-		List notDeleted= getNotDeletedInfos();
-		if (notDeleted == null || notDeleted.size() == 0)
+		int notDeletedInfosCount= getNotDeletedInfosCount();
+		if (notDeletedInfosCount == 0)
 			return false;
 		int[] indc= getTable().getSelectionIndices();
 		if (indc.length == 0)
 			return false;
-		int invalid= up ? 0 : notDeleted.size() - 1;
+		int invalid= up ? 0 : notDeletedInfosCount - 1;
 		for (int i= 0; i < indc.length; i++) {
 			if (indc[i] == invalid)
 				return false;
@@ -655,14 +684,14 @@ public class ChangeParametersControl extends Composite {
 		return true;
 	}
 	
-	private List getNotDeletedInfos(){
-		if (fParameterInfos == null)
-			return null;
-		List result= new ArrayList(fParameterInfos.size());
+	private int getNotDeletedInfosCount(){
+		if (fParameterInfos == null) // during initialization
+			return 0;
+		int result= 0;
 		for (Iterator iter= fParameterInfos.iterator(); iter.hasNext();) {
 			ParameterInfo info= (ParameterInfo) iter.next();
 			if (! info.isDeleted())
-				result.add(info);
+				result++;
 		}
 		return result;
 	}
