@@ -1,16 +1,16 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
 package org.eclipse.pde.internal.core;
 
-import org.eclipse.core.resources.IProject;
+import org.eclipse.core.resources.*;
 import org.eclipse.core.runtime.*;
 import org.eclipse.jdt.core.*;
 
@@ -37,7 +37,7 @@ public class RequiredPluginsInitializer extends ClasspathContainerInitializer {
 		if (entry == null) {
 			ModelEntry.updateUnknownClasspathContainer(javaProject);
 		} else {
-			entry.updateClasspathContainer(true);
+			entry.updateClasspathContainer(true, false);
 		}
 	}
-}
\ No newline at end of file
+}
