@@ -1,18 +1,18 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2004 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
 package org.eclipse.pde.internal.core.plugin;
 
-import org.eclipse.core.runtime.PlatformObject;
-import org.eclipse.pde.core.plugin.IPlugin;
-import org.eclipse.pde.internal.core.PDECore;
+import org.eclipse.core.runtime.*;
+import org.eclipse.pde.core.plugin.*;
+import org.eclipse.pde.internal.core.*;
 
 public class PluginReference extends PlatformObject {
 	private String id;
@@ -41,7 +41,7 @@ public class PluginReference extends PlatformObject {
 		if (plugin!=null) {
 			return plugin.getTranslatedName();
 		}
-		return id!=null?id:"?";
+		return id!=null?id:"?"; //$NON-NLS-1$
 	}
 	public boolean isResolved() {
 		return plugin!=null;
