@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2004 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -26,7 +26,7 @@ public class SystemFileEditorInputFactory implements IElementFactory {
 	}
 	
 	public IAdaptable createElement (IMemento memento) {
-		String path = memento.getString("path");
+		String path = memento.getString("path"); //$NON-NLS-1$
 		File file = new File(path);
 		SystemFileEditorInput input = new SystemFileEditorInput(file);
 		return input;
