@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -43,7 +43,9 @@ public class Accessible {
 	int refCount = 0, enumIndex = 0;
 	COMObject objIAccessible, objIEnumVARIANT;
 	IAccessible iaccessible;
-	Vector accessibleListeners = new Vector(), accessibleControlListeners = new Vector();
+	Vector accessibleListeners = new Vector();
+	Vector accessibleControlListeners = new Vector();
+	Vector textListeners = new Vector ();
 	Object[] variants;
 	Control control;
 
@@ -51,7 +53,7 @@ public class Accessible {
 		this.control = control;
 		int[] ppvObject = new int[1];
 		int result = COM.CreateStdAccessibleObject(control.handle, COM.OBJID_CLIENT, COM.IIDIAccessible, ppvObject);
-		if (result == COM.E_NOTIMPL) return;
+		if (ppvObject[0] == 0) return;
 		if (result != COM.S_OK) OLE.error(OLE.ERROR_CANNOT_CREATE_OBJECT, result);
 		iaccessible = new IAccessible(ppvObject[0]);
 		iaccessible.AddRef();
@@ -118,7 +120,7 @@ public class Accessible {
 
 	/**
 	 * Adds the listener to the collection of listeners who will
-	 * be notifed when an accessible client asks for certain strings,
+	 * be notified when an accessible client asks for certain strings,
 	 * such as name, description, help, or keyboard shortcut. The
 	 * listener is notified by sending it one of the messages defined
 	 * in the <code>AccessibleListener</code> interface.
@@ -143,34 +145,10 @@ public class Accessible {
 		accessibleListeners.addElement(listener);
 	}
 	
-	/**
-	 * Removes the listener from the collection of listeners who will
-	 * be notifed when an accessible client asks for certain strings,
-	 * such as name, description, help, or keyboard shortcut.
-	 *
-	 * @param listener the listener that should no longer be notified when the receiver
-	 * is asked for a name, description, help, or keyboard shortcut string
-	 *
-	 * @exception IllegalArgumentException <ul>
-	 *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
-	 * </ul>
-	 * @exception SWTException <ul>
-	 *    <li>ERROR_WIDGET_DISPOSED - if the receiver's control has been disposed</li>
-	 *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver's control</li>
-	 * </ul>
-	 *
-	 * @see AccessibleListener
-	 * @see #addAccessibleListener
-	 */
-	public void removeAccessibleListener(AccessibleListener listener) {
-		checkWidget();
-		if (listener == null) SWT.error(SWT.ERROR_NULL_ARGUMENT);
-		accessibleListeners.removeElement(listener);
-	}
 	
 	/**
 	 * Adds the listener to the collection of listeners who will
-	 * be notifed when an accessible client asks for custom control
+	 * be notified when an accessible client asks for custom control
 	 * specific information. The listener is notified by sending it
 	 * one of the messages defined in the <code>AccessibleControlListener</code>
 	 * interface.
@@ -196,12 +174,14 @@ public class Accessible {
 	}
 
 	/**
-	 * Removes the listener from the collection of listeners who will
-	 * be notifed when an accessible client asks for custom control
-	 * specific information.
+	 * Adds the listener to the collection of listeners who will
+	 * be notified when an accessible client asks for custom text control
+	 * specific information. The listener is notified by sending it
+	 * one of the messages defined in the <code>AccessibleTextListener</code>
+	 * interface.
 	 *
-	 * @param listener the listener that should no longer be notified when the receiver
-	 * is asked for custom control specific information
+	 * @param listener the listener that should be notified when the receiver
+	 * is asked for custom text control specific information
 	 *
 	 * @exception IllegalArgumentException <ul>
 	 *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
@@ -211,33 +191,25 @@ public class Accessible {
 	 *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver's control</li>
 	 * </ul>
 	 *
-	 * @see AccessibleControlListener
-	 * @see #addAccessibleControlListener
+	 * @see AccessibleTextListener
+	 * @see #removeAccessibleTextListener
+	 * 
+	 * @since 3.0
 	 */
-	public void removeAccessibleControlListener(AccessibleControlListener listener) {
-		checkWidget();
-		if (listener == null) SWT.error(SWT.ERROR_NULL_ARGUMENT);
-		accessibleControlListeners.removeElement(listener);
+	public void addAccessibleTextListener (AccessibleTextListener listener) {
+		checkWidget ();
+		if (listener == null) SWT.error (SWT.ERROR_NULL_ARGUMENT);
+		textListeners.addElement (listener);		
 	}
 	
 	/**
-	 * Sends a message to accessible clients indicating that the focus
-	 * has changed within a custom control.
+	 * Returns the control for this Accessible object. 
 	 *
-	 * @param childID an identifier specifying a child of the control
-	 * 
-	 * @exception SWTException <ul>
-	 *    <li>ERROR_WIDGET_DISPOSED - if the receiver's control has been disposed</li>
-	 *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver's control</li>
-	 * </ul>
+	 * @return the receiver's control
+	 * @since 3.0
 	 */
-	public void setFocus(int childID) {
-		checkWidget();
-		int id;
-		if (childID == ACC.CHILDID_SELF) id = COM.CHILDID_SELF;
-		else if (control instanceof Tree) id = childID; // Tree item childIDs are pointers
-		else id = childID + 1; // All other childIDs are 1-based indices
-		COM.NotifyWinEvent (COM.EVENT_OBJECT_FOCUS, control.handle, COM.OBJID_CLIENT, id);
+	public Control getControl() {
+		return control;
 	}
 
 	/**
@@ -275,6 +247,173 @@ public class Accessible {
 		return 0;
 	}
 
+	/**
+	 * Removes the listener from the collection of listeners who will
+	 * be notified when an accessible client asks for certain strings,
+	 * such as name, description, help, or keyboard shortcut.
+	 *
+	 * @param listener the listener that should no longer be notified when the receiver
+	 * is asked for a name, description, help, or keyboard shortcut string
+	 *
+	 * @exception IllegalArgumentException <ul>
+	 *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
+	 * </ul>
+	 * @exception SWTException <ul>
+	 *    <li>ERROR_WIDGET_DISPOSED - if the receiver's control has been disposed</li>
+	 *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver's control</li>
+	 * </ul>
+	 *
+	 * @see AccessibleListener
+	 * @see #addAccessibleListener
+	 */
+	public void removeAccessibleListener(AccessibleListener listener) {
+		checkWidget();
+		if (listener == null) SWT.error(SWT.ERROR_NULL_ARGUMENT);
+		accessibleListeners.removeElement(listener);
+	}
+
+	/**
+	 * Removes the listener from the collection of listeners who will
+	 * be notified when an accessible client asks for custom control
+	 * specific information.
+	 *
+	 * @param listener the listener that should no longer be notified when the receiver
+	 * is asked for custom control specific information
+	 *
+	 * @exception IllegalArgumentException <ul>
+	 *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
+	 * </ul>
+	 * @exception SWTException <ul>
+	 *    <li>ERROR_WIDGET_DISPOSED - if the receiver's control has been disposed</li>
+	 *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver's control</li>
+	 * </ul>
+	 *
+	 * @see AccessibleControlListener
+	 * @see #addAccessibleControlListener
+	 */
+	public void removeAccessibleControlListener(AccessibleControlListener listener) {
+		checkWidget();
+		if (listener == null) SWT.error(SWT.ERROR_NULL_ARGUMENT);
+		accessibleControlListeners.removeElement(listener);
+	}
+
+	/**
+	 * Removes the listener from the collection of listeners who will
+	 * be notified when an accessible client asks for custom text control
+	 * specific information.
+	 *
+	 * @param listener the listener that should no longer be notified when the receiver
+	 * is asked for custom text control specific information
+	 *
+	 * @exception IllegalArgumentException <ul>
+	 *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
+	 * </ul>
+	 * @exception SWTException <ul>
+	 *    <li>ERROR_WIDGET_DISPOSED - if the receiver's control has been disposed</li>
+	 *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver's control</li>
+	 * </ul>
+	 *
+	 * @see AccessibleTextListener
+	 * @see #addAccessibleTextListener
+	 * 
+	 * @since 3.0
+	 */
+	public void removeAccessibleTextListener (AccessibleTextListener listener) {
+		checkWidget ();
+		if (listener == null) SWT.error (SWT.ERROR_NULL_ARGUMENT);
+		textListeners.removeElement (listener);
+	}
+
+	/**
+	 * Sends a message to accessible clients that the child selection
+	 * within a custom container control has changed.
+	 *
+	 * @exception SWTException <ul>
+	 *    <li>ERROR_WIDGET_DISPOSED - if the receiver's control has been disposed</li>
+	 *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver's control</li>
+	 * </ul>
+	 * 
+	 * @since 3.0
+	 */
+	public void selectionChanged () {
+		checkWidget();
+		COM.NotifyWinEvent (COM.EVENT_OBJECT_SELECTIONWITHIN, control.handle, COM.OBJID_CLIENT, COM.CHILDID_SELF);
+	}
+
+	/**
+	 * Sends a message to accessible clients indicating that the focus
+	 * has changed within a custom control.
+	 *
+	 * @param childID an identifier specifying a child of the control
+	 * 
+	 * @exception SWTException <ul>
+	 *    <li>ERROR_WIDGET_DISPOSED - if the receiver's control has been disposed</li>
+	 *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver's control</li>
+	 * </ul>
+	 */
+	public void setFocus(int childID) {
+		checkWidget();
+		COM.NotifyWinEvent (COM.EVENT_OBJECT_FOCUS, control.handle, COM.OBJID_CLIENT, childIDToOs(childID));
+	}
+
+	/**
+	 * Sends a message to accessible clients that the text
+	 * caret has moved within a custom control.
+	 *
+	 * @param index the new caret index within the control
+	 * 
+	 * @exception SWTException <ul>
+	 *    <li>ERROR_WIDGET_DISPOSED - if the receiver's control has been disposed</li>
+	 *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver's control</li>
+	 * </ul>
+	 *
+	 * @since 3.0
+	 */
+	public void textCaretMoved (int index) {
+		checkWidget();
+		COM.NotifyWinEvent (COM.EVENT_OBJECT_LOCATIONCHANGE, control.handle, COM.OBJID_CARET, COM.CHILDID_SELF);
+	}
+	
+	/**
+	 * Sends a message to accessible clients that the text
+	 * within a custom control has changed.
+	 *
+	 * @param type the type of change, one of <code>ACC.NOTIFY_TEXT_INSERT</code>
+	 * or <code>ACC.NOTIFY_TEXT_DELETE</code>
+	 * @param startIndex the text index within the control where the insertion or deletion begins
+	 * @param length the non-negative length in characters of the insertion or deletion
+	 *
+	 * @exception SWTException <ul>
+	 *    <li>ERROR_WIDGET_DISPOSED - if the receiver's control has been disposed</li>
+	 *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver's control</li>
+	 * </ul>
+	 * 
+	 * @see ACC#TEXT_INSERT
+	 * @see ACC#TEXT_DELETE
+	 * 
+	 * @since 3.0
+	 */
+	public void textChanged (int type, int startIndex, int length) {
+		checkWidget();
+		COM.NotifyWinEvent (COM.EVENT_OBJECT_VALUECHANGE, control.handle, COM.OBJID_CLIENT, COM.CHILDID_SELF);
+	}
+	
+	/**
+	 * Sends a message to accessible clients that the text
+	 * selection has changed within a custom control.
+	 *
+	 * @exception SWTException <ul>
+	 *    <li>ERROR_WIDGET_DISPOSED - if the receiver's control has been disposed</li>
+	 *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver's control</li>
+	 * </ul>
+	 *
+	 * @since 3.0
+	 */
+	public void textSelectionChanged () {
+		checkWidget();
+		// not an MSAA event
+	}
+	
 	int QueryInterface(int arg1, int arg2) {
 		if (iaccessible == null) return COM.CO_E_OBJNOTCONNECTED;
 		GUID guid = new GUID();
@@ -357,10 +496,8 @@ public class Accessible {
 		if (childID == ACC.CHILDID_NONE) {
 			return iaccessible.accHitTest(xLeft, yTop, pvarChild);
 		}
-		if (childID == ACC.CHILDID_SELF) childID = COM.CHILDID_SELF;
-		else if (!(control instanceof Tree)) childID++; // Tree item childIDs are pointers (not 1-based indices)
 		COM.MoveMemory(pvarChild, new short[] { COM.VT_I4 }, 2);
-		COM.MoveMemory(pvarChild + 8, new int[] { childID }, 4);
+		COM.MoveMemory(pvarChild + 8, new int[] { childIDToOs(childID) }, 4);
 		return COM.S_OK;
 	}
 	
@@ -383,9 +520,7 @@ public class Accessible {
 		}
 
 		AccessibleControlEvent event = new AccessibleControlEvent(this);
-		if (varChild_lVal == COM.CHILDID_SELF) event.childID = ACC.CHILDID_SELF;
-		else if (control instanceof Tree) event.childID = varChild_lVal; // Tree item childIDs are pointers
-		else event.childID = varChild_lVal - 1; // All other childIDs are 1-based indices
+		event.childID = osToChildID(varChild_lVal);
 		event.x = osLeft;
 		event.y = osTop;
 		event.width = osWidth;
@@ -427,9 +562,7 @@ public class Accessible {
 		}
 
 		AccessibleControlEvent event = new AccessibleControlEvent(this);
-		if (varChild_lVal == COM.CHILDID_SELF) event.childID = ACC.CHILDID_SELF;
-		else if (control instanceof Tree) event.childID = varChild_lVal; // Tree item childIDs are pointers
-		else event.childID = varChild_lVal - 1; // All other childIDs are 1-based indices
+		event.childID = osToChildID(varChild_lVal);
 		for (int i = 0; i < accessibleControlListeners.size(); i++) {
 			AccessibleControlListener listener = (AccessibleControlListener) accessibleControlListeners.elementAt(i);
 			listener.getChild(event);
@@ -488,9 +621,7 @@ public class Accessible {
 		}
 
 		AccessibleControlEvent event = new AccessibleControlEvent(this);
-		if (varChild_lVal == COM.CHILDID_SELF) event.childID = ACC.CHILDID_SELF;
-		else if (control instanceof Tree) event.childID = varChild_lVal; // Tree item childIDs are pointers
-		else event.childID = varChild_lVal - 1; // All other childIDs are 1-based indices
+		event.childID = osToChildID(varChild_lVal);
 		event.result = osDefaultAction;
 		for (int i = 0; i < accessibleControlListeners.size(); i++) {
 			AccessibleControlListener listener = (AccessibleControlListener) accessibleControlListeners.elementAt(i);
@@ -511,7 +642,8 @@ public class Accessible {
 		String osDescription = null;
 		int code = iaccessible.get_accDescription(varChild_vt, varChild_reserved1, varChild_lVal, varChild_reserved2, pszDescription);
 		if (code == COM.E_INVALIDARG) code = COM.S_FALSE; // proxy doesn't know about app childID
-		if (accessibleListeners.size() == 0) return code;
+		// TEMPORARY CODE - process tree even if there are no apps listening
+		if (accessibleListeners.size() == 0 && !(control instanceof Tree)) return code;
 		if (code == COM.S_OK) {
 			int[] pDescription = new int[1];
 			COM.MoveMemory(pDescription, pszDescription, 4);
@@ -524,10 +656,36 @@ public class Accessible {
 		}
 		
 		AccessibleEvent event = new AccessibleEvent(this);
-		if (varChild_lVal == COM.CHILDID_SELF) event.childID = ACC.CHILDID_SELF;
-		else if (control instanceof Tree) event.childID = varChild_lVal; // Tree item childIDs are pointers
-		else event.childID = varChild_lVal - 1; // All other childIDs are 1-based indices
+		event.childID = osToChildID(varChild_lVal);
 		event.result = osDescription;
+		
+		// TEMPORARY CODE
+		/* Currently our tree columns are emulated using custom draw,
+		 * so we need to create the description using the tree column
+		 * header text and tree item text. */
+		if (varChild_lVal != COM.CHILDID_SELF) {
+			if (control instanceof Tree) {
+				Tree tree = (Tree) control;
+				int columnCount = tree.getColumnCount ();
+				if (columnCount > 1) {
+					int hwnd = control.handle, hItem = 0;
+					if (OS.COMCTL32_MAJOR >= 6) {
+						hItem = OS.SendMessage (hwnd, OS.TVM_MAPACCIDTOHTREEITEM, varChild_lVal, 0);
+					} else {
+						hItem = varChild_lVal;
+					}
+					Widget widget = tree.getDisplay ().findWidget (hwnd, hItem);
+					event.result = "";
+					if (widget != null && widget instanceof TreeItem) {
+						TreeItem item = (TreeItem) widget;
+						for (int i = 1; i < columnCount; i++) {
+							event.result += tree.getColumn(i).getText() + ": " + item.getText(i);
+							if (i + 1 < columnCount) event.result += ", ";
+						}
+					}
+				}
+			}
+		}
 		for (int i = 0; i < accessibleListeners.size(); i++) {
 			AccessibleListener listener = (AccessibleListener) accessibleListeners.elementAt(i);
 			listener.getDescription(event);
@@ -552,7 +710,7 @@ public class Accessible {
 			if (pvt[0] == COM.VT_I4) {
 				int[] pChild = new int[1];
 				COM.MoveMemory(pChild, pvarChild + 8, 4);
-				osChild = (pChild[0] == COM.CHILDID_SELF) ? ACC.CHILDID_SELF : pChild[0] - 1;
+				osChild = osToChildID(pChild[0]);
 			}
 		}
 
@@ -579,7 +737,7 @@ public class Accessible {
 			return COM.S_OK;
 		}
 		COM.MoveMemory(pvarChild, new short[] { COM.VT_I4 }, 2);
-		COM.MoveMemory(pvarChild + 8, new int[] { childID + 1 }, 4);
+		COM.MoveMemory(pvarChild + 8, new int[] { childIDToOs(childID) }, 4);
 		return COM.S_OK;
 	}
 	
@@ -604,9 +762,7 @@ public class Accessible {
 		}
 
 		AccessibleEvent event = new AccessibleEvent(this);
-		if (varChild_lVal == COM.CHILDID_SELF) event.childID = ACC.CHILDID_SELF;
-		else if (control instanceof Tree) event.childID = varChild_lVal; // Tree item childIDs are pointers
-		else event.childID = varChild_lVal - 1; // All other childIDs are 1-based indices
+		event.childID = osToChildID(varChild_lVal);
 		event.result = osHelp;
 		for (int i = 0; i < accessibleListeners.size(); i++) {
 			AccessibleListener listener = (AccessibleListener) accessibleListeners.elementAt(i);
@@ -648,9 +804,7 @@ public class Accessible {
 		}
 
 		AccessibleEvent event = new AccessibleEvent(this);
-		if (varChild_lVal == COM.CHILDID_SELF) event.childID = ACC.CHILDID_SELF;
-		else if (control instanceof Tree) event.childID = varChild_lVal; // Tree item childIDs are pointers
-		else event.childID = varChild_lVal - 1; // All other childIDs are 1-based indices
+		event.childID = osToChildID(varChild_lVal);
 		event.result = osKeyboardShortcut;
 		for (int i = 0; i < accessibleListeners.size(); i++) {
 			AccessibleListener listener = (AccessibleListener) accessibleListeners.elementAt(i);
@@ -684,9 +838,7 @@ public class Accessible {
 		}
 
 		AccessibleEvent event = new AccessibleEvent(this);
-		if (varChild_lVal == COM.CHILDID_SELF) event.childID = ACC.CHILDID_SELF;
-		else if (control instanceof Tree) event.childID = varChild_lVal; // Tree item childIDs are pointers
-		else event.childID = varChild_lVal - 1; // All other childIDs are 1-based indices
+		event.childID = osToChildID(varChild_lVal);
 		event.result = osName;
 		for (int i = 0; i < accessibleListeners.size(); i++) {
 			AccessibleListener listener = (AccessibleListener) accessibleListeners.elementAt(i);
@@ -726,27 +878,14 @@ public class Accessible {
 		}
 
 		AccessibleControlEvent event = new AccessibleControlEvent(this);
+		event.childID = osToChildID(varChild_lVal);
 		event.detail = osToRole(osRole);
-		if (varChild_lVal == COM.CHILDID_SELF) {
-			event.childID = ACC.CHILDID_SELF;
-		} else {
-			if (control instanceof Tree) {
-				/* Tree item childIDs are pointers (not 1-based indices). */
-				event.childID = varChild_lVal;
-				
-				// TEMPORARY CODE
-				/* Currently our checkbox tree is emulated using state mask images,
-				 * so we need to specify 'checkbox' role for the items here. */
+		// TEMPORARY CODE
+		/* Currently our checkbox table and tree are emulated using state mask
+		 * images, so we need to specify 'checkbox' role for the items. */
+		if (varChild_lVal != COM.CHILDID_SELF) {
+			if (control instanceof Tree || control instanceof Table) {
 				if ((control.getStyle() & SWT.CHECK) != 0) event.detail = ACC.ROLE_CHECKBUTTON;
-			} else if (control instanceof Table) {
-				event.childID = varChild_lVal - 1;
-				
-				// TEMPORARY CODE
-				/* Currently our checkbox table is emulated using state mask images,
-				 * so we need to specify 'checkbox' role for the items here. */
-				if ((control.getStyle() & SWT.CHECK) != 0) event.detail = ACC.ROLE_CHECKBUTTON;
-			} else {
-				event.childID = varChild_lVal - 1;
 			}
 		}
 		for (int i = 0; i < accessibleControlListeners.size(); i++) {
@@ -772,7 +911,7 @@ public class Accessible {
 			if (pvt[0] == COM.VT_I4) {
 				int[] pChild = new int[1];
 				COM.MoveMemory(pChild, pvarChildren + 8, 4);
-				osChild = (pChild[0] == COM.CHILDID_SELF) ? ACC.CHILDID_SELF : pChild[0] - 1;
+				osChild = osToChildID(pChild[0]);
 			} else if (pvt[0] == COM.VT_UNKNOWN) {
 				osChild = ACC.CHILDID_MULTIPLE;
 				/* Should get IEnumVARIANT from punkVal field... need better API here... */
@@ -808,7 +947,7 @@ public class Accessible {
 			return COM.S_OK;
 		}
 		COM.MoveMemory(pvarChildren, new short[] { COM.VT_I4 }, 2);
-		COM.MoveMemory(pvarChildren + 8, new int[] { childID + 1 }, 4);
+		COM.MoveMemory(pvarChildren + 8, new int[] { childIDToOs(childID) }, 4);
 		return COM.S_OK;
 	}
 	
@@ -833,38 +972,31 @@ public class Accessible {
 		}
 
 		AccessibleControlEvent event = new AccessibleControlEvent(this);
+		event.childID = osToChildID(varChild_lVal);
 		event.detail = osToState(osState);
-		if (varChild_lVal == COM.CHILDID_SELF) {
-			event.childID = ACC.CHILDID_SELF;
-		} else {
-			if (control instanceof Tree) {
-				/* Tree item childIDs are pointers (not 1-based indices). */
-				event.childID = varChild_lVal;
-
-				// TEMPORARY CODE
-				/* Currently our checkbox tree is emulated using state mask images,
-				 * so we need to determine if the item is 'checked' here. */
+		// TEMPORARY CODE
+		/* Currently our checkbox table and tree are emulated using state mask
+		 * images, so we need to determine if the item state is 'checked'. */
+		if (varChild_lVal != COM.CHILDID_SELF) {
+			if (control instanceof Tree && (control.getStyle() & SWT.CHECK) != 0) {
 				int hwnd = control.handle;
 				TVITEM tvItem = new TVITEM ();
 				tvItem.mask = OS.TVIF_HANDLE | OS.TVIF_STATE;
-				tvItem.hItem = varChild_lVal;
 				tvItem.stateMask = OS.TVIS_STATEIMAGEMASK;
+				if (OS.COMCTL32_MAJOR >= 6) {
+					tvItem.hItem = OS.SendMessage (hwnd, OS.TVM_MAPACCIDTOHTREEITEM, varChild_lVal, 0);
+				} else {
+					tvItem.hItem = varChild_lVal;
+				}
 				int result = OS.SendMessage (hwnd, OS.TVM_GETITEM, 0, tvItem);
 				boolean checked = (result != 0) && (((tvItem.state >> 12) & 1) == 0);
 				if (checked) event.detail |= ACC.STATE_CHECKED;
-			} else if (control instanceof Table) {
-				event.childID = varChild_lVal - 1;
-				
-				// TEMPORARY CODE
-				/* Currently our checkbox table is emulated using state mask images,
-				 * so we need to determine if the item is 'checked' here. */
+			} else if (control instanceof Table && (control.getStyle() & SWT.CHECK) != 0) {
 				Table table = (Table) control;
 				TableItem item = table.getItem(event.childID);
 				if (item != null) {
 					if (item.getChecked()) event.detail |= ACC.STATE_CHECKED;
 				}
-			} else {
-				event.childID = varChild_lVal - 1;
 			}
 		}
 		for (int i = 0; i < accessibleControlListeners.size(); i++) {
@@ -898,9 +1030,7 @@ public class Accessible {
 		}
 
 		AccessibleControlEvent event = new AccessibleControlEvent(this);
-		if (varChild_lVal == COM.CHILDID_SELF) event.childID = ACC.CHILDID_SELF;
-		else if (control instanceof Tree) event.childID = varChild_lVal; // Tree item childIDs are pointers
-		else event.childID = varChild_lVal - 1; // All other childIDs are 1-based indices
+		event.childID = osToChildID(varChild_lVal);
 		event.result = osValue;
 		for (int i = 0; i < accessibleControlListeners.size(); i++) {
 			AccessibleControlListener listener = (AccessibleControlListener) accessibleControlListeners.elementAt(i);
@@ -973,7 +1103,7 @@ public class Accessible {
 				for (int i = 0; i < nextItems.length; i++) {
 					Object child = variants[enumIndex];
 					if (child instanceof Integer) {
-						nextItems[i] = new Integer(((Integer)child).intValue() + 1);
+						nextItems[i] = new Integer(childIDToOs(((Integer)child).intValue()));
 					} else {
 						nextItems[i] = child;
 					}
@@ -1045,6 +1175,34 @@ public class Accessible {
 		return COM.S_OK;
 	}
 	
+	int childIDToOs(int childID) {
+		if (childID == ACC.CHILDID_SELF) return COM.CHILDID_SELF;
+		/*
+		* Feature of Windows:
+		* In Windows XP, tree item ids are 1-based indices. Previous versions
+		* of Windows use the tree item handle for the accessible child ID.
+		* For backward compatibility, we still take a handle childID for tree
+		* items on XP. All other childIDs are 1-based indices.
+		*/
+		if (!(control instanceof Tree)) return childID + 1;
+		if (OS.COMCTL32_MAJOR < 6) return childID;
+		return OS.SendMessage (control.handle, OS.TVM_MAPHTREEITEMTOACCID, childID, 0);
+	}
+
+	int osToChildID(int osChildID) {
+		if (osChildID == COM.CHILDID_SELF) return ACC.CHILDID_SELF;
+		/*
+		* Feature of Windows:
+		* In Windows XP, tree item ids are 1-based indices. Previous versions
+		* of Windows use the tree item handle for the accessible child ID.
+		* For backward compatibility, we still take a handle childID for tree
+		* items on XP. All other childIDs are 1-based indices.
+		*/
+		if (!(control instanceof Tree)) return osChildID - 1;
+		if (OS.COMCTL32_MAJOR < 6) return osChildID;
+		return OS.SendMessage (control.handle, OS.TVM_MAPACCIDTOHTREEITEM, osChildID, 0);
+	}
+	
 	int stateToOs(int state) {
 		int osState = 0;
 		if ((state & ACC.STATE_SELECTED) != 0) osState |= COM.STATE_SYSTEM_SELECTED;
@@ -1062,6 +1220,7 @@ public class Accessible {
 		if ((state & ACC.STATE_INVISIBLE) != 0) osState |= COM.STATE_SYSTEM_INVISIBLE;
 		if ((state & ACC.STATE_OFFSCREEN) != 0) osState |= COM.STATE_SYSTEM_OFFSCREEN;
 		if ((state & ACC.STATE_SIZEABLE) != 0) osState |= COM.STATE_SYSTEM_SIZEABLE;
+		if ((state & ACC.STATE_LINKED) != 0) osState |= COM.STATE_SYSTEM_LINKED;
 		return osState;
 	}
 	
@@ -1082,6 +1241,7 @@ public class Accessible {
 		if ((osState & COM.STATE_SYSTEM_INVISIBLE) != 0) state |= ACC.STATE_INVISIBLE;
 		if ((osState & COM.STATE_SYSTEM_OFFSCREEN) != 0) state |= ACC.STATE_OFFSCREEN;
 		if ((osState & COM.STATE_SYSTEM_SIZEABLE) != 0) state |= ACC.STATE_SIZEABLE;
+		if ((osState & COM.STATE_SYSTEM_LINKED) != 0) state |= ACC.STATE_LINKED;
 		return state;
 	}
 
@@ -1115,6 +1275,7 @@ public class Accessible {
 			case ACC.ROLE_TABITEM: return COM.ROLE_SYSTEM_PAGETAB;
 			case ACC.ROLE_PROGRESSBAR: return COM.ROLE_SYSTEM_PROGRESSBAR;
 			case ACC.ROLE_SLIDER: return COM.ROLE_SYSTEM_SLIDER;
+			case ACC.ROLE_LINK: return COM.ROLE_SYSTEM_LINK;
 		}
 		return COM.ROLE_SYSTEM_CLIENT;
 	}
@@ -1150,6 +1311,7 @@ public class Accessible {
 			case COM.ROLE_SYSTEM_PAGETAB: return ACC.ROLE_TABITEM;
 			case COM.ROLE_SYSTEM_PROGRESSBAR: return ACC.ROLE_PROGRESSBAR;
 			case COM.ROLE_SYSTEM_SLIDER: return ACC.ROLE_SLIDER;
+			case COM.ROLE_SYSTEM_LINK: return ACC.ROLE_LINK;
 		}
 		return role;
 	}
