@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -70,35 +70,47 @@ import org.eclipse.swt.internal.gtk.*;
  */
 public class DropTarget extends Widget {
 
-	static Callback DragMotion;
-	static Callback DragLeave;
-	static Callback DragDataReceived;
-	static Callback DragDrop;
-	static {	
-		DragMotion = new Callback(DropTarget.class, "DragMotion", 5);
-		DragLeave = new Callback(DropTarget.class, "DragLeave", 3);
-		DragDataReceived = new Callback(DropTarget.class, "DragDataReceived", 7);
-		DragDrop = new Callback(DropTarget.class, "DragDrop", 5);
-	}
-	private static final String DROPTARGETID = "DropTarget"; //$NON-NLS-1$
-	
-	// Track key state changes
-	int lastOperation = -1;
+	Control control;
+	Listener controlListener;
+	Transfer[] transferAgents = new Transfer[0];
+	DragUnderEffect effect;
 	
 	// Track application selections
 	TransferData selectedDataType;
 	int selectedOperation;
 	
-	DragUnderEffect effect;
-	Transfer[] transferAgents;
-	Listener controlListener;
-	Control control;
-
-	private static final int DRAGOVER_HYSTERESIS = 50;
-	private long dragOverStart;
-	private Runnable dragOverHeartbeat;
-	private DNDEvent dragOverEvent;
+	// workaround - There is no event for "operation changed" so track operation based on key state
+	int keyOperation = -1;
 	
+	// workaround - Simulate events when the mouse is not moving
+	long dragOverStart;
+	Runnable dragOverHeartbeat;
+	DNDEvent dragOverEvent;
+	
+	int drag_motion_handler;
+	int drag_leave_handler;
+	int drag_data_received_handler;
+	int drag_drop_handler;
+	
+	static final String DROPTARGETID = "DropTarget"; //$NON-NLS-1$
+	static final int DRAGOVER_HYSTERESIS = 50;
+	
+	static Callback Drag_Motion;
+	static Callback Drag_Leave;
+	static Callback Drag_Data_Received;
+	static Callback Drag_Drop;
+	
+	 static {	
+		Drag_Motion = new Callback(DropTarget.class, "Drag_Motion", 5); //$NON-NLS-1$
+		if (Drag_Motion.getAddress() == 0) SWT.error(SWT.ERROR_NO_MORE_CALLBACKS);
+		Drag_Leave = new Callback(DropTarget.class, "Drag_Leave", 3); //$NON-NLS-1$
+		if (Drag_Leave.getAddress() == 0) SWT.error(SWT.ERROR_NO_MORE_CALLBACKS);
+		Drag_Data_Received = new Callback(DropTarget.class, "Drag_Data_Received", 7); //$NON-NLS-1$
+		if (Drag_Data_Received.getAddress() == 0) SWT.error(SWT.ERROR_NO_MORE_CALLBACKS);
+		Drag_Drop = new Callback(DropTarget.class, "Drag_Drop", 5); //$NON-NLS-1$
+		if (Drag_Drop.getAddress() == 0) SWT.error(SWT.ERROR_NO_MORE_CALLBACKS);
+	}
+
 /**
  * Creates a new <code>DropTarget</code> to allow data to be dropped on the specified 
  * <code>Control</code>.
@@ -113,43 +125,52 @@ public class DropTarget extends Widget {
  * @exception SWTException <ul>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the parent</li>
  *    <li>ERROR_INVALID_SUBCLASS - if this class is not an allowed subclass</li>
+ * </ul>
  * @exception SWTError <ul>
  *    <li>ERROR_CANNOT_INIT_DROP - unable to initiate drop target; this will occur if more than one
  *        drop target is created for a control or if the operating system will not allow the creation
  *        of the drop target</li>
  * </ul>
+ *
+ * <p>NOTE: ERROR_CANNOT_INIT_DROP should be an SWTException, since it is a
+ * recoverable error, but can not be changed due to backward compatability.</p>
  * 
- * @see DropTarget#dispose
+ * @see Widget#dispose
  * @see DropTarget#checkSubclass
  * @see DND#DROP_NONE
  * @see DND#DROP_COPY
  * @see DND#DROP_MOVE
  * @see DND#DROP_LINK
- */	
+ */
 public DropTarget(Control control, int style) {
 	super(control, checkStyle(style));
 	this.control = control;
-	if (control.getData(DROPTARGETID) != null) DND.error(DND.ERROR_CANNOT_INIT_DROP);
+	if (Drag_Motion == null || Drag_Leave == null || Drag_Data_Received == null || Drag_Drop == null) {
+		 DND.error(DND.ERROR_CANNOT_INIT_DROP);
+	}
+	if (control.getData(DROPTARGETID) != null) {
+		DND.error(DND.ERROR_CANNOT_INIT_DROP);
+	}
 	control.setData(DROPTARGETID, this);
-	byte[] buffer = Converter.wcsToMbcs(null, "drag_motion", true);
-	OS.g_signal_connect(control.handle, buffer, DragMotion.getAddress(), 0);
-	buffer = Converter.wcsToMbcs(null, "drag_leave", true);
-	OS.g_signal_connect(control.handle, buffer, DragLeave.getAddress(), 0);
-	buffer = Converter.wcsToMbcs(null, "drag_data_received", true);
-	OS.g_signal_connect(control.handle, buffer, DragDataReceived.getAddress(), 0);
-	buffer = Converter.wcsToMbcs(null, "drag_drop", true);
-	OS.g_signal_connect(control.handle, buffer, DragDrop.getAddress(), 0);
+	
+	drag_motion_handler = OS.g_signal_connect(control.handle, OS.drag_motion, Drag_Motion.getAddress(), 0);
+	drag_leave_handler = OS.g_signal_connect(control.handle, OS.drag_leave, Drag_Leave.getAddress(), 0);
+	drag_data_received_handler = OS.g_signal_connect(control.handle, OS.drag_data_received, Drag_Data_Received.getAddress(), 0);
+	drag_drop_handler = OS.g_signal_connect(control.handle, OS.drag_drop, Drag_Drop.getAddress(), 0);
 
 	// Dispose listeners	
 	controlListener = new Listener(){
 		public void handleEvent(Event event){
-			DropTarget.this.dispose();
+			if (!DropTarget.this.isDisposed()){
+				DropTarget.this.dispose();
+			}
 		}
 	};
-	control.addListener(SWT.Dispose, controlListener);	
+	control.addListener(SWT.Dispose, controlListener);
+		
 	this.addListener(SWT.Dispose, new Listener(){
 		public void handleEvent(Event event){
-			DropTarget.this.onDispose();
+			onDispose();
 		}	
 	});
 
@@ -164,28 +185,33 @@ public DropTarget(Control control, int style) {
 
 	dragOverHeartbeat = new Runnable() {
 		public void run() {
-			if (DropTarget.this.control.isDisposed() || dragOverStart == 0) return;
+			Control control = DropTarget.this.control;
+			if (control == null || control.isDisposed() || dragOverStart == 0) return;
 			long time = System.currentTimeMillis();
 			int delay = DRAGOVER_HYSTERESIS;
-			if (time >= dragOverStart) {
-				if (selectedDataType == null) {
-					selectedDataType = dragOverEvent.dataTypes[0];
-				}
+			if (time < dragOverStart) {
+				delay = (int)(dragOverStart - time);
+			} else {	
+				int allowedOperations = dragOverEvent.operations;
+				TransferData[] allowedTypes = dragOverEvent.dataTypes;
+				//pass a copy of data types in to listeners in case application modifies it
+				TransferData[] dataTypes = new TransferData[allowedTypes.length];
+				System.arraycopy(allowedTypes, 0, dataTypes, 0, dataTypes.length);
+	
 				DNDEvent event = new DNDEvent();
-				event.widget = DropTarget.this.control;
-				event.time = (int)time;
+				event.widget = dragOverEvent.widget;
 				event.x = dragOverEvent.x;
 				event.y = dragOverEvent.y;
-				event.dataTypes  = dragOverEvent.dataTypes;
+				event.time = (int)time;
 				event.feedback = DND.FEEDBACK_SELECT;
-				int allowedOperations = dragOverEvent.operations;
-				TransferData[] allowedTypes = event.dataTypes;
-				event.operations = allowedOperations;
-				event.dataType  = selectedDataType;
+				event.dataTypes = dataTypes;
+				event.dataType = selectedDataType;
+				event.operations = dragOverEvent.operations;
 				event.detail  = selectedOperation;
+				selectedDataType = null;
+				selectedOperation = DND.DROP_NONE;
 				notifyListeners(DND.DragOver, event);
 				effect.show(event.feedback, event.x, event.y);
-				selectedDataType = null;
 				if (event.dataType != null) {
 					for (int i = 0; i < allowedTypes.length; i++) {
 						if (allowedTypes[i].type == event.dataType.type) {
@@ -194,46 +220,56 @@ public DropTarget(Control control, int style) {
 						}
 					}
 				}
-			
-				selectedOperation = DND.DROP_NONE;
 				if (selectedDataType != null && (event.detail & allowedOperations) != 0) {
 					selectedOperation = event.detail;
 				}
-			} else {
-				delay = (int)(dragOverStart - time);
 			}
-			DropTarget.this.control.getDisplay().timerExec(delay, dragOverHeartbeat);
+			control = DropTarget.this.control;
+			if (control == null || control.isDisposed()) return;
+			control.getDisplay().timerExec(delay, dragOverHeartbeat);
 		}
 	};
 }
 
-static DropTarget FindDropTarget(int handle) {
+static int checkStyle (int style) {
+	if (style == SWT.NONE) return DND.DROP_MOVE;	
+	return style;
+}
+
+static int /*long*/ Drag_Data_Received ( int /*long*/ widget, int /*long*/ context, int /*long*/ x, int /*long*/ y, int /*long*/ data, int /*long*/ info, int /*long*/ time){
+	DropTarget target = FindDropTarget(widget);
+	if (target == null) return 0;
+	target.drag_data_received (widget, context, (int)/*64*/x, (int)/*64*/y, data, (int)/*64*/info, (int)/*64*/time);
+	return 0;
+}
+
+static int /*long*/ Drag_Drop(int /*long*/ widget, int /*long*/ context, int /*long*/ x, int /*long*/ y, int /*long*/ time) {
+	DropTarget target = FindDropTarget(widget);
+	if (target == null) return 0;
+	return target.drag_drop (widget, context, (int)/*64*/x, (int)/*64*/y, (int)/*64*/time) ? 1 : 0;
+}
+
+static int /*long*/ Drag_Leave ( int /*long*/ widget, int /*long*/ context, int /*long*/ time){
+	DropTarget target = FindDropTarget(widget);
+	if (target == null) return 0;
+	target.drag_leave (widget, context, (int)/*64*/time);
+	return 0;
+}
+
+static int /*long*/ Drag_Motion ( int /*long*/ widget, int /*long*/ context, int /*long*/ x, int /*long*/ y, int /*long*/ time){
+	DropTarget target = FindDropTarget(widget);
+	if (target == null) return 0;
+	return target.drag_motion (widget, context, (int)/*64*/x, (int)/*64*/y, (int)/*64*/time) ? 1 : 0;
+}
+	
+static DropTarget FindDropTarget(int /*long*/ handle) {
 	Display display = Display.findDisplay(Thread.currentThread());
 	if (display == null || display.isDisposed()) return null;
 	Widget widget = display.findWidget(handle);
 	if (widget == null) return null;
 	return (DropTarget)widget.getData(DROPTARGETID);
 }
-static int DragDataReceived ( int widget, int context, int x, int y, int data, int info, int time){
-	DropTarget target = FindDropTarget(widget);
-	if (target == null) return 0;
-	return target.dragDataReceived (widget, context, x, y, data, info, time);
-}
-static int DragDrop(int widget, int context, int x, int y, int time) {
-	DropTarget target = FindDropTarget(widget);
-	if (target == null) return 0;
-	return target.dragDrop (widget, context, x, y, time);
-}
-static int DragLeave ( int widget, int context, int time){
-	DropTarget target = FindDropTarget(widget);
-	if (target == null) return 0;
-	return target.dragLeave (widget, context, time);
-}
-static int DragMotion ( int widget, int context, int x, int y, int time){
-	DropTarget target = FindDropTarget(widget);
-	if (target == null) return 0;
-	return target.dragMotion (widget, context, x, y, time);
-}
+
 /**
  * Adds the listener to the collection of listeners who will
  * be notified when a drag and drop operation is in progress, by sending
@@ -275,189 +311,158 @@ public void addDropListener(DropTargetListener listener) {
 	addListener (DND.DragOperationChanged, typedListener);
 	addListener (DND.Drop, typedListener);
 	addListener (DND.DropAccept, typedListener);	
-}
-
-static int checkStyle (int style) {
-	if (style == SWT.NONE) return DND.DROP_MOVE;	
-	return style;
 }	
 
-int dragDataReceived ( int widget, int context, int x, int y, int data, int info, int time){
+protected void checkSubclass () {
+	String name = getClass().getName ();
+	String validName = DropTarget.class.getName();
+	if (!validName.equals(name)) {
+		DND.error (SWT.ERROR_INVALID_SUBCLASS);
+	}
+}
+
+void drag_data_received ( int /*long*/ widget, int /*long*/ context, int x, int y, int /*long*/ data, int info, int time){
+	DNDEvent event = new DNDEvent();
+	if (data == 0 || !setEventData(context, x, y, time, event)) {
+		keyOperation = -1;
+		return;
+	}
+	keyOperation = -1;
+	
+	int allowedOperations = event.operations;
+	
 	// Get data in a Java format	
 	Object object = null;
 	TransferData transferData = new TransferData();
-	if (data != 0) {
-		GtkSelectionData selectionData = new GtkSelectionData(); 
-		OS.memmove(selectionData, data, GtkSelectionData.sizeof);
-		if (selectionData.data != 0) {
-			transferData.type = selectionData.type;
-			transferData.length = selectionData.length;
-			transferData.pValue = selectionData.data;
-			transferData.format = selectionData.format;
-			Transfer transfer = null;
-			for (int i = 0; i < transferAgents.length; i++) {
-				transfer = transferAgents[i];
-				if (transfer.isSupportedType(transferData)) break;
-			}
-			if (transfer != null) {
-				object = transfer.nativeToJava(transferData);
+	GtkSelectionData selectionData = new GtkSelectionData(); 
+	OS.memmove(selectionData, data, GtkSelectionData.sizeof);
+	if (selectionData.data != 0) {
+		transferData.type = selectionData.type;
+		transferData.length = selectionData.length;
+		transferData.pValue = selectionData.data;
+		transferData.format = selectionData.format;
+		for (int i = 0; i < transferAgents.length; i++) {
+			if (transferAgents[i].isSupportedType(transferData)) {
+				object = transferAgents[i].nativeToJava(transferData);
+				break;
 			}
 		}
 	}
+	if (object == null) {
+		selectedOperation = DND.DROP_NONE;
+	}
 	
-	//Notify listeners
-	DNDEvent event = new DNDEvent();
-	if (!setEventData(context, x, y, time, event)) return 0;
 	event.detail = selectedOperation;
 	event.dataType = transferData;
 	event.data = object;
-	try {
-		notifyListeners(DND.Drop, event);
-	} catch (Throwable e) {
-		event.detail = DND.DROP_NONE;
+	selectedOperation = DND.DROP_NONE;
+	notifyListeners(DND.Drop, event);
+	if ((allowedOperations & event.detail) == event.detail) {
+		selectedOperation = event.detail;
 	}
+	//notify source of action taken
+	OS.gtk_drag_finish(context, selectedOperation != DND.DROP_NONE, selectedOperation== DND.DROP_MOVE, time); 			
+	return;	
+}
+
+boolean drag_drop(int /*long*/ widget, int /*long*/ context, int x, int y, int time) {
+	DNDEvent event = new DNDEvent();
+	if (!setEventData(context, x, y, time, event)) {
+		keyOperation = -1;
+		return false;
+	}
+	keyOperation = -1;
 	
-	//Terminate DND and indicate if a move operation occurred
-	OS.gtk_drag_finish(context, event.detail != DND.DROP_NONE, event.detail == DND.DROP_MOVE, time); 			
-	return 1;	
-}
-
-int dragLeave ( int widget, int context, int time){
-	updateDragOverHover(0, null);
-	effect.show(DND.FEEDBACK_NONE, 0, 0);
-	lastOperation = -1;
-					
-	DNDEvent event = new DNDEvent();
-	event.widget = this;
-	event.time = (int)(long)time;
-	event.detail = DND.DROP_NONE;
-	try {
-		notifyListeners(DND.DragLeave, event);
-	} catch (Throwable e) {
-	}
-	return 1;
-		
-}
-
-int dragDrop(int widget, int context, int x, int y, int time) {
-	updateDragOverHover(0, null);
-	DNDEvent event = new DNDEvent();
-	if (!setEventData(context, x, y, time, event)) return 0;
 	int allowedOperations = event.operations;
-	TransferData[] allowedTypes = event.dataTypes;
+	TransferData[] allowedDataTypes = new TransferData[event.dataTypes.length];
+	System.arraycopy(event.dataTypes, 0, allowedDataTypes, 0, allowedDataTypes.length);
+	
 	event.dataType = selectedDataType;
 	event.detail = selectedOperation;
-	
-	try {
-		notifyListeners(DND.DropAccept,event);
-	} catch (Throwable err) {
-		event.detail = DND.DROP_NONE;
-		event.dataType = null;
-	}
-
 	selectedDataType = null;
+	selectedOperation = DND.DROP_NONE;
+	notifyListeners(DND.DropAccept,event);
 	if (event.dataType != null) {
-		for (int i = 0; i < allowedTypes.length; i++) {
-			if (allowedTypes[i].type == event.dataType.type) {
-				selectedDataType = event.dataType;
+		for (int i = 0; i < allowedDataTypes.length; i++) {
+			if (allowedDataTypes[i].type == event.dataType.type) {
+				selectedDataType = allowedDataTypes[i];
 				break;
 			}
 		}
 	}
-	selectedOperation = DND.DROP_NONE;
 	if (selectedDataType != null && ((event.detail & allowedOperations) == event.detail)) {
 		selectedOperation = event.detail;
 	}
-
 	if (selectedOperation == DND.DROP_NONE) {
 		// this was not a successful drop
-		return 0;
+		return false;
 	}
-
+	// ask drag source for dropped data
 	OS.gtk_drag_get_data(widget, context, selectedDataType.type, time);
-	return 1;
+	return true;
 }
 
-private void updateDragOverHover(long delay, DNDEvent event) {
-	if (delay == 0) {
-		dragOverStart = 0;
-		dragOverEvent = null;
-		return;
+void drag_leave ( int /*long*/ widget, int /*long*/ context, int time){
+	updateDragOverHover(0, null);
+	effect.show(DND.FEEDBACK_NONE, 0, 0);
+	
+	if (keyOperation == -1) return;
+	keyOperation = -1;
+	
+	DNDEvent event = new DNDEvent();
+	event.widget = this;
+	event.time = time;
+	event.detail = DND.DROP_NONE;
+	notifyListeners(DND.DragLeave, event);
+}
+
+boolean drag_motion ( int /*long*/ widget, int /*long*/ context, int x, int y, int time){
+	int oldKeyOperation = keyOperation;
+	
+	if (oldKeyOperation == -1) { //drag enter
+		selectedDataType = null;
+		selectedOperation = DND.DROP_NONE;
 	}
-	dragOverStart = System.currentTimeMillis() + delay;
-	if (dragOverEvent == null) dragOverEvent = new DNDEvent();
-	dragOverEvent.x = event.x;
-	dragOverEvent.y = event.y;
-	dragOverEvent.dataTypes  = event.dataTypes;
-	dragOverEvent.operations = event.operations;
-	dragOverEvent.dataType  = event.dataType;
-	dragOverEvent.detail  = event.detail;
-}
-
-int dragMotion ( int widget, int context, int x, int y, int time){
+		
 	DNDEvent event = new DNDEvent();
 	if (!setEventData(context, x, y, time, event)) {
+		keyOperation = -1;
 		OS.gdk_drag_status(context, 0, time);
-		return 0;
+		return false;
 	}
+	
 	int allowedOperations = event.operations;
-	TransferData[] allowedTypes = event.dataTypes;
-		
-	if (event.detail == DND.DROP_DEFAULT && (getStyle() & DND.DROP_DEFAULT) == 0) {
-		if ((allowedOperations & DND.DROP_MOVE) != 0) {
-			event.detail = DND.DROP_MOVE;
+	TransferData[] allowedDataTypes = new TransferData[event.dataTypes.length];
+	System.arraycopy(event.dataTypes, 0, allowedDataTypes, 0, allowedDataTypes.length);
+
+	if (oldKeyOperation == -1) {
+		event.type = DND.DragEnter;
+	} else {
+		if (keyOperation == oldKeyOperation) {
+			event.type = DND.DragOver;
+			event.dataType = selectedDataType;
+			event.detail = selectedOperation;
 		} else {
-			event.detail = DND.DROP_NONE;
+			event.type = DND.DragOperationChanged;
+			event.dataType = selectedDataType;
 		}
 	}
-
-	if (lastOperation == -1) {
-			event.type = DND.DragEnter;
-			int atom = OS.gtk_drag_dest_find_target(control.handle, context, 0);
-			if (atom == 0) {
-				OS.gdk_drag_status(context, 0, time);
-				return 0;
-			}
-			event.dataType = new TransferData();
-			event.dataType.type = atom;
-			lastOperation = event.detail;
-	} else {
-			if (lastOperation != event.detail) {
-				event.type = DND.DragOperationChanged;
-				lastOperation = event.detail;
-			} else {
-				event.type = DND.DragOver;
-				event.detail = selectedOperation;
-			}
-			event.dataType = selectedDataType;
-	}
-	event.feedback = DND.FEEDBACK_SELECT;
-	
 	updateDragOverHover(DRAGOVER_HYSTERESIS, event);
-						
-	try {
-		notifyListeners(event.type, event);	
-	} catch (Throwable e) {
-		event.detail = DND.DROP_NONE;
-		event.dataType = null;
-	}
-
-	if (event.detail == DND.DROP_DEFAULT) {
-		event.detail = DND.DROP_MOVE;
-	}
-	
 	selectedDataType = null;
+	selectedOperation = DND.DROP_NONE;
+	notifyListeners(event.type, event);	
+	if (event.detail == DND.DROP_DEFAULT) {
+		event.detail = (allowedOperations & DND.DROP_MOVE) != 0 ? DND.DROP_MOVE : DND.DROP_NONE;
+	}
 	if (event.dataType != null) {
-		for (int i = 0; i < allowedTypes.length; i++) {
-			if (allowedTypes[i].type == event.dataType.type) {
-				selectedDataType = event.dataType;
+		for (int i = 0; i < allowedDataTypes.length; i++) {
+			if (allowedDataTypes[i].type == event.dataType.type) {
+				selectedDataType = allowedDataTypes[i];
 				break;
 			}
 		}
 	}
-
-	selectedOperation = DND.DROP_NONE;
-	if (selectedDataType != null && (event.detail & allowedOperations) != 0) {
+	if (selectedDataType != null && (allowedOperations & event.detail) != 0) {
 		selectedOperation = event.detail;
 	}
 	effect.show(event.feedback, event.x, event.y);
@@ -477,10 +482,10 @@ int dragMotion ( int widget, int context, int x, int y, int time){
 			break;
 	}
 	
-	if (event.type == DND.DragEnter) {
+	if (oldKeyOperation == -1) {
 		dragOverHeartbeat.run();
 	}
-	return 1;
+	return true;
 }
 
 /**
@@ -493,9 +498,15 @@ public Control getControl () {
 	return control;
 }
 
-public Display getDisplay () {
-	if (control == null) DND.error(SWT.ERROR_WIDGET_DISPOSED);
-	return control.getDisplay ();
+int getOperationFromKeyState() {
+	int[] state = new int[1];
+	OS.gdk_window_get_pointer(0, null, null, state);
+	boolean ctrl = (state[0] & OS.GDK_CONTROL_MASK) != 0;
+	boolean shift = (state[0] & OS.GDK_SHIFT_MASK) != 0;
+	if (ctrl && shift) return DND.DROP_LINK;
+	if (ctrl)return DND.DROP_COPY;
+	if (shift)return DND.DROP_MOVE;
+	return DND.DROP_DEFAULT;
 }
 
 /**
@@ -511,26 +522,59 @@ public void notifyListeners (int eventType, Event event) {
 	Point coordinates = new Point(event.x, event.y);
 	coordinates = control.toControl(coordinates);
 	if (this.control instanceof Tree) {
-		event.item = ((Tree)control).getItem(coordinates);
+		Tree tree = (Tree)control;
+		event.item = tree.getItem(coordinates);
+		if (event.item == null) {
+			Rectangle area = tree.getClientArea();
+			if (area.contains(coordinates)) {
+				// Scan across the width of the tree.
+				for (int x1 = area.x; x1 < area.x + area.width; x1++) {
+					Point pt = new Point(x1, coordinates.y);
+					event.item = tree.getItem(pt);
+					if (event.item != null) {
+						break;
+					}
+				}
+			}
+		}
 	}
 	if (this.control instanceof Table) {
-		event.item = ((Table)control).getItem(coordinates);
+		Table table = (Table)control;
+		event.item = table.getItem(coordinates);
+		if (event.item == null) {
+			Rectangle area = table.getClientArea();
+			if (area.contains(coordinates)) {
+				// Scan across the width of the tree.
+				for (int x1 = area.x; x1 < area.x + area.width; x1++) {
+					Point pt = new Point(x1, coordinates.y);
+					event.item = table.getItem(pt);
+					if (event.item != null) {
+						break;
+					}
+				}
+			}
+		}
 	}
 	super.notifyListeners(eventType, event);
 }
 
-private void onDispose(){
-	if (transferAgents != null){
+void onDispose(){
+	if (control == null) return;
+	OS.g_signal_handler_disconnect(control.handle, drag_motion_handler);
+	OS.g_signal_handler_disconnect(control.handle, drag_leave_handler);
+	OS.g_signal_handler_disconnect(control.handle, drag_data_received_handler);
+	OS.g_signal_handler_disconnect(control.handle, drag_drop_handler);
+	if (transferAgents.length != 0)
 		OS.gtk_drag_dest_unset(control.handle);
+	transferAgents = null;
 	if (controlListener != null)
 		control.removeListener(SWT.Dispose, controlListener);
-	}
 	control.setData(DROPTARGETID, null);
 	control = null;
 	controlListener = null;
 }
 
-private int opToOsOp(int operation){
+int opToOsOp(int operation){
 	int osOperation = 0;
 	if ((operation & DND.DROP_COPY) == DND.DROP_COPY)
 		osOperation |= OS.GDK_ACTION_COPY;
@@ -541,7 +585,7 @@ private int opToOsOp(int operation){
 	return osOperation;
 }
 
-private int osOpToOp(int osOperation){
+int osOpToOp(int osOperation){
 	int operation = DND.DROP_NONE;
 	if ((osOperation & OS.GDK_ACTION_COPY) == OS.GDK_ACTION_COPY)
 		operation |= DND.DROP_COPY;
@@ -595,7 +639,7 @@ public void removeDropListener(DropTargetListener listener) {
 public void setTransfer(Transfer[] transferAgents){
 	if (transferAgents == null) DND.error(SWT.ERROR_NULL_ARGUMENT);
 	
-	if (this.transferAgents != null) {
+	if (this.transferAgents.length != 0) {
 		OS.gtk_drag_dest_unset(control.handle);
 	}
 	this.transferAgents = transferAgents;
@@ -618,7 +662,7 @@ public void setTransfer(Transfer[] transferAgents){
 		}	
 	}
 	
-	int pTargets = OS.g_malloc(targets.length * GtkTargetEntry.sizeof);
+	int /*long*/ pTargets = OS.g_malloc(targets.length * GtkTargetEntry.sizeof);
 	for (int i = 0; i < targets.length; i++) {
 		OS.memmove(pTargets + i*GtkTargetEntry.sizeof, targets[i], GtkTargetEntry.sizeof);		
 	}			
@@ -631,51 +675,77 @@ public void setTransfer(Transfer[] transferAgents){
 	}
 }
 
-boolean setEventData(int context, int x, int y, int time, DNDEvent event) {
+boolean setEventData(int /*long*/ context, int x, int y, int time, DNDEvent event) {
 	if (context == 0) return false;
 	GdkDragContext dragContext = new GdkDragContext();
 	OS.memmove(dragContext, context, GdkDragContext.sizeof);
 	if (dragContext.targets == 0) return false;
+	
+	// get allowed operations
+	int style = getStyle();
+	int operations = osOpToOp(dragContext.actions) & style;
+	if (operations == DND.DROP_NONE) return false;
+	
+	// get current operation
+	int operation = getOperationFromKeyState();
+	keyOperation = operation;
+	if (operation == DND.DROP_DEFAULT) {
+		if ((style & DND.DROP_DEFAULT) == 0) {
+			operation = (operations & DND.DROP_MOVE) != 0 ? DND.DROP_MOVE : DND.DROP_NONE;
+		}
+	} else {
+		if ((operation & operations) == 0) operation = DND.DROP_NONE;
+	}
+
+	// Get allowed transfer types
 	int length = OS.g_list_length(dragContext.targets);
-	TransferData[] tdata = new TransferData[0];
+	TransferData[] dataTypes = new TransferData[0];
 	for (int i = 0; i < length; i++) {
-		int pData = OS.g_list_nth(dragContext.targets, i);
+		int /*long*/ pData = OS.g_list_nth(dragContext.targets, i);
 		GtkTargetPair gtkTargetPair = new GtkTargetPair();
 		OS.memmove(gtkTargetPair, pData, GtkTargetPair.sizeof);
 		TransferData data = new TransferData();
 		data.type = gtkTargetPair.target;
-		TransferData[] newTdata = new TransferData[tdata.length + 1];
-		System.arraycopy(tdata, 0, newTdata, 0, tdata.length);
-		newTdata[tdata.length] = data;
-		tdata = newTdata;
+		for (int j = 0; j < transferAgents.length; j++) {
+			if (transferAgents[j].isSupportedType(data)) {
+				TransferData[] newDataTypes = new TransferData[dataTypes.length + 1];
+				System.arraycopy(dataTypes, 0, newDataTypes, 0, dataTypes.length);
+				newDataTypes[dataTypes.length] = data;
+				dataTypes = newDataTypes;	
+				break;
+			}
+		}
 	}
-	if (tdata.length == 0) return false;
+	if (dataTypes.length == 0) return false;
 
 	Point coordinates = control.toDisplay( new Point( x, y) );
-
+	
 	event.widget = this;
 	event.x = coordinates.x;
 	event.y = coordinates.y;
-	event.dataTypes	= tdata;
-	int operations = osOpToOp(dragContext.actions);
-	if (operations == DND.DROP_COPY ||
-	    operations == DND.DROP_LINK || 
-	    operations == DND.DROP_MOVE || 
-	    operations == DND.DROP_NONE) {
-			event.detail = operations;
-	} else {
-			event.detail = DND.DROP_DEFAULT;
-	}
+	event.time = time;
+	event.feedback = DND.FEEDBACK_SELECT;
+	event.dataTypes = dataTypes;
+	event.dataType = dataTypes[0];
 	event.operations = operations;
-	event.time = (int)(long)time;
+	event.detail = operation;
 	return true;
 }
 
-protected void checkSubclass () {
-	String name = getClass().getName ();
-	String validName = DropTarget.class.getName();
-	if (!validName.equals(name)) {
-		DND.error (SWT.ERROR_INVALID_SUBCLASS);
+void updateDragOverHover(long delay, DNDEvent event) {
+	if (delay == 0) {
+		dragOverStart = 0;
+		dragOverEvent = null;
+		return;
 	}
+	dragOverStart = System.currentTimeMillis() + delay;
+	if (dragOverEvent == null) dragOverEvent = new DNDEvent();
+	dragOverEvent.x = event.x;
+	dragOverEvent.y = event.y;
+	TransferData[] dataTypes = new TransferData[ event.dataTypes.length];
+	System.arraycopy( event.dataTypes, 0, dataTypes, 0, dataTypes.length);
+	dragOverEvent.dataTypes  = dataTypes;
+	dragOverEvent.operations = event.operations;
 }
+
 }
