@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -23,6 +23,11 @@ import org.eclipse.swt.graphics.*;
  * shares a significant amount of code with this class,
  * and is a subclass.
  * <p>
+ * IMPORTANT: This class was intended to be abstract and
+ * should <em>never</em> be referenced or instantiated.
+ * Instead, the class <code>Shell</code> should be used.
+ * </p>
+ * <p>
  * Instances are always displayed in one of the maximized, 
  * minimized or normal states:
  * <ul>
@@ -90,11 +95,12 @@ import org.eclipse.swt.graphics.*;
  */
 public class Decorations extends Canvas {
 	Image image;
+	Image [] images = new Image [0];
 	Menu menuBar;
 	String text = "";
 	boolean minimized, maximized;
 	Control savedFocus;
-	Button defaultButton, saveDefault;
+	Button defaultButton;
 	
 Decorations () {
 	/* Do nothing */
@@ -143,13 +149,16 @@ public Decorations (Composite parent, int style) {
 }
 
 static int checkStyle (int style) {
+	if ((style & SWT.NO_TRIM) != 0) {
+		style &= ~(SWT.CLOSE | SWT.TITLE | SWT.MIN | SWT.MAX | SWT.RESIZE | SWT.BORDER);
+	}
 	if ((style & (SWT.MENU | SWT.MIN | SWT.MAX | SWT.CLOSE)) != 0) {
 		style |= SWT.TITLE;
 	}
 	return style;
 }
 
-void bringToTop () {
+void bringToTop (boolean force) {
 	moveAbove (null);
 }
 
@@ -209,6 +218,41 @@ public Image getImage () {
 	return image;
 }
 
+/**
+ * Returns the receiver's images if they had previously been 
+ * set using <code>setImages()</code>. Images are typically
+ * displayed by the window manager when the instance is
+ * marked as iconified, and may also be displayed somewhere
+ * in the trim when the instance is in normal or maximized
+ * states. Depending where the icon is displayed, the platform
+ * chooses the icon with the "best" attributes.  It is expected
+ * that the array will contain the same icon rendered at different
+ * sizes, with different depth and transparency attributes.
+ * 
+ * <p>
+ * Note: This method will return an empty array if called before
+ * <code>setImages()</code> is called. It does not provide
+ * access to a window manager provided, "default" image
+ * even if one exists.
+ * </p>
+ * 
+ * @return the images
+ *
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ * 
+ * @since 3.0
+ */
+public Image [] getImages () {
+	checkWidget ();
+	if (images == null) return new Image [0];
+	Image [] result = new Image [images.length];
+	System.arraycopy (images, 0, result, 0, images.length);
+	return result;
+}
+
 /**
  * Returns <code>true</code> if the receiver is currently
  * maximized, and false otherwise. 
@@ -320,7 +364,10 @@ void releaseWidget () {
 	}
 	menus = null;
 	super.releaseWidget ();
-	defaultButton = saveDefault = null;
+	image = null;
+	images = null;
+	savedFocus = null;
+	defaultButton = null;
 }
 
 boolean restoreFocus () {
@@ -332,7 +379,9 @@ boolean restoreFocus () {
 void saveFocus () {
 	int window = OS.GetControlOwner (handle);
 	Control control = display.getFocusControl (window);
-	if (control != null) savedFocus = control;
+	if (control != null && control != this && this == control.menuShell ()) {
+		setSavedFocus (control);
+	}
 }
 
 /**
@@ -343,9 +392,13 @@ void saveFocus () {
  * <em>saved default button</em>). If no default button had
  * previously been set, or the saved default button was
  * disposed, the receiver's default button will be set to
- * null. 
+ * null.
+ * <p>
+ * The default button is the button that is selected when
+ * the receiver is active and the user presses ENTER.
+ * </p>
  *
- * @param the new default button
+ * @param button the new default button
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_INVALID_ARGUMENT - if the button has been disposed</li> 
@@ -357,29 +410,18 @@ void saveFocus () {
  */
 public void setDefaultButton (Button button) {
 	checkWidget();
-	setDefaultButton (button, true);
-}
-
-void setDefaultButton (Button button, boolean save) {
-	if (button == null) {
-		if (defaultButton == saveDefault) {
-			if (save) saveDefault = null;
-			return;
-		}
-	} else {
+	if (button != null) {
 		if (button.isDisposed()) error(SWT.ERROR_INVALID_ARGUMENT);
 		if ((button.style & SWT.PUSH) == 0) return;
-		if (button == defaultButton) return;
 	}
+	if (button == defaultButton) return;
 	if (defaultButton != null) {
 		if (!defaultButton.isDisposed ()) defaultButton.setDefault (false);
 	}
-	if ((defaultButton = button) == null) defaultButton = saveDefault;
+	defaultButton = button;
 	if (defaultButton != null) {
 		if (!defaultButton.isDisposed ()) defaultButton.setDefault (true);
 	}
-	if (save) saveDefault = defaultButton;
-	if (saveDefault != null && saveDefault.isDisposed ()) saveDefault = null;
 }
 
 /**
@@ -401,10 +443,61 @@ void setDefaultButton (Button button, boolean save) {
  */
 public void setImage (Image image) {
 	checkWidget();
-	if (image != null) {
-		if (image.isDisposed()) error(SWT.ERROR_INVALID_ARGUMENT);
-	}
+	if (image != null && image.isDisposed()) error(SWT.ERROR_INVALID_ARGUMENT);
 	this.image = image;
+	if (display.dockImage == 0) {
+		if (image != null) {
+			OS.SetApplicationDockTileImage (image.handle);
+		} else {
+			OS.RestoreApplicationDockTileImage ();
+		}
+	}
+}
+
+/**
+ * Sets the receiver's images to the argument, which may
+ * be an empty array. Images are typically displayed by the
+ * window manager when the instance is marked as iconified,
+ * and may also be displayed somewhere in the trim when the
+ * instance is in normal or maximized states. Depending where
+ * the icon is displayed, the platform chooses the icon with
+ * the "best" attributes. It is expected that the array will
+ * contain the same icon rendered at different sizes, with
+ * different depth and transparency attributes.
+ * 
+ * @param images the new image array
+ *
+ * @exception IllegalArgumentException <ul>
+ *    <li>ERROR_NULL_ARGUMENT - if the array of images is null</li>
+ *    <li>ERROR_INVALID_ARGUMENT - if one of the images is null or has been disposed</li>
+ * </ul>
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ * 
+ * @since 3.0
+ */
+public void setImages (Image [] images) {
+	checkWidget();
+	if (images == null) error(SWT.ERROR_INVALID_ARGUMENT);
+	for (int i = 0; i < images.length; i++) {
+		if (images [i] == null || images [i].isDisposed ()) error (SWT.ERROR_INVALID_ARGUMENT);
+	}
+	this.images = images;
+	if (display.dockImage == 0) {
+		int index = -1, width = 0, height = 0;
+		for (int i = 0; i < images.length; i++) {
+			Image image = images[i];
+			Rectangle bounds = image.getBounds();
+			if (bounds.width > width || bounds.height > height) {
+				width = bounds.width;
+				height = bounds.height;
+				index = i;
+			}
+		}
+		OS.SetApplicationDockTileImage (images [index].handle);
+	}
 }
 
 /**
@@ -415,13 +508,13 @@ public void setImage (Image image) {
  * causes the receiver to switch back to either the minimized
  * or normal states.
  * <p>
- * Note: The result of intermixing calls to<code>setMaximized(true)</code>
+ * Note: The result of intermixing calls to <code>setMaximized(true)</code>
  * and <code>setMinimized(true)</code> will vary by platform. Typically,
  * the behavior will match the platform user's expectations, but not
  * always. This should be avoided if possible.
  * </p>
  *
- * @param the new maximized state
+ * @param maximized the new maximized state
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -469,13 +562,13 @@ public void setMenuBar (Menu menu) {
  * causes the receiver to switch back to either the maximized
  * or normal states.
  * <p>
- * Note: The result of intermixing calls to<code>setMaximized(true)</code>
+ * Note: The result of intermixing calls to <code>setMaximized(true)</code>
  * and <code>setMinimized(true)</code> will vary by platform. Typically,
  * the behavior will match the platform user's expectations, but not
  * always. This should be avoided if possible.
  * </p>
  *
- * @param the new maximized state
+ * @param minimized the new maximized state
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -490,18 +583,13 @@ public void setMinimized (boolean minimized) {
 }
 
 void setSavedFocus (Control control) {
-	if (this == control) {
-		savedFocus = null;
-		return;
-	}
-	if (this != control.menuShell ()) return;
 	savedFocus = control;
 }
 
 /**
  * Sets the receiver's text, which is the string that the
  * window manager will typically display as the receiver's
- * <em>title</em>, to the argument, which may not be null. 
+ * <em>title</em>, to the argument, which must not be null. 
  *
  * @param string the new text
  *
