@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -35,6 +35,7 @@ import org.eclipse.swt.internal.carbon.*;
 public class MenuItem extends Item {
 	Menu parent, menu;
 	int accelerator;
+//	int x, y, width, height;
 
 /**
  * Constructs a new instance of this class given its parent
@@ -117,21 +118,6 @@ public MenuItem (Menu parent, int style, int index) {
 	parent.createItem (this, index);
 }
 
-public void _setEnabled (boolean enabled) {
-	int index = parent.indexOf (this);
-	if (index == -1) return;
-	int outMenuRef [] = new int [1];
-	short menuIndex = (short) (index + 1);
-	OS.GetMenuItemHierarchicalMenu (parent.handle, menuIndex, outMenuRef);
-	if (enabled) {
-		if (outMenuRef [0] != 0) OS.EnableMenuItem (outMenuRef [0], (short) 0);
-		OS.EnableMenuItem (parent.handle, menuIndex);
-	} else {
-		if (outMenuRef [0] != 0) OS.DisableMenuItem (outMenuRef [0], (short) 0);
-		OS.DisableMenuItem (parent.handle, menuIndex);
-	}
-}
-
 /**
  * Adds the listener to the collection of listeners who will
  * be notified when the arm events are generated for the control, by sending
@@ -186,7 +172,7 @@ public void addHelpListener (HelpListener listener) {
 
 /**
  * Adds the listener to the collection of listeners who will
- * be notified when the control is selected, by sending
+ * be notified when the menu item is selected, by sending
  * it one of the messages defined in the <code>SelectionListener</code>
  * interface.
  * <p>
@@ -224,12 +210,44 @@ static int checkStyle (int style) {
 	return checkBits (style, SWT.PUSH, SWT.CHECK, SWT.RADIO, SWT.SEPARATOR, SWT.CASCADE, 0);
 }
 
+int createEmptyMenu () {
+	/*
+	* Bug in the Macintosh.  When a menu bar item does not have
+	* an associated pull down menu, the Macintosh segment faults.
+	* The fix is to temporarily attach an empty menu. 
+	*/
+	if ((parent.style & SWT.BAR) != 0) {
+		int outMenuRef [] = new int [1];
+		if (OS.CreateNewMenu ((short) 0, 0, outMenuRef) != OS.noErr) {
+			error (SWT.ERROR_NO_HANDLES);
+		}
+		return outMenuRef [0];
+	}
+	return 0;
+}
+
+void destroyEmptyMenu (int index) {
+	if ((parent.style & SWT.BAR) != 0) {
+		if (index == -1) index = parent.indexOf (this);
+		if (index != -1) {
+			short menuIndex = (short) (index + 1);
+			int [] outMenuRef = new int [1];
+			OS.GetMenuItemHierarchicalMenu (parent.handle, menuIndex, outMenuRef);
+			if (outMenuRef [0] != 0) {
+				OS.DisposeMenu (outMenuRef [0]);
+			}
+		}
+	}
+}
+
 /**
- * Return the widget accelerator.  An accelerator is the bit-wise
+ * Returns the widget accelerator.  An accelerator is the bit-wise
  * OR of zero or more modifier masks and a key. Examples:
  * <code>SWT.CONTROL | SWT.SHIFT | 'T', SWT.ALT | SWT.F2</code>.
+ * The default value is zero, indicating that the menu item does
+ * not have an accelerator.
  *
- * @return the accelerator
+ * @return the accelerator or 0
  *
  * </ul>
  * @exception SWTException <ul>
@@ -242,9 +260,34 @@ public int getAccelerator () {
 	return accelerator;
 }
 
+/*public*/ Rectangle getBounds () {
+	checkWidget ();
+	if ((parent.style & SWT.BAR) != 0) {
+		int index = parent.indexOf (this);
+		if (index == -1) return new Rectangle (0 ,0, 0, 0);
+		Menu menu = display.getMenuBar ();
+		if (parent != menu) return new Rectangle (0 ,0, 0, 0);
+		int outMenuRef [] = new int [1];
+		if (OS.GetMenuItemHierarchicalMenu (menu.handle, (short)(index + 1), outMenuRef) != OS.noErr) {
+			return new Rectangle (0 ,0, 0, 0);
+		};
+		Rect rect = new Rect ();
+		//TODO - get the bounds of the menu item from the menu title
+//		if (... code needed to do this ... != OS.noErr) {
+//			return new Rectangle (0 ,0, 0, 0);
+//		}
+		int width = rect.right - rect.left;
+		int height = rect.bottom - rect.top;
+		return new Rectangle (rect.left, rect.top, width, height);
+	}
+	return new Rectangle (0 ,0, 0, 0);
+//	return new Rectangle (x, y, width, height);
+
+}
+
 /**
  * Returns <code>true</code> if the receiver is enabled, and
- * <code>false</code> otherwise. A disabled control is typically
+ * <code>false</code> otherwise. A disabled menu item is typically
  * not selectable from the user interface and draws with an
  * inactive or "grayed" look.
  *
@@ -351,7 +394,7 @@ int kEventProcessCommand (int nextHandler, int theEvent, int userData) {
 /**
  * Returns <code>true</code> if the receiver is enabled and all
  * of the receiver's ancestors are enabled, and <code>false</code>
- * otherwise. A disabled control is typically not selectable from the
+ * otherwise. A disabled menu item is typically not selectable from the
  * user interface and draws with an inactive or "grayed" look.
  *
  * @return the receiver's enabled state
@@ -405,27 +448,15 @@ int keyGlyph (int key) {
 
 void releaseChild () {
 	super.releaseChild ();
-	if (menu != null) menu.dispose ();
-	menu = null;
 	parent.destroyItem (this);
 }
 
 void releaseWidget () {
-	if (menu != null) {
+	if (menu == null) {
+		destroyEmptyMenu (-1);
+	} else {
 		menu.releaseWidget ();
 		menu.destroyWidget ();
-	} else {
-		if ((parent.style & SWT.BAR) != 0) {
-//			short [] outIndex = new short [1];
-//			if (OS.GetIndMenuItemWithCommandID (parent.handle, id, 1, null, outIndex) == OS.noErr) {
-//				int [] outMenuRef = new int [1];
-//				OS.GetMenuItemHierarchicalMenu (parent.handle, outIndex [0], outMenuRef);
-//				if (outMenuRef [0] != 0) {
-//					OS.DeleteMenu (OS.GetMenuID (outMenuRef [0]));
-//					OS.DisposeMenu (outMenuRef [0]);
-//				}
-//			}
-		}
 	}
 	menu = null;
 	super.releaseWidget ();
@@ -438,7 +469,7 @@ void releaseWidget () {
  * Removes the listener from the collection of listeners who will
  * be notified when the arm events are generated for the control.
  *
- * @param listener the listener which should be notified
+ * @param listener the listener which should no longer be notified
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
@@ -462,7 +493,7 @@ public void removeArmListener (ArmListener listener) {
  * Removes the listener from the collection of listeners who will
  * be notified when the help events are generated for the control.
  *
- * @param listener the listener which should be notified
+ * @param listener the listener which should no longer be notified
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
@@ -486,7 +517,7 @@ public void removeHelpListener (HelpListener listener) {
  * Removes the listener from the collection of listeners who will
  * be notified when the control is selected.
  *
- * @param listener the listener which should be notified
+ * @param listener the listener which should no longer be notified
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
@@ -523,6 +554,8 @@ void selectRadio () {
  * OR of zero or more modifier masks and a key. Examples:
  * <code>SWT.MOD1 | SWT.MOD2 | 'T', SWT.MOD3 | SWT.F2</code>.
  * <code>SWT.CONTROL | SWT.SHIFT | 'T', SWT.ALT | SWT.F2</code>.
+ * The default value is zero, indicating that the menu item does
+ * not have an accelerator.
  *
  * @param accelerator an integer that is the bit-wise OR of masks and a key
  *
@@ -566,7 +599,7 @@ public void setAccelerator (int accelerator) {
 
 /**
  * Enables the receiver if the argument is <code>true</code>,
- * and disables it otherwise. A disabled control is typically
+ * and disables it otherwise. A disabled menu item is typically
  * not selectable from the user interface and draws with an
  * inactive or "grayed" look.
  *
@@ -579,21 +612,30 @@ public void setAccelerator (int accelerator) {
  */
 public void setEnabled (boolean enabled) {
 	checkWidget ();
+	int index = parent.indexOf (this);
+	if (index == -1) return;
+	int outMenuRef [] = new int [1];
+	short menuIndex = (short) (index + 1);
+	OS.GetMenuItemHierarchicalMenu (parent.handle, menuIndex, outMenuRef);
 	if (enabled) {
 		state &= ~DISABLED;
+		if (outMenuRef [0] != 0) OS.EnableMenuItem (outMenuRef [0], (short) 0);
+		OS.EnableMenuItem (parent.handle, menuIndex);
 	} else {
 		state |= DISABLED;
+		if (outMenuRef [0] != 0) OS.DisableMenuItem (outMenuRef [0], (short) 0);
+		OS.DisableMenuItem (parent.handle, menuIndex);
 	}
-	_setEnabled (enabled);
 }
 
 /**
  * Sets the image the receiver will display to the argument.
  * <p>
- * Note: This feature is not available on all window systems (for example, Window NT),
- * in which case, calling this method will silently do nothing.
+ * Note: This operation is a hint and is not supported on
+ * platforms that do not have this concept (for example, Windows NT).
+ * </p>
  *
- * @param menu the image to display
+ * @param image the image to display
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -657,36 +699,24 @@ public void setMenu (Menu menu) {
 	/* Update the menu in the OS */
 	int index = parent.indexOf (this);
 	if (index == -1) return;
-	short menuIndex = (short) (index + 1);
-	int outMenuRef [] = new int [1];
+	int menuHandle = 0;
 	if (menu == null) {
-		if ((parent.style & SWT.BAR) != 0) {
-//			Display display = getDisplay ();
-//			short menuID = display.nextMenuId ();
-//			if (OS.CreateNewMenu (menuID, 0, outMenuRef) != OS.noErr) {
-//				error (SWT.ERROR_NO_HANDLES);
-//			}
-		}
+		menuHandle = createEmptyMenu ();
 	} else {
 		menu.cascade = this;
-		if ((parent.style & SWT.BAR) != 0) {
-			if (oldMenu == null) {
-//				OS.GetMenuItemHierarchicalMenu (parent.handle, outIndex [0], outMenuRef);
-//				if (outMenuRef [0] != 0) {
-//					OS.DeleteMenu (OS.GetMenuID (outMenuRef [0]));
-//					OS.DisposeMenu (outMenuRef [0]);
-//				}
-			}
-		}
-		outMenuRef [0] = menu.handle;
+		if (oldMenu == null) 	destroyEmptyMenu (index);
+		menuHandle = menu.handle;
+	}
+	short menuIndex = (short) (index + 1);
+	if (menuHandle != 0) {
 		int [] outString = new int [1];
 		if (OS.CopyMenuItemTextAsCFString (parent.handle, menuIndex, outString) != OS.noErr) {
 			error (SWT.ERROR_CANNOT_SET_MENU);
 		}
-		OS.SetMenuTitleWithCFString (outMenuRef [0], outString [0]);
+		OS.SetMenuTitleWithCFString (menuHandle, outString [0]);
 		OS.CFRelease (outString [0]);
 	}
-	if (OS.SetMenuItemHierarchicalMenu (parent.handle, menuIndex, outMenuRef [0]) != OS.noErr) {
+	if (OS.SetMenuItemHierarchicalMenu (parent.handle, menuIndex, menuHandle) != OS.noErr) {
 		error (SWT.ERROR_CANNOT_SET_MENU);
 	}
 }
@@ -763,6 +793,7 @@ public void setText (String string) {
 	checkWidget ();
 	if (string == null) error (SWT.ERROR_NULL_ARGUMENT);
 	if ((style & SWT.SEPARATOR) != 0) return;
+	if (text.equals (string)) return;
 	int index = parent.indexOf (this);
 	if (index == -1) return;
 	super.setText (string);
@@ -775,10 +806,10 @@ void updateText (short menuIndex) {
 	text.getChars (0, buffer.length, buffer, 0);
 	int i=0, j=0;
 	while (i < buffer.length) {
-		if (accelerator != 0 && buffer [i] == '\t') break;
-		if ((buffer [j++] = buffer [i++]) == Mnemonic) {
+		if (buffer [i] == '\t') break;
+		if ((buffer [j++] = buffer [i++]) == '&') {
 			if (i == buffer.length) {continue;}
-			if (buffer [i] == Mnemonic) {i++; continue;}
+			if (buffer [i] == '&') {i++; continue;}
 			j--;
 		}
 	}
