@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -12,6 +12,7 @@ package org.eclipse.swt.widgets;
 
  
 import org.eclipse.swt.internal.carbon.OS;
+import org.eclipse.swt.internal.carbon.Rect;
 
 import org.eclipse.swt.*;
 import org.eclipse.swt.events.*;
@@ -22,7 +23,7 @@ import org.eclipse.swt.graphics.*;
  * objects that represent a range of positive, numeric values. 
  * <p>
  * At any given moment, a given slider will have a 
- * single <em>selection</em> that is considered to be its
+ * single 'selection' that is considered to be its
  * value, which is constrained to be within the range of
  * values the slider represents (that is, between its
  * <em>minimum</em> and <em>maximum</em> values).
@@ -66,7 +67,7 @@ import org.eclipse.swt.graphics.*;
  * @see ScrollBar
  */
 public class Slider extends Control {
-	boolean dragging;
+	boolean dragging, tracking;
 	int increment = 1;
 	int pageIncrement = 10;
 
@@ -110,7 +111,7 @@ public Slider (Composite parent, int style) {
  * interface.
  * <p>
  * When <code>widgetSelected</code> is called, the event object detail field contains one of the following values:
- * <code>0</code> - for the end of a drag.
+ * <code>SWT.NONE</code> - for the end of a drag.
  * <code>SWT.DRAG</code>.
  * <code>SWT.HOME</code>.
  * <code>SWT.END</code>.
@@ -168,7 +169,7 @@ int actionProc (int theControl, int partCode) {
 	        event.detail = SWT.ARROW_DOWN;
 	        break;
 	    case OS.kControlIndicatorPart:
-	    	dragging = true;
+	    		dragging = true;
 			event.detail = SWT.DRAG;
 	        break;
 		default:
@@ -278,7 +279,7 @@ public int getPageIncrement () {
 }
 
 /**
- * Returns the single <em>selection</em> that is the receiver's value.
+ * Returns the 'selection', which is the receiver's value.
  *
  * @return the selection
  *
@@ -308,16 +309,52 @@ public int getThumb () {
     return OS.GetControlViewSize (handle);
 }
 
+int kEventControlTrack (int nextHandler, int theEvent, int userData) {
+	int result = super.kEventControlTrack (nextHandler, theEvent, userData);
+	if (result == OS.noErr) return result;
+	tracking = true;
+	return OS.eventNotHandledErr;
+}
+
 int kEventMouseDown (int nextHandler, int theEvent, int userData) {
 	int status = super.kEventMouseDown (nextHandler, theEvent, userData);
 	if (status == OS.noErr) return status;
-	dragging = false;
+	/*
+	* Feature in the Macintosh.  Some controls call TrackControl() or
+	* HandleControlClick() to track the mouse.  Unfortunately, mouse move
+	* events and the mouse up events are consumed.  The fix is to call the
+	* default handler and send a fake mouse up when tracking is finished.
+	* 
+	* NOTE: No mouse move events are sent while tracking.  There is no
+	* fix for this at this time.
+	*/
+	display.grabControl = null;
+	display.runDeferredEvents ();
+	dragging = tracking = false;
 	status = OS.CallNextEventHandler (nextHandler, theEvent);
 	if (dragging) {
 		Event event = new Event ();
 		sendEvent (SWT.Selection, event);
 	}
 	dragging = false;
+	if (tracking) {
+		org.eclipse.swt.internal.carbon.Point outPt = new org.eclipse.swt.internal.carbon.Point ();
+		OS.GetGlobalMouse (outPt);
+		Rect rect = new Rect ();
+		int window = OS.GetControlOwner (handle);
+		OS.GetWindowBounds (window, (short) OS.kWindowContentRgn, rect);
+		int x = outPt.h - rect.left;
+		int y = outPt.v - rect.top;
+		OS.GetControlBounds (parent.handle, rect);
+		x -= rect.left;
+		y -=  rect.top;
+		short [] button = new short [1];
+		OS.GetEventParameter (theEvent, OS.kEventParamMouseButton, OS.typeMouseButton, null, 2, null, button);
+		int chord = OS.GetCurrentEventButtonState ();
+		int modifiers = OS.GetCurrentEventKeyModifiers ();
+		sendMouseEvent (SWT.MouseUp, button [0], chord, (short)x, (short)y, modifiers, true);
+	}
+	tracking = false;
 	return status;
 }
 
@@ -366,11 +403,12 @@ public void setIncrement (int value) {
 }
 
 /**
- * Sets the maximum value which the receiver will allow
- * to be the argument which must be greater than or
- * equal to zero.
+ * Sets the maximum. If this value is negative or less than or
+ * equal to the minimum, the value is ignored. If necessary, first
+ * the thumb and then the selection are adjusted to fit within the
+ * new range.
  *
- * @param value the new maximum (must be zero or greater)
+ * @param value the new maximum, which must be greater than the current minimum
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -381,17 +419,22 @@ public void setMaximum (int value) {
 	checkWidget();
 	if (value < 0) return;
 	int minimum = OS.GetControl32BitMinimum (handle);
+	if (value <= minimum) return;
 	int viewSize = OS.GetControlViewSize (handle);
-	if (value - minimum - viewSize < 0) return;
+	if (value - minimum < viewSize) {
+		viewSize = value - minimum;
+		OS.SetControlViewSize (handle, viewSize);
+	}
 	OS.SetControl32BitMaximum (handle, value - viewSize);
 }
 
 /**
- * Sets the minimum value which the receiver will allow
- * to be the argument which must be greater than or
- * equal to zero.
+ * Sets the minimum value. If this value is negative or greater
+ * than or equal to the maximum, the value is ignored. If necessary,
+ * first the thumb and then the selection are adjusted to fit within
+ * the new range.
  *
- * @param value the new minimum (must be zero or greater)
+ * @param value the new minimum
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -401,9 +444,14 @@ public void setMaximum (int value) {
 public void setMinimum (int value) {
 	checkWidget();
 	if (value < 0) return;
-	int maximum = OS.GetControl32BitMaximum (handle);
 	int viewSize = OS.GetControlViewSize (handle);
-	if (maximum - value - viewSize < 0) return;
+	int maximum = OS.GetControl32BitMaximum (handle) + viewSize;
+	if (value >= maximum) return;
+	if (maximum - value < viewSize) {
+		viewSize = maximum - value;
+		OS.SetControl32BitMaximum (handle, maximum - viewSize);
+		OS.SetControlViewSize (handle, viewSize);
+	}
 	OS.SetControl32BitMinimum (handle, value);
 }
 
@@ -413,7 +461,7 @@ public void setMinimum (int value) {
  * are selected to the argument, which must be at least
  * one.
  *
- * @return the page increment (must be greater than zero)
+ * @param value the page increment (must be greater than zero)
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -427,8 +475,8 @@ public void setPageIncrement (int value) {
 }
 
 /**
- * Sets the single <em>selection</em> that is the receiver's
- * value to the argument which must be greater than or equal
+ * Sets the 'selection', which is the receiver's
+ * value, to the argument which must be greater than or equal
  * to zero.
  *
  * @param value the new selection (must be zero or greater)
@@ -445,24 +493,26 @@ public void setSelection (int value) {
 
 /**
  * Sets the size of the receiver's thumb relative to the
- * difference between its maximum and minimum values to the
- * argument which must be at least one.
+ * difference between its maximum and minimum values.  This new
+ * value will be ignored if it is less than one, and will be
+ * clamped if it exceeds the receiver's current range.
  *
- * @param value the new thumb value (must be at least one)
+ * @param value the new thumb value, which must be at least one and not
+ * larger than the size of the current range
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
- *
- * @see ScrollBar
  */
 public void setThumb (int value) {
 	checkWidget();
 	if (value < 1) return;
-	int maximum = OS.GetControl32BitMaximum (handle);
+	int minimum = OS.GetControl32BitMinimum (handle);
 	int viewSize = OS.GetControlViewSize (handle);
-	OS.SetControl32BitMaximum (handle, maximum + viewSize - value);
+	int maximum = OS.GetControl32BitMaximum (handle) + viewSize;
+	value = Math.min (value, maximum - minimum);
+	OS.SetControl32BitMaximum (handle, maximum - value);
 	OS.SetControlViewSize (handle, value);
 }
 
@@ -492,9 +542,9 @@ public void setValues (int selection, int minimum, int maximum, int thumb, int i
 	if (minimum < 0) return;
 	if (maximum < 0) return;
 	if (thumb < 1) return;
-	if (maximum - minimum - thumb < 0) return;
 	if (increment < 1) return;
 	if (pageIncrement < 1) return;
+	thumb = Math.min (thumb, maximum - minimum);
 	OS.SetControl32BitMinimum (handle, minimum);
 	OS.SetControl32BitMaximum (handle, maximum - thumb);
 	OS.SetControlViewSize (handle, thumb);
