@@ -1,16 +1,16 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
 package org.eclipse.swt.widgets;
 
- 
+
 import org.eclipse.swt.internal.carbon.OS;
 import org.eclipse.swt.internal.carbon.ControlButtonContentInfo;
 import org.eclipse.swt.internal.carbon.HMHelpContentRec;
@@ -37,9 +37,10 @@ import org.eclipse.swt.graphics.*;
  * </p>
  */
 public class ToolItem extends Item {
-	int handle, iconHandle, labelHandle, arrowHandle;
-	int cIcon, labelCIcon, arrowCIcon;
+	int handle, iconHandle, labelHandle;
+	int cIcon, labelCIcon;
 	int visibleRgn;
+	int width = DEFAULT_SEPARATOR_WIDTH;
 	ToolBar parent;
 	Image hotImage, disabledImage;
 	String toolTipText;
@@ -48,7 +49,8 @@ public class ToolItem extends Item {
 
 	static final int DEFAULT_WIDTH = 24;
 	static final int DEFAULT_HEIGHT = 22;
-	static final int DEFAULT_SEPARATOR_WIDTH = 8;
+	static final int DEFAULT_SEPARATOR_WIDTH = 6;
+	static final int ARROW_WIDTH = 9;
 
 /**
  * Constructs a new instance of this class given its parent
@@ -88,7 +90,6 @@ public ToolItem (ToolBar parent, int style) {
 	super (parent, checkStyle (style));
 	this.parent = parent;
 	parent.createItem (this, parent.getItemCount ());
-	parent.relayout ();
 }
 
 /**
@@ -130,7 +131,6 @@ public ToolItem (ToolBar parent, int style, int index) {
 	super (parent, checkStyle (style));
 	this.parent = parent;
 	parent.createItem (this, index);
-	parent.relayout ();
 }
 
 /**
@@ -174,44 +174,73 @@ protected void checkSubclass () {
 	if (!isValidSubclass ()) error (SWT.ERROR_INVALID_SUBCLASS);
 }
 
-Point computeSize () {
+int colorProc (int inControl, int inMessage, int inDrawDepth, int inDrawInColor) {
+	switch (inMessage) {
+		case OS.kControlMsgApplyTextColor: {
+			if (parent.foreground != null) {
+				OS.RGBForeColor (toRGBColor (parent.foreground));
+			} else {
+				OS.SetThemeTextColor ((short) OS.kThemeTextColorDialogActive, (short) inDrawDepth, inDrawInColor != 0);
+			}
+			return OS.noErr;
+		}
+		case OS.kControlMsgSetUpBackground: {
+			float [] background = parent.background != null ? parent.background : parent.getParentBackground ();
+			if (background != null) {
+				OS.RGBBackColor (toRGBColor (background));
+			} else {
+				OS.SetThemeBackground ((short) OS.kThemeBrushDialogBackgroundActive, (short) inDrawDepth, inDrawInColor != 0);
+			}
+			return OS.noErr;
+		}
+	}
+	return OS.eventNotHandledErr;
+}
+
+Point computeSize (GC gc) {
 	checkWidget();
 	int width = 0, height = 0;
 	if ((style & SWT.SEPARATOR) != 0) {
-		if ((style & SWT.HORIZONTAL) != 0) {
+		if ((parent.style & SWT.HORIZONTAL) != 0) {
 			width = getWidth ();
 			height = DEFAULT_HEIGHT;
 		} else {
 			width = DEFAULT_WIDTH;
 			height = getWidth ();
 		}
+		if (control != null) {
+			height = Math.max (height, control.getMininumHeight ());
+		}
 	} else {
 		int space = 0;
-		int stringWidth = 0, stringHeight = 0;
-		if (text.length () != 0) {
-			GC gc = new GC (parent);
-			Point size = gc.stringExtent (text);
-			stringWidth = size.x;
-			stringHeight = size.y;
-			gc.dispose ();
-		}
-		int imageWidth = 0, imageHeight = 0;
-		if (image != null) {
-			if (text.length () != 0) space = 2;
-			Rectangle rect = image.getBounds ();
-			imageWidth = rect.width;
-			imageHeight = rect.height;
-		}
-		if ((parent.style & SWT.RIGHT) != 0) {
-			width = stringWidth + imageWidth;
-			height = Math.max (stringHeight, imageHeight);
+		if (text.length () != 0 || image != null) {
+			int stringWidth = 0, stringHeight = 0;
+			if (text.length () != 0) {
+				int flags = SWT.DRAW_DELIMITER | SWT.DRAW_TAB | SWT.DRAW_MNEMONIC | SWT.DRAW_TRANSPARENT;
+				Point size = gc.textExtent (text, flags);
+				stringWidth = size.x;
+				stringHeight = size.y;
+			}
+			int imageWidth = 0, imageHeight = 0;
+			if (image != null) {
+				if (text.length () != 0) space = 2;
+				Rectangle rect = image.getBounds ();
+				imageWidth = rect.width;
+				imageHeight = rect.height;
+			}
+			if ((parent.style & SWT.RIGHT) != 0) {
+				width = stringWidth + imageWidth;
+				height = Math.max (stringHeight, imageHeight);
+			} else {
+				width = Math.max (stringWidth, imageWidth);
+				height = stringHeight + imageHeight;
+			}
 		} else {
-			width = Math.max (stringWidth, imageWidth);
-			height = stringHeight + imageHeight;
+			width = DEFAULT_WIDTH;
+			height = DEFAULT_HEIGHT;
 		}
 		if ((style & SWT.DROP_DOWN) != 0) {
-			int arrowWidth = 6; //NOT DONE
-			width += 3 + arrowWidth;
+			width += ARROW_WIDTH;
 		}
 		int inset = 3;
 		width += space + inset * 2;
@@ -227,30 +256,15 @@ void createHandle () {
 	OS.CreateUserPaneControl (window, null, features, outControl);
 	if (outControl [0] == 0) error (SWT.ERROR_NO_HANDLES);
 	handle = outControl [0];
-	int width = DEFAULT_WIDTH, height = DEFAULT_HEIGHT;
 	if ((style & SWT.SEPARATOR) == 0) {
 		ControlButtonContentInfo inContent = new ControlButtonContentInfo ();
-		if ((style & SWT.DROP_DOWN) != 0) {
-			OS.CreateIconControl(window, null, inContent, false, outControl);
-			if (outControl [0] == 0) error (SWT.ERROR_NO_HANDLES);
-			arrowHandle = outControl [0];
-			updateArrow ();
-		}
 		OS.CreateIconControl(window, null, inContent, false, outControl);
 		if (outControl [0] == 0) error (SWT.ERROR_NO_HANDLES);
 		iconHandle = outControl [0];
 		OS.CreateIconControl(window, null, inContent, false, outControl);
 		if (outControl [0] == 0) error (SWT.ERROR_NO_HANDLES);
 		labelHandle = outControl [0];
-	} else {
-		if ((parent.style & SWT.HORIZONTAL) != 0) {
-			width = DEFAULT_SEPARATOR_WIDTH;
-		} else {
-			height = DEFAULT_SEPARATOR_WIDTH;
-		}
-	}	
-	setBounds (0, 0, width, height);
-	parent.relayout ();
+	}
 }
 
 void createWidget () {
@@ -259,24 +273,18 @@ void createWidget () {
 	toolTipText = "";
 }
 
-int defaultThemeFont () {	
-	return OS.kThemeToolbarFont;
-}
-
 void deregister () {
 	super.deregister ();
-	WidgetTable.remove (handle);
-	if (iconHandle != 0) WidgetTable.remove (iconHandle);
-	if (labelHandle != 0) WidgetTable.remove (labelHandle);
-	if (arrowHandle != 0) WidgetTable.remove (arrowHandle);
+	display.removeWidget (handle);
+	if (iconHandle != 0) display.removeWidget (iconHandle);
+	if (labelHandle != 0) display.removeWidget (labelHandle);
 }
 
 void destroyWidget () {
-	Display display = this.display;
 	int theControl = handle;
 	releaseHandle ();
 	if (theControl != 0) {
-		display.addDisposeControl (theControl);
+		OS.DisposeControl (theControl);
 	}
 }
 
@@ -288,16 +296,30 @@ public void dispose () {
 }
 
 void drawBackground (int control) {
-	drawBackground (control, null);
+	drawBackground (control, parent.background);
 }
 
 void drawWidget (int control, int damageRgn, int visibleRgn, int theEvent) {
-	if ((style & SWT.SEPARATOR) != 0) {
+	if (control == handle && (style & (SWT.DROP_DOWN | SWT.SEPARATOR)) != 0) {
+		int state;
+		if (OS.IsControlEnabled (control)) {
+			state = OS.IsControlActive (control) ? OS.kThemeStateActive : OS.kThemeStateInactive;
+		} else {
+			state = OS.IsControlActive (control) ? OS.kThemeStateUnavailable : OS.kThemeStateUnavailableInactive;
+		}
 		Rect rect = new Rect ();
 		OS.GetControlBounds (handle, rect);
-		rect.top += 2;
-		rect.bottom -= 2;
-		OS.DrawThemeSeparator (rect, 0);
+		if ((style & SWT.SEPARATOR) != 0 && this.control == null) {
+			rect.top += 2;
+			rect.bottom -= 2;
+			OS.DrawThemeSeparator (rect, state);
+		}
+		if ((style & SWT.DROP_DOWN) != 0) {
+			int height = rect.bottom - rect.top;
+			rect.top = (short) (rect.bottom - (height / 2) - 1);
+			rect.left = (short) (rect.right - ARROW_WIDTH);
+			OS.DrawThemePopupArrow (rect, (short) OS.kThemeArrowDown, (short) OS.kThemeArrow5pt, state, 0, 0);
+		}
 	}
 }
 
@@ -463,9 +485,7 @@ public String getToolTipText () {
  */
 public int getWidth () {
 	checkWidget();
-	Rect rect = new Rect ();
-	OS.GetControlBounds (handle, rect);
-	return rect.right - rect.left;
+	return width;
 }
 
 int getVisibleRegion (int control, boolean clipChildren) {
@@ -485,20 +505,13 @@ int helpProc (int inControl, int inGlobalMouse, int inRequest, int outContentPro
 			if (toolTipText != null && toolTipText.length () != 0) {
 				char [] buffer = new char [toolTipText.length ()];
 				toolTipText.getChars (0, buffer.length, buffer, 0);
-				int i=0, j=0;
-				while (i < buffer.length) {
-					if ((buffer [j++] = buffer [i++]) == Mnemonic) {
-						if (i == buffer.length) {continue;}
-						if (buffer [i] == Mnemonic) {i++; continue;}
-						j--;
-					}
-				}
+				int length = fixMnemonic (buffer);
 				if (display.helpString != 0) OS.CFRelease (display.helpString);
-		    	display.helpString = OS.CFStringCreateWithCharacters (OS.kCFAllocatorDefault, buffer, j);
+		    	display.helpString = OS.CFStringCreateWithCharacters (OS.kCFAllocatorDefault, buffer, length);
 				HMHelpContentRec helpContent = new HMHelpContentRec ();
 				OS.memcpy (helpContent, ioHelpContent, HMHelpContentRec.sizeof);
 		        helpContent.version = OS.kMacHelpVersion;
-		        helpContent.tagSide = OS.kHMDefaultSide;
+		        helpContent.tagSide = (short) OS.kHMDefaultSide;
 				display.helpControl = null;
 		        helpContent.absHotRect_left = (short) 0;
 		     	helpContent.absHotRect_top = (short) 0;
@@ -526,6 +539,7 @@ int helpProc (int inControl, int inGlobalMouse, int inRequest, int outContentPro
 void hookEvents () {
 	super.hookEvents ();
 	int controlProc = display.controlProc;
+	int colorProc = display.colorProc;
 	int [] mask1 = new int [] {
 		OS.kEventClassControl, OS.kEventControlDraw,
 		OS.kEventClassControl, OS.kEventControlHit,
@@ -542,17 +556,16 @@ void hookEvents () {
 	if (iconHandle != 0) {
 		controlTarget = OS.GetControlEventTarget (iconHandle);
 		OS.InstallEventHandler (controlTarget, controlProc, mask2.length / 2, mask2, iconHandle, null);
+		OS.SetControlColorProc (iconHandle, colorProc);
 	}
 	if (labelHandle != 0) {
 		controlTarget = OS.GetControlEventTarget (labelHandle);
 		OS.InstallEventHandler (controlTarget, controlProc, mask2.length / 2, mask2, labelHandle, null);
-	}
-	if (arrowHandle != 0) {
-		controlTarget = OS.GetControlEventTarget (arrowHandle);
-		OS.InstallEventHandler (controlTarget, controlProc, mask2.length / 2, mask2, arrowHandle, null);
+		OS.SetControlColorProc (labelHandle, colorProc);
 	}
 	int helpProc = display.helpProc;
 	OS.HMInstallControlContentCallback (handle, helpProc);
+	OS.SetControlColorProc (handle, colorProc);
 }
 
 void invalidateVisibleRegion (int control) {
@@ -560,6 +573,10 @@ void invalidateVisibleRegion (int control) {
 	parent.resetVisibleRegion (control);
 }
 
+void invalWindowRgn (int window, int rgn) {
+	parent.invalWindowRgn (window, rgn);
+}
+
 /**
  * Returns <code>true</code> if the receiver is enabled and all
  * of the receiver's ancestors are enabled, and <code>false</code>
@@ -587,24 +604,13 @@ int kEventControlContextualMenuClick (int nextHandler, int theEvent, int userDat
 int kEventControlHit (int nextHandler, int theEvent, int userData) {
 	int result = super.kEventControlHit (nextHandler, theEvent, userData);
 	if (result == OS.noErr) return result;
-	Event event = new Event ();
 	if ((style & SWT.RADIO) != 0) {
 		if ((parent.getStyle () & SWT.NO_RADIO_GROUP) == 0) {
 			selectRadio ();
 		}
 	}
 	if ((style & SWT.CHECK) != 0) setSelection (!getSelection ());
-	if ((style & SWT.DROP_DOWN) != 0) {
-		int [] theControl = new int [1];
-		OS.GetEventParameter (theEvent, OS.kEventParamDirectObject, OS.typeControlRef, null, 4, null, theControl);
-		if (theControl [0] == arrowHandle) {
-			event.detail = SWT.ARROW;
-			Rect rect = getControlBounds (handle);
-			event.x = rect.left;
-			event.y = rect.bottom;
-		}
-	}
-	postEvent (SWT.Selection, event);
+	postEvent (SWT.Selection);
 	return OS.eventNotHandledErr;
 }
 
@@ -616,9 +622,35 @@ int kEventControlTrack (int nextHandler, int theEvent, int userData) {
 int kEventMouseDown (int nextHandler, int theEvent, int userData) {
 	int result = parent.kEventMouseDown (nextHandler, theEvent, userData);
 	if (result == OS.noErr) return result;
+	
+	if ((style & SWT.DROP_DOWN) != 0) {
+		int sizeof = org.eclipse.swt.internal.carbon.Point.sizeof;
+		org.eclipse.swt.internal.carbon.Point pt = new org.eclipse.swt.internal.carbon.Point ();
+		OS.GetEventParameter (theEvent, OS.kEventParamMouseLocation, OS.typeQDPoint, null, sizeof, null, pt);
+		Rect rect = new Rect ();
+		int window = OS.GetControlOwner (handle);
+		OS.GetWindowBounds (window, (short) OS.kWindowContentRgn, rect);
+		int x = pt.h - rect.left;
+		int y = pt.v - rect.top;
+		OS.GetControlBounds (handle, rect);
+		x -= rect.left;
+		y -= rect.top;
+		int width = rect.right - rect.left;
+		if (width - x < 12) {
+			x = rect.left;
+			y = rect.bottom;
+			OS.GetControlBounds (parent.handle, rect);
+			x -= rect.left;
+			y -= rect.top;
+			Event event = new Event ();
+			event.detail = SWT.ARROW;
+			event.x = x;
+			event.y = y;
+			postEvent (SWT.Selection, event);
+		}
+	}
 	/*
-	* Feature in the Macintosh.  When some controls get kEventControlClick
-	* (which gets sent from kEventMouseDown), they call TrackControl() or
+	* Feature in the Macintosh.  Some controls call TrackControl() or
 	* HandleControlClick() to track the mouse.  Unfortunately, mouse move
 	* events and the mouse up events are consumed.  The fix is to call the
 	* default handler and send a fake mouse up when tracking is finished.
@@ -638,16 +670,14 @@ int kEventMouseDown (int nextHandler, int theEvent, int userData) {
 		OS.GetWindowBounds (window, (short) OS.kWindowContentRgn, rect);
 		int x = outPt.h - rect.left;
 		int y = outPt.v - rect.top;
-		int [] theControl = new int [1];
-		OS.GetEventParameter (theEvent, OS.kEventParamDirectObject, OS.typeControlRef, null, 4, null, theControl);
-		OS.GetControlBounds (theControl [0], rect);
+		OS.GetControlBounds (parent.handle, rect);
 		x -= rect.left;
 		y -=  rect.top;
 		short [] button = new short [1];
 		OS.GetEventParameter (theEvent, OS.kEventParamMouseButton, OS.typeMouseButton, null, 2, null, button);
 		int chord = OS.GetCurrentEventButtonState ();
 		int modifiers = OS.GetCurrentEventKeyModifiers ();
-		parent.sendMouseEvent (SWT.MouseUp, button [0], chord, (short)x, (short)y, modifiers);
+		parent.sendMouseEvent (SWT.MouseUp, button [0], chord, (short)x, (short)y, modifiers, true);
 	}
 	tracking = false;
 	return result;
@@ -667,28 +697,27 @@ int kEventMouseUp (int nextHandler, int theEvent, int userData) {
 
 void register () {
 	super.register ();
-	WidgetTable.put (handle, this);
-	if (iconHandle != 0) WidgetTable.put (iconHandle, this);
-	if (labelHandle != 0) WidgetTable.put (labelHandle, this);
-	if (arrowHandle != 0) WidgetTable.put (arrowHandle, this);
+	display.addWidget (handle, this);
+	if (iconHandle != 0) display.addWidget (iconHandle, this);
+	if (labelHandle != 0) display.addWidget (labelHandle, this);
 }
 
 void releaseChild () {
 	super.releaseChild ();
+	setVisible (handle, false);
 	parent.destroyItem (this);
 }
 
 void releaseHandle () {
 	super.releaseHandle ();
-	handle = iconHandle = labelHandle = arrowHandle = 0;
+	handle = iconHandle = labelHandle = 0;
 }
 
 void releaseWidget () {
 	super.releaseWidget ();
 	if (cIcon != 0) destroyCIcon (cIcon);
 	if (labelCIcon != 0) destroyCIcon (labelCIcon);
-	if (arrowCIcon != 0) destroyCIcon (arrowCIcon);
-	cIcon = labelCIcon = arrowCIcon = 0;
+	cIcon = labelCIcon = 0;
 	if (visibleRgn != 0) OS.DisposeRgn (visibleRgn);
 	visibleRgn = 0;
 	parent = null;
@@ -701,7 +730,7 @@ void releaseWidget () {
  * Removes the listener from the collection of listeners who will
  * be notified when the control is selected.
  *
- * @param listener the listener which should be notified
+ * @param listener the listener which should no longer be notified
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
@@ -740,19 +769,26 @@ void selectRadio () {
 	setSelection (true);
 }
 
-void setBounds (int x, int y, int width, int height) {
-	if (control != null) control.setBounds (x, y, width, height);
+void setBackground (float [] color) {
+	parent.setBackground (handle, color);
+	if (labelHandle != 0) {
+		parent.setBackground (labelHandle, color);
+		updateText (false);
+	}
+	if (iconHandle != 0) parent.setBackground (iconHandle, color);
+}
+
+void setBounds (int x, int y, int width, int height, GC gc) {
 	setBounds (handle, x, y, width, height, true, true, false);
 	if ((style & SWT.SEPARATOR) != 0) return;
 	int space = 0;
 	int inset = 3;
 	int stringWidth = 0, stringHeight = 0;
 	if (text.length () != 0) {
-		GC gc = new GC (parent);
-		Point size = gc.stringExtent (text);
+		int flags = SWT.DRAW_DELIMITER | SWT.DRAW_TAB | SWT.DRAW_MNEMONIC | SWT.DRAW_TRANSPARENT;
+		Point size = gc.textExtent (text, flags);
 		stringWidth = size.x;
 		stringHeight = size.y;
-		gc.dispose ();
 	}
 	int imageWidth = 0, imageHeight = 0;
 	if (image != null) {
@@ -761,10 +797,9 @@ void setBounds (int x, int y, int width, int height) {
 		imageWidth = rect.width;
 		imageHeight = rect.height;
 	}
-	int arrowWidth = 0, arrowHeight = 0;
+	int arrowWidth = 0;
 	if ((style & SWT.DROP_DOWN) != 0) {
-		arrowWidth = 6;
-		arrowHeight = 4; //NOT DONE
+		arrowWidth = ARROW_WIDTH;
 	}
 	if ((parent.style & SWT.RIGHT) != 0) {
 		int imageX = inset;
@@ -774,18 +809,13 @@ void setBounds (int x, int y, int width, int height) {
 		int labelY = inset + (height - (inset * 2) - stringHeight) / 2;
 		setBounds (labelHandle, labelX, labelY, stringWidth, stringHeight, true, true, false);
 	} else {
-		int imageX = inset + (width - (inset * 2) - (arrowWidth + 3) - imageWidth) / 2;
-		int imageY = inset;
+		int imageX = inset + (width - (inset * 2) - arrowWidth - imageWidth) / 2;
+		int imageY = inset + (height - imageHeight - stringHeight - inset * 2) / 2;
 		setBounds (iconHandle, imageX, imageY, imageWidth, imageHeight, true, true, false);
-		int labelX = inset + (width - (inset * 2) - (arrowWidth + 3) - stringWidth) / 2;
+		int labelX = inset + (width - (inset * 2) - arrowWidth - stringWidth) / 2;
 		int labelY = imageY + imageHeight + space;
 		setBounds (labelHandle, labelX, labelY, stringWidth, stringHeight, true, true, false);
 	}
-	if ((style & SWT.DROP_DOWN) != 0) {
-		int arrowX = width - inset - arrowWidth;
-		int arrowY = inset + (height - (inset * 2) - arrowHeight) / 2;
-		setBounds (arrowHandle, arrowX, arrowY, arrowWidth, arrowHeight, true, true, false);
-	}
 }
 
 /**
@@ -810,10 +840,13 @@ public void setControl (Control control) {
 		if (control.parent != parent) error (SWT.ERROR_INVALID_PARENT);
 	}
 	if ((style & SWT.SEPARATOR) == 0) return;
+	if (this.control == control) return;
 	this.control = control;
+	redrawWidget (handle, false);
 	if (control != null && !control.isDisposed ()) {
-		control.setBounds (getBounds ());
+		control.moveAbove (null);
 	}
+	parent.relayout ();
 }
 
 /**
@@ -847,18 +880,17 @@ public void setEnabled (boolean enabled) {
 
 void setFontStyle (Font font) {
 	/* This code is intentionaly commented. */
-//	ControlFontStyleRec fontStyle = new ControlFontStyleRec ();
-//	if (font != null) {
-//		fontStyle.flags |= OS.kControlUseFontMask | OS.kControlUseSizeMask | OS.kControlUseFaceMask;
-//		fontStyle.font = font.id;
-//		fontStyle.style = font.style;
-//		fontStyle.size = font.size;
-//	} else {
-//		fontStyle.flags |= OS.kControlUseThemeFontIDMask;
-//		fontStyle.font = (short) defaultThemeFont ();
-//	}
-//	OS.SetControlFontStyle (labelHandle, fontStyle);
-	updateText ();
+//	parent.setFontStyle (labelHandle, font);
+	updateText (false);
+}
+
+void setForeground (float [] color) {
+	parent.setForeground (handle, color);
+	if (labelHandle != 0) {
+		parent.setForeground (labelHandle, color);
+		updateText (false);
+	}
+	if (iconHandle != 0) parent.setForeground (iconHandle, color);
 }
 
 /**
@@ -883,7 +915,7 @@ public void setDisabledImage (Image image) {
 	if (image != null && image.isDisposed()) error(SWT.ERROR_INVALID_ARGUMENT);
 	if ((style & SWT.SEPARATOR) != 0) return;
 	disabledImage = image;
-	updateImage ();
+	updateImage (true);
 }
 
 /**
@@ -908,7 +940,7 @@ public void setHotImage (Image image) {
 	if (image != null && image.isDisposed()) error(SWT.ERROR_INVALID_ARGUMENT);
 	if ((style & SWT.SEPARATOR) != 0) return;
 	hotImage = image;
-	updateImage ();
+	updateImage (true);
 }
 
 public void setImage (Image image) {
@@ -916,7 +948,7 @@ public void setImage (Image image) {
 	if (image != null && image.isDisposed()) error(SWT.ERROR_INVALID_ARGUMENT);
 	if ((style & SWT.SEPARATOR) != 0) return;
 	super.setImage (image);
-	updateImage ();
+	updateImage (true);
 }
 
 boolean setRadioSelection (boolean value) {
@@ -948,17 +980,10 @@ public void setSelection (boolean selected) {
 	if ((style & (SWT.CHECK | SWT.RADIO)) == 0) return;
 	int transform = selected ? OS.kTransformSelected : 0;
 	OS.SetControlData (iconHandle, OS.kControlEntireControl, OS.kControlIconTransformTag, 2, new short [] {(short)transform});
-	OS.SetControlData (labelHandle, OS.kControlEntireControl, OS.kControlIconTransformTag, 2, new short [] {(short)transform});
-	redrawWidget (handle, true);
-}
-
-void setSize (int width, int height, boolean layout) {
-	Rect rect = new Rect();
-	OS.GetControlBounds (handle, rect);
-	if ((rect.right - rect.left) != width || (rect.bottom - rect.top) != height) {
-		setBounds (handle, 0, 0, width, height, false, true, false);
-		if (layout) parent.relayout ();
+	if (image == null) {
+		OS.SetControlData (labelHandle, OS.kControlEntireControl, OS.kControlIconTransformTag, 2, new short [] {(short)transform});
 	}
+	redrawWidget (handle, true);
 }
 
 /**
@@ -991,7 +1016,7 @@ public void setText (String string) {
 	if (string == null) error (SWT.ERROR_NULL_ARGUMENT);
 	if ((style & SWT.SEPARATOR) != 0) return;
 	super.setText (string);
-	updateText ();
+	updateText (true);
 }
 
 /**
@@ -1011,7 +1036,7 @@ public void setToolTipText (String string) {
 }
 
 /**
- * Sets the width of the receiver.
+ * Sets the width of the receiver, for <code>SEPARATOR</code> ToolItems.
  *
  * @param width the new width
  *
@@ -1023,23 +1048,20 @@ public void setToolTipText (String string) {
 public void setWidth (int width) {
 	checkWidget();
 	if ((style & SWT.SEPARATOR) == 0) return;
-	if (width < 0) return;
-	Rect rect = new Rect ();
-	OS.GetControlBounds (handle, rect);
-	setSize (width, rect.bottom - rect.top, true);
-	if (control != null && !control.isDisposed ()) {
-		control.setBounds (getBounds ());
-	}
+	if (width < 0 || this.width == width) return;
+	this.width = width;
+	redrawWidget (handle, false);
+	parent.relayout();
 }
 
 void setZOrder () {
 	OS.HIViewAddSubview (parent.handle, handle);
 	if (iconHandle != 0) OS.HIViewAddSubview (handle, iconHandle);
 	if (labelHandle != 0) OS.HIViewAddSubview (handle, labelHandle);
-	if (arrowHandle != 0) OS.HIViewAddSubview (handle, arrowHandle);
 }
 
-void updateImage () {
+void updateImage (boolean layout) {
+	if ((style & SWT.SEPARATOR) != 0) return;
 	if (cIcon != 0) destroyCIcon (cIcon);
 	cIcon = 0;
 	Image image = null;
@@ -1059,71 +1081,50 @@ void updateImage () {
 		inContent.iconRef = cIcon;
 	}
 	OS.SetBevelButtonContentInfo (iconHandle, inContent);
-	redrawWidget (iconHandle, false);
-	Point size = computeSize ();
-	setSize (size.x, size.y, true);
+	if (layout) {
+		redrawWidget (iconHandle, false);
+		parent.relayout();
+	}
 }
 
-void updateArrow () {
-	if (arrowCIcon != 0) destroyCIcon (arrowCIcon);
-	arrowCIcon = 0;
-	Image image = new Image (display, 7, 4);
-	GC gc = new GC (image);
-	int startX = 0, startY = 0;
-	int [] arrow = {startX, startY, startX + 3, startY + 3, startX + 6, startY};
-	gc.setBackground (parent.getForeground ());
-	gc.fillPolygon (arrow);
-	gc.drawPolygon (arrow);
-	gc.dispose ();
-	ImageData data = image.getImageData ();
-	data.transparentPixel = 0xFFFFFFFF;
-	image.dispose ();
-	image = new Image (display, data, data.getTransparencyMask());
-	arrowCIcon = createCIcon (image);
-	image.dispose ();
-	ControlButtonContentInfo inContent = new ControlButtonContentInfo ();
-	inContent.contentType = (short) OS.kControlContentCIconHandle;
-	inContent.iconRef = arrowCIcon;
-	OS.SetBevelButtonContentInfo (arrowHandle, inContent);	
-}
-
-void updateText () {
+void updateText (boolean layout) {
+	if ((style & SWT.SEPARATOR) != 0) return;
 	if (labelCIcon != 0) destroyCIcon (labelCIcon);
 	labelCIcon = 0;
 	ControlButtonContentInfo inContent = new ControlButtonContentInfo ();
 	if (text.length () > 0) {
-		char [] buffer = new char [text.length ()];
-		text.getChars (0, buffer.length, buffer, 0);
-		int i=0, j=0;
-		while (i < buffer.length) {
-			if ((buffer [j++] = buffer [i++]) == Mnemonic) {
-				if (i == buffer.length) {continue;}
-				if (buffer [i] == Mnemonic) {i++; continue;}
-				j--;
-			}
-		}
 		Font font = parent.getFont ();
 		GC gc = new GC (parent);
-		Point size = gc.stringExtent (text);
+		int flags = SWT.DRAW_DELIMITER | SWT.DRAW_TAB | SWT.DRAW_MNEMONIC | SWT.DRAW_TRANSPARENT;
+		Point size = gc.textExtent (text, flags);
 		gc.dispose ();
 		Image image = new Image (display, size.x, size.y);
 		gc = new GC (image);
+		Color foreground = parent.getForeground ();
+		gc.setForeground (foreground);
+		if (parent.background != null) {
+			gc.setBackground (parent.getBackground ());
+			gc.fillRectangle (0, 0, size.x, size.y);
+		}
 		gc.setFont (font);
-		gc.drawString (text, 0, 0);
+		gc.drawText (text, 0, 0, flags);
 		gc.dispose ();
-		ImageData data = image.getImageData ();
-		data.transparentPixel = 0xFFFFFFFF;
-		image.dispose ();
-		image = new Image (display, data, data.getTransparencyMask());
+		if (parent.background == null) {
+			ImageData data = image.getImageData ();
+			data.transparentPixel = 0xFFFFFF;
+			image.dispose ();
+			image = new Image (display, data, data.getTransparencyMask ());
+		}
 		labelCIcon = createCIcon (image);
 		image.dispose ();
 		inContent.contentType = (short) OS.kControlContentCIconHandle;
 		inContent.iconRef = labelCIcon;
 	}
 	OS.SetBevelButtonContentInfo (labelHandle, inContent);	
-	redrawWidget (labelHandle, false);
-	Point size = computeSize ();
-	setSize (size.x, size.y, true);
+	if (layout) {
+		redrawWidget (labelHandle, false);
+		parent.relayout();
+	}
 }
 
 }
