@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -22,7 +22,12 @@ import org.eclipse.swt.internal.gtk.*;
  * can be drawn on by sending messages to the associated GC.
  */
 public abstract class Device implements Drawable {
-	
+	/**
+	 * the handle to the X Display
+	 */
+	int /*long*/ xDisplay;
+	int /*long*/ shellHandle;
+
 	/* Debugging */
 	public static boolean DEBUG;
 	boolean debug = DEBUG;
@@ -38,13 +43,18 @@ public abstract class Device implements Drawable {
 	boolean disposed;
 	
 	/* Warning and Error Handlers */
-	int logProc;
+	int /*long*/ logProc;
 	Callback logCallback;
 	//NOT DONE - get list of valid names
 	String [] log_domains = {"GLib-GObject", "GLib", "GObject", "Pango", "ATK", "GdkPixbuf", "Gdk", "Gtk", "GnomeVFS"};
 	int [] handler_ids = new int [log_domains.length];
 	int warningLevel;
 	
+	/* X Warning and Error Handlers */
+	static Callback XErrorCallback, XIOErrorCallback;
+	static int /*long*/ XErrorProc, XIOErrorProc, XNullErrorProc, XNullIOErrorProc;
+	static Device[] Devices = new Device[4];
+
 	/*
 	* The following colors are listed in the Windows
 	* Programmer's Reference as the colors in the default
@@ -57,6 +67,10 @@ public abstract class Device implements Drawable {
 	/* System Font */
 	Font systemFont;
 	
+	int /*long*/ emptyTab;
+
+	static final Object CREATE_LOCK = new Object();
+
 	/*
 	* TEMPORARY CODE. When a graphics object is
 	* created and the device parameter is null,
@@ -80,13 +94,28 @@ public abstract class Device implements Drawable {
 /*
 * TEMPORARY CODE.
 */
-static Device getDevice () {
+static synchronized Device getDevice () {
 	if (DeviceFinder != null) DeviceFinder.run();
 	Device device = CurrentDevice;
 	CurrentDevice = null;
 	return device;
 }
 
+/**
+ * Constructs a new instance of this class.
+ * <p>
+ * You must dispose the device when it is no longer required. 
+ * </p>
+ *
+ * @see #create
+ * @see #init
+ * 
+ * @since 3.1
+ */
+public Device() {
+	this(null);
+}
+
 /**
  * Constructs a new instance of this class.
  * <p>
@@ -100,19 +129,30 @@ static Device getDevice () {
  * @see DeviceData
  */
 public Device(DeviceData data) {
-	if (data != null) {
-		debug = data.debug;
-		tracking = data.tracking;
+	synchronized (CREATE_LOCK) {
+		if (data != null) {
+			debug = data.debug;
+			tracking = data.tracking;
+		}
+		if (tracking) {
+			errors = new Error [128];
+			objects = new Object [128];
+		}
+		create (data);
+		init ();
+		register (this);
+	
+		/* Initialize the system font slot */
+		systemFont = getSystemFont ();
 	}
-	if (tracking) {
-		errors = new Error [128];
-		objects = new Object [128];
-	}
-	create (data);
-	init ();
+}
 
-	/* Initialize the system font slot */
-	systemFont = getSystemFont ();
+void checkCairo() {
+	try {
+		Class.forName("org.eclipse.swt.internal.cairo.Cairo");
+	} catch (Throwable t) {
+		SWT.error(SWT.ERROR_NO_GRAPHICS_LIBRARY, t, " [Cairo is required]");
+	}
 }
 
 /**
@@ -171,6 +211,8 @@ public void dispose () {
 	checkDevice ();
 	release ();
 	destroy ();
+	deregister (this);
+	xDisplay = 0;
 	disposed = true;
 	if (tracking) {
 		objects = null;
@@ -188,6 +230,22 @@ void dispose_Object (Object object) {
 	}
 }
 
+static synchronized Device findDevice (int /*long*/ xDisplay) {
+	for (int i=0; i<Devices.length; i++) {
+		Device device = Devices [i];
+		if (device != null && device.xDisplay == xDisplay) {
+			return device;
+		}
+	}
+	return null;
+}
+
+synchronized static void deregister (Device device) {
+	for (int i=0; i<Devices.length; i++) {
+		if (device == Devices [i]) Devices [i] = null;
+	}
+}
+
 /**
  * Destroys the device in the operating system and releases
  * the device's handle.  If the device does not have a handle,
@@ -311,50 +369,58 @@ public Point getDPI () {
  * <code>faceName</code> is null, all fonts will be returned.
  *
  * @param faceName the name of the font to look for, or null
- * @param scalable true if scalable fonts should be returned.
+ * @param scalable if true only scalable fonts are returned, otherwise only non-scalable fonts are returned.
  * @return the matching font data
  *
  * @exception SWTException <ul>
  *    <li>ERROR_DEVICE_DISPOSED - if the receiver has been disposed</li>
  * </ul>
  */
-public FontData[] getFontList (String faceName, boolean scalable) {	
+public FontData[] getFontList (String faceName, boolean scalable) {
 	checkDevice ();
-
-	//FIXME - check scalable flag
-	int[] family = new int[1];
-	int[] face = new int[1];
-	int[] families = new int[1];
+	if (!scalable) return new FontData[0];
+	int /*long*/[] family = new int /*long*/[1];
+	int /*long*/[] face = new int /*long*/[1];
+	int /*long*/[] families = new int /*long*/[1];
 	int[] n_families = new int[1];
-	int[] faces = new int[1];
+	int /*long*/[] faces = new int /*long*/[1];
 	int[] n_faces = new int[1];
-	int context = OS.gdk_pango_context_get();
+	int /*long*/ context = OS.gdk_pango_context_get();
 	OS.pango_context_list_families(context, families, n_families);
 	int nFds = 0;
 	FontData[] fds = new FontData[faceName != null ? 4 : n_families[0]];
 	for (int i=0; i<n_families[0]; i++) {
-		OS.memmove(family, families[0] + i * 4, 4);
-		OS.pango_font_family_list_faces(family[0], faces, n_faces);
-		for (int j=0; j<n_faces[0]; j++) {
-			OS.memmove(face, faces[0] + j * 4, 4);
-			int fontDesc = OS.pango_font_face_describe(face[0]);
-			Font font = Font.gtk_new(this, fontDesc);
-			FontData data = font.getFontData()[0];
-			if (faceName == null || Compatibility.equalsIgnoreCase(faceName, data.name)) {
+		OS.memmove(family, families[0] + i * OS.PTR_SIZEOF, OS.PTR_SIZEOF);
+		boolean match = true;
+		if (faceName != null) {
+			int /*long*/ familyName = OS.pango_font_family_get_name(family[0]);
+			int length = OS.strlen(familyName);
+			byte[] buffer = new byte[length];
+			OS.memmove(buffer, familyName, length);
+			String name = new String(Converter.mbcsToWcs(null, buffer));
+			match = Compatibility.equalsIgnoreCase(faceName, name);
+		}
+		if (match) {
+		    OS.pango_font_family_list_faces(family[0], faces, n_faces);
+		    for (int j=0; j<n_faces[0]; j++) {
+		        OS.memmove(face, faces[0] + j * OS.PTR_SIZEOF, OS.PTR_SIZEOF);
+		        int /*long*/ fontDesc = OS.pango_font_face_describe(face[0]);
+		        Font font = Font.gtk_new(this, fontDesc);
+		        FontData data = font.getFontData()[0];
 				if (nFds == fds.length) {
 					FontData[] newFds = new FontData[fds.length + n_families[0]];
 					System.arraycopy(fds, 0, newFds, 0, nFds);
 					fds = newFds;
 				}
 				fds[nFds++] = data;
-			}
-			OS.pango_font_description_free(fontDesc);
+				OS.pango_font_description_free(fontDesc);
+		    }
+		    OS.g_free(faces[0]);
+		    if (faceName != null) break;
 		}
-		OS.g_free(faces[0]);
 	}
 	OS.g_free(families[0]);
 	OS.g_object_unref(context);
-
 	if (nFds == fds.length) return fds;
 	FontData[] result = new FontData[nFds];
 	System.arraycopy(fds, 0, result, 0, nFds);
@@ -367,7 +433,7 @@ public FontData[] getFontList (String faceName, boolean scalable) {
  * specified in class <code>SWT</code>. Any value other
  * than one of the SWT color constants which is passed
  * in will result in the color black. This color should
- * not be free'd because it was allocated by the system,
+ * not be freed because it was allocated by the system,
  * not the application.
  *
  * @param id the color constant
@@ -406,7 +472,7 @@ public Color getSystemColor (int id) {
  * Returns a reasonable font for applications to use.
  * On some platforms, this will match the "default font"
  * or "system font" if such can be found.  This font
- * should not be free'd because it was allocated by the
+ * should not be freed because it was allocated by the
  * system, not the application.
  * <p>
  * Typically, applications which want the default look
@@ -456,6 +522,34 @@ public boolean getWarnings () {
  * @see #create
  */
 protected void init () {
+	if (OS.GDK_WINDOWING_X11()) {
+		xDisplay = OS.GDK_DISPLAY ();
+	}
+
+	if (debug) {
+		if (OS.GDK_WINDOWING_X11()) {
+			/* Create the warning and error callbacks */
+			Class clazz = getClass ();
+			synchronized (clazz) {
+				int index = 0;
+				while (index < Devices.length) {
+					if (Devices [index] != null) break;
+					index++;
+				}
+				if (index == Devices.length) {
+					XErrorCallback = new Callback (clazz, "XErrorProc", 2);
+					XNullErrorProc = XErrorCallback.getAddress ();
+					if (XNullErrorProc == 0) SWT.error (SWT.ERROR_NO_MORE_CALLBACKS);
+					XIOErrorCallback = new Callback (clazz, "XIOErrorProc", 1);
+					XNullIOErrorProc = XIOErrorCallback.getAddress ();
+					if (XNullIOErrorProc == 0) SWT.error (SWT.ERROR_NO_MORE_CALLBACKS);
+					XErrorProc = OS.XSetErrorHandler (XNullErrorProc);
+					XIOErrorProc = OS.XSetIOErrorHandler (XNullIOErrorProc);
+				}
+			}
+			OS.XSynchronize (xDisplay, true);
+		}
+	}
 	
 	/* Create GTK warnings and error callbacks */
 	logCallback = new Callback (this, "logProc", 4);
@@ -488,6 +582,14 @@ protected void init () {
 	COLOR_MAGENTA = new Color (this, 0xFF,0,0xFF);
 	COLOR_CYAN = new Color (this, 0,0xFF,0xFF);
 	COLOR_WHITE = new Color (this, 0xFF,0xFF,0xFF);
+
+	emptyTab = OS.pango_tab_array_new(1, false);
+	if (emptyTab == 0) SWT.error(SWT.ERROR_NO_HANDLES);
+	OS.pango_tab_array_set_tab(emptyTab, 0, OS.PANGO_TAB_LEFT, 1);
+
+	shellHandle = OS.gtk_window_new(OS.GTK_WINDOW_TOPLEVEL);
+	if (shellHandle == 0) SWT.error(SWT.ERROR_NO_HANDLES);
+	OS.gtk_widget_realize(shellHandle);
 }
 
 /**	 
@@ -503,7 +605,7 @@ protected void init () {
  * @param data the platform specific GC data 
  * @return the platform specific GC handle
  */
-public abstract int internal_new_GC (GCData data);
+public abstract int /*long*/ internal_new_GC (GCData data);
 
 /**	 
  * Invokes platform specific functionality to dispose a GC handle.
@@ -515,10 +617,10 @@ public abstract int internal_new_GC (GCData data);
  * application code.
  * </p>
  *
- * @param handle the platform specific GC handle
+ * @param hDC the platform specific GC handle
  * @param data the platform specific GC data 
  */
-public abstract void internal_dispose_GC (int handle, GCData data);
+public abstract void internal_dispose_GC (int /*long*/ handle, GCData data);
 
 /**
  * Returns <code>true</code> if the device has been disposed,
@@ -534,10 +636,12 @@ public boolean isDisposed () {
 	return disposed;
 }
 
-int logProc (int log_domain, int log_level, int message, int user_data) {
-	if (DEBUG || (debug && warningLevel == 0)) {
-		new Error ().printStackTrace ();
-		OS.g_log_default_handler (log_domain, log_level, message, 0);
+int /*long*/ logProc (int /*long*/ log_domain, int /*long*/ log_level, int /*long*/ message, int /*long*/ user_data) {
+	if (warningLevel == 0) {
+		if (DEBUG || debug) {
+			new Error ().printStackTrace ();
+		}
+		OS.g_log_default_handler (log_domain, (int)/*64*/log_level, message, 0);
 	}
 	return 0;
 }
@@ -560,6 +664,19 @@ void new_Object (Object object) {
 	errors = newErrors;
 }
 
+static synchronized void register (Device device) {
+	for (int i=0; i<Devices.length; i++) {
+		if (Devices [i] == null) {
+			Devices [i] = device;
+			return;
+		}
+	}
+	Device [] newDevices = new Device [Devices.length + 4];
+	System.arraycopy (Devices, 0, newDevices, 0, Devices.length);
+	newDevices [Devices.length] = device;
+	Devices = newDevices;
+}
+
 /**
  * Releases any internal resources back to the operating
  * system and clears all fields except the device handle.
@@ -584,8 +701,11 @@ void new_Object (Object object) {
  * @see #destroy
  */
 protected void release () {
+	if (shellHandle != 0) OS.gtk_widget_destroy(shellHandle);
+	shellHandle = 0;
+
 	if (gdkColors != null) {
-		int colormap = OS.gdk_colormap_get_system();
+		int /*long*/ colormap = OS.gdk_colormap_get_system();
 		for (int i = 0; i < gdkColors.length; i++) {
 			GdkColor color = gdkColors [i];
 			if (color != null) {
@@ -602,6 +722,9 @@ protected void release () {
 	COLOR_DARK_MAGENTA = COLOR_DARK_CYAN = COLOR_GRAY = COLOR_DARK_GRAY = COLOR_RED =
 	COLOR_GREEN = COLOR_YELLOW = COLOR_BLUE = COLOR_MAGENTA = COLOR_CYAN = COLOR_WHITE = null;
 		
+	if (emptyTab != 0) OS.pango_tab_array_free(emptyTab);
+	emptyTab = 0;
+
 	/* Free the GTK error and warning handler */
 	for (int i=0; i<handler_ids.length; i++) {
 		if (handler_ids [i] != 0) {
@@ -617,11 +740,11 @@ protected void release () {
 
 /**
  * If the underlying window system supports printing warning messages
- * to the console, setting warnings to <code>true</code> prevents these
- * messages from being printed. If the argument is <code>false</code>
+ * to the console, setting warnings to <code>false</code> prevents these
+ * messages from being printed. If the argument is <code>true</code> then
  * message printing is not blocked.
  *
- * @param warnings <code>true</code>if warnings should be handled, and <code>false</code> otherwise
+ * @param warnings <code>true</code>if warnings should be printed, and <code>false</code> otherwise
  *
  * @exception SWTException <ul>
  *    <li>ERROR_DEVICE_DISPOSED - if the receiver has been disposed</li>
@@ -652,4 +775,33 @@ public void setWarnings (boolean warnings) {
 	}
 }
 
+static int XErrorProc (int xDisplay, int xErrorEvent) {
+	Device device = findDevice (xDisplay);
+	if (device != null) {
+		if (device.warningLevel == 0) {
+			if (DEBUG || device.debug) {
+				new SWTError ().printStackTrace ();
+			}
+			OS.Call (XErrorProc, xDisplay, xErrorEvent);
+		}
+	} else {
+		if (DEBUG) new SWTError ().printStackTrace ();
+		OS.Call (XErrorProc, xDisplay, xErrorEvent);
+	}
+	return 0;
+}
+
+static int XIOErrorProc (int xDisplay) {
+	Device device = findDevice (xDisplay);
+	if (device != null) {
+		if (DEBUG || device.debug) {
+			new SWTError ().printStackTrace ();
+		}
+	} else {
+		if (DEBUG) new SWTError ().printStackTrace ();
+	}
+	OS.Call (XIOErrorProc, xDisplay, 0);
+	return 0;
+}
+
 }
