@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -45,8 +45,7 @@ import org.eclipse.swt.events.*;
  * <dd>DefaultSelection, Modify, Selection</dd>
  * </dl>
  * <p>
- * Note: Only one of the styles DROP_DOWN and SIMPLE 
- * may be specified.
+ * Note: Only one of the styles DROP_DOWN and SIMPLE may be specified.
  * </p><p>
  * IMPORTANT: This class is <em>not</em> intended to be subclassed.
  * </p>
@@ -54,6 +53,8 @@ import org.eclipse.swt.events.*;
  * @see List
  */
 public class Combo extends Composite {
+	int visibleCount = 5;
+
 	/**
 	 * the operating system limit for the number of characters
 	 * that the text field in an instance of this class can hold
@@ -115,9 +116,6 @@ public Combo (Composite parent, int style) {
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
- * @exception SWTError <ul>
- *    <li>ERROR_ITEM_NOT_ADDED - if the operation fails because of an operating system failure</li>
- * </ul>
  *
  * @see #add(String,int)
  */
@@ -151,9 +149,6 @@ public void add (String string) {
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
- * @exception SWTError <ul>
- *    <li>ERROR_ITEM_NOT_ADDED - if the operation fails because of an operating system failure</li>
- * </ul>
  *
  * @see #add(String)
  */
@@ -235,6 +230,33 @@ public void addSelectionListener(SelectionListener listener) {
 	addListener (SWT.Selection,typedListener);
 	addListener (SWT.DefaultSelection,typedListener);
 }
+/**
+ * Adds the listener to the collection of listeners who will
+ * be notified when the receiver's text is verified, by sending
+ * it one of the messages defined in the <code>VerifyListener</code>
+ * interface.
+ *
+ * @param listener the listener which should be notified
+ *
+ * @exception IllegalArgumentException <ul>
+ *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
+ * </ul>
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ *
+ * @see VerifyListener
+ * @see #removeVerifyListener
+ * 
+ * @since 3.1
+ */
+public void addVerifyListener (VerifyListener listener) {
+	checkWidget ();
+	if (listener == null) error (SWT.ERROR_NULL_ARGUMENT);
+	TypedListener typedListener = new TypedListener (listener);
+	addListener (SWT.Verify, typedListener);
+}
 static int checkStyle (int style) {
 	/*
 	* Feature in Windows.  It is not possible to create
@@ -378,6 +400,10 @@ void createHandle (int index) {
 	};
 	handle = OS.XmCreateComboBox (formHandle, null, argList2, argList2.length / 2);
 	if (handle == 0) error (SWT.ERROR_NO_HANDLES);
+	int [] argList3 = {OS.XmNtextField, 0};
+	OS.XtGetValues (handle, argList3, argList3.length / 2);
+	int [] argList4 = {OS.XmNverifyBell, 0};
+	OS.XtSetValues (argList3 [1], argList4, argList4.length / 2);
 }
 /**
  * Cuts the selected text.
@@ -386,9 +412,6 @@ void createHandle (int index) {
  * clipboard and then deleted from the widget.
  * </p>
  *
- * @exception IllegalArgumentException <ul>
- *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
- * </ul>
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
@@ -423,7 +446,6 @@ public void deselect (int index) {
 	OS.XtGetValues (handle, argList, argList.length / 2);
 	
 	if (OS.XmListPosSelected (argList[3], index + 1)) {
-		Display display = getDisplay ();
 		boolean warnings = display.getWarnings ();
 		display.setWarnings (false);
 		OS.XmTextSetString (argList[1], new byte[1]);
@@ -449,15 +471,12 @@ public void deselectAll () {
 	checkWidget();
 	int [] argList = {OS.XmNtextField, 0, OS.XmNlist, 0};
 	OS.XtGetValues (handle, argList, argList.length / 2);
-	Display display = getDisplay ();
 	boolean warnings = display.getWarnings ();
 	display.setWarnings (false);
 	OS.XmTextSetString (argList[1], new byte[1]);
 	display.setWarnings(warnings);	
 	OS.XmListDeselectAllItems (argList[3]);
 }
-
-
 /**
  * Returns the item at the given, zero-relative index in the
  * receiver's list. Throws an exception if the index is out
@@ -473,9 +492,6 @@ public void deselectAll () {
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
- * @exception SWTError <ul>
- *    <li>ERROR_CANNOT_GET_ITEM - if the operation fails because of an operating system failure</li>
- * </ul>
  */
 public String getItem (int index) {
 	checkWidget();
@@ -489,13 +505,14 @@ public String getItem (int index) {
 	int [] buffer1 = new int [1]; 
 	OS.memmove (buffer1, ptr, 4);
 	ptr = buffer1 [0];
+	int [] table = new int [] {display.tabMapping, display.crMapping};
 	int address = OS.XmStringUnparse (
 		ptr,
 		null,
 		OS.XmCHARSET_TEXT,
 		OS.XmCHARSET_TEXT,
-		null,
-		0,
+		table,
+		table.length,
 		OS.XmOUTPUT_ALL);
 	if (address == 0) error (SWT.ERROR_CANNOT_GET_ITEM);
 	int length = OS.strlen (address);
@@ -513,9 +530,6 @@ public String getItem (int index) {
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
- * @exception SWTError <ul>
- *    <li>ERROR_CANNOT_GET_COUNT - if the operation fails because of an operating system failure</li>
- * </ul>
  */
 public int getItemCount () {
 	checkWidget();
@@ -533,23 +547,20 @@ public int getItemCount () {
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
- * @exception SWTError <ul>
- *    <li>ERROR_CANNOT_GET_ITEM_HEIGHT - if the operation fails because of an operating system failure</li>
- * </ul>
  */
 public int getItemHeight () {
 	checkWidget();
 	int [] listHandleArgs = {OS.XmNlist, 0};
 	OS.XtGetValues (handle, listHandleArgs, listHandleArgs.length / 2);
-	int [] argList = {OS.XmNlistSpacing, 0, OS.XmNhighlightThickness, 0};
+	int [] argList = {OS.XmNlistSpacing, 0, OS.XmNhighlightThickness, 0, OS.XmNfontList, 0};
 	OS.XtGetValues (listHandleArgs[1], argList, argList.length / 2);
-	int spacing = argList [1], highlight = argList [3];
+	int spacing = argList [1], highlight = argList [3], fontList = argList [5];
 	/* Result is from empirical analysis on Linux and AIX */
-	return getFontHeight () + spacing + (2 * highlight);
+	return getFontHeight (fontList) + spacing + (2 * highlight);
 }
 /**
- * Returns an array of <code>String</code>s which are the items
- * in the receiver's list. 
+ * Returns a (possibly empty) array of <code>String</code>s which are
+ * the items in the receiver's list. 
  * <p>
  * Note: This is not the actual structure used by the receiver
  * to maintain its list of items, so modifying the array will
@@ -562,9 +573,6 @@ public int getItemHeight () {
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
- * @exception SWTError <ul>
- *    <li>ERROR_CANNOT_GET_ITEM - if the operation fails because of an operating system failure</li>
- * </ul>
  */
 public String [] getItems () {
 	checkWidget();
@@ -577,13 +585,14 @@ public String [] getItems () {
 	for (int i = 0; i < itemCount; i++) {
 		OS.memmove (buffer1, items, 4);
 		int ptr = buffer1 [0];
+		int [] table = new int [] {display.tabMapping, display.crMapping};
 		int address = OS.XmStringUnparse (
 			ptr,
 			null,
 			OS.XmCHARSET_TEXT,
 			OS.XmCHARSET_TEXT,
-			null,
-			0,
+			table,
+			table.length,
 			OS.XmOUTPUT_ALL);
 		if (address == 0) error (SWT.ERROR_CANNOT_GET_ITEM);
 		int length = OS.strlen (address);
@@ -595,13 +604,16 @@ public String [] getItems () {
 	}
 	return result;
 }
+int getMinimumHeight () {
+	return getTextHeight ();
+}
 String getNameText () {
 	return getText ();
 }
 /**
  * Returns the orientation of the receiver.
  *
- * @return the orientation bit.
+ * @return the orientation style
  * 
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -615,11 +627,16 @@ public int getOrientation () {
 	return style & (SWT.LEFT_TO_RIGHT | SWT.RIGHT_TO_LEFT);
 }
 /**
- * Returns a <code>Point</code> whose x coordinate is the start
- * of the selection in the receiver's text field, and whose y
- * coordinate is the end of the selection. The returned values
- * are zero-relative. An "empty" selection as indicated by
- * the the x and y coordinates having the same value.
+ * Returns a <code>Point</code> whose x coordinate is the
+ * character position representing the start of the selection
+ * in the receiver's text field, and whose y coordinate is the
+ * character position representing the end of the selection.
+ * An "empty" selection is indicated by the x and y coordinates
+ * having the same value.
+ * <p>
+ * Indexing is zero based.  The range of a selection is from
+ * 0..N where N is the number of characters in the widget.
+ * </p>
  *
  * @return a point representing the selection start and end
  *
@@ -668,7 +685,8 @@ public int getSelectionIndex () {
 }
 /**
  * Returns a string containing a copy of the contents of the
- * receiver's text field.
+ * receiver's text field, or an empty string if there are no
+ * contents.
  *
  * @return the receiver's text
  *
@@ -699,9 +717,6 @@ public String getText () {
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
- * @exception SWTError <ul>
- *    <li>ERROR_CANNOT_GET_ITEM_HEIGHT - if the operation fails because of an operating system failure</li>
- * </ul>
  */
 public int getTextHeight () {
 	checkWidget();
@@ -723,7 +738,7 @@ public int getTextHeight () {
 		OS.XtGetValues (handle, argList, argList.length / 2);	
 		int [] argList2 = {OS.XmNmarginHeight, 0};
 		OS.XtGetValues (argList[1], argList2, argList2.length / 2);	
-		int height = getFontHeight ();
+		int height = getFontHeight (font.handle);
 		XRectangle rect = new XRectangle ();
 		OS.XmWidgetGetDisplayRect (argList[1], rect);
 		height += (rect.y * 2) + (2 * argList2[1]);
@@ -748,6 +763,8 @@ public int getTextHeight () {
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
+ *
+ * @see #LIMIT
  */
 public int getTextLimit () {
 	checkWidget();
@@ -755,17 +772,50 @@ public int getTextLimit () {
 	OS.XtGetValues (handle, argList, argList.length / 2);
 	return OS.XmTextGetMaxLength (argList[1]);
 }
+/**
+ * Gets the number of items that are visible in the drop
+ * down portion of the receiver's list.
+ * <p>
+ * Note: This operation is a hint and is not supported on
+ * platforms that do not have this concept.
+ * </p>
+ *
+ * @return the number of items that are visible
+ *
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ * 
+ * @since 3.0
+ */
+public int getVisibleItemCount () {
+	checkWidget ();
+	if ((style & SWT.SIMPLE) != 0) return visibleCount;
+	int [] argList = new int [] {OS.XmNvisibleItemCount, 0};
+	OS.XtGetValues (handle, argList, argList.length / 2);
+	return argList [1];
+}
 void hookEvents () {
 	super.hookEvents ();
-	int windowProc = getDisplay ().windowProc;
-	OS.XtAddCallback (handle, OS.XmNselectionCallback, windowProc, SELECTION_CALLBACK);
-	int [] argList = {OS.XmNtextField, 0};
+	int windowProc = display.windowProc;
+	int [] argList = {OS.XmNlist, 0, OS.XmNtextField, 0};
 	OS.XtGetValues (handle, argList, argList.length / 2);
-	OS.XtAddCallback (argList[1], OS.XmNactivateCallback, windowProc, ACTIVATE_CALLBACK);
-	OS.XtAddCallback (argList[1], OS.XmNvalueChangedCallback, windowProc, VALUE_CHANGED_CALLBACK);
-	OS.XtAddEventHandler (argList[1], OS.KeyPressMask, false, windowProc, KEY_PRESS);
-	OS.XtAddEventHandler (argList[1], OS.KeyReleaseMask, false, windowProc, KEY_RELEASE);
-	OS.XtInsertEventHandler (argList[1], OS.FocusChangeMask, false, windowProc, FOCUS_CHANGE, OS.XtListTail);
+	int listHandle = argList [1];
+	int textHandle = argList [3];
+	OS.XtAddCallback (listHandle, OS.XmNbrowseSelectionCallback, windowProc, BROWSE_SELECTION_CALLBACK);
+	OS.XtAddEventHandler (listHandle, OS.KeyPressMask, false, windowProc, KEY_PRESS);
+	OS.XtAddEventHandler (listHandle, OS.KeyReleaseMask, false, windowProc, KEY_RELEASE);
+	OS.XtAddCallback (textHandle, OS.XmNactivateCallback, windowProc, ACTIVATE_CALLBACK);
+	OS.XtAddCallback (textHandle, OS.XmNvalueChangedCallback, windowProc, VALUE_CHANGED_CALLBACK);
+	OS.XtAddCallback (textHandle, OS.XmNmodifyVerifyCallback, windowProc, MODIFY_VERIFY_CALLBACK);
+	OS.XtAddEventHandler (textHandle, OS.ButtonPressMask, false, windowProc, BUTTON_PRESS);
+	OS.XtAddEventHandler (textHandle, OS.ButtonReleaseMask, false, windowProc, BUTTON_RELEASE);
+	OS.XtAddEventHandler (textHandle, OS.EnterWindowMask, false, windowProc, ENTER_WINDOW);
+	OS.XtAddEventHandler (textHandle, OS.LeaveWindowMask, false, windowProc, LEAVE_WINDOW);
+	OS.XtAddEventHandler (textHandle, OS.KeyPressMask, false, windowProc, KEY_PRESS);
+	OS.XtAddEventHandler (textHandle, OS.KeyReleaseMask, false, windowProc, KEY_RELEASE);
+	OS.XtInsertEventHandler (textHandle, OS.FocusChangeMask, false, windowProc, FOCUS_CHANGE, OS.XtListTail);
 }
 /**
  * Searches the receiver's list starting at the first item
@@ -807,6 +857,7 @@ public int indexOf (String string) {
  * returns -1.
  *
  * @param string the search item
+ * @param start the zero-relative index at which to begin the search
  * @return the index of the item
  *
  * @exception IllegalArgumentException <ul>
@@ -855,7 +906,6 @@ public int indexOf (String string, int start) {
  */
 public void paste () {
 	checkWidget();
-	Display display = getDisplay ();
 	boolean warnings = display.getWarnings ();
 	display.setWarnings (false);
 	int [] argList = {OS.XmNtextField, 0};
@@ -876,15 +926,12 @@ public void paste () {
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
- * @exception SWTError <ul>
- *    <li>ERROR_ITEM_NOT_REMOVED - if the operation fails because of an operating system failure</li>
- * </ul>
  */
 public void remove (int index) {
 	checkWidget();
 	if (index == -1) error (SWT.ERROR_INVALID_RANGE);
 	/*
-	* Feature in Motif.  An index out of range handled
+	* Feature in Motif.  An index out of range is handled
 	* correctly by the list widget but causes an unwanted
 	* Xm Warning.  The fix is to check the range before
 	* deleting an item.
@@ -894,7 +941,11 @@ public void remove (int index) {
 	if (!(0 <= index && index < argList [1])) {
 		error (SWT.ERROR_INVALID_RANGE);
 	}
-	OS.XmComboBoxDeletePos (handle, index + 1);
+	if (argList [1] == 1) {
+		removeAll ();
+	} else {
+		OS.XmComboBoxDeletePos (handle, index + 1);
+	}
 }
 /**
  * Removes the items from the receiver's list which are
@@ -911,35 +962,46 @@ public void remove (int index) {
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
- * @exception SWTError <ul>
- *    <li>ERROR_ITEM_NOT_REMOVED - if the operation fails because of an operating system failure</li>
- * </ul>
  */
 public void remove (int start, int end) {
 	checkWidget();
 	if (start > end) return;
-	/*
-	* Feature in Motif.  An index out of range handled
-	* correctly by the list widget but causes an unwanted
-	* Xm Warning.  The fix is to check the range before
-	* deleting an item.
-	*/
 	int [] argList = {OS.XmNitemCount, 0};
 	OS.XtGetValues (handle, argList, argList.length / 2);
-	if (!(0 <= start && start < argList [1])) {
+	if (!(0 <= start && start <= end && end < argList [1])) {
 		error (SWT.ERROR_INVALID_RANGE);
 	}
-	int newEnd = Math.min (end, argList [1] - 1);
-	for (int i = start; i <= newEnd; i++) {
-		OS.XmComboBoxDeletePos (handle, start + 1);	
+	if (argList [1] == (end - start + 1)) {
+		removeAll ();
+	} else {
+		for (int i = start; i <= end; i++) {
+			OS.XmComboBoxDeletePos (handle, start + 1);	
+		}
 	}
-	if (end >= argList [1]) error (SWT.ERROR_INVALID_RANGE);
 }
 void register () {
 	super.register ();
-	int [] argList = {OS.XmNtextField, 0};
+	int [] argList = {OS.XmNlist, 0, OS.XmNtextField, 0};
 	OS.XtGetValues (handle, argList, argList.length / 2);
-	WidgetTable.put(argList[1], this);
+	display.addWidget (argList[1], this);
+	display.addWidget (argList[3], this);
+}
+void releaseWidget () {
+	/*
+	* Bug in Motif.  Disposing a Combo while its list is visible
+	* causes Motif to crash.  The fix is to hide the drop down
+	* list before disposing the Combo. 
+	*/
+	if ((style & SWT.DROP_DOWN) != 0) {
+		int[] argList = new int[] {OS.XmNlist, 0};
+		OS.XtGetValues (handle, argList, argList.length / 2);
+		int parent = OS.XtParent (argList [1]);
+		while (parent != 0 && !OS.XtIsSubclass (parent, OS.shellWidgetClass ())) {
+			parent = OS.XtParent (parent);
+		}
+		if (parent != 0) OS.XtPopdown (parent);
+	}
+	super.releaseWidget ();
 }
 /**
  * Searches the receiver's list starting at the first item
@@ -956,28 +1018,27 @@ void register () {
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
- * @exception SWTError <ul>
- *    <li>ERROR_ITEM_NOT_REMOVED - if the operation fails because of an operating system failure</li>
- * </ul>
  */
 public void remove (String string) {
 	checkWidget();
 	if (string == null) error (SWT.ERROR_NULL_ARGUMENT);
-
 	byte [] buffer = Converter.wcsToMbcs (getCodePage (), encodeString(string), true);
 	int xmString = OS.XmStringCreateLocalized (buffer);
 	if (xmString == 0) error (SWT.ERROR_ITEM_NOT_REMOVED);
-	
-	int [] argList = {OS.XmNlist, 0};
+	int [] argList = {OS.XmNlist, 0, OS.XmNitemCount, 0};
 	OS.XtGetValues (handle, argList, argList.length / 2);
 	int index = OS.XmListItemPos (argList[1], xmString);
-	
 	OS.XmStringFree (xmString);	
 	if (index == 0) error (SWT.ERROR_INVALID_ARGUMENT);
-	OS.XmComboBoxDeletePos (handle, index);
+	if (argList [3] == 1) {
+		removeAll ();
+	} else {
+		OS.XmComboBoxDeletePos (handle, index);
+	}
 }
 /**
- * Removes all of the items from the receiver's list.
+ * Removes all of the items from the receiver's list and clear the
+ * contents of receiver's text field.
  * <p>
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -988,14 +1049,11 @@ public void removeAll () {
 	checkWidget();
 	int [] argList = {OS.XmNtextField, 0, OS.XmNlist, 0, OS.XmNitemCount, 0};
 	OS.XtGetValues (handle, argList, argList.length / 2);
-	
-	Display display = getDisplay ();
 	boolean warnings = display.getWarnings ();
 	display.setWarnings (false);
 	OS.XmTextSetString (argList[1], new byte[1]);
 	display.setWarnings(warnings);	
 	OS.XmListDeselectAllItems (argList[3]);
-	
 	for (int i = 0; i < argList[5]; i++) {
 		OS.XmComboBoxDeletePos(handle, 1);
 	}
@@ -1047,6 +1105,31 @@ public void removeSelectionListener (SelectionListener listener) {
 	eventTable.unhook (SWT.Selection, listener);
 	eventTable.unhook (SWT.DefaultSelection,listener);	
 }
+/**
+ * Removes the listener from the collection of listeners who will
+ * be notified when the control is verified.
+ *
+ * @param listener the listener which should no longer be notified
+ *
+ * @exception IllegalArgumentException <ul>
+ *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
+ * </ul>
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ *
+ * @see VerifyListener
+ * @see #addVerifyListener
+ * 
+ * @since 3.1
+ */
+public void removeVerifyListener (VerifyListener listener) {
+	checkWidget ();
+	if (listener == null) error (SWT.ERROR_NULL_ARGUMENT);
+	if (eventTable == null) return;
+	eventTable.unhook (SWT.Verify, listener);	
+}
 /**
  * Selects the item at the given zero-relative index in the receiver's 
  * list.  If the item at the index was already selected, it remains
@@ -1064,7 +1147,6 @@ public void select (int index) {
 	if (index == -1) {
 		int [] argList = {OS.XmNtextField, 0, OS.XmNlist, 0};
 		OS.XtGetValues (handle, argList, argList.length / 2);
-		Display display = getDisplay ();
 		boolean warnings = display.getWarnings ();
 		display.setWarnings (false);
 		OS.XmTextSetString (argList[1], new byte[1]);
@@ -1080,39 +1162,10 @@ public void select (int index) {
 		ignoreSelect = false;
 	}
 }
-void sendIMKeyEvent (int type, XKeyEvent xEvent, byte [] mbcs, char [] chars) {
-	/*
-	* Bug in Motif. On Solaris and Linux, XmImMbLookupString() clears
-	* the characters from the IME. This causes the characters to be
-	* stolen from the text widget. The fix is to detect that the IME
-	* has been cleared and use XmTextInsert() to insert the stolen
-	* characters. This problem does not happen on AIX.
-	*/
-	super.sendIMKeyEvent (type, xEvent, mbcs, chars);
-	if (mbcs == null || xEvent.keycode != 0) return;
+boolean sendIMKeyEvent (int type, XKeyEvent xEvent) {
 	int [] argList = {OS.XmNtextField, 0};
 	OS.XtGetValues (handle, argList, argList.length / 2);
-	int [] unused = new int [1];
-	byte [] buffer = new byte [2];
-	int length = OS.XmImMbLookupString (argList [1], xEvent, buffer, buffer.length, unused, unused);
-	if (length != 0) return;
-	int [] start = new int [1], end = new int [1];
-	OS.XmTextGetSelectionPosition (argList [1], start, end);
-	if (start [0] == end [0]) {
-		start [0] = end [0] = OS.XmTextGetInsertionPosition (argList [1]);
-	}
-	Display display = getDisplay();
-	boolean warnings = display.getWarnings ();
-	display.setWarnings (false);
-	OS.XmTextReplace (argList [1], start [0], end [0], mbcs);
-	int index = 0;
-	while (index < chars.length) {
-		if (chars [index] == 0) break;
-		index++;
-	}
-	int position = start [0] + index;
-	OS.XmTextSetInsertionPosition (argList [1], position);
-	display.setWarnings (warnings);
+	return super.sendIMKeyEvent (type, xEvent, argList [1]);
 }
 void setBackgroundPixel (int pixel) {
 	super.setBackgroundPixel (pixel);
@@ -1135,24 +1188,41 @@ boolean setBounds (int x, int y, int width, int height, boolean move, boolean re
 }
 public void setFont (Font font) {
 	checkWidget();
-	super.setFont (font);
 	
+	/*
+	* Bug in Motif. Setting the font in a combo widget that does
+	* not have any items causes a GP on UTF-8 locale.
+	* The fix is to add an item, change the font, then
+	* remove the added item at the end. 
+	*/
+	int [] argList1 = {OS.XmNitems, 0, OS.XmNitemCount, 0,};
+	OS.XtGetValues (handle, argList1, argList1.length / 2);
+	boolean fixString = OS.IsDBLocale && argList1 [3] == 0;
+	if (fixString) {
+		byte [] buffer = Converter.wcsToMbcs (getCodePage (), "string", true);
+		int xmString = OS.XmStringCreateLocalized (buffer);
+		OS.XmComboBoxAddItem (handle, xmString, -1, false);
+		OS.XmStringFree (xmString);
+	}
+	super.setFont (font);
+	if (fixString) OS.XtSetValues (handle, argList1, argList1.length / 2);	
+
 	/*
 	* Bug in Motif.  When a font is set in a combo box after the widget
 	* is realized, the combo box does not lay out properly.  For example,
 	* the drop down arrow may be positioned in the middle of the text
 	* field or may be invisible, positioned outside the bounds of the
 	* widget.  The fix is to detect these cases and force the combo box
-	* to be layed out properly by temporarily growing and then shrinking
+	* to be laid out properly by temporarily growing and then shrinking
 	* the widget.
 	* 
 	* NOTE: This problem also occurs for simple combo boxes.
 	*/
 	if (OS.XtIsRealized (handle)) {
-		int [] argList = {OS.XmNwidth, 0, OS.XmNheight, 0, OS.XmNborderWidth, 0};
-		OS.XtGetValues (handle, argList, argList.length / 2);
-		OS.XtResizeWidget (handle, argList [1], argList [3] + 1, argList [5]);
-		OS.XtResizeWidget (handle, argList [1], argList [3], argList [5]);
+		int [] argList2 = {OS.XmNwidth, 0, OS.XmNheight, 0, OS.XmNborderWidth, 0};
+		OS.XtGetValues (handle, argList2, argList2.length / 2);
+		OS.XtResizeWidget (handle, argList2 [1], argList2 [3] + 1, argList2 [5]);
+		OS.XtResizeWidget (handle, argList2 [1], argList2 [3], argList2 [5]);
 	}
 }
 void setForegroundPixel (int pixel) {
@@ -1166,23 +1236,20 @@ void setForegroundPixel (int pixel) {
 /**
  * Sets the text of the item in the receiver's list at the given
  * zero-relative index to the string argument. This is equivalent
- * to <code>remove</code>'ing the old item at the index, and then
- * <code>add</code>'ing the new item at that index.
+ * to removing the old item at the index, and then adding the new
+ * item at that index.
  *
  * @param index the index for the item
  * @param string the new text for the item
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_INVALID_RANGE - if the index is not between 0 and the number of elements in the list minus 1 (inclusive)</li>
+ *    <li>ERROR_NULL_ARGUMENT - if the string is null</li>
  * </ul>
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
- * @exception SWTError <ul>
- *    <li>ERROR_ITEM_NOT_REMOVED - if the remove operation fails because of an operating system failure</li>
- *    <li>ERROR_ITEM_NOT_ADDED - if the add operation fails because of an operating system failure</li>
- * </ul>
  */
 public void setItem (int index, String string) {
 	checkWidget();
@@ -1206,17 +1273,21 @@ public void setItem (int index, String string) {
  *
  * @param items the array of items
  *
+ * @exception IllegalArgumentException <ul>
+ *    <li>ERROR_NULL_ARGUMENT - if the items array is null</li>
+ *    <li>ERROR_INVALID_ARGUMENT - if an item in the items array is null</li>
+ * </ul>
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
- * @exception SWTError <ul>
- *    <li>ERROR_ITEM_NOT_ADDED - if the operation fails because of an operating system failure</li>
- * </ul>
  */
 public void setItems (String [] items) {
 	checkWidget();
 	if (items == null) error (SWT.ERROR_NULL_ARGUMENT);
+	for (int i=0; i<items.length; i++) {
+		if (items [i] == null) error (SWT.ERROR_INVALID_ARGUMENT);
+	}
 
 	if (items.length == 0) {
 		removeAll();
@@ -1228,7 +1299,6 @@ public void setItems (String [] items) {
 	String codePage = getCodePage ();
 	while (index < items.length) {
 		String string = items [index];
-		if (string == null) break; 
 		byte [] buffer = Converter.wcsToMbcs (codePage, encodeString(string), true);
 		int xmString = OS.XmStringCreateLocalized (buffer);
 		if (xmString == 0) break;
@@ -1244,7 +1314,6 @@ public void setItems (String [] items) {
 	
 	int [] argList2 = {OS.XmNtextField, 0};
 	OS.XtGetValues (handle, argList2, argList2.length / 2);
-	Display display = getDisplay ();
 	boolean warnings = display.getWarnings ();
 	display.setWarnings (false);
 	OS.XmTextSetString (argList2[1], new byte[1]);
@@ -1256,10 +1325,10 @@ public void setItems (String [] items) {
 }
 /**
  * Sets the orientation of the receiver, which must be one
- * of the constants <code>SWT.LEFT_TO_RIGHT</code> or <code>SWT.LEFT_TO_RIGHT</code>.
+ * of the constants <code>SWT.LEFT_TO_RIGHT</code> or <code>SWT.RIGHT_TO_LEFT</code>.
  * <p>
  *
- * @param orientation new orientation bit
+ * @param orientation new orientation style
  * 
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -1277,7 +1346,7 @@ public void setOrientation (int orientation) {
  * start of the selection and whose y coordinate is the end
  * of the selection. 
  *
- * @param a point representing the new selection start and end
+ * @param selection a point representing the new selection start and end
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the point is null</li>
@@ -1289,6 +1358,7 @@ public void setOrientation (int orientation) {
  */
 public void setSelection (Point selection) {
 	checkWidget();
+	if (selection == null) error (SWT.ERROR_NULL_ARGUMENT);
 	
 	int [] argList = {OS.XmNtextField, 0};
 	OS.XtGetValues (handle, argList, argList.length / 2);
@@ -1311,7 +1381,6 @@ public void setSelection (Point selection) {
 	if (xDisplay == 0) return;
 	int nStart = Math.min (Math.max (Math.min (selection.x, selection.y), 0), position);
 	int nEnd = Math.min (Math.max (Math.max (selection.x, selection.y), 0), position);
-	Display display = getDisplay ();
 	boolean warnings = display.getWarnings ();
 	display.setWarnings (false);
 	OS.XmTextSetSelection (argList[1], nStart, nEnd, OS.XtLastTimestampProcessed (xDisplay));
@@ -1331,7 +1400,7 @@ public void setSelection (Point selection) {
  * display incorrectly.
  * </p>
  *
- * @param text the new text
+ * @param string the new text
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the string is null</li>
@@ -1367,7 +1436,6 @@ public void setText (String string) {
 			OS.XmComboBoxSelectItem(handle, xmString);
 		} else {
 			/* The list does not contain the item. */
-			Display display = getDisplay ();
 			boolean warnings = display.getWarnings ();
 			display.setWarnings (false);
 			OS.XmTextSetString (argList[1], buffer);
@@ -1380,7 +1448,11 @@ public void setText (String string) {
 /**
  * Sets the maximum number of characters that the receiver's
  * text field is capable of holding to be the argument.
- *
+ * <p>
+ * To reset this value to the default, use <code>setTextLimit(Combo.LIMIT)</code>.
+ * Specifying a limit value larger than <code>Combo.LIMIT</code> sets the
+ * receiver's limit to <code>Combo.LIMIT</code>.
+ * </p>
  * @param limit new text limit
  *
  * @exception IllegalArgumentException <ul>
@@ -1390,6 +1462,8 @@ public void setText (String string) {
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
+ * 
+ * @see #LIMIT
  */
 public void setTextLimit (int limit) {
 	checkWidget();
@@ -1398,12 +1472,46 @@ public void setTextLimit (int limit) {
 	OS.XtGetValues (handle, argList, argList.length / 2);
 	OS.XmTextSetMaxLength (argList[1], limit);
 }
-
+/**
+ * Sets the number of items that are visible in the drop
+ * down portion of the receiver's list.
+ * <p>
+ * Note: This operation is a hint and is not supported on
+ * platforms that do not have this concept.
+ * </p>
+ *
+ * @param count the new number of items to be visible
+ *
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ * 
+ * @since 3.0
+ */
+public void setVisibleItemCount (int count) {
+	checkWidget ();
+	if (count < 0) return;
+	this.visibleCount = count;
+	/*
+	* But in Motif.  Setting the XmNvisibleItemCount resource
+	* for the combo box after it has been realized causes the
+	* widget to layout badly, sometimes moving the drop down
+	* arrow part of the combo box outside of the bounds.
+	* The fix is to set the XmNvisibleItemCount resource on
+	* the list instead.
+	*/
+	int [] argList1 = new int [] {OS.XmNlist, 0};
+	OS.XtGetValues (handle, argList1, argList1.length / 2);
+	int [] argList2 = {OS.XmNvisibleItemCount, count};
+	OS.XtSetValues (argList1 [1], argList2, argList2.length / 2);
+}
 void deregister () {
 	super.deregister ();
-	int [] argList = {OS.XmNtextField, 0};
+	int [] argList = {OS.XmNlist, 0, OS.XmNtextField, 0};
 	OS.XtGetValues (handle, argList, argList.length / 2);
-	WidgetTable.remove (argList[1]);
+	display.removeWidget (argList[1]);
+	display.removeWidget (argList[3]);
 }
 void enableWidget (boolean enabled) {
 	super.enableWidget (enabled);
@@ -1459,7 +1567,7 @@ int XmNactivateCallback (int w, int client_data, int call_data) {
 	postEvent (SWT.DefaultSelection);
 	return 0;
 }
-int XmNselectionCallback (int w, int client_data, int call_data) {
+int XmNbrowseSelectionCallback (int w, int client_data, int call_data) {
 	/*
 	* Bug in MOTIF.  If items have been added and removed from a
 	* combo then users are able to select an empty drop-down item
@@ -1471,6 +1579,45 @@ int XmNselectionCallback (int w, int client_data, int call_data) {
 	postEvent (SWT.Selection);
 	return 0;
 }
+int XmNmodifyVerifyCallback (int w, int client_data, int call_data) {
+	int result = super.XmNmodifyVerifyCallback (w, client_data, call_data);
+	if (result != 0) return result;
+	if (!hooks (SWT.Verify) && !filters (SWT.Verify)) return result;
+	XmTextVerifyCallbackStruct textVerify = new XmTextVerifyCallbackStruct ();
+	OS.memmove (textVerify, call_data, XmTextVerifyCallbackStruct.sizeof);
+	XmTextBlockRec textBlock = new XmTextBlockRec ();
+	OS.memmove (textBlock, textVerify.text, XmTextBlockRec.sizeof);
+	byte [] buffer = new byte [textBlock.length];
+	OS.memmove (buffer, textBlock.ptr, textBlock.length);
+	String codePage = getCodePage ();
+	String text = new String (Converter.mbcsToWcs (codePage, buffer));
+	Event event = new Event ();
+	if (textVerify.event != 0) {
+		XKeyEvent xEvent = new XKeyEvent ();
+		OS.memmove (xEvent, textVerify.event, XKeyEvent.sizeof);
+		event.time = xEvent.time;
+		setKeyState (event, xEvent);
+	}
+	event.start = textVerify.startPos;
+	event.end = textVerify.endPos;
+	event.doit = textVerify.doit == 1;
+	event.text = text;
+	sendEvent (SWT.Verify, event);
+	String newText = event.text;
+	textVerify.doit = (byte) ((event.doit && newText != null) ? 1 : 0);
+	if (newText != null && newText != text) {
+		OS.XtFree(textBlock.ptr);
+		byte [] buffer2 = Converter.wcsToMbcs (codePage, newText, true);
+		int length = buffer2.length;
+		int ptr = OS.XtMalloc (length);
+		OS.memmove (ptr, buffer2, length);
+		textBlock.ptr = ptr;
+		textBlock.length = buffer2.length - 1;
+		OS.memmove (textVerify.text, textBlock, XmTextBlockRec.sizeof);
+	}
+	OS.memmove (call_data, textVerify, XmTextVerifyCallbackStruct.sizeof);
+	return result;
+}
 int XmNvalueChangedCallback (int w, int client_data, int call_data) {
 	sendEvent (SWT.Modify);
 	return 0;
