@@ -1,16 +1,15 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
 package org.eclipse.swt.widgets;
 
-
 import org.eclipse.swt.internal.motif.*;
 import org.eclipse.swt.*;
 import org.eclipse.swt.graphics.*;
@@ -22,6 +21,11 @@ import org.eclipse.swt.graphics.*;
  * shares a significant amount of code with this class,
  * and is a subclass.
  * <p>
+ * IMPORTANT: This class was intended to be abstract and
+ * should <em>never</em> be referenced or instantiated.
+ * Instead, the class <code>Shell</code> should be used.
+ * </p>
+ * <p>
  * Instances are always displayed in one of the maximized, 
  * minimized or normal states:
  * <ul>
@@ -90,6 +94,7 @@ import org.eclipse.swt.graphics.*;
 public class Decorations extends Canvas {
 	String label;
 	Image image;
+	Image [] images = new Image [0];
 	int dialogHandle;
 	boolean minimized, maximized;
 	Menu menuBar;
@@ -140,6 +145,37 @@ Decorations () {
 public Decorations (Composite parent, int style) {
 	super (parent, checkStyle (style));
 }
+
+void _setImages (Image [] images) {
+	if (images != null && images.length > 1) {
+		Image [] bestImages = new Image [images.length];
+		System.arraycopy (images, 0, bestImages, 0, images.length);
+		sort (bestImages);
+		images = bestImages;
+	}
+	Image icon = images != null && images.length > 0 ? icon = images [0] : null;
+	int pixmap = 0, mask = 0;
+	if (icon != null) {
+		switch (icon.type) {
+			case SWT.BITMAP:
+				pixmap = icon.pixmap;
+				break;
+			case SWT.ICON:
+				pixmap = icon.pixmap;
+				mask = icon.mask;
+				break;
+			default:
+				error (SWT.ERROR_INVALID_IMAGE);
+		}
+	}
+	int [] argList = {
+		OS.XmNiconPixmap, pixmap,
+		OS.XmNiconMask, mask,
+	};
+	int topHandle = topHandle ();
+	OS.XtSetValues (topHandle, argList, argList.length / 2);
+}
+
 void add (Menu menu) {
 	if (menus == null) menus = new Menu [4];
 	for (int i=0; i<menus.length; i++) {
@@ -157,6 +193,9 @@ void bringToTop (boolean force) {
 	moveAbove (null);
 }
 static int checkStyle (int style) {
+	if ((style & SWT.NO_TRIM) != 0) {
+		style &= ~(SWT.CLOSE | SWT.TITLE | SWT.MIN | SWT.MAX | SWT.RESIZE | SWT.BORDER);
+	}
 	if ((style & (SWT.MENU | SWT.MIN | SWT.MAX | SWT.CLOSE)) != 0) {
 		style |= SWT.TITLE;
 	}
@@ -166,6 +205,23 @@ protected void checkSubclass () {
 	if (!isValidSubclass ()) error (SWT.ERROR_INVALID_SUBCLASS);
 }
 
+int compare (ImageData data1, ImageData data2) {
+	int transparent1 = data1.getTransparencyType ();
+	int transparent2 = data2.getTransparencyType ();
+	if (transparent1 != transparent2) {
+		if (transparent1 == SWT.TRANSPARENCY_ALPHA) return 1;
+		if (transparent2 == SWT.TRANSPARENCY_ALPHA) return -1;
+	}
+	if (data1.width == data2.width && data1.height == data2.height) {
+		if (transparent1 == SWT.TRANSPARENCY_MASK) return -1;
+		if (transparent2 == SWT.TRANSPARENCY_MASK) return 1;
+		if (transparent1 == SWT.TRANSPARENCY_PIXEL) return -1;
+		if (transparent2 == SWT.TRANSPARENCY_PIXEL) return 1;
+		return 0;
+	}
+	return data1.width > data2.width || data1.height > data2.height ? -1 : 1;
+}
+
 Control computeTabGroup () {
 	return this;
 }
@@ -189,7 +245,7 @@ public Rectangle computeTrim (int x, int y, int width, int height) {
 }
 void createHandle (int index) {
 	state |= HANDLE | CANVAS;
-	createScrolledHandle (parent.handle);
+	createHandle (index, parent.handle, true);
 }
 void createWidget (int index) {
 	super.createWidget (index);
@@ -241,6 +297,40 @@ public Image getImage () {
 	checkWidget();
 	return image;
 }
+/**
+ * Returns the receiver's images if they had previously been 
+ * set using <code>setImages()</code>. Images are typically
+ * displayed by the window manager when the instance is
+ * marked as iconified, and may also be displayed somewhere
+ * in the trim when the instance is in normal or maximized
+ * states. Depending where the icon is displayed, the platform
+ * chooses the icon with the "best" attributes.  It is expected
+ * that the array will contain the same icon rendered at different
+ * sizes, with different depth and transparency attributes.
+ * 
+ * <p>
+ * Note: This method will return an empty array if called before
+ * <code>setImages()</code> is called. It does not provide
+ * access to a window manager provided, "default" image
+ * even if one exists.
+ * </p>
+ * 
+ * @return the images
+ *
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ * 
+ * @since 3.0
+ */
+public Image [] getImages () {
+	checkWidget ();
+	if (images == null) return new Image [0];
+	Image [] result = new Image [images.length];
+	System.arraycopy (images, 0, result, 0, images.length);
+	return result;
+}
 /**
  * Returns <code>true</code> if the receiver is currently
  * maximized, and false otherwise. 
@@ -325,7 +415,7 @@ void propagateWidget (boolean enabled) {
 	super.propagateWidget (enabled);
 	int [] argList = {OS.XmNmenuBar, 0};
 	OS.XtGetValues (scrolledHandle, argList, argList.length / 2);
-	if (argList [1] != 0) propagateHandle (enabled, argList [1]);
+	if (argList [1] != 0) propagateHandle (enabled, argList [1], OS.None);
 }
 void releaseHandle () {
 	super.releaseHandle ();
@@ -341,6 +431,9 @@ void releaseWidget () {
 	menuBar = null;
 	menus = null;
 	super.releaseWidget ();
+	image = null;
+	images = null;
+	savedFocus = null;
 	defaultButton = saveDefault = null;
 	label = null;
 }
@@ -377,9 +470,13 @@ void remove (Menu menu) {
  * <em>saved default button</em>). If no default button had
  * previously been set, or the saved default button was
  * disposed, the receiver's default button will be set to
- * null. 
+ * null.
+ * <p>
+ * The default button is the button that is selected when
+ * the receiver is active and the user presses ENTER.
+ * </p>
  *
- * @param the new default button
+ * @param button the new default button
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_INVALID_ARGUMENT - if the button has been disposed</li> 
@@ -433,29 +530,45 @@ void setDefaultButton (Button button, boolean save) {
  */
 public void setImage (Image image) {
 	checkWidget();
-	int pixmap = 0, mask = 0;
-	if (image != null) {
-		if (image.isDisposed()) error(SWT.ERROR_INVALID_ARGUMENT);
-		switch (image.type) {
-			case SWT.BITMAP:
-				pixmap = image.pixmap;
-				break;
-			case SWT.ICON:
-				pixmap = image.pixmap;
-				mask = image.mask;
-				break;
-			default:
-				error (SWT.ERROR_INVALID_IMAGE);
-		}
-	}
+	if (image != null && image.isDisposed()) error(SWT.ERROR_INVALID_ARGUMENT);
 	this.image = image;
-	int [] argList = {
-		OS.XmNiconPixmap, pixmap,
-		OS.XmNiconMask, mask,
-	};
-	int topHandle = topHandle ();
-	OS.XtSetValues (topHandle, argList, argList.length / 2);
+	_setImages (image != null ? new Image [] {image} : null);
 }
+
+/**
+ * Sets the receiver's images to the argument, which may
+ * be an empty array. Images are typically displayed by the
+ * window manager when the instance is marked as iconified,
+ * and may also be displayed somewhere in the trim when the
+ * instance is in normal or maximized states. Depending where
+ * the icon is displayed, the platform chooses the icon with
+ * the "best" attributes. It is expected that the array will
+ * contain the same icon rendered at different sizes, with
+ * different depth and transparency attributes.
+ * 
+ * @param images the new image array
+ *
+ * @exception IllegalArgumentException <ul>
+ *    <li>ERROR_NULL_ARGUMENT - if the array of images is null</li>
+ *    <li>ERROR_INVALID_ARGUMENT - if one of the images is null or has been disposed</li>
+ * </ul>
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ * 
+ * @since 3.0
+ */
+public void setImages (Image [] images) {
+	checkWidget ();
+	if (images == null) error (SWT.ERROR_INVALID_ARGUMENT);
+	for (int i = 0; i < images.length; i++) {
+		if (images [i] == null || images [i].isDisposed ()) error (SWT.ERROR_INVALID_ARGUMENT);
+	}
+	this.images = images;
+	_setImages (images);
+}
+
 /**
  * Sets the maximized state of the receiver.
  * If the argument is <code>true</code> causes the receiver
@@ -464,13 +577,13 @@ public void setImage (Image image) {
  * causes the receiver to switch back to either the minimized
  * or normal states.
  * <p>
- * Note: The result of intermixing calls to<code>setMaximized(true)</code>
+ * Note: The result of intermixing calls to <code>setMaximized(true)</code>
  * and <code>setMinimized(true)</code> will vary by platform. Typically,
  * the behavior will match the platform user's expectations, but not
  * always. This should be avoided if possible.
  * </p>
  *
- * @param the new maximized state
+ * @param maximized the new maximized state
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -510,13 +623,13 @@ public void setMenuBar (Menu menu) {
 	/* Ensure the new menu bar is correctly enabled */
 	if (menuBar != null) {
 		if (!isEnabled () && menuBar.getEnabled ()) {
-			propagateHandle (true, menuBar.handle); 
+			propagateHandle (true, menuBar.handle, OS.None); 
 		}
 		menuBar.removeAccelerators ();
 	}
 	if (menu != null) {
 		if (!isEnabled ()) {
-			propagateHandle (false, menu.handle); 
+			propagateHandle (false, menu.handle, OS.None); 
 		}
 		menu.addAccelerators ();
 	}
@@ -529,7 +642,7 @@ public void setMenuBar (Menu menu) {
 	* Bug in Motif.  When a XmMainWindowSetAreas () is used
 	* to replace an existing menu, both menus must be managed
 	* before the call to XmMainWindowSetAreas () or the new
-	* menu will not be layed out properly.
+	* menu will not be laid out properly.
 	*/
 	int newHandle = (menu != null) ? menu.handle : 0;
 	int oldHandle = (menuBar != null) ? menuBar.handle : 0;
@@ -570,7 +683,10 @@ public void setMenuBar (Menu menu) {
 	OS.XtGetValues (handle, argList2, argList2.length / 2);
 	if (argList1 [1] != argList2 [1] || argList1 [3] != argList2 [3]) {
 		sendEvent (SWT.Resize);
-		if (layout != null) layout (false);
+		if (layout != null) {
+			markLayout (false, false);
+			updateLayout (false);
+		}
 	}
 }
 /**
@@ -581,13 +697,13 @@ public void setMenuBar (Menu menu) {
  * causes the receiver to switch back to either the maximized
  * or normal states.
  * <p>
- * Note: The result of intermixing calls to<code>setMaximized(true)</code>
+ * Note: The result of intermixing calls to <code>setMaximized(true)</code>
  * and <code>setMinimized(true)</code> will vary by platform. Typically,
  * the behavior will match the platform user's expectations, but not
  * always. This should be avoided if possible.
  * </p>
  *
- * @param the new maximized state
+ * @param minimized the new maximized state
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -607,12 +723,12 @@ void setSavedFocus (Control control) {
 /**
  * Sets the receiver's text, which is the string that the
  * window manager will typically display as the receiver's
- * <em>title</em>, to the argument, which may not be null. 
+ * <em>title</em>, to the argument, which must not be null. 
  *
- * @param text the new text
+ * @param string the new text
  *
  * @exception IllegalArgumentException <ul>
- *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
+ *    <li>ERROR_NULL_ARGUMENT - if the text is null</li>
  * </ul>
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -624,73 +740,38 @@ public void setText (String string) {
 	if (string == null) error (SWT.ERROR_NULL_ARGUMENT);
 	label = string;
 }
-boolean translateAccelerator (int key, int keysym, XKeyEvent xEvent) {
-	/*
-	* Bug in Solaris.  When accelerators are set more
-	* than once in the same menu bar, the time it takes
-	* to set the accelerator increases exponentially.
-	* The fix is to implement our own accelerator table
-	* on Solaris.
-	*/
-	if (OS.IsSunOS) {
-		if (menuBar != null && menuBar.getEnabled ()) {
-			/* Ignore modifiers. */
-			switch (keysym) {
-				case OS.XK_Control_L:
-				case OS.XK_Control_R:
-				case OS.XK_Alt_L:
-				case OS.XK_Alt_R:
-				case OS.XK_Shift_L:
-				case OS.XK_Shift_R:
-					return false;					
-			}
-			
-			/*
-			* Bug in MOTIF.  On Solaris only, XK_F11 and XK_F12 are not
-			* translated correctly by XLookupString().  They are mapped
-			* to 0x1005FF10 and 0x1005FF11 respectively.  The fix is to
-			* look for these values explicitly and correct them.
-			*/
-			if (keysym != 0) {
-				switch (keysym) {
-					case 0x1005FF10: 
-						keysym = OS.XK_F11;
-						key = 0;
-						break;
-					case 0x1005FF11:
-						keysym = OS.XK_F12;
-						key = 0;
-						break;
-				}
-				/*
-				* Bug in MOTIF.  On Solaris only, there is garbage in the
-				* high 16-bits for Keysyms such as XK_Down.  Since Keysyms
-				* must be 16-bits to fit into a Character, mask away the
-				* high 16-bits on all platforms.
-				*/
-				keysym &= 0xFFFF;
-			}
-			
-			/*
-			* Bug in Motif.  There are some keycodes for which 
-			* XLookupString() does not translate the character.
-			* Some of examples are Shift+Tab and Ctrl+Space.
-			*/
-			switch (keysym) {
-				case OS.XK_ISO_Left_Tab: key = '\t'; break;
-				case OS.XK_space: key = ' '; break;
-			}
-				
-			int accelerator = Display.translateKey (keysym);
-			if (accelerator == 0) accelerator = key;
-			if (accelerator == 0) return false;
-			if ((xEvent.state & OS.Mod1Mask) != 0) accelerator |= SWT.ALT;
-			if ((xEvent.state & OS.ShiftMask) != 0) accelerator |= SWT.SHIFT;
-			if ((xEvent.state & OS.ControlMask) != 0) accelerator |= SWT.CONTROL;
-			return menuBar.translateAccelerator (accelerator);
-		}
+void sort (Image [] images) {
+	/* Shell Sort from K&R, pg 108 */
+	int length = images.length;
+	if (length <= 1) return; 
+	ImageData [] datas = new ImageData [length];
+	for (int i = 0; i < length; i++) {
+		datas [i] = images [i].getImageData ();
 	}
-	return false;
+	for (int gap=length/2; gap>0; gap/=2) {
+		for (int i=gap; i<length; i++) {
+			for (int j=i-gap; j>=0; j-=gap) {
+		   		if (compare (datas [j], datas [j + gap]) >= 0) {
+					Image swap = images [j];
+					images [j] = images [j + gap];
+					images [j + gap] = swap;
+					ImageData swapData = datas [j];
+					datas [j] = datas [j + gap];
+					datas [j + gap] = swapData;
+		   		}
+	    	}
+	    }
+	}
+}
+boolean translateAccelerator (char key, int keysym, XKeyEvent xEvent, boolean doit) {
+	if (menuBar == null || !menuBar.getEnabled ()) return false;
+	int accelerator = Display.translateKey (keysym);
+	if (accelerator == 0) accelerator = key;
+	if (accelerator == 0) return false;
+	if ((xEvent.state & OS.Mod1Mask) != 0) accelerator |= SWT.ALT;
+	if ((xEvent.state & OS.ShiftMask) != 0) accelerator |= SWT.SHIFT;
+	if ((xEvent.state & OS.ControlMask) != 0) accelerator |= SWT.CONTROL;
+	return menuBar.translateAccelerator (accelerator, doit);
 }
 boolean traverseItem (boolean next) {
 	return false;
