@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -57,7 +57,7 @@ public class Menu extends Widget {
  * @see Widget#getStyle
  */
 public Menu (Control parent) {
-	this (checkNull(parent).getShell (), SWT.POP_UP);
+	this (checkNull(parent).menuShell (), SWT.POP_UP);
 }
 /**
  * Constructs a new instance of this class given its parent
@@ -101,7 +101,7 @@ public Menu (Decorations parent, int style) {
  * for the instance so that the instance will be a drop-down
  * menu on the given parent's parent.
  *
- * @param parent a menu which will be the parent of the new instance (cannot be null)
+ * @param parentMenu a menu which will be the parent of the new instance (cannot be null)
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the parent is null</li>
@@ -124,7 +124,7 @@ public Menu (Menu parentMenu) {
  * for the instance so that the instance will be a drop-down
  * menu on the given parent's parent menu.
  *
- * @param parent a menu item which will be the parent of the new instance (cannot be null)
+ * @param parentItem a menu item which will be the parent of the new instance (cannot be null)
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the parent is null</li>
@@ -141,6 +141,55 @@ public Menu (Menu parentMenu) {
 public Menu (MenuItem parentItem) {
 	this (checkNull(parentItem).parent);
 }
+void _setVisible (boolean visible) {
+	if ((style & (SWT.BAR | SWT.DROP_DOWN)) != 0) return;
+	if (visible) {
+		sendEvent (SWT.Show);
+		if (getItemCount () != 0) {
+			int xDisplay = OS.XtDisplay (handle);
+			if (xDisplay == 0) return;
+			int xWindow = OS.XDefaultRootWindow (xDisplay);
+			if (xWindow == 0) return;
+			int [] rootX = new int [1], rootY = new int [1], unused = new int [1], mask = new int [1];
+			if (OS.XQueryPointer (xDisplay, xWindow, unused, unused, rootX, rootY, unused, unused, mask) == 0) {
+				return;
+			}
+			if (!hasLocation) {
+				/*
+				* Bug in Motif.  For some reason, when a menu is popped up
+				* under the mouse, the menu will not highlight until the
+				* mouse exits and then enters the menu again.  The fix is
+				* to pop the menu up outside the current mouse position
+				* causing highlighting to work properly when the user
+				* waits for the menu to appear.
+				*/
+				rootX[0] += 1;  rootY[0] += 1;
+				int [] argList = {OS.XmNx, rootX [0], OS.XmNy, rootY [0]};
+				OS.XtSetValues (handle, argList, argList.length / 2);
+			}
+			/*
+			* Feature in Motif.  If an X grab is active, then
+			* the menu pops us but issues and error message an
+			* fails to grab the pointer.  The fix is to ensure
+			* that no grab is active before showing the menu.
+			*/
+			OS.XUngrabPointer (xDisplay, OS.CurrentTime);
+			OS.XtManageChild (handle);
+			/*
+			* Feature in Motif.  There is no API to force the menu
+			* to accept keyboard traversal when popped up using
+			* XtManageChild.  The fix is to call undocumented API
+			* to do this.
+			*/
+			int flags = OS.Button1Mask | OS.Button2Mask | OS.Button3Mask;
+			if ((mask [0] & flags) == 0) OS._XmSetMenuTraversal (handle, true);
+		} else {
+			sendEvent (SWT.Hide);
+		}
+	} else {
+		OS.XtUnmanageChild (handle);
+	}
+}
 void addAccelerators () {
 	MenuItem [] items = getItems ();
 	for (int i = 0; i < items.length; i++) {
@@ -229,7 +278,7 @@ void createHandle (int index) {
 		int [] argc = new int [] {0};
 		int xtContext = OS.XtCreateApplicationContext ();
 		xDisplay = OS.XtOpenDisplay (xtContext, null, null, null, 0, 0, argc, 0);
-		shellHandle = OS.XtAppCreateShell (null, null, OS.TopLevelShellWidgetClass (), xDisplay, null, 0);
+		shellHandle = OS.XtAppCreateShell (null, null, OS.topLevelShellWidgetClass (), xDisplay, null, 0);
 	}
 	
 	/* BAR menu */
@@ -285,6 +334,20 @@ void createWidget (int index) {
 	super.createWidget (index);
 	parent.add (this);
 }
+/*public*/ Rectangle getBounds () {
+	checkWidget();
+	if (!OS.XtIsManaged (handle)) return new Rectangle (0, 0, 0, 0);
+	int [] argList = {OS.XmNx, 0, OS.XmNy, 0, OS.XmNwidth, 0, OS.XmNheight, 0};
+	OS.XtGetValues (handle, argList, argList.length / 2);
+	int x = argList [1], y = argList [3];
+	if ((style & SWT.BAR) != 0) {
+		short [] root_x = new short [1], root_y = new short [1];
+		OS.XtTranslateCoords (handle, (short) x, (short) x, root_x, root_y);
+		x = root_x [0];
+		y = root_y [0];
+	}
+	return new Rectangle (x, y, argList [5], argList [7]);
+}
 /**
  * Returns the default menu item or null if none has
  * been previously set.
@@ -301,14 +364,9 @@ public MenuItem getDefaultItem () {
 	checkWidget();
 	return defaultItem;
 }
-public Display getDisplay () {
-	Decorations parent = this.parent;
-	if (parent == null) error (SWT.ERROR_WIDGET_DISPOSED);
-	return parent.getDisplay ();
-}
 /**
  * Returns <code>true</code> if the receiver is enabled, and
- * <code>false</code> otherwise. A disabled control is typically
+ * <code>false</code> otherwise. A disabled menu is typically
  * not selectable from the user interface and draws with an
  * inactive or "grayed" look.
  *
@@ -358,7 +416,7 @@ public MenuItem getItem (int index) {
 		i++;
 	}
 	if (index != count) error (SWT.ERROR_INVALID_RANGE);
-	Widget widget = WidgetTable.get (handles [i]);
+	Widget widget = display.getWidget (handles [i]);
 	if (!(widget instanceof MenuItem)) error (SWT.ERROR_CANNOT_GET_ITEM);
 	return (MenuItem) widget;
 }
@@ -386,8 +444,8 @@ public int getItemCount () {
 	return count;	
 }
 /**
- * Returns an array of <code>MenuItem</code>s which are the items
- * in the receiver. 
+ * Returns a (possibly empty) array of <code>MenuItem</code>s which
+ * are the items in the receiver. 
  * <p>
  * Note: This is not the actual structure used by the receiver
  * to maintain its list of items, so modifying the array will
@@ -412,7 +470,7 @@ public MenuItem [] getItems () {
 	MenuItem [] items = new MenuItem [count];
 	int i = 0, j = 0;
 	while (i < count) {
-		Widget item = WidgetTable.get (handles [i]);
+		Widget item = display.getWidget (handles [i]);
 		if (item != null) items [j++] = (MenuItem) item;
 		i++;
 	}
@@ -518,10 +576,18 @@ public Shell getShell () {
  */
 public boolean getVisible () {
 	checkWidget();
+	if ((style & SWT.POP_UP) != 0) {
+		Menu [] popups = display.popups;
+		if (popups != null) {
+			for (int i=0; i<popups.length; i++) {
+				if (popups [i] == this) return true;
+			}
+		}
+	}
 	return OS.XtIsManaged (handle);
 }
 void hookEvents () {
-	int windowProc = getDisplay ().windowProc;
+	int windowProc = display.windowProc;
 	OS.XtAddCallback (handle, OS.XmNhelpCallback, windowProc, HELP_CALLBACK);
 	OS.XtAddCallback (handle, OS.XmNmapCallback, windowProc, MAP_CALLBACK);
 	OS.XtAddCallback (handle, OS.XmNunmapCallback, windowProc, UNMAP_CALLBACK);
@@ -561,7 +627,7 @@ public int indexOf (MenuItem item) {
 /**
  * Returns <code>true</code> if the receiver is enabled and all
  * of the receiver's ancestors are enabled, and <code>false</code>
- * otherwise. A disabled control is typically not selectable from the
+ * otherwise. A disabled menu is typically not selectable from the
  * user interface and draws with an inactive or "grayed" look.
  *
  * @return the receiver's enabled state
@@ -624,7 +690,7 @@ void removeAccelerators () {
  * Removes the listener from the collection of listeners who will
  * be notified when the help events are generated for the control.
  *
- * @param listener the listener which should be notified
+ * @param listener the listener which should no longer be notified
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
@@ -647,7 +713,7 @@ public void removeHelpListener (HelpListener listener) {
  * Removes the listener from the collection of listeners who will
  * be notified when the menu events are generated for the control.
  *
- * @param listener the listener which should be notified
+ * @param listener the listener which should no longer be notified
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
@@ -695,7 +761,7 @@ public void setDefaultItem (MenuItem item) {
 }
 /**
  * Enables the receiver if the argument is <code>true</code>,
- * and disables it otherwise. A disabled control is typically
+ * and disables it otherwise. A disabled menu is typically
  * not selectable from the user interface and draws with an
  * inactive or "grayed" look.
  *
@@ -712,11 +778,15 @@ public void setEnabled (boolean enabled) {
 	OS.XtSetValues (handle, argList, argList.length / 2);
 }
 /**
- * Sets the receiver's location to the point specified by
- * the arguments which are relative to the display.
+ * Sets the location of the receiver, which must be a popup,
+ * to the point specified by the arguments which are relative
+ * to the display.
  * <p>
- * Note:  This is different from most widgets where the
+ * Note that this is different from most widgets where the
  * location of the widget is relative to the parent.
+ * </p><p>
+ * Note that the platform window manager ultimately has control
+ * over the location of popup menus.
  * </p>
  *
  * @param x the new x coordinate for the receiver
@@ -735,11 +805,15 @@ public void setLocation (int x, int y) {
 	hasLocation = true;
 }
 /**
- * Sets the receiver's location to the point specified by
- * the arguments which are relative to the display.
+ * Sets the location of the receiver, which must be a popup,
+ * to the point specified by the argument which is relative
+ * to the display.
  * <p>
- * Note:  This is different from most widgets where the
+ * Note that this is different from most widgets where the
  * location of the widget is relative to the parent.
+ * </p><p>
+ * Note that the platform window manager ultimately has control
+ * over the location of popup menus.
  * </p>
  *
  * @param location the new location for the receiver
@@ -779,58 +853,18 @@ public void setVisible (boolean visible) {
 	checkWidget();
 	if ((style & (SWT.BAR | SWT.DROP_DOWN)) != 0) return;
 	if (visible) {
-		sendEvent (SWT.Show);
-		if (getItemCount () != 0) {
-			int xDisplay = OS.XtDisplay (handle);
-			if (xDisplay == 0) return;
-			int xWindow = OS.XDefaultRootWindow (xDisplay);
-			if (xWindow == 0) return;
-			int [] rootX = new int [1], rootY = new int [1], unused = new int [1], mask = new int [1];
-			if (OS.XQueryPointer (xDisplay, xWindow, unused, unused, rootX, rootY, unused, unused, mask) == 0) {
-				return;
-			}
-			if (!hasLocation) {
-				/*
-				* Bug in Motif.  For some reason, when a menu is popped up
-				* under the mouse, the menu will not highlight until the
-				* mouse exits and then enters the menu again.  The fix is
-				* to pop the menu up outside the current mouse position
-				* causing highlighting to work properly when the user
-				* waits for the menu to appear.
-				*/
-				rootX[0] += 1;  rootY[0] += 1;
-				int [] argList = {OS.XmNx, rootX [0], OS.XmNy, rootY [0]};
-				OS.XtSetValues (handle, argList, argList.length / 2);
-			}
-			/*
-			* Feature in Motif.  If an X grab is active, then
-			* the menu pops us but issues and error message an
-			* fails to grab the pointer.  The fix is to ensure
-			* that no grab is active before showing the menu.
-			*/
-			OS.XUngrabPointer (xDisplay, OS.CurrentTime);
-			OS.XtManageChild (handle);
-			/*
-			* Feature in Motif.  There is no API to force the menu
-			* to accept keyboard traversal when popped up using
-			* XtManageChild.  The fix is to call undocumented API
-			* to do this.
-			*/
-			int flags = OS.Button1Mask | OS.Button2Mask | OS.Button3Mask;
-			if ((mask [0] & flags) == 0) OS._XmSetMenuTraversal (handle, true);
-		} else {
-			sendEvent (SWT.Hide);
-		}
+		display.addPopup (this);
 	} else {
-		OS.XtUnmanageChild (handle);
+		display.removePopup (this);
+		_setVisible (false);
 	}
 }
-boolean translateAccelerator (int accel) {
+boolean translateAccelerator (int accel, boolean doit) {
 	if (!getEnabled ()) return false;
 	MenuItem [] items = getItems ();
 	for (int i = 0; i < items.length; i++) {
 		MenuItem item = items [i];
-		if (item.translateAccelerator (accel)) return true;
+		if (item.translateAccelerator (accel, doit)) return true;
 	}
 	return false;
 }
@@ -840,25 +874,6 @@ int XmNhelpCallback (int w, int client_data, int call_data) {
 }
 int XmNmapCallback (int w, int client_data, int call_data) {
 	if ((style & SWT.POP_UP) != 0) return 0;
-	/*
-	* SWT.Selection events are posted to allow stepping
-	* in the VA/Java debugger.  SWT.Show events are
-	* sent to ensure that application event handler
-	* code runs before the menu is displayed.  This
-	* means that SWT.Show events would normally occur
-	* before SWT.Selection events.  While this is not 
-	* strictly incorrect, applications often use the 
-	* SWT.Selection event to update the state of menu
-	* items and would like the ordering of events to 
-	* be the other way around.
-	*
-	* The fix is to run the deferred events before
-	* the menu is shown.  This means that stepping
-	* through a selection event that was caused by
-	* a popup menu will fail in VA/Java.
-	*/
-	Display display = getDisplay ();
-	display.runDeferredEvents ();
 	sendEvent (SWT.Show);
 	return 0;
 }
