@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -37,6 +37,7 @@ public class ToolItem extends Item {
 	Image hotImage, disabledImage;
 	String toolTipText;
 	Control control;
+	int width = DEFAULT_SEPARATOR_WIDTH;
 	boolean set;
 	
 	static final int DEFAULT_WIDTH = 24;
@@ -81,7 +82,6 @@ public ToolItem (ToolBar parent, int style) {
 	super (parent, checkStyle (style));
 	this.parent = parent;
 	parent.createItem (this, parent.getItemCount ());
-	parent.relayout ();
 }
 
 /**
@@ -123,7 +123,6 @@ public ToolItem (ToolBar parent, int style, int index) {
 	super (parent, checkStyle (style));
 	this.parent = parent;
 	parent.createItem (this, index);
-	parent.relayout ();
 }
 
 /**
@@ -199,7 +198,7 @@ void createHandle (int index) {
 	setBackgroundPixel (pixel);
 }
 
-void click (boolean dropDown, XInputEvent xEvent) {
+void click (boolean dropDown, int state) {
 	if ((style & SWT.RADIO) != 0) {
 		selectRadio ();
 	} else {
@@ -215,18 +214,23 @@ void click (boolean dropDown, XInputEvent xEvent) {
 			event.y = (short) argList [3] + (short) argList [5];
 		}
 	}
-	if (xEvent != null) setInputState (event, xEvent);
+	if (state != 0) setInputState (event, state);
 	postEvent (SWT.Selection, event);
 }
 
-Point computeSize () {
+Point computeSize (GC gc) {
+	int width = 0, height = 0;
 	if ((style & SWT.SEPARATOR) != 0) {
-		int [] argList = {
-			OS.XmNwidth, 0,
-			OS.XmNheight, 0,
-		};
-		OS.XtGetValues (handle, argList, argList.length / 2);
-		int width = argList [1], height = argList [3];
+		if ((parent.style & SWT.HORIZONTAL) != 0) {
+			width = getWidth ();
+			height = DEFAULT_HEIGHT;
+		} else {
+			width = DEFAULT_WIDTH;
+			height = getWidth ();
+		}
+		if (control != null) {
+			height = Math.max (height, control.getMinimumHeight ());
+		}
 		return new Point(width, height);
 	}
 	int [] argList = {
@@ -238,36 +242,38 @@ Point computeSize () {
 	int marginHeight = argList [1], marginWidth = argList [3];
 	int shadowThickness = argList [5];
 	if ((parent.style & SWT.FLAT) != 0) {
-		Display display = getDisplay ();
 		shadowThickness = Math.min (2, display.buttonShadowThickness);
 	}
-	int textWidth = 0, textHeight = 0;
-	if (text.length () != 0) {
-		GC gc = new GC (parent);
-		int flags = SWT.DRAW_DELIMITER | SWT.DRAW_TAB | SWT.DRAW_MNEMONIC;
-		Point textExtent = gc.textExtent (text, flags);
-		textWidth = textExtent.x;
-		textHeight = textExtent.y;
-		gc.dispose ();
-	}
-	int imageWidth = 0, imageHeight = 0;
-	if (image != null) {
-		Rectangle rect = image.getBounds ();
-		imageWidth = rect.width;
-		imageHeight = rect.height;
-	}
-	int width = 0, height = 0;
-	if ((parent.style & SWT.RIGHT) != 0) {
-		width = imageWidth + textWidth;
-		height = Math.max (imageHeight, textHeight);
-		if (imageWidth != 0 && textWidth != 0) width += 2;
+	if (text.length() != 0 || image != null) {
+		int textWidth = 0, textHeight = 0;
+		if (text.length () != 0) {
+			int flags = SWT.DRAW_DELIMITER | SWT.DRAW_TAB | SWT.DRAW_MNEMONIC;
+			Point textExtent = gc.textExtent (text, flags);
+			textWidth = textExtent.x;
+			textHeight = textExtent.y;
+		}
+		int imageWidth = 0, imageHeight = 0;
+		if (image != null) {
+			Rectangle rect = image.getBounds ();
+			imageWidth = rect.width;
+			imageHeight = rect.height;
+		}
+		if ((parent.style & SWT.RIGHT) != 0) {
+			width = imageWidth + textWidth;
+			height = Math.max (imageHeight, textHeight);
+			if (imageWidth != 0 && textWidth != 0) width += 2;
+		} else {
+			height = imageHeight + textHeight;
+			if (imageHeight != 0 && textHeight != 0) height += 2;
+			width = Math.max (imageWidth, textWidth);
+		}
 	} else {
-		height = imageHeight + textHeight;
-		if (imageHeight != 0 && textHeight != 0) height += 2;
-		width = Math.max (imageWidth, textWidth);
+		width = DEFAULT_WIDTH;
+		height = DEFAULT_HEIGHT;
+	}
+	if ((style & SWT.DROP_DOWN) != 0) {
+		width += 12;
 	}
-	if ((style & SWT.DROP_DOWN) != 0) width += 12;
-	
 	if (width != 0) {
 		width += (marginWidth + shadowThickness) * 2 + 2;
 	} else {
@@ -280,10 +286,6 @@ Point computeSize () {
 	}
 	return new Point (width, height);
 }
-void createWidget (int index) {
-	super.createWidget (index);
-	parent.relayout ();
-}
 public void dispose () {
 	if (isDisposed()) return;
 	ToolBar parent = this.parent;
@@ -361,11 +363,6 @@ public boolean getEnabled () {
 	OS.XtGetValues (handle, argList, argList.length / 2);
 	return argList [1] != 0;
 }
-public Display getDisplay () {
-	Composite parent = this.parent;
-	if (parent == null) error (SWT.ERROR_WIDGET_DISPOSED);
-	return parent.getDisplay ();
-}
 /**
  * Returns the receiver's hot image if it has one, or null
  * if it does not.
@@ -446,9 +443,7 @@ public String getToolTipText () {
  */
 public int getWidth () {
 	checkWidget();
-	int [] argList = {OS.XmNwidth, 0};
-	OS.XtGetValues (handle, argList, argList.length / 2);
-	return argList [1];
+	return width;
 }
 boolean hasCursor () {
 	int [] unused = new int [1], buffer = new int [1];
@@ -465,7 +460,7 @@ boolean hasCursor () {
 void hookEvents () {
 	super.hookEvents ();
 	if ((style & SWT.SEPARATOR) != 0) return;
-	int windowProc = getDisplay ().windowProc;
+	int windowProc = display.windowProc;
 	OS.XtAddEventHandler (handle, OS.KeyPressMask, false, windowProc, KEY_PRESS);
 	OS.XtAddEventHandler (handle, OS.KeyReleaseMask, false, windowProc, KEY_RELEASE);
 	OS.XtAddEventHandler (handle, OS.ButtonPressMask, false, windowProc, BUTTON_PRESS);
@@ -480,7 +475,6 @@ int hoverProc (int id) {
 	boolean showTip = toolTipText != null;
 	parent.hoverProc (id, !showTip);
 	if (showTip) {
-		Display display = getDisplay ();
 		display.showToolTip (handle, toolTipText);
 	}
 	return 0;
@@ -519,9 +513,9 @@ void releaseChild () {
 	parent.destroyItem (this);
 }
 void releaseWidget () {
-	Display display = getDisplay ();
 	display.releaseToolTipHandle (handle);
 	super.releaseWidget ();
+	if (parent.lastFocus == this) parent.lastFocus = null;
 	parent = null;
 	control = null;
 	toolTipText = null;
@@ -531,7 +525,7 @@ void releaseWidget () {
  * Removes the listener from the collection of listeners who will
  * be notified when the control is selected.
  *
- * @param listener the listener which should be notified
+ * @param listener the listener which should no longer be notified
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
@@ -568,7 +562,6 @@ void setBackgroundPixel(int pixel) {
 	OS.XtSetValues (handle, argList, argList.length / 2);
 }
 void setBounds (int x, int y, int width, int height) {
-	if (control != null) control.setBounds(x, y, width, height);
 	/*
 	* Feature in Motif.  Motif will not allow a window
 	* to have a zero width or zero height.  The fix is
@@ -599,9 +592,13 @@ public void setControl (Control control) {
 		if (control.parent != parent) error (SWT.ERROR_INVALID_PARENT);
 	}
 	if ((style & SWT.SEPARATOR) == 0) return;
+	if (this.control == control) return;
 	this.control = control;
+	int [] argList = {
+		OS.XmNseparatorType, control == null ? ((parent.style & SWT.FLAT) != 0 ? OS.XmSHADOW_ETCHED_IN : OS.XmSHADOW_ETCHED_OUT) : OS.XmNO_LINE,
+	};
+	OS.XtSetValues (handle, argList, argList.length / 2);
 	if (control != null && !control.isDisposed ()) {
-		control.setBounds (getBounds ());
 		/*
 		 * It is possible that the control was created with a 
 		 * z-order below that of the current tool item. In this
@@ -630,6 +627,7 @@ public void setControl (Control control) {
 		int flags = OS.CWStackMode | OS.CWSibling;
 		OS.XReconfigureWMWindow (xDisplay, window1, screen, flags, struct);
 	}
+	parent.relayout ();
 }
 /**
  * Enables the receiver if the argument is <code>true</code>,
@@ -655,11 +653,6 @@ public void setEnabled (boolean enabled) {
 void setForegroundPixel(int pixel) {
 	int [] argList = {OS.XmNforeground, pixel};
 	OS.XtSetValues (handle, argList, argList.length / 2);
-	int xDisplay = OS.XtDisplay (handle);
-	if (xDisplay == 0) return;
-	int xWindow = OS.XtWindow (handle);
-	if (xWindow == 0) return;
-	OS.XClearArea (xDisplay, xWindow, 0, 0, 0, 0, true);
 }
 /**
  * Sets the receiver's disabled image to the argument, which may be
@@ -685,6 +678,10 @@ public void setDisabledImage (Image image) {
 	disabledImage = image;
 	if (!getEnabled ()) redraw ();
 }
+boolean setFocus () {
+	if ((style & SWT.SEPARATOR) != 0) return false;
+	return XmProcessTraversal (handle, OS.XmTRAVERSE_CURRENT);
+}
 /**
  * Sets the receiver's hot image to the argument, which may be
  * null indicating that no hot image should be displayed.
@@ -714,8 +711,7 @@ public void setImage (Image image) {
 	if (image != null && image.isDisposed()) error(SWT.ERROR_INVALID_ARGUMENT);
 	if ((style & SWT.SEPARATOR) != 0) return;
 	super.setImage (image);
-	Point size = computeSize ();
-	setSize (size.x, size.y, true);
+	parent.relayout();
 	redraw ();
 }
 boolean setRadioSelection (boolean value) {
@@ -748,15 +744,6 @@ public void setSelection (boolean selected) {
 	set = selected;
 	setDrawPressed (set);
 }
-
-void setSize (int width, int height, boolean layout) {
-	int [] argList = {OS.XmNwidth, 0, OS.XmNheight, 0};
-	OS.XtGetValues (handle, argList, argList.length / 2);
-	if (argList [1] != width || argList [3] != height) {
-		OS.XtResizeWidget (handle, width, height, 0);
-		if (layout) parent.relayout ();
-	}
-}
 /**
  * Sets the receiver's text. The string may include
  * the mnemonic character.
@@ -787,8 +774,7 @@ public void setText (String string) {
 	if (string == null) error (SWT.ERROR_NULL_ARGUMENT);
 	if ((style & SWT.SEPARATOR) != 0) return;
 	super.setText (string);
-	Point size = computeSize ();
-	setSize (size.x, size.y, true);
+	parent.relayout();
 	redraw ();
 }
 
@@ -807,8 +793,11 @@ public void setToolTipText (String string) {
 	checkWidget();
 	toolTipText = string;
 }
+void setVisible (boolean visible) {
+	OS.XtSetMappedWhenManaged (handle, visible);
+}
 /**
- * Sets the width of the receiver.
+ * Sets the width of the receiver, for <code>SEPARATOR</code> ToolItems.
  *
  * @param width the new width
  *
@@ -820,33 +809,28 @@ public void setToolTipText (String string) {
 public void setWidth (int width) {
 	checkWidget();
 	if ((style & SWT.SEPARATOR) == 0) return;
-	if (width < 0) return;
-	int [] argList = {OS.XmNheight, 0};
-	OS.XtGetValues (handle, argList, argList.length / 2);
-	setSize (width, argList [1], true);
-	if (control != null && !control.isDisposed ()) {
-		control.setBounds (getBounds ());
-	}
+	if (width < 0 || this.width == width) return;
+	this.width = width;
+	parent.relayout();
 }
 void setDrawPressed (boolean value) {
 	int shadowType = value ? OS.XmSHADOW_IN : OS.XmSHADOW_OUT;
 	int [] argList = {OS.XmNshadowType, shadowType};
 	OS.XtSetValues(handle, argList, argList.length / 2);
 }
-boolean translateAccelerator (int key, int keysym, XKeyEvent xEvent) {
-	return parent.translateAccelerator (key, keysym, xEvent);
+boolean translateAccelerator (char key, int keysym, XKeyEvent xEvent, boolean doit) {
+	return parent.translateAccelerator (key, keysym, xEvent, doit);
 }
-boolean translateMnemonic (int key, XKeyEvent xEvent) {
-	return parent.translateMnemonic (key, xEvent);
+boolean translateMnemonic (char key, int keysym, XKeyEvent xEvent) {
+	return parent.translateMnemonic (key, keysym, xEvent);
 }
 boolean translateTraversal (int key, XKeyEvent xEvent) {
 	return parent.translateTraversal (key, xEvent);
 }
 void propagateWidget (boolean enabled) {
-	propagateHandle (enabled, handle);
+	propagateHandle (enabled, handle, OS.None);
 }
 int XButtonPress (int w, int client_data, int call_data, int continue_to_dispatch) {
-	Display display = getDisplay ();
 //	Shell shell = parent.getShell ();
 	display.hideToolTip ();
 	XButtonEvent xEvent = new XButtonEvent ();
@@ -884,7 +868,6 @@ int XButtonPress (int w, int client_data, int call_data, int continue_to_dispatc
 	return 0;
 }
 int XButtonRelease (int w, int client_data, int call_data, int continue_to_dispatch) {
-	Display display = getDisplay ();
 	display.hideToolTip(); 
 	XButtonEvent xEvent = new XButtonEvent ();
 	OS.memmove (xEvent, call_data, XButtonEvent.sizeof);
@@ -893,7 +876,7 @@ int XButtonRelease (int w, int client_data, int call_data, int continue_to_dispa
 		OS.XtGetValues (handle, argList, argList.length / 2);
 		int width = argList [1], height = argList [3];
 		if (0 <= xEvent.x && xEvent.x < width && 0 <= xEvent.y && xEvent.y < height) {
-			click (xEvent.x > width - 12, xEvent);
+			click (xEvent.x > width - 12, xEvent.state);
 		}
 		setDrawPressed(set);
 	}
@@ -938,12 +921,11 @@ int XFocusChange (int w, int client_data, int call_data, int continue_to_dispatc
 	OS.memmove (xEvent, call_data, XFocusChangeEvent.sizeof);
 	xEvent.window = OS.XtWindow (parent.handle);
 //	OS.memmove (call_data, xEvent, XFocusChangeEvent.sizeof);
-	if (OS.IsDBLocale) {
-		parent.XFocusChange (w, client_data, call_data, continue_to_dispatch);
-	} 
+	parent.XFocusChange (w, client_data, call_data, continue_to_dispatch);
 	return 0;
 }
 int XKeyPress (int w, int client_data, int call_data, int continue_to_dispatch) {
+	int result = 0;
 	XKeyEvent xEvent = new XKeyEvent ();
 	OS.memmove (xEvent, call_data, XKeyEvent.sizeof);
 	int [] keysym = new int [1];
@@ -951,11 +933,12 @@ int XKeyPress (int w, int client_data, int call_data, int continue_to_dispatch)
 	keysym [0] &= 0xFFFF;
 	switch (keysym [0]) {
 		case OS.XK_space:
-			click (false, xEvent);
+			click (false, xEvent.state);
+			result = 1;
 			break;
-		case OS.XK_KP_Enter:
-		case OS.XK_Return:
-			click (true, xEvent);
+		case OS.XK_Down:
+			click (true, xEvent.state);
+			result = 1;
 			break;
 	}
 	/*
@@ -969,7 +952,10 @@ int XKeyPress (int w, int client_data, int call_data, int continue_to_dispatch)
 	xEvent.window = OS.XtWindow (parent.handle);
 //	OS.memmove (callData, xEvent, XKeyEvent.sizeof);
 	parent.XKeyPress (w, client_data, call_data, continue_to_dispatch);
-	return 0;
+	if (result == 1) {
+		OS.memmove (continue_to_dispatch, new int [1], 4);
+	}
+	return result;
 }
 int XKeyRelease (int w, int client_data, int call_data, int continue_to_dispatch) {
 	XKeyEvent xEvent = new XKeyEvent ();
@@ -989,7 +975,6 @@ int XKeyRelease (int w, int client_data, int call_data, int continue_to_dispatch
 	return 0;
 }
 int XLeaveWindow (int w, int client_data, int call_data, int continue_to_dispatch) {
-	Display display = getDisplay ();
 	display.removeMouseHoverTimeOut ();
 	display.hideToolTip ();
 	XCrossingEvent xEvent = new XCrossingEvent ();
@@ -1019,7 +1004,6 @@ int XmNexposureCallback (int w, int client_data, int call_data) {
 	boolean enabled = getEnabled();
 
 	if ((parent.style & SWT.FLAT) != 0) {
-		Display display = getDisplay ();
 		boolean hasCursor = hasCursor ();
 		
 		/* Set the shadow thickness */
@@ -1036,13 +1020,15 @@ int XmNexposureCallback (int w, int client_data, int call_data) {
 		}
 	}
 
-	ToolDrawable wrapper = new ToolDrawable ();
-	wrapper.device = getDisplay ();
-	wrapper.display = xDisplay;
-	wrapper.drawable = xWindow;
-	wrapper.font = parent.font;
-	wrapper.colormap = argList [1];	
-	GC gc = new GC (wrapper);
+	GCData data = new GCData ();
+	data.device = display;
+	data.display = xDisplay;
+	data.drawable = xWindow;
+	data.font = parent.font;
+	data.colormap = argList [1];
+	int xGC = OS.XCreateGC (xDisplay, xWindow, 0, null);
+	if (xGC == 0) SWT.error (SWT.ERROR_NO_HANDLES);
+	GC gc = GC.motif_new (xGC, data);
 	
 	XmAnyCallbackStruct cb = new XmAnyCallbackStruct ();
 	OS.memmove (cb, call_data, XmAnyCallbackStruct.sizeof);
@@ -1054,7 +1040,6 @@ int XmNexposureCallback (int w, int client_data, int call_data) {
 	}
 	
 	if (!enabled) {
-		Display display = getDisplay ();
 		currentImage = disabledImage;
 		if (currentImage == null && image != null) {
 			currentImage = new Image (display, image, SWT.IMAGE_DISABLE);
@@ -1110,6 +1095,7 @@ int XmNexposureCallback (int w, int client_data, int call_data) {
 		gc.drawPolygon (arrow);
 	}
 	gc.dispose ();
+	OS.XFreeGC (xDisplay, xGC);
 	
 	if (!enabled && disabledImage == null) {
 		if (currentImage != null) currentImage.dispose ();
@@ -1117,7 +1103,6 @@ int XmNexposureCallback (int w, int client_data, int call_data) {
 	return 0;
 }
 int XPointerMotion (int w, int client_data, int call_data, int continue_to_dispatch) {
-	Display display = getDisplay ();
 	display.addMouseHoverTimeOut (handle);
 
 	/*
@@ -1128,8 +1113,8 @@ int XPointerMotion (int w, int client_data, int call_data, int continue_to_dispa
 	* in X in the parent.  This is done to be
 	* compatible with Windows.
 	*/
-	XButtonEvent xEvent = new XButtonEvent ();
-	OS.memmove (xEvent, call_data, XButtonEvent.sizeof);
+	XMotionEvent xEvent = new XMotionEvent ();
+	OS.memmove (xEvent, call_data, XMotionEvent.sizeof);
 	int [] argList = {OS.XmNx, 0, OS.XmNy, 0};
 	OS.XtGetValues (handle, argList, argList.length / 2);
 	xEvent.window = OS.XtWindow (parent.handle);
@@ -1142,7 +1127,7 @@ int XPointerMotion (int w, int client_data, int call_data, int continue_to_dispa
 	*/
 //	OS.memmove (callData, xEvent, XButtonEvent.sizeof);
 //	parent.XPointerMotion (w, client_data, call_data, continue_to_dispatch);
-	parent.sendMouseEvent (SWT.MouseMove, 0, xEvent);
+	parent.sendMouseEvent (SWT.MouseMove, xEvent);
 
 	return 0;
 }
