@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -53,7 +53,7 @@ import java.io.*;
  * loading process should use the support provided in class
  * <code>ImageLoader</code>.
  * </p><p>
- * Application code must explicitely invoke the <code>Image.dispose()</code> 
+ * Application code must explicitly invoke the <code>Image.dispose()</code> 
  * method to release the operating system resources managed by each instance
  * when those instances are no longer required.
  * </p>
@@ -62,7 +62,7 @@ import java.io.*;
  * @see ImageData
  * @see ImageLoader
  */
-public final class Image implements Drawable {
+public final class Image extends Resource implements Drawable {
 
 	/**
 	 * specifies whether the receiver is a bitmap or an icon
@@ -71,31 +71,29 @@ public final class Image implements Drawable {
 	public int type;
 
 	/**
-	 * the OS resource of the image
+	 * the handle to the OS image resource
 	 * (Warning: This field is platform dependent)
+	 * <p>
+	 * <b>IMPORTANT:</b> This field is <em>not</em> part of the SWT
+	 * public API. It is marked public only so that it can be shared
+	 * within the packages provided by SWT. It is not available on all
+	 * platforms and should never be accessed from application code.
+	 * </p>
 	 */
 	public int handle;
 	
-	/**
-	 * the device where this image was created
-	 */
-	Device device;
-	
 	/**
 	 * specifies the transparent pixel
-	 * (Warning: This field is platform dependent)
 	 */
 	int transparentPixel = -1;
 	
 	/**
 	 * the GC which is drawing on the image
-	 * (Warning: This field is platform dependent)
 	 */
 	GC memGC;
 
 	/**
 	 * specifies the default scanline padding
-	 * (Warning: This field is platform dependent)
 	 */
 	static final int DEFAULT_SCANLINE_PAD = 4;
 
@@ -165,6 +163,7 @@ public Image(Device device, int width, int height) {
  * @exception SWTException <ul>
  *    <li>ERROR_INVALID_IMAGE - if the image is not a bitmap or an icon, or
  *          is otherwise in an invalid state</li>
+ *    <li>ERROR_UNSUPPORTED_DEPTH - if the depth of the Image is not supported</li>
  * </ul>
  * @exception SWTError <ul>
  *    <li>ERROR_NO_HANDLES if a handle could not be obtained for image creation</li>
@@ -256,6 +255,7 @@ public Image(Device device, Image srcImage, int flag) {
 				newData = new ImageData(r.width, r.height, 8, new PaletteData(rgbs));
 				newData.maskData = data.maskData;
 				newData.maskPad = data.maskPad;
+				if (data.transparentPixel != -1) newData.transparentPixel = 254; 
 
 				/* Convert the pixels. */
 				int[] scanline = new int[r.width];
@@ -270,14 +270,20 @@ public Image(Device device, Image srcImage, int flag) {
 					data.getPixels(0, y, r.width, scanline, 0);
 					for (int x=0; x<r.width; x++) {
 						int pixel = scanline[x];
-						int red = pixel & redMask;
-						red = (redShift < 0) ? red >>> -redShift : red << redShift;
-						int green = pixel & greenMask;
-						green = (greenShift < 0) ? green >>> -greenShift : green << greenShift;
-						int blue = pixel & blueMask;
-						blue = (blueShift < 0) ? blue >>> -blueShift : blue << blueShift;
-						newData.data[offset++] =
-							(byte)((red+red+green+green+green+green+green+blue) >> 3);
+						if (pixel != data.transparentPixel) {
+							int red = pixel & redMask;
+							red = (redShift < 0) ? red >>> -redShift : red << redShift;
+							int green = pixel & greenMask;
+							green = (greenShift < 0) ? green >>> -greenShift : green << greenShift;
+							int blue = pixel & blueMask;
+							blue = (blueShift < 0) ? blue >>> -blueShift : blue << blueShift;
+							int intensity = (red+red+green+green+green+green+green+blue) >> 3;
+							if (newData.transparentPixel == intensity) intensity = 255;
+							newData.data[offset] = (byte)intensity;
+						} else {
+							newData.data[offset] = (byte)254;
+						}
+						offset++;
 					}
 				}
 			}
@@ -338,6 +344,9 @@ public Image(Device device, Rectangle bounds) {
  *    <li>ERROR_NULL_ARGUMENT - if device is null and there is no current device</li>
  *    <li>ERROR_NULL_ARGUMENT - if the image data is null</li>
  * </ul>
+ * @exception SWTException <ul>
+ *    <li>ERROR_UNSUPPORTED_DEPTH - if the depth of the ImageData is not supported</li>
+ * </ul>
  * @exception SWTError <ul>
  *    <li>ERROR_NO_HANDLES if a handle could not be obtained for image creation</li>
  * </ul>
@@ -352,10 +361,8 @@ public Image(Device device, ImageData data) {
 /**
  * Constructs an instance of this class, whose type is 
  * <code>SWT.ICON</code>, from the two given <code>ImageData</code>
- * objects. The two images must be the same size, and the mask image
- * must have a color depth of 1. Pixel transparency in either image
- * will be ignored. If either image is an icon to begin with, an
- * exception is thrown.
+ * objects. The two images must be the same size. Pixel transparency
+ * in either image will be ignored.
  * <p>
  * The mask image should contain white wherever the icon is to be visible,
  * and black wherever the icon is to be transparent. In addition,
@@ -370,9 +377,7 @@ public Image(Device device, ImageData data) {
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if device is null and there is no current device</li>
  *    <li>ERROR_NULL_ARGUMENT - if either the source or mask is null </li>
- *    <li>ERROR_INVALID_ARGUMENT - if source and mask are different sizes or
- *          if the mask is not monochrome, or if either the source or mask
- *          is already an icon</li>
+ *    <li>ERROR_INVALID_ARGUMENT - if source and mask are different sizes</li>
  * </ul>
  * @exception SWTError <ul>
  *    <li>ERROR_NO_HANDLES if a handle could not be obtained for image creation</li>
@@ -386,7 +391,7 @@ public Image(Device device, ImageData source, ImageData mask) {
 	if (source.width != mask.width || source.height != mask.height) {
 		SWT.error(SWT.ERROR_INVALID_ARGUMENT);
 	}
-	if (mask.depth != 1) SWT.error(SWT.ERROR_INVALID_ARGUMENT);
+	mask = ImageData.convertMask(mask);
 	ImageData image = new ImageData(source.width, source.height, source.depth, source.palette, source.scanlinePad, source.data);
 	image.maskPad = mask.scanlinePad;
 	image.maskData = mask.data;
@@ -398,7 +403,8 @@ public Image(Device device, ImageData source, ImageData mask) {
  * Constructs an instance of this class by loading its representation
  * from the specified input stream. Throws an error if an error
  * occurs while loading the image, or if the result is an image
- * of an unsupported type.
+ * of an unsupported type.  Application code is still responsible
+ * for closing the input stream.
  * <p>
  * This constructor is provided for convenience when loading a single
  * image only. If the stream contains multiple images, only the first
@@ -408,7 +414,20 @@ public Image(Device device, ImageData source, ImageData mask) {
  * This constructor may be used to load a resource as follows:
  * </p>
  * <pre>
- *     new Image(device, clazz.getResourceAsStream("file.gif"));
+ *     static Image loadImage (Display display, Class clazz, String string) {
+ *          InputStream stream = clazz.getResourceAsStream (string);
+ *          if (stream == null) return null;
+ *          Image image = null;
+ *          try {
+ *               image = new Image (display, stream);
+ *          } catch (SWTException ex) {
+ *          } finally {
+ *               try {
+ *                    stream.close ();
+ *               } catch (IOException ex) {}
+ *          }
+ *          return image;
+ *     }
  * </pre>
  *
  * @param device the device on which to create the image
@@ -421,7 +440,9 @@ public Image(Device device, ImageData source, ImageData mask) {
  * @exception SWTException <ul>
  *    <li>ERROR_INVALID_IMAGE - if the image file contains invalid data </li>
  *    <li>ERROR_IO - if an IO error occurs while reading data</li>
- * </ul>
+ *    <li>ERROR_UNSUPPORTED_DEPTH - if the InputStream describes an image with an unsupported depth</li>
+ *    <li>ERROR_UNSUPPORTED_FORMAT - if the image file contains an unrecognized format</li>
+ *  * </ul>
  * @exception SWTError <ul>
  *    <li>ERROR_NO_HANDLES if a handle could not be obtained for image creation</li>
  * </ul>
@@ -453,6 +474,8 @@ public Image (Device device, InputStream stream) {
  * @exception SWTException <ul>
  *    <li>ERROR_INVALID_IMAGE - if the image file contains invalid data </li>
  *    <li>ERROR_IO - if an IO error occurs while reading data</li>
+ *    <li>ERROR_UNSUPPORTED_DEPTH - if the image file has an unsupported depth</li>
+ *    <li>ERROR_UNSUPPORTED_FORMAT - if the image file contains an unrecognized format</li>
  * </ul>
  * @exception SWTError <ul>
  *    <li>ERROR_NO_HANDLES if a handle could not be obtained for image creation</li>
@@ -473,6 +496,7 @@ public Image (Device device, String filename) {
 public void dispose () {
 	if (handle == 0) return;
 	if (device.isDisposed()) return;
+	if (memGC != null) memGC.dispose();
 	destroyImage(handle);
 	handle = 0;
 	memGC = null;
@@ -587,6 +611,7 @@ public Rectangle getBounds() {
  */
 public ImageData getImageData() {
 	if (isDisposed()) SWT.error(SWT.ERROR_GRAPHIC_DISPOSED);
+	if (memGC != null) memGC.flushImage();
 	PhImage_t phImage = new PhImage_t();
 	OS.memmove(phImage, handle, PhImage_t.sizeof);
 	int depth = 0;
@@ -632,11 +657,11 @@ public ImageData getImageData() {
 			SWT.error(SWT.ERROR_UNSUPPORTED_DEPTH);
 	}
 
-	int calcBpl, scanLinePad, bpl = phImage.bpl;
+	int scanLinePad, bpl = phImage.bpl;
 	int width = phImage.size_w, height = phImage.size_h;
 	int dataBytesPerLine = (width * depth + 7) / 8;
 	for (scanLinePad = 1; scanLinePad < 128; scanLinePad++) {
-		calcBpl = (dataBytesPerLine + (scanLinePad - 1)) / scanLinePad * scanLinePad;
+		int calcBpl = (dataBytesPerLine + (scanLinePad - 1)) / scanLinePad * scanLinePad;
 		if (bpl == calcBpl) break;
 	}	
 	byte[] data = new byte[height * bpl];
@@ -649,7 +674,7 @@ public ImageData getImageData() {
 	} else if (phImage.mask_bm != 0) {
 		imageData.maskData = new byte[height * phImage.mask_bpl];
 		OS.memmove(imageData.maskData, phImage.mask_bm, imageData.maskData.length);
-		imageData.maskPad = 4;
+		imageData.maskPad = 2;
 	} else if (phImage.alpha != 0) {
 		PgAlpha_t alpha = new PgAlpha_t();
 		OS.memmove(alpha, phImage.alpha, PgAlpha_t.sizeof);
@@ -666,7 +691,7 @@ public ImageData getImageData() {
 
 /**
  * Returns an integer hash code for the receiver. Any two 
- * objects which return <code>true</code> when passed to 
+ * objects that return <code>true</code> when passed to 
  * <code>equals</code> must return the same value for this
  * method.
  *
@@ -679,7 +704,7 @@ public int hashCode () {
 }
 
 void init(Device device, int width, int height) {
-	if (width <= 0 | height <= 0) {
+	if (width <= 0 || height <= 0) {
 		SWT.error (SWT.ERROR_INVALID_ARGUMENT);
 	}
 	this.device = device;
@@ -698,7 +723,7 @@ void init(Device device, ImageData i) {
 	* memory contexts can not be created on 1 & 4-bit depth images.  The
 	* fix is to create 8-bit depth images instead.
 	*/
-	if (i.depth == 1 || i.depth == 2 || i.depth == 4) {
+	if ((i.depth == 1 || i.depth == 2 || i.depth == 4) && !i.palette.isDirect) {
 		ImageData img = new ImageData(i.width, i.height, 8, i.palette);
 		ImageData.blit(ImageData.BLIT_SRC, 
 			i.data, i.depth, i.bytesPerLine, img.getByteOrder(), 0, 0, i.width, i.height, null, null, null,
@@ -796,7 +821,7 @@ void init(Device device, ImageData i) {
 	if (handle == 0) SWT.error(SWT.ERROR_NO_HANDLES);
 	PhImage_t phImage = new PhImage_t();
 	phImage.type = type;
-	phImage.flags = OS.Ph_RELEASE_IMAGE_ALL;
+	phImage.flags = (byte)OS.Ph_RELEASE_IMAGE_ALL;
 	int size = i.data.length;
 	int ptr = OS.malloc(size);
 	if (ptr == 0) {
@@ -930,7 +955,7 @@ public int internal_new_GC (GCData data) {
  * application code.
  * </p>
  *
- * @param handle the platform specific GC handle
+ * @param hDC the platform specific GC handle
  * @param data the platform specific GC data 
  */
 public void internal_dispose_GC (int pmMC, GCData data) {
@@ -962,7 +987,7 @@ public boolean isDisposed() {
  * these cases. For example:
  * <pre>
  *    Button b = new Button();
- *    image.setBackground(b.getBackground());>
+ *    image.setBackground(b.getBackground());
  *    b.setImage(image);
  * </pre>
  * </p><p>
@@ -1003,7 +1028,7 @@ static void destroyImage(int image) {
 	if (image == 0) return;
 	PhImage_t phImage = new PhImage_t();
 	OS.memmove(phImage, image, PhImage_t.sizeof);
-	phImage.flags = OS.Ph_RELEASE_IMAGE_ALL;
+	phImage.flags = (byte)OS.Ph_RELEASE_IMAGE_ALL;
 	OS.memmove(image, phImage, PhImage_t.sizeof);
 	OS.PhReleaseImage(image);
 	OS.free(image);
