@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -72,7 +72,9 @@ public class MenuItem extends Item {
  * @see Widget#getStyle
  */
 public MenuItem (Menu parent, int style) {
-	this (parent, style, parent.getItemCount());
+	super (parent, checkStyle (style));
+	this.parent = parent;
+	createWidget (parent.getItemCount ());
 }
 
 /**
@@ -123,7 +125,6 @@ void addAccelerator () {
 	if ((accelerator & SWT.SHIFT) != 0) keyMods |= OS.Pk_KM_Shift;
 	if ((accelerator & SWT.CONTROL) != 0) keyMods |= OS.Pk_KM_Ctrl;
 	int key = (accelerator & ~(SWT.ALT | SWT.SHIFT | SWT.CONTROL));
-	Display display = getDisplay ();
 	int keyCode = Display.untranslateKey (key);
 	if (keyCode != 0) key = keyCode;
 	else key = Character.toLowerCase ((char)key);
@@ -185,7 +186,7 @@ public void addHelpListener (HelpListener listener) {
 
 /**
  * Adds the listener to the collection of listeners who will
- * be notified when the control is selected, by sending
+ * be notified when the menu item is selected, by sending
  * it one of the messages defined in the <code>SelectionListener</code>
  * interface.
  * <p>
@@ -260,11 +261,13 @@ void createHandle (int index) {
 }
 
 /**
- * Return the widget accelerator.  An accelerator is the bit-wise
+ * Returns the widget accelerator.  An accelerator is the bit-wise
  * OR of zero or more modifier masks and a key. Examples:
  * <code>SWT.CONTROL | SWT.SHIFT | 'T', SWT.ALT | SWT.F2</code>.
+ * The default value is zero, indicating that the menu item does
+ * not have an accelerator.
  *
- * @return the accelerator
+ * @return the accelerator or 0
  *
  * </ul>
  * @exception SWTException <ul>
@@ -277,15 +280,9 @@ public int getAccelerator () {
 	return accelerator;
 }
 
-public Display getDisplay () {
-	Menu parent = this.parent;
-	if (parent == null) error (SWT.ERROR_WIDGET_DISPOSED);
-	return parent.getDisplay ();
-}
-
 /**
  * Returns <code>true</code> if the receiver is enabled, and
- * <code>false</code> otherwise. A disabled control is typically
+ * <code>false</code> otherwise. A disabled menu item is typically
  * not selectable from the user interface and draws with an
  * inactive or "grayed" look.
  *
@@ -372,7 +369,7 @@ public boolean getSelection () {
 
 void hookEvents () {
 	if ((style & SWT.SEPARATOR) != 0) return;
-	int windowProc = getDisplay ().windowProc;
+	int windowProc = display.windowProc;
 	if ((style & SWT.CASCADE) != 0) {
 		OS.PtAddCallback (handle, OS.Pt_CB_ARM, windowProc, OS.Pt_CB_ARM);
 	}
@@ -390,7 +387,7 @@ int hotkeyProc (int widget, int data, int info) {
 /**
  * Returns <code>true</code> if the receiver is enabled and all
  * of the receiver's ancestors are enabled, and <code>false</code>
- * otherwise. A disabled control is typically not selectable from the
+ * otherwise. A disabled menu item is typically not selectable from the
  * user interface and draws with an inactive or "grayed" look.
  *
  * @return the receiver's enabled state
@@ -423,13 +420,13 @@ int Pt_CB_ACTIVATE (int widget, int info) {
 					case OS.Ph_EV_KEY:
 						PhKeyEvent_t ke = new PhKeyEvent_t ();
 						OS.memmove (ke, data, PhKeyEvent_t.sizeof);
-						setKeyState (event, ke);
+						setInputState (event, SWT.Selection, ke.key_mods, ke.button_state);
 						break;
 					case OS.Ph_EV_BUT_PRESS:
 					case OS.Ph_EV_BUT_RELEASE:
 						PhPointerEvent_t pe = new PhPointerEvent_t ();
 						OS.memmove (pe, data, PhPointerEvent_t.sizeof);
-						setMouseState (event, pe, ev);
+						setInputState (event, SWT.Selection, pe.key_mods, pe.button_state);
 						break;
 				}	
 			}
@@ -445,7 +442,7 @@ int Pt_CB_ACTIVATE (int widget, int info) {
 }
 
 int Pt_CB_ARM (int widget, int info) {
-	postEvent (SWT.Arm);
+	sendEvent (SWT.Arm);
 	showMenu ();
 	return OS.Pt_CONTINUE;
 }
@@ -459,7 +456,7 @@ int Pt_CB_REALIZED (int widget, int info) {
 	*/
 	int topHandle = topHandle ();
 	int flags = (state & DISABLED) != 0 ? OS.Pt_BLOCKED | OS.Pt_GHOST : 0;
-	OS.PtSetResource (handle, OS.Pt_ARG_FLAGS, flags, OS.Pt_BLOCKED | OS.Pt_GHOST);
+	OS.PtSetResource (topHandle, OS.Pt_ARG_FLAGS, flags, OS.Pt_BLOCKED | OS.Pt_GHOST);
 	return OS.Pt_CONTINUE;
 }
 
@@ -482,7 +479,7 @@ void releaseWidget () {
  * Removes the listener from the collection of listeners who will
  * be notified when the arm events are generated for the control.
  *
- * @param listener the listener which should be notified
+ * @param listener the listener which should no longer be notified
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
@@ -506,7 +503,7 @@ public void removeArmListener (ArmListener listener) {
  * Removes the listener from the collection of listeners who will
  * be notified when the help events are generated for the control.
  *
- * @param listener the listener which should be notified
+ * @param listener the listener which should no longer be notified
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
@@ -533,7 +530,6 @@ void removeAccelerator () {
 	if ((accelerator & SWT.SHIFT) != 0) keyMods |= OS.Pk_KM_Shift;
 	if ((accelerator & SWT.CONTROL) != 0) keyMods |= OS.Pk_KM_Ctrl;
 	int key = (accelerator & ~(SWT.ALT | SWT.SHIFT | SWT.CONTROL));
-	Display display = getDisplay ();
 	int keyCode = Display.untranslateKey (key);
 	if (keyCode != 0) key = keyCode;
 	else key = Character.toLowerCase ((char)key);
@@ -545,7 +541,7 @@ void removeAccelerator () {
  * Removes the listener from the collection of listeners who will
  * be notified when the control is selected.
  *
- * @param listener the listener which should be notified
+ * @param listener the listener which should no longer be notified
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
@@ -582,6 +578,8 @@ void selectRadio () {
  * OR of zero or more modifier masks and a key. Examples:
  * <code>SWT.MOD1 | SWT.MOD2 | 'T', SWT.MOD3 | SWT.F2</code>.
  * <code>SWT.CONTROL | SWT.SHIFT | 'T', SWT.ALT | SWT.F2</code>.
+ * The default value is zero, indicating that the menu item does
+ * not have an accelerator.
  *
  * @param accelerator an integer that is the bit-wise OR of masks and a key
  *
@@ -601,7 +599,7 @@ public void setAccelerator (int accelerator) {
 
 /**
  * Enables the receiver if the argument is <code>true</code>,
- * and disables it otherwise. A disabled control is typically
+ * and disables it otherwise. A disabled menu item is typically
  * not selectable from the user interface and draws with an
  * inactive or "grayed" look.
  *
@@ -627,10 +625,11 @@ public void setEnabled (boolean enabled) {
 /**
  * Sets the image the receiver will display to the argument.
  * <p>
- * Note: This feature is not available on all window systems (for example, Window NT),
- * in which case, calling this method will silently do nothing.
+ * Note: This operation is a hint and is not supported on
+ * platforms that do not have this concept (for example, Windows NT).
+ * </p>
  *
- * @param menu the image to display
+ * @param image the image to display
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -656,7 +655,6 @@ public void setImage (Image image) {
 		* PtMenuButton, it can never be removed.  The fix is to set it to
 		* a small blank image.
 		*/
-		Display display = getDisplay ();
 		imageHandle = copyPhImage (display.nullImage);
 	}
 	int [] args = {
@@ -799,10 +797,10 @@ public void setText (String string) {
 	int i=0, j=0;
 	char mnemonic=0;
 	while (i < text.length) {
-		if (text [i] == '\t') {accel = true; break;};
-		if ((text [j++] = text [i++]) == Mnemonic) {
+		if (text [i] == '\t') {accel = true; break;}
+		if ((text [j++] = text [i++]) == '&') {
 			if (i == text.length) {continue;}
-			if (text [i] == Mnemonic) {i++; continue;}
+			if (text [i] == '&') {i++; continue;}
 			if (mnemonic == 0) mnemonic = text [i];
 			j--;
 		}
