@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -41,21 +41,26 @@ import org.eclipse.swt.events.*;
  */
 
 public class Button extends Control {
-	Image image;
+	String text = "";
+	Image image, image2;
+	ImageList imageList;
+	boolean ignoreMouse;
 	static final int ButtonProc;
 	static final TCHAR ButtonClass = new TCHAR (0,"BUTTON", true);
-	static final int CheckWidth, CheckHeight;
+	static final char [] SCROLLBAR = new char [] {'S', 'C', 'R', 'O', 'L', 'L', 'B', 'A', 'R', 0};
+	static final int CHECK_WIDTH, CHECK_HEIGHT;
+	static final int ICON_WIDTH = 128, ICON_HEIGHT = 128;
 	static {
 		int hBitmap = OS.LoadBitmap (0, OS.OBM_CHECKBOXES);
 		if (hBitmap == 0) {
-			CheckWidth = OS.GetSystemMetrics (OS.IsWinCE ? OS.SM_CXSMICON : OS.SM_CXVSCROLL);
-			CheckHeight = OS.GetSystemMetrics (OS.IsWinCE ? OS.SM_CYSMICON : OS.SM_CYVSCROLL);
+			CHECK_WIDTH = OS.GetSystemMetrics (OS.IsWinCE ? OS.SM_CXSMICON : OS.SM_CXVSCROLL);
+			CHECK_HEIGHT = OS.GetSystemMetrics (OS.IsWinCE ? OS.SM_CYSMICON : OS.SM_CYVSCROLL);
 		} else {
 			BITMAP bitmap = new BITMAP ();
 			OS.GetObject (hBitmap, BITMAP.sizeof, bitmap);
 			OS.DeleteObject (hBitmap);
-			CheckWidth = bitmap.bmWidth / 4;
-			CheckHeight =  bitmap.bmHeight / 3;
+			CHECK_WIDTH = bitmap.bmWidth / 4;
+			CHECK_HEIGHT =  bitmap.bmHeight / 3;
 		}
 		WNDCLASS lpWndClass = new WNDCLASS ();
 		OS.GetClassInfo (0, ButtonClass, lpWndClass);
@@ -102,6 +107,121 @@ public Button (Composite parent, int style) {
 	super (parent, checkStyle (style));
 }
 
+void _setImage (Image image) {
+	if (OS.COMCTL32_MAJOR >= 6 && OS.IsAppThemed ()) {
+		OS.SendMessage (handle, OS.BCM_SETIMAGELIST, 0, 0);
+		if (imageList != null) imageList.dispose ();
+		imageList = null;
+		if (image != null) {
+			imageList = new ImageList (style & SWT.RIGHT_TO_LEFT);
+			imageList.add (image);
+			BUTTON_IMAGELIST buttonImageList = new BUTTON_IMAGELIST ();
+			buttonImageList.himl = imageList.getHandle ();
+			if ((style & SWT.LEFT) != 0) buttonImageList.uAlign = OS.BUTTON_IMAGELIST_ALIGN_LEFT;
+			if ((style & SWT.CENTER) != 0) buttonImageList.uAlign = OS.BUTTON_IMAGELIST_ALIGN_CENTER;
+			if ((style & SWT.RIGHT) != 0) buttonImageList.uAlign = OS.BUTTON_IMAGELIST_ALIGN_RIGHT;
+			TCHAR buffer = new TCHAR (getCodePage (), "", true);
+			OS.SetWindowText (handle, buffer);
+			OS.SendMessage (handle, OS.BCM_SETIMAGELIST, 0, buttonImageList);
+		} else {
+			TCHAR buffer = new TCHAR (getCodePage (), text, true);
+			OS.SetWindowText (handle, buffer);
+			OS.SendMessage (handle, OS.BCM_SETIMAGELIST, 0, 0);
+		}
+	} else {
+		if (image2 != null) image2.dispose ();
+		image2 = null;
+		int hImage = 0, imageBits = 0, fImageType = 0;
+		if (image != null) {
+			switch (image.type) {
+				case SWT.BITMAP: {
+					Rectangle rect = image.getBounds ();
+					ImageData data = image.getImageData ();
+					switch (data.getTransparencyType ()) {
+						case SWT.TRANSPARENCY_PIXEL: 
+							if (rect.width <= ICON_WIDTH && rect.height <= ICON_HEIGHT) {
+								image2 = new Image (display, data, data.getTransparencyMask ());
+								hImage = image2.handle;
+								imageBits = OS.BS_ICON;
+								fImageType = OS.IMAGE_ICON;
+								break;
+							}
+							//FALL THROUGH
+						case SWT.TRANSPARENCY_ALPHA:
+							image2 = new Image (display, rect.width, rect.height);
+							GC gc = new GC (image2);
+							gc.setBackground (getBackground ());
+							gc.fillRectangle (rect);
+							gc.drawImage (image, 0, 0);
+							gc.dispose ();
+							hImage = image2.handle;
+							imageBits = OS.BS_BITMAP;
+							fImageType = OS.IMAGE_BITMAP;
+							break;
+						case SWT.TRANSPARENCY_NONE:
+							hImage = image.handle;
+							imageBits = OS.BS_BITMAP;
+							fImageType = OS.IMAGE_BITMAP;
+							break;
+					}
+					break;
+				}
+				case SWT.ICON: {
+					hImage = image.handle;
+					imageBits = OS.BS_ICON;
+					fImageType = OS.IMAGE_ICON;
+					break;
+				}
+			}
+			
+			/*
+			* Feature in Windows.  The button control mirrors its image when the
+			* flag WS_EX_LAYOUTRTL is set. This behaviour is not desirable in SWT.
+			* The fix is to set a mirrored version of real image in the button.
+			*/
+			if ((style & SWT.RIGHT_TO_LEFT) != 0) {
+				if (!OS.IsWinCE && OS.WIN32_VERSION >= OS.VERSION (4, 10)) {
+					Rectangle rect = image.getBounds ();
+					int hDC = OS.GetDC (handle);
+					int dstHdc = OS.CreateCompatibleDC (hDC);
+					int hBitmap = OS.CreateCompatibleBitmap (hDC, rect.width, rect.height);
+					int oldBitmap = OS.SelectObject (dstHdc, hBitmap);
+					OS.SetLayout (dstHdc, OS.LAYOUT_RTL);
+					if (fImageType == OS.IMAGE_BITMAP) {
+						int srcHdc = OS.CreateCompatibleDC (hDC);
+						int oldSrcBitmap = OS.SelectObject (srcHdc, hImage);
+						OS.SetLayout (dstHdc, 0);
+						OS.BitBlt (dstHdc, 0, 0, rect.width, rect.height, srcHdc, 0, 0, OS.SRCCOPY);
+						OS.SelectObject (srcHdc, oldSrcBitmap);
+						OS.DeleteDC (srcHdc);
+					} else {
+						int newBrush = OS.CreateSolidBrush (getBackgroundPixel ());
+						int oldBrush = OS.SelectObject (dstHdc, newBrush);
+						OS.PatBlt (dstHdc, 0, 0, rect.width, rect.height, OS.PATCOPY);
+						OS.DrawIconEx (dstHdc, 0, 0, hImage, 0, 0, 0, 0, OS.DI_NORMAL);
+						OS.SelectObject (dstHdc, oldBrush);
+						OS.DeleteObject (newBrush);
+					}
+					OS.SelectObject (dstHdc, oldBitmap);
+					OS.DeleteDC (dstHdc);
+					OS.ReleaseDC (handle, hDC);
+					if (image2 != null) image2.dispose ();
+					image2 = Image.win32_new (display, SWT.BITMAP, hBitmap);
+					imageBits = OS.BS_BITMAP;
+					fImageType = OS.IMAGE_BITMAP;
+					hImage = hBitmap;
+				}
+			}
+		}
+		int newBits = OS.GetWindowLong (handle, OS.GWL_STYLE);
+		int oldBits = newBits;
+		newBits &= ~(OS.BS_BITMAP | OS.BS_ICON);
+		newBits |= imageBits;
+		if (newBits != oldBits) OS.SetWindowLong (handle, OS.GWL_STYLE, newBits);
+		OS.SendMessage (handle, OS.BM_SETIMAGE, fImageType, hImage);
+	}
+}
+
 /**
  * Adds the listener to the collection of listeners who will
  * be notified when the control is selected, by sending
@@ -134,20 +254,21 @@ public void addSelectionListener (SelectionListener listener) {
 	addListener (SWT.DefaultSelection,typedListener);
 }
 
-int callWindowProc (int msg, int wParam, int lParam) {
+int callWindowProc (int hwnd, int msg, int wParam, int lParam) {
 	if (handle == 0) return 0;
-	return OS.CallWindowProc (ButtonProc, handle, msg, wParam, lParam);
+	return OS.CallWindowProc (ButtonProc, hwnd, msg, wParam, lParam);
 }
 
 static int checkStyle (int style) {
 	style = checkBits (style, SWT.PUSH, SWT.ARROW, SWT.CHECK, SWT.RADIO, SWT.TOGGLE, 0);
-	if ((style & SWT.PUSH) != 0) {
+	if ((style & (SWT.PUSH | SWT.TOGGLE)) != 0) {
 		return checkBits (style, SWT.CENTER, SWT.LEFT, SWT.RIGHT, 0, 0, 0);
 	}
-	if ((style & (SWT.CHECK | SWT.RADIO | SWT.TOGGLE)) != 0) {
+	if ((style & (SWT.CHECK | SWT.RADIO)) != 0) {
 		return checkBits (style, SWT.LEFT, SWT.RIGHT, SWT.CENTER, 0, 0, 0);
 	}
 	if ((style & SWT.ARROW) != 0) {
+		style |= SWT.NO_FOCUS;
 		return checkBits (style, SWT.UP, SWT.DOWN, SWT.LEFT, SWT.RIGHT, 0, 0);
 	}
 	return style;
@@ -155,15 +276,20 @@ static int checkStyle (int style) {
 
 void click () {
 	/*
-	* Note: BM_CLICK sends WM_LBUTTONDOWN and WM_LBUTTONUP.
+	* Feature in Windows.  BM_CLICK sends a fake WM_LBUTTONDOWN and
+	* WM_LBUTTONUP in order to click the button.  This causes the
+	* application to get unexpected mouse events.  The fix is to
+	* ignore mouse events when they are caused by BM_CLICK.
 	*/
+	ignoreMouse = true;
 	OS.SendMessage (handle, OS.BM_CLICK, 0, 0);
+	ignoreMouse = false;
 }
 
 public Point computeSize (int wHint, int hHint, boolean changed) {
 	checkWidget ();
 	int border = getBorderWidth ();
-	int width = border * 2, height = border * 2;
+	int width = 0, height = 0;
 	if ((style & SWT.ARROW) != 0) {
 		if ((style & (SWT.UP | SWT.DOWN)) != 0) {
 			width += OS.GetSystemMetrics (OS.SM_CXVSCROLL);
@@ -172,29 +298,37 @@ public Point computeSize (int wHint, int hHint, boolean changed) {
 			width += OS.GetSystemMetrics (OS.SM_CXHSCROLL);
 			height += OS.GetSystemMetrics (OS.SM_CYHSCROLL);
 		}
-		if (wHint != SWT.DEFAULT) width = wHint + (border * 2);
-		if (hHint != SWT.DEFAULT) height = hHint + (border * 2);
+		if (wHint != SWT.DEFAULT) width = wHint;
+		if (hHint != SWT.DEFAULT) height = hHint;
+		width += border * 2; height += border * 2;
 		return new Point (width, height);
 	}
 	int extra = 0;
-	int bits = OS.GetWindowLong (handle, OS.GWL_STYLE);
-	if ((bits & (OS.BS_BITMAP | OS.BS_ICON)) == 0) {
+	boolean hasImage;
+	if (OS.COMCTL32_MAJOR >= 6 && OS.IsAppThemed ()) {
+		BUTTON_IMAGELIST buttonImageList = new BUTTON_IMAGELIST();
+		OS.SendMessage (handle, OS.BCM_GETIMAGELIST, 0, buttonImageList);
+		hasImage = buttonImageList.himl != 0;
+	} else {
+		int bits = OS.GetWindowLong (handle, OS.GWL_STYLE);
+		hasImage = (bits & (OS.BS_BITMAP | OS.BS_ICON)) != 0;		
+	}
+	if (!hasImage) {
 		int oldFont = 0;
 		int hDC = OS.GetDC (handle);
 		int newFont = OS.SendMessage (handle, OS.WM_GETFONT, 0, 0);
 		if (newFont != 0) oldFont = OS.SelectObject (hDC, newFont);
-		TEXTMETRIC lptm = new TEXTMETRIC ();
+		TEXTMETRIC lptm = OS.IsUnicode ? (TEXTMETRIC) new TEXTMETRICW () : new TEXTMETRICA ();
 		OS.GetTextMetrics (hDC, lptm);
-		int length = OS.GetWindowTextLength (handle);
+		int length = text.length ();
 		if (length == 0) {
 			height += lptm.tmHeight;
 		} else {
 			extra = Math.max (8, lptm.tmAveCharWidth);
-			TCHAR buffer = new TCHAR (getCodePage (), length + 1);
-			OS.GetWindowText (handle, buffer, length + 1);
+			TCHAR buffer = new TCHAR (getCodePage (), text, true);
 			RECT rect = new RECT ();
 			int flags = OS.DT_CALCRECT | OS.DT_SINGLELINE;
-			OS.DrawText (hDC, buffer, length, rect, flags);
+			OS.DrawText (hDC, buffer, -1, rect, flags);
 			width += rect.right - rect.left;
 			height += rect.bottom - rect.top;
 		}
@@ -209,14 +343,15 @@ public Point computeSize (int wHint, int hHint, boolean changed) {
 		}
 	}
 	if ((style & (SWT.CHECK | SWT.RADIO)) != 0) {
-		width += CheckWidth + extra;
-		height = Math.max (height, CheckHeight + 3);
+		width += CHECK_WIDTH + extra;
+		height = Math.max (height, CHECK_HEIGHT + 3);
 	}
 	if ((style & (SWT.PUSH | SWT.TOGGLE)) != 0) {
 		width += 12;  height += 10;
 	}
-	if (wHint != SWT.DEFAULT) width = wHint + (border * 2);
-	if (hHint != SWT.DEFAULT) height = hHint + (border * 2);
+	if (wHint != SWT.DEFAULT) width = wHint;
+	if (hHint != SWT.DEFAULT) height = hHint;
+	width += border * 2; height += border * 2;
 	return new Point (width, height);
 }
 
@@ -231,6 +366,26 @@ int defaultForeground () {
 	return OS.GetSysColor (OS.COLOR_BTNTEXT);
 }
 
+void enableWidget (boolean enabled) {
+	super.enableWidget (enabled);
+	/*
+	* Bug in Windows.  When a Button control is right-to-left and
+	* is disabled, the first pixel of the text is clipped.  The fix
+	* is to append a space to the text.
+	*/
+	if ((style & SWT.RIGHT_TO_LEFT) != 0) {
+		if (OS.COMCTL32_MAJOR < 6 || !OS.IsAppThemed ()) {
+			int bits = OS.GetWindowLong (handle, OS.GWL_STYLE);
+			boolean hasImage = (bits & (OS.BS_BITMAP | OS.BS_ICON)) != 0;
+			if (!hasImage) {
+					String string = enabled ? text : text + " ";
+					TCHAR buffer = new TCHAR (getCodePage (), string, true);
+					OS.SetWindowText (handle, buffer);
+			}
+		}
+	}
+}
+
 /**
  * Returns a value which describes the position of the
  * text or image in the receiver. The value will be one of
@@ -313,7 +468,8 @@ public boolean getSelection () {
 
 /**
  * Returns the receiver's text, which will be an empty
- * string if it has never been set.
+ * string if it has never been set or if the receiver is
+ * an <code>ARROW</code> button.
  *
  * @return the receiver's text
  *
@@ -324,11 +480,8 @@ public boolean getSelection () {
  */
 public String getText () {
 	checkWidget ();
-	int length = OS.GetWindowTextLength (handle);
-	if (length == 0) return "";
-	TCHAR buffer = new TCHAR (getCodePage (), length + 1);
-	OS.GetWindowText (handle, buffer, length + 1);
-	return buffer.toString (0, length);
+	if ((style & SWT.ARROW) != 0) return "";
+	return text;
 }
 
 boolean isTabItem () {
@@ -357,6 +510,11 @@ boolean mnemonicMatch (char key) {
 
 void releaseWidget () {
 	super.releaseWidget ();
+	if (imageList != null) imageList.dispose ();
+	imageList = null;
+	if (image2 != null) image2.dispose ();
+	image2 = null;
+	text = null;
 	image = null;
 }
 
@@ -364,7 +522,7 @@ void releaseWidget () {
  * Removes the listener from the collection of listeners who will
  * be notified when the control is selected.
  *
- * @param listener the listener which should be notified
+ * @param listener the listener which should no longer be notified
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
@@ -444,6 +602,16 @@ public void setAlignment (int alignment) {
 	if ((style & SWT.CENTER) != 0) bits |= OS.BS_CENTER;
 	if ((style & SWT.RIGHT) != 0) bits |= OS.BS_RIGHT;
 	OS.SetWindowLong (handle, OS.GWL_STYLE, bits);
+	if (OS.COMCTL32_MAJOR >= 6 && OS.IsAppThemed ()) {
+		if (imageList != null) {
+			BUTTON_IMAGELIST buttonImageList = new BUTTON_IMAGELIST ();
+			buttonImageList.himl = imageList.getHandle ();
+			if ((style & SWT.LEFT) != 0) buttonImageList.uAlign = OS.BUTTON_IMAGELIST_ALIGN_LEFT;
+			if ((style & SWT.CENTER) != 0) buttonImageList.uAlign = OS.BUTTON_IMAGELIST_ALIGN_CENTER;
+			if ((style & SWT.RIGHT) != 0) buttonImageList.uAlign = OS.BUTTON_IMAGELIST_ALIGN_RIGHT;
+			OS.SendMessage (handle, OS.BCM_SETIMAGELIST, 0, buttonImageList);
+		}
+	}
 	OS.InvalidateRect (handle, null, true);
 }
 
@@ -461,17 +629,21 @@ void setDefault (boolean value) {
 	OS.SendMessage (handle, OS.BM_SETSTYLE, bits, 1);
 }
 
-public boolean setFocus () {
-	checkWidget();
-	if ((style & SWT.ARROW) != 0) return false;
-	return super.setFocus ();
+boolean setFixedFocus () {
+	/*
+	* Feature in Windows.  When a radio button gets focus, 
+	* it selects the button in WM_SETFOCUS.  The fix is to
+	* not assign focus to an unselected radio button.
+	*/
+	if ((style & SWT.RADIO) != 0 && !getSelection ()) return false;
+	return super.setFixedFocus ();
 }
 
 /**
  * Sets the receiver's image to the argument, which may be
- * null indicating that no image should be displayed.
+ * <code>null</code> indicating that no image should be displayed.
  *
- * @param image the image to display on the receiver (may be null)
+ * @param image the image to display on the receiver (may be <code>null</code>)
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_INVALID_ARGUMENT - if the image has been disposed</li>
@@ -483,32 +655,8 @@ public boolean setFocus () {
  */
 public void setImage (Image image) {
 	checkWidget ();
-	int hImage = 0, imageBits = 0, fImageType = 0;
-	if (image != null) {
-		if (image.isDisposed()) error(SWT.ERROR_INVALID_ARGUMENT);
-		hImage = image.handle;
-		switch (image.type) {
-			case SWT.BITMAP:
-				imageBits = OS.BS_BITMAP;
-				fImageType = OS.IMAGE_BITMAP;
-				break;
-			case SWT.ICON:
-				imageBits = OS.BS_ICON;
-				fImageType = OS.IMAGE_ICON;
-				break;
-			default:
-				return;
-		}
-	}
-	this.image = image;
-	int newBits = OS.GetWindowLong (handle, OS.GWL_STYLE);
-	int oldBits = newBits;
-	newBits &= ~(OS.BS_BITMAP | OS.BS_ICON);
-	newBits |= imageBits;
-	if (newBits != oldBits) {
-		OS.SetWindowLong (handle, OS.GWL_STYLE, newBits);
-	}
-	OS.SendMessage (handle, OS.BM_SETIMAGE, fImageType, hImage);
+	if (image != null && image.isDisposed()) error(SWT.ERROR_INVALID_ARGUMENT);
+	_setImage (this.image = image);
 }
 
 boolean setRadioFocus () {
@@ -531,9 +679,8 @@ boolean setSavedFocus () {
 	* it selects the button in WM_SETFOCUS.  If the previous
 	* saved focus widget was a radio button, allowing the shell
 	* to automatically restore the focus to the previous radio
-	* button will unexpectedly check that button.  The fix is
-	* to disallow focus to be restored to radio button that is
-	* not selected.
+	* button will unexpectedly check that button.  The fix is to
+	* not assign focus to an unselected radio button.
 	*/
 	if ((style & SWT.RADIO) != 0 && !getSelection ()) return false;
 	return super.setSavedFocus ();
@@ -602,18 +749,36 @@ public void setSelection (boolean selected) {
 public void setText (String string) {
 	checkWidget ();
 	if (string == null) error (SWT.ERROR_NULL_ARGUMENT);
-	int newBits = OS.GetWindowLong (handle, OS.GWL_STYLE);
-	int oldBits = newBits;
-	newBits &= ~(OS.BS_BITMAP | OS.BS_ICON);
-	if (newBits != oldBits) {
-		OS.SetWindowLong (handle, OS.GWL_STYLE, newBits);
+	if ((style & SWT.ARROW) != 0) return;
+	if (OS.COMCTL32_MAJOR >= 6 && OS.IsAppThemed ()) {
+		OS.SendMessage (handle, OS.BCM_SETIMAGELIST, 0, 0);
+		if (imageList != null) imageList.dispose ();
+		imageList = null;
+	} else {
+		int newBits = OS.GetWindowLong (handle, OS.GWL_STYLE);
+		int oldBits = newBits;
+		newBits &= ~(OS.BS_BITMAP | OS.BS_ICON);
+		if (newBits != oldBits) {
+			OS.SetWindowLong (handle, OS.GWL_STYLE, newBits);
+		}
+	}
+	text = string;
+	/*
+	* Bug in Windows.  When a Button control is right-to-left and
+	* is disabled, the first pixel of the text is clipped.  The fix
+	* is to append a space to the text.
+	*/
+	if ((style & SWT.RIGHT_TO_LEFT) != 0) {
+		if (OS.COMCTL32_MAJOR < 6 || !OS.IsAppThemed ()) {
+			string = OS.IsWindowEnabled (handle) ? text : text + " ";
+		}
 	}
 	TCHAR buffer = new TCHAR (getCodePage (), string, true);
 	OS.SetWindowText (handle, buffer);
 }
 
 int widgetStyle () {
-	int bits = super.widgetStyle () | OS.BS_NOTIFY;
+	int bits = super.widgetStyle ();
 	if ((style & SWT.FLAT) != 0) bits |= OS.BS_FLAT;
 	if ((style & SWT.ARROW) != 0) return bits | OS.BS_OWNERDRAW;
 	if ((style & SWT.LEFT) != 0) bits |= OS.BS_LEFT;
@@ -622,7 +787,7 @@ int widgetStyle () {
 	if ((style & SWT.PUSH) != 0) return bits | OS.BS_PUSHBUTTON | OS.WS_TABSTOP;
 	if ((style & SWT.CHECK) != 0) return bits | OS.BS_CHECKBOX | OS.WS_TABSTOP;
 	if ((style & SWT.RADIO) != 0) return bits | OS.BS_RADIOBUTTON;
-	if ((style & SWT.TOGGLE) != 0) return bits | OS.BS_PUSHLIKE | OS.BS_CHECKBOX;
+	if ((style & SWT.TOGGLE) != 0) return bits | OS.BS_PUSHLIKE | OS.BS_CHECKBOX | OS.WS_TABSTOP;
 	return bits | OS.BS_PUSHBUTTON | OS.WS_TABSTOP;
 }
 
@@ -651,6 +816,16 @@ LRESULT WM_KILLFOCUS (int wParam, int lParam) {
 	return result;
 }
 
+LRESULT WM_LBUTTONDOWN (int wParam, int lParam) {
+	if (ignoreMouse) return null;
+	return super.WM_LBUTTONDOWN (wParam, lParam);
+}
+
+LRESULT WM_LBUTTONUP (int wParam, int lParam) {
+	if (ignoreMouse) return null;
+	return super.WM_LBUTTONUP (wParam, lParam);
+}
+
 LRESULT WM_SETFOCUS (int wParam, int lParam) {
 	/*
 	* Feature in Windows. When Windows sets focus to
@@ -672,6 +847,29 @@ LRESULT WM_SETFOCUS (int wParam, int lParam) {
 	return result;
 }
 
+LRESULT WM_SYSCOLORCHANGE (int wParam, int lParam) {
+	LRESULT result = super.WM_SYSCOLORCHANGE (wParam, lParam);
+	if (result != null) return result;
+	if (image2 != null) _setImage (image);
+	return result;
+}
+
+LRESULT wmColorChild (int wParam, int lParam) {
+	LRESULT result = super.wmColorChild (wParam, lParam);
+	if (OS.COMCTL32_MAJOR >= 6 && OS.IsAppThemed ()) {
+		Control control = findThemeControl ();
+		if (control != null) {
+			OS.SetBkMode (wParam, OS.TRANSPARENT);
+			RECT rect = new RECT ();
+			OS.GetClientRect (control.handle, rect);
+			OS.MapWindowPoints (control.handle, handle, rect, 2);
+			control.drawThemeBackground (wParam, rect);
+			return new LRESULT (OS.GetStockObject (OS.NULL_BRUSH));
+		}
+	}
+	return result;
+}
+
 LRESULT wmCommandChild (int wParam, int lParam) {
 	int code = wParam >> 16;
 	switch (code) {
@@ -697,19 +895,39 @@ LRESULT wmDrawChild (int wParam, int lParam) {
 	if ((style & SWT.ARROW) == 0) return super.wmDrawChild (wParam, lParam);
 	DRAWITEMSTRUCT struct = new DRAWITEMSTRUCT ();
 	OS.MoveMemory (struct, lParam, DRAWITEMSTRUCT.sizeof);
-	int uState = OS.DFCS_SCROLLLEFT;
-	switch (style & (SWT.UP | SWT.DOWN | SWT.LEFT | SWT.RIGHT)) {
-		case SWT.UP: uState = OS.DFCS_SCROLLUP; break;
-		case SWT.DOWN: uState = OS.DFCS_SCROLLDOWN; break;
-		case SWT.LEFT: uState = OS.DFCS_SCROLLLEFT; break;
-		case SWT.RIGHT: uState = OS.DFCS_SCROLLRIGHT; break;
-	}
-	if (!getEnabled ()) uState |= OS.DFCS_INACTIVE;
-	if ((style & SWT.FLAT) == SWT.FLAT) uState |= OS.DFCS_FLAT;
-	if ((struct.itemState & OS.ODS_SELECTED) != 0) uState |= OS.DFCS_PUSHED;
 	RECT rect = new RECT ();
 	OS.SetRect (rect, struct.left, struct.top, struct.right, struct.bottom);
-	OS.DrawFrameControl (struct.hDC, rect, OS.DFC_SCROLL, uState);
+	if (OS.COMCTL32_MAJOR >= 6 && OS.IsAppThemed ()) {
+		int hTheme = OS.OpenThemeData (handle, SCROLLBAR);
+		int iStateId = OS.ABS_LEFTNORMAL;
+		switch (style & (SWT.UP | SWT.DOWN | SWT.LEFT | SWT.RIGHT)) {
+			case SWT.UP: iStateId = OS.ABS_UPNORMAL; break;
+			case SWT.DOWN: iStateId = OS.ABS_DOWNNORMAL; break;
+			case SWT.LEFT: iStateId = OS.ABS_LEFTNORMAL; break;
+			case SWT.RIGHT: iStateId = OS.ABS_RIGHTNORMAL; break;
+		}
+		/*
+		* NOTE: The normal, hot, pressed and disabled state is
+		* computed relying on the fact that the increment between
+		* the direction states is invariant (always separated by 4).
+		*/
+		if (!getEnabled ()) iStateId += OS.ABS_UPDISABLED - OS.ABS_UPNORMAL;
+		if ((struct.itemState & OS.ODS_SELECTED) != 0) iStateId += OS.ABS_UPPRESSED - OS.ABS_UPNORMAL;
+		OS.DrawThemeBackground (hTheme, struct.hDC, OS.SBP_ARROWBTN, iStateId, rect, null);
+		OS.CloseThemeData (hTheme);
+	} else {
+		int uState = OS.DFCS_SCROLLLEFT;
+		switch (style & (SWT.UP | SWT.DOWN | SWT.LEFT | SWT.RIGHT)) {
+			case SWT.UP: uState = OS.DFCS_SCROLLUP; break;
+			case SWT.DOWN: uState = OS.DFCS_SCROLLDOWN; break;
+			case SWT.LEFT: uState = OS.DFCS_SCROLLLEFT; break;
+			case SWT.RIGHT: uState = OS.DFCS_SCROLLRIGHT; break;
+		}
+		if (!getEnabled ()) uState |= OS.DFCS_INACTIVE;
+		if ((style & SWT.FLAT) == SWT.FLAT) uState |= OS.DFCS_FLAT;
+		if ((struct.itemState & OS.ODS_SELECTED) != 0) uState |= OS.DFCS_PUSHED;
+		OS.DrawFrameControl (struct.hDC, rect, OS.DFC_SCROLL, uState);
+	}
 	return null;
 }
 
