@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2004 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -28,7 +28,7 @@ import org.eclipse.swt.graphics.*;
  * </p><p>
  * <dl>
  * <dt><b>Styles:</b></dt>
- * <dd>(none)</dd>
+ * <dd>FLAT</dd>
  * <dt><b>Events:</b></dt>
  * <dd>(none)</dd>
  * </dl>
@@ -53,6 +53,7 @@ public class CoolBar extends Composite {
 		OS.GetClassInfo (0, ReBarClass, lpWndClass);
 		ReBarProc = lpWndClass.lpfnWndProc;
 	}
+	static final int SEPARATOR_WIDTH = 2;
 	static final int MAX_WIDTH = 0x7FFF;
 
 /**
@@ -87,9 +88,9 @@ public CoolBar (Composite parent, int style) {
 	super (parent, checkStyle (style));
 }
 
-int callWindowProc (int msg, int wParam, int lParam) {
+int callWindowProc (int hwnd, int msg, int wParam, int lParam) {
 	if (handle == 0) return 0;
-	return OS.CallWindowProc (ReBarProc, handle, msg, wParam, lParam);
+	return OS.CallWindowProc (ReBarProc, hwnd, msg, wParam, lParam);
 }
 
 static int checkStyle (int style) {
@@ -119,7 +120,7 @@ public Point computeSize (int wHint, int hHint, boolean changed) {
 		ignoreResize = true;
 		boolean redraw = false;
 		if (OS.IsWindowVisible (handle)) {
-			if (COMCTL32_MAJOR >= 6) {
+			if (OS.COMCTL32_MAJOR >= 6) {
 				redraw = true;
 				OS.UpdateWindow (handle);
 				OS.DefWindowProc (handle, OS.WM_SETREDRAW, 0, 0);
@@ -136,27 +137,27 @@ public Point computeSize (int wHint, int hHint, boolean changed) {
 		int oldWidth = oldRect.right - oldRect.left;
 		int oldHeight = oldRect.bottom - oldRect.top;
 		int flags = OS.SWP_NOACTIVATE | OS.SWP_NOMOVE | OS.SWP_NOREDRAW | OS.SWP_NOZORDER;	
-		OS.SetWindowPos (handle, 0, 0, 0, newWidth, newHeight, flags);
+		SetWindowPos (handle, 0, 0, 0, newWidth, newHeight, flags);
 		RECT rect = new RECT ();
 		OS.SendMessage (handle, OS.RB_GETRECT, count - 1, rect);
 		height = Math.max (height, rect.bottom);
-		OS.SetWindowPos (handle, 0, 0, 0, oldWidth, oldHeight, flags);
+		SetWindowPos (handle, 0, 0, 0, oldWidth, oldHeight, flags);
 		REBARBANDINFO rbBand = new REBARBANDINFO ();
 		rbBand.cbSize = REBARBANDINFO.sizeof;
 		rbBand.fMask = OS.RBBIM_IDEALSIZE | OS.RBBIM_STYLE;
 		int rowWidth = 0;
+		int separator = (style & SWT.FLAT) == 0 ? SEPARATOR_WIDTH : 0;
 		for (int i = 0; i < count; i++) {
 			OS.SendMessage(handle, OS.RB_GETBANDINFO, i, rbBand);
-			OS.SendMessage(handle, OS.RB_GETBANDBORDERS, i, rect);
 			if ((rbBand.fStyle & OS.RBBS_BREAK) != 0) {
-				width = Math.max(width, rowWidth);
+				width = Math.max(width, rowWidth - separator);
 				rowWidth = 0;
 			}
-			rowWidth += rbBand.cxIdeal + rect.left + rect.right + 2;
+			rowWidth += rbBand.cxIdeal + getMargin (i) + separator;
 		}
-		width = Math.max(width, rowWidth);
+		width = Math.max(width, rowWidth - separator);
 		if (redraw) {
-			if (COMCTL32_MAJOR >= 6) {
+			if (OS.COMCTL32_MAJOR >= 6) {
 				OS.DefWindowProc (handle, OS.WM_SETREDRAW, 1, 0);
 			} else {
 				OS.SendMessage (handle, OS.WM_SETREDRAW, 1, 0);
@@ -333,6 +334,32 @@ void destroyItem (CoolItem item) {
 	originalItems = newOriginals;
 }
 
+int getMargin (int index) {
+	int margin = 0;
+	if (OS.COMCTL32_MAJOR >= 6) {
+		MARGINS margins = new MARGINS ();
+		OS.SendMessage (handle, OS.RB_GETBANDMARGINS, 0, margins);
+		margin += margins.cxLeftWidth + margins.cxRightWidth;
+	}
+	RECT rect = new RECT ();
+	OS.SendMessage (handle, OS.RB_GETBANDBORDERS, index, rect);
+	if ((style & SWT.FLAT) != 0) {
+		/*
+		* Bug in Windows.  When the style bit  RBS_BANDBORDERS is not set
+		* the rectangle returned by RBS_BANDBORDERS is four pixels too small.
+		* The fix is to add four pixels to the result.
+		*/	
+		margin += rect.left + 4;
+	} else {
+		margin += rect.left + rect.right; 
+	}
+	return margin;
+}
+
+Control findThemeControl () {
+	return null;
+}
+
 /**
  * Returns the item that is currently displayed at the given,
  * zero-relative index. Throws an exception if the index is
@@ -348,9 +375,6 @@ void destroyItem (CoolItem item) {
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
- * @exception SWTError <ul>
- *    <li>ERROR_CANNOT_GET_ITEM - if the operation fails because of an operating system failure</li>
- * </ul>
  */
 public CoolItem getItem (int index) {
 	checkWidget ();
@@ -372,9 +396,6 @@ public CoolItem getItem (int index) {
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
- * @exception SWTError <ul>
- *    <li>ERROR_CANNOT_GET_COUNT - if the operation fails because of an operating system failure</li>
- * </ul>
  */
 public int getItemCount () {
 	checkWidget ();
@@ -401,9 +422,6 @@ public int getItemCount () {
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
- * @exception SWTError <ul>
- *    <li>ERROR_CANNOT_GET_ITEM - if the operation fails because of an operating system failure</li>
- * </ul>
  */
 public int [] getItemOrder () {
 	checkWidget ();
@@ -441,9 +459,6 @@ public int [] getItemOrder () {
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
- * @exception SWTError <ul>
- *    <li>ERROR_CANNOT_GET_ITEM - if the operation fails because of an operating system failure</li>
- * </ul>
  */
 public CoolItem [] getItems () {
 	checkWidget ();
@@ -478,11 +493,19 @@ public Point [] getItemSizes () {
 	REBARBANDINFO rbBand = new REBARBANDINFO ();
 	rbBand.cbSize = REBARBANDINFO.sizeof;
 	rbBand.fMask = OS.RBBIM_CHILDSIZE;
+	int separator = (style & SWT.FLAT) == 0 ? SEPARATOR_WIDTH : 0;
+	MARGINS margins = new MARGINS ();
 	for (int i=0; i<count; i++) {
 		RECT rect = new RECT ();
 		OS.SendMessage (handle, OS.RB_GETRECT, i, rect);
 		OS.SendMessage (handle, OS.RB_GETBANDINFO, i, rbBand);
-		sizes [i] = new Point (rect.right - rect.left + 2, rbBand.cyChild);
+		if (OS.COMCTL32_MAJOR >= 6) {
+			OS.SendMessage (handle, OS.RB_GETBANDMARGINS, 0, margins);
+			rect.left -= margins.cxLeftWidth;
+			rect.right += margins.cxRightWidth;
+		}
+		if (!isLastItemOfRow(i)) rect.right += separator;
+		sizes [i] = new Point (rect.right - rect.left, rbBand.cyChild);
 	}
 	return sizes;
 }
@@ -502,6 +525,16 @@ int getLastIndexOfRow (int index) {
 	return count - 1;
 }
 
+boolean isLastItemOfRow (int index) {
+	int count = OS.SendMessage (handle, OS.RB_GETBANDCOUNT, 0, 0);
+	if (index + 1 == count) return true;
+	REBARBANDINFO rbBand = new REBARBANDINFO ();
+	rbBand.cbSize = REBARBANDINFO.sizeof;
+	rbBand.fMask = OS.RBBIM_STYLE;
+	OS.SendMessage (handle, OS.RB_GETBANDINFO, index + 1, rbBand);
+	return (rbBand.fStyle & OS.RBBS_BREAK) != 0;
+}
+
 /**
  * Returns whether or not the receiver is 'locked'. When a coolbar
  * is locked, its items cannot be repositioned.
@@ -585,7 +618,8 @@ void resizeToPreferredWidth (int index) {
 		OS.SendMessage (handle, OS.RB_GETBANDINFO, index, rbBand);
 		RECT rect = new RECT ();
 		OS.SendMessage (handle, OS.RB_GETBANDBORDERS, index, rect);
-		rbBand.cx = rbBand.cxIdeal + rect.left + rect.right;
+		rbBand.cx = rbBand.cxIdeal + rect.left;
+		if ((style & SWT.FLAT) == 0) rbBand.cx += rect.right;
 		rbBand.fMask = OS.RBBIM_SIZE;
 		OS.SendMessage (handle, OS.RB_SETBANDINFO, index, rbBand);
 	}
@@ -610,12 +644,35 @@ void releaseWidget () {
 	super.releaseWidget();
 }
 
+void removeControl (Control control) {
+	super.removeControl (control);
+	for (int i=0; i<items.length; i++) {
+		CoolItem item = items [i];
+		if (item != null && item.control == control) {
+			item.setControl (null);
+		}
+	}
+}
+
 void setBackgroundPixel (int pixel) {
 	if (background == pixel) return;
 	background = pixel;
 	if (pixel == -1) pixel = defaultBackground ();
 	OS.SendMessage (handle, OS.RB_SETBKCOLOR, 0, pixel);
 	setItemColors (OS.SendMessage (handle, OS.RB_GETTEXTCOLOR, 0, 0), pixel);
+	/*
+	* Feature in Windows.  For some reason, Windows
+	* does not fully erase the coolbar area and coolbar
+	* items when you set the background.  The fix is
+	* to invalidate the coolbar area.
+	*/
+	if (!OS.IsWindowVisible (handle)) return;
+	if (OS.IsWinCE) {
+		OS.InvalidateRect (handle, null, true);
+	} else {
+		int flags = OS.RDW_ERASE | OS.RDW_FRAME | OS.RDW_INVALIDATE | OS.RDW_ALLCHILDREN;
+		OS.RedrawWindow (handle, null, 0, flags);
+	}
 }
 
 void setForegroundPixel (int pixel) {
@@ -670,9 +727,6 @@ void setItemColors (int foreColor, int backColor) {
  *    <li>ERROR_NULL_ARGUMENT - if item order or sizes is null</li>
  *    <li>ERROR_INVALID_ARGUMENT - if item order or sizes is not the same length as the number of items</li>
  * </ul>
- * @exception SWTError <ul>
- *    <li>ERROR_CANNOT_GET_ITEM - if the operation fails because of an operating system failure</li>
- * </ul>
  */
 public void setItemLayout (int [] itemOrder, int [] wrapIndices, Point [] sizes) {
 	checkWidget ();
@@ -699,9 +753,6 @@ public void setItemLayout (int [] itemOrder, int [] wrapIndices, Point [] sizes)
  *    <li>ERROR_NULL_ARGUMENT - if the item order is null</li>
  *    <li>ERROR_INVALID_ARGUMENT - if the item order is not the same length as the number of items</li>
  * </ul>
- * @exception SWTError <ul>
- *    <li>ERROR_CANNOT_GET_ITEM - if the operation fails because of an operating system failure</li>
- * </ul>
  */
 void setItemOrder (int [] itemOrder) {
 	if (itemOrder == null) error (SWT.ERROR_NULL_ARGUMENT);
@@ -853,7 +904,8 @@ public void setWrapIndices (int [] indices) {
 
 int widgetStyle () {
 	int bits = super.widgetStyle () | OS.CCS_NODIVIDER | OS.CCS_NORESIZE;
-	bits |= OS.RBS_VARHEIGHT | OS.RBS_BANDBORDERS | OS.RBS_DBLCLKTOGGLE;
+	bits |= OS.RBS_VARHEIGHT | OS.RBS_DBLCLKTOGGLE;
+	if ((style & SWT.FLAT) == 0) bits |= OS.RBS_BANDBORDERS; 
 	return bits;
 }
 
@@ -899,14 +951,17 @@ LRESULT WM_ERASEBKGND (int wParam, int lParam) {
 	* occupies when the size of the cool bar is larger
 	* than the space occupied by the cool bar items.
 	* The fix is to erase the cool bar background.
+	* 
+	* NOTE: On versions of Windows prior to XP, for
+	* some reason, the cool bar draws separators in
+	* WM_ERASEBKGND.  Therefore it is essential to run
+	* the cool bar window proc after the background has
+	* been erased.
+	* 
+	* On XP, this work around is unnecessary because
+	* the background is drawn using NM_CUSTOMDRAW.
 	*/
-	drawBackground (wParam);
-	
-	/*
-	* NOTE: The cool bar draws separators in WM_ERASEBKGND
-	* so it is essential to run the cool bar window proc
-	* after the background has been erased.
-	*/
+	if (OS.COMCTL32_MAJOR < 6) drawBackground (wParam);
 	return null;
 }
 
@@ -956,9 +1011,9 @@ LRESULT WM_SETREDRAW (int wParam, int lParam) {
 	* not running the default window proc or the rebar window
 	* proc.
 	*/
-	if (COMCTL32_MAJOR >= 6) return LRESULT.ZERO;
+	if (OS.COMCTL32_MAJOR >= 6) return LRESULT.ZERO;
 	Rectangle rect = getBounds ();		
-	int code = callWindowProc (OS.WM_SETREDRAW, wParam, lParam);
+	int code = callWindowProc (handle, OS.WM_SETREDRAW, wParam, lParam);
 	OS.DefWindowProc (handle, OS.WM_SETREDRAW, wParam, lParam);
 	if (!rect.equals (getBounds ())) {
 		parent.redraw (rect.x, rect.y, rect.width, rect.height, true);
@@ -966,19 +1021,41 @@ LRESULT WM_SETREDRAW (int wParam, int lParam) {
 	return new LRESULT (code);
 }
 
-LRESULT WM_SIZE(int wParam, int lParam) {
+LRESULT WM_SIZE (int wParam, int lParam) {
 	if (ignoreResize) {
-		int code = callWindowProc (OS.WM_SIZE, wParam, lParam);
+		int code = callWindowProc (handle, OS.WM_SIZE, wParam, lParam);
 		if (code == 0) return LRESULT.ZERO;
 		return new LRESULT (code);
 	}
-	return super.WM_SIZE(wParam, lParam);
+	return super.WM_SIZE (wParam, lParam);
 }
 
 LRESULT wmNotifyChild (int wParam, int lParam) {
 	NMHDR hdr = new NMHDR ();
 	OS.MoveMemory (hdr, lParam, NMHDR.sizeof);
 	switch (hdr.code) {
+		case OS.RBN_CHILDSIZE:
+			/*
+			* Bug in Windows.  When Windows sets the size of the rebar band
+			* child and the child is a combo box, the size of the drop down
+			* portion of the combo box is resized to zero.  The fix is to set
+			* the size of the control to the current size after the rebar has
+			* already resized it.  If the control is not a combo, this does
+			* nothing.  If the control is a combo, the drop down portion is
+			* recalculated.
+			*/
+			NMREBARCHILDSIZE lprbcs  = new NMREBARCHILDSIZE ();
+			OS.MoveMemory (lprbcs, lParam, NMREBARCHILDSIZE.sizeof);
+			if (lprbcs.uBand != -1) {
+				CoolItem item = items [lprbcs.wID];
+				Control control = item.control;
+				if (control != null) {
+					int width = lprbcs.rcChild_right - lprbcs.rcChild_left;
+					int height = lprbcs.rcChild_bottom - lprbcs.rcChild_top;
+					control.setBounds (lprbcs.rcChild_left, lprbcs.rcChild_top, width, height);
+				}
+			}
+			break;
 		case OS.RBN_HEIGHTCHANGE:
 			if (!ignoreResize) {
 				Point size = getSize ();
@@ -990,13 +1067,33 @@ LRESULT wmNotifyChild (int wParam, int lParam) {
 		case OS.RBN_CHEVRONPUSHED:
 			NMREBARCHEVRON lpnm = new NMREBARCHEVRON ();
 			OS.MoveMemory (lpnm, lParam, NMREBARCHEVRON.sizeof);
-			CoolItem child = items [lpnm.wID];
-			if (child != null) {
+			CoolItem item = items [lpnm.wID];
+			if (item != null) {
 				Event event = new Event();
 				event.detail = SWT.ARROW;
 				event.x = lpnm.left;
 				event.y = lpnm.bottom;
-				child.postEvent (SWT.Selection, event);
+				item.postEvent (SWT.Selection, event);
+			}
+			break;
+		case OS.NM_CUSTOMDRAW:
+			/*
+			* Bug in Windows.  On versions of Windows prior to XP,
+			* drawing the background color in NM_CUSTOMDRAW erases
+			* the separators.  The fix is to draw the background
+			* in WM_ERASEBKGND.
+			*/
+			if (OS.COMCTL32_MAJOR < 6) break;
+			if (background != -1 || (style & SWT.FLAT) != 0) {
+				NMCUSTOMDRAW nmcd = new NMCUSTOMDRAW ();
+				OS.MoveMemory (nmcd, lParam, NMCUSTOMDRAW.sizeof);
+				switch (nmcd.dwDrawStage) {
+					case OS.CDDS_PREERASE:
+						return new LRESULT (OS.CDRF_NOTIFYPOSTERASE);
+					case OS.CDDS_POSTERASE:
+						drawBackground (nmcd.hdc);
+						break;
+				}
 			}
 			break;
 	}
