@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -175,9 +175,9 @@ protected void checkSubclass () {
  * @see Layout
  * @see #getBounds
  * @see #getSize
- * @see CoolBar#getBorderWidth
- * @see CoolBar#computeTrim
- * @see CoolBar#getClientArea
+ * @see Control#getBorderWidth
+ * @see Scrollable#computeTrim
+ * @see Scrollable#getClientArea
  */
 public Point computeSize (int wHint, int hHint) {
 	checkWidget ();
@@ -186,10 +186,7 @@ public Point computeSize (int wHint, int hHint) {
 	int width = wHint, height = hHint;
 	if (wHint == SWT.DEFAULT) width = 32;
 	if (hHint == SWT.DEFAULT) height = 32;
-	int hwnd = parent.handle;
-	RECT rect = new RECT ();
-	OS.SendMessage (hwnd, OS.RB_GETBANDBORDERS, index, rect);
-	width += rect.left + rect.right + 2;
+	width += parent.getMargin (index);
 	return new Point (width, height);
 }
 
@@ -211,7 +208,16 @@ public Rectangle getBounds () {
 	int hwnd = parent.handle;
 	RECT rect = new RECT ();
 	OS.SendMessage (hwnd, OS.RB_GETRECT, index, rect);
-	int width = rect.right - rect.left + 2;
+	if (OS.COMCTL32_MAJOR >= 6) {
+		MARGINS margins = new MARGINS ();
+		OS.SendMessage (hwnd, OS.RB_GETBANDMARGINS, 0, margins);
+		rect.left -= margins.cxLeftWidth;
+		rect.right += margins.cxRightWidth;
+	}
+	if (!parent.isLastItemOfRow (index)) {
+		rect.right += (parent.style & SWT.FLAT) == 0 ? CoolBar.SEPARATOR_WIDTH : 0;
+	}
+	int width = rect.right - rect.left;
 	int height = rect.bottom - rect.top;
 	return new Rectangle (rect.left, rect.top, width, height);
 }
@@ -229,9 +235,14 @@ Rectangle getClientArea () {
 	RECT rect = new RECT ();
 	OS.SendMessage (hwnd, OS.RB_GETRECT, index, rect);
 	int x = rect.left + insetRect.left;
-	int y = rect.top + insetRect.top;
-	int width = rect.right - rect.left - (insetRect.left + insetRect.right);
-	int height = rect.bottom - rect.top - (insetRect.top + insetRect.bottom);
+	int y = rect.top;
+	int width = rect.right - rect.left - insetRect.left;
+	int height = rect.bottom - rect.top;
+	if ((parent.style & SWT.FLAT) == 0) {
+		y += insetRect.top;
+		width -= insetRect.right;
+		height -= insetRect.top + insetRect.bottom; 
+	}
 	if (index == 0) {
 		REBARBANDINFO rbBand = new REBARBANDINFO ();
 		rbBand.cbSize = REBARBANDINFO.sizeof;
@@ -257,12 +268,6 @@ public Control getControl () {
 	return control;
 }
 
-public Display getDisplay () {
-	CoolBar parent = this.parent;
-	if (parent == null) error (SWT.ERROR_WIDGET_DISPOSED);
-	return parent.getDisplay ();
-}
-
 /**
  * Returns the receiver's parent, which must be a <code>CoolBar</code>.
  *
@@ -317,8 +322,7 @@ public void setControl (Control control) {
 	}
 	Control oldControl = this.control, newControl = control;
 	int hwnd = parent.handle;
-	int hwndChild = 0;
-	if (newControl != null) hwndChild = control.handle;
+	int hwndChild = newControl != null ? control.topHandle () : 0;
 	REBARBANDINFO rbBand = new REBARBANDINFO ();
 	rbBand.cbSize = REBARBANDINFO.sizeof;
 	rbBand.fMask = OS.RBBIM_CHILD;
@@ -342,7 +346,7 @@ public void setControl (Control control) {
 	if (showOld) oldControl.setVisible (true);
 	if (hwndAbove != 0 && hwndAbove != hwndChild) {
 		int flags = OS.SWP_NOSIZE | OS.SWP_NOMOVE | OS.SWP_NOACTIVATE; 
-		OS.SetWindowPos (hwndChild, hwndAbove, 0, 0, 0, 0, flags);
+		SetWindowPos (hwndChild, hwndAbove, 0, 0, 0, 0, flags);
 	}
 }
 
@@ -367,9 +371,7 @@ public Point getPreferredSize () {
 	rbBand.cbSize = REBARBANDINFO.sizeof;
 	rbBand.fMask = OS.RBBIM_CHILDSIZE | OS.RBBIM_IDEALSIZE;
 	OS.SendMessage (hwnd, OS.RB_GETBANDINFO, index, rbBand);
-	RECT rect = new RECT ();
-	OS.SendMessage (hwnd, OS.RB_GETBANDBORDERS, index, rect);
-	int width = rbBand.cxIdeal + rect.left + rect.right + 2;
+	int width = rbBand.cxIdeal + parent.getMargin (index);
 	return new Point (width, rbBand.cyMinChild);
 }
 
@@ -388,10 +390,10 @@ public void setPreferredSize (int width, int height) {
 	checkWidget ();
 	int index = parent.indexOf (this);
 	if (index == -1) return;
+	width = Math.max (0, width);
+	height = Math.max (0, height);
 	ideal = true;
 	int hwnd = parent.handle;
-	RECT rect = new RECT ();
-	OS.SendMessage (hwnd, OS.RB_GETBANDBORDERS, index, rect);
 	REBARBANDINFO rbBand = new REBARBANDINFO ();
 	rbBand.cbSize = REBARBANDINFO.sizeof;
 	
@@ -401,7 +403,7 @@ public void setPreferredSize (int width, int height) {
 	
 	/* Set the size fields we are currently modifying. */
 	rbBand.fMask = OS.RBBIM_CHILDSIZE | OS.RBBIM_IDEALSIZE;
-	rbBand.cxIdeal = width - rect.left - rect.right - 2;
+	rbBand.cxIdeal = Math.max (0, width - parent.getMargin (index));
 	rbBand.cyMaxChild = height;
 	if (!minimum) rbBand.cyMinChild = height;
 	OS.SendMessage (hwnd, OS.RB_SETBANDINFO, index, rbBand);
@@ -411,7 +413,6 @@ public void setPreferredSize (int width, int height) {
  * Sets the receiver's ideal size to the point specified by the argument.
  *
  * @param size the new ideal size for the receiver
- * @param height the new ideal height for the receiver
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the point is null</li>
@@ -447,7 +448,16 @@ public Point getSize() {
 	int hwnd = parent.handle;
 	RECT rect = new RECT ();
 	OS.SendMessage (hwnd, OS.RB_GETRECT, index, rect);
-	int width = rect.right - rect.left + 2;
+	if (OS.COMCTL32_MAJOR >= 6) {
+		MARGINS margins = new MARGINS ();
+		OS.SendMessage (hwnd, OS.RB_GETBANDMARGINS, 0, margins);
+		rect.left -= margins.cxLeftWidth;
+		rect.right += margins.cxRightWidth;
+	}
+	if (!parent.isLastItemOfRow (index)) {
+		rect.right += (parent.style & SWT.FLAT) == 0 ? CoolBar.SEPARATOR_WIDTH : 0;
+	}
+	int width = rect.right - rect.left;
 	int height = rect.bottom - rect.top;
 	return new Point (width, height);
 }
@@ -472,39 +482,33 @@ public void setSize (int width, int height) {
 	checkWidget ();
 	int index = parent.indexOf (this);
 	if (index == -1) return;
+	width = Math.max (0, width);
+	height = Math.max (0, height);
 	int hwnd = parent.handle;
 	REBARBANDINFO rbBand = new REBARBANDINFO ();
 	rbBand.cbSize = REBARBANDINFO.sizeof;
 	
-	/*
-	* Do not set the size for the last item on the row.
-	*/
-	int count = OS.SendMessage (hwnd, OS.RB_GETBANDCOUNT, 0, 0);
-	boolean isLastItem;
-	if (index + 1 == count) {
-		isLastItem = true;
-	} else {
-		rbBand.fMask = OS.RBBIM_STYLE;
-		OS.SendMessage (hwnd, OS.RB_GETBANDINFO, index + 1, rbBand);
-		isLastItem = (rbBand.fStyle & OS.RBBS_BREAK) != 0;
-		rbBand.fMask = 0;
-	}
-	
 	/* Get the child size fields first so we don't overwrite them. */
 	rbBand.fMask = OS.RBBIM_CHILDSIZE | OS.RBBIM_IDEALSIZE;
 	OS.SendMessage (hwnd, OS.RB_GETBANDINFO, index, rbBand);
 	
 	/* Set the size fields we are currently modifying. */
-	rbBand.fMask = OS.RBBIM_CHILDSIZE | OS.RBBIM_IDEALSIZE;
-	if (!ideal) {
-		RECT rect = new RECT ();
-		OS.SendMessage (hwnd, OS.RB_GETBANDBORDERS, index, rect);
-		rbBand.cxIdeal = width - rect.left - rect.right - 2;
-	}
+	if (!ideal) rbBand.cxIdeal = Math.max (0, width - parent.getMargin (index));
 	if (!minimum) rbBand.cyMinChild = height;
 	rbBand.cyChild = rbBand.cyMaxChild = height;
-	if (!isLastItem) {
-		rbBand.cx = width - 2;
+	
+	/*
+	* Do not set the size for the last item on the row.
+	*/	
+	if (!parent.isLastItemOfRow (index)) {
+		if (OS.COMCTL32_MAJOR >= 6) {
+			MARGINS margins = new MARGINS ();
+			OS.SendMessage (hwnd, OS.RB_GETBANDMARGINS, 0, margins);
+			width -= margins.cxLeftWidth;
+			width -= margins.cxRightWidth;
+		}
+		int separator = (parent.style & SWT.FLAT) == 0 ? CoolBar.SEPARATOR_WIDTH : 0;
+		rbBand.cx = width - separator;
 		rbBand.fMask |= OS.RBBIM_SIZE;
 	}
 	OS.SendMessage (hwnd, OS.RB_SETBANDINFO, index, rbBand);
@@ -519,7 +523,6 @@ public void setSize (int width, int height) {
  * </p>
  *
  * @param size the new size for the receiver
- * @param height the new height for the receiver
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the point is null</li>
@@ -577,6 +580,8 @@ public void setMinimumSize (int width, int height) {
 	checkWidget ();
 	int index = parent.indexOf (this);
 	if (index == -1) return;
+	width = Math.max (0, width);
+	height = Math.max (0, height);
 	minimum = true;
 	int hwnd = parent.handle;
 	REBARBANDINFO rbBand = new REBARBANDINFO ();
@@ -643,7 +648,7 @@ void setWrap(boolean wrap) {
  * Removes the listener from the collection of listeners that
  * will be notified when the control is selected.
  *
- * @param listener the listener which should be notified
+ * @param listener the listener which should no longer be notified
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
