@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -66,7 +66,7 @@ import org.eclipse.swt.events.*;
  * it would be upgraded to <code>APPLICATION_MODAL</code>.
  * <dl>
  * <dt><b>Styles:</b></dt>
- * <dd>BORDER, CLOSE, MIN, MAX, NO_TRIM, RESIZE, TITLE</dd>
+ * <dd>BORDER, CLOSE, MIN, MAX, NO_TRIM, RESIZE, TITLE, ON_TOP, TOOL</dd>
  * <dd>APPLICATION_MODAL, MODELESS, PRIMARY_MODAL, SYSTEM_MODAL</dd>
  * <dt><b>Events:</b></dt>
  * <dd>Activate, Close, Deactivate, Deiconify, Iconify</dd>
@@ -99,14 +99,14 @@ import org.eclipse.swt.events.*;
  * @see SWT
  */
 public class Shell extends Decorations {
-	Display display;
 	Menu activeMenu;
-	int hIMC;
+	int hIMC, hwndMDIClient, toolTipHandle, lpstrTip;
+	int minWidth = SWT.DEFAULT, minHeight = SWT.DEFAULT;
 	int [] brushes;
 	boolean showWithParent;
-	int toolTipHandle, lpstrTip;
 	Control lastActive;
 	SHACTIVATEINFO psai;
+	Region region;
 	static final int DialogProc;
 	static final TCHAR DialogClass = new TCHAR (0, OS.IsWinCE ? "Dialog" : "#32770", true);
 	static {
@@ -245,6 +245,9 @@ Shell (Display display, Shell parent, int style, int handle) {
 	if (!display.isValidThread ()) {
 		error (SWT.ERROR_THREAD_INVALID_ACCESS);
 	}
+	if (parent != null && parent.isDisposed ()) {
+		error (SWT.ERROR_INVALID_ARGUMENT);	
+	}
 	this.style = checkStyle (style);
 	this.parent = parent;
 	this.display = display;
@@ -267,7 +270,7 @@ Shell (Display display, Shell parent, int style, int handle) {
  * @param parent a shell which will be the parent of the new instance
  *
  * @exception IllegalArgumentException <ul>
- *    <li>ERROR_NULL_ARGUMENT - if the parent is null</li>
+ *    <li>ERROR_INVALID_ARGUMENT - if the parent is disposed</li> 
  * </ul>
  * @exception SWTException <ul>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the parent</li>
@@ -301,6 +304,9 @@ public Shell (Shell parent) {
  * @param parent a shell which will be the parent of the new instance
  * @param style the style of control to construct
  *
+ * @exception IllegalArgumentException <ul>
+ *    <li>ERROR_INVALID_ARGUMENT - if the parent is disposed</li> 
+ * </ul>
  * @exception SWTException <ul>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the parent</li>
  *    <li>ERROR_INVALID_SUBCLASS - if this class is not an allowed subclass</li>
@@ -315,13 +321,15 @@ public Shell (Shell parent) {
  * @see SWT#NO_TRIM
  * @see SWT#SHELL_TRIM
  * @see SWT#DIALOG_TRIM
+ * @see SWT#ON_TOP
+ * @see SWT#TOOL
  * @see SWT#MODELESS
  * @see SWT#PRIMARY_MODAL
  * @see SWT#APPLICATION_MODAL
  * @see SWT#SYSTEM_MODAL
  */
 public Shell (Shell parent, int style) {
-	this (parent != null ? parent.getDisplay () : null, parent, style, 0);
+	this (parent != null ? parent.display : null, parent, style, 0);
 }
 
 /**	 
@@ -336,6 +344,7 @@ public Shell (Shell parent, int style) {
  *
  * @param display the display for the shell
  * @param handle the handle for the shell
+ * @return a new shell object containing the specified display and handle
  */
 public static Shell win32_new (Display display, int handle) {
 	return new Shell (display, null, SWT.NO_TRIM, handle);
@@ -381,17 +390,24 @@ public void addShellListener (ShellListener listener) {
 	addListener (SWT.Deactivate, typedListener);
 }
 
-int callWindowProc (int msg, int wParam, int lParam) {
+int callWindowProc (int hwnd, int msg, int wParam, int lParam) {
+	if (handle == 0) return 0;
+	if ((style & SWT.TOOL) != 0) {
+		int trim = SWT.TITLE | SWT.CLOSE | SWT.MIN | SWT.MAX | SWT.BORDER | SWT.RESIZE;
+		if ((style & trim) == 0) return OS.DefWindowProc (hwnd, msg, wParam, lParam);
+	}
 	if (parent != null) {
-		if (handle == 0) return 0;
 		switch (msg) {
 			case OS.WM_KILLFOCUS:
 			case OS.WM_SETFOCUS: 
-				return OS.DefWindowProc (handle, msg, wParam, lParam);
+				return OS.DefWindowProc (hwnd, msg, wParam, lParam);
 		}
-		return OS.CallWindowProc (DialogProc, handle, msg, wParam, lParam);
+		return OS.CallWindowProc (DialogProc, hwnd, msg, wParam, lParam);
 	}
-	return super.callWindowProc (msg, wParam, lParam);
+	if (hwndMDIClient != 0) {
+		return OS.DefFrameProc (hwnd, hwndMDIClient, msg, wParam, lParam);
+	}
+	return OS.DefWindowProc (hwnd, msg, wParam, lParam);
 }
 
 /**
@@ -406,11 +422,12 @@ int callWindowProc (int msg, int wParam, int lParam) {
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
  *
+ * @see SWT#Close
  * @see #dispose
  */
 public void close () {
 	checkWidget ();
-	OS.PostMessage (handle, OS.WM_CLOSE, 0, 0);
+	closeWidget ();
 }
 
 void createHandle () {
@@ -429,6 +446,7 @@ void createHandle () {
 	*/
 //	if ((style & SWT.ON_TOP) != 0) display.lockActiveWindow = true;
 	super.createHandle ();
+	
 	/*
 	* The following code is intentionally commented.
 	*/
@@ -442,7 +460,16 @@ void createHandle () {
 		if ((style & SWT.NO_TRIM) == 0) {
 			if ((style & (SWT.BORDER | SWT.RESIZE)) == 0) bits |= OS.WS_BORDER;
 		}
+		/*
+		* Bug in Windows.  When the WS_CAPTION bits are cleared using
+		* SetWindowLong(), Windows does not resize the client area of
+		* the window to get rid of the caption until the first resize.
+		* The fix is to use SetWindowPos() with SWP_DRAWFRAME to force
+		* the frame to be redrawn and resized.
+		*/
 		OS.SetWindowLong (handle, OS.GWL_STYLE, bits);
+		int flags = OS.SWP_DRAWFRAME | OS.SWP_NOMOVE | OS.SWP_NOSIZE | OS.SWP_NOZORDER | OS.SWP_NOACTIVATE;
+		SetWindowPos (handle, 0, 0, 0, 0, 0, flags);
 		if (OS.IsWinCE) setMaximized (true);
 		if (OS.IsPPC) {
 			psai = new SHACTIVATEINFO ();
@@ -474,6 +501,19 @@ public void dispose () {
 //	if (oldDisplay != null) oldDisplay.update ();
 }
 
+void enableWidget (boolean enabled) {
+	if (enabled) {
+		state &= ~DISABLED;
+	} else {
+		state |= DISABLED;
+	}
+	if (Display.TrimEnabled) {
+		if (isActive ()) setItemEnabled (OS.SC_CLOSE, enabled);
+	} else {
+		OS.EnableWindow (handle, enabled);
+	}
+}
+
 int findBrush (int pixel) {
 	if (pixel == OS.GetSysColor (OS.COLOR_BTNFACE)) {
 		return OS.GetSysColorBrush (OS.COLOR_BTNFACE);
@@ -497,16 +537,29 @@ int findBrush (int pixel) {
 	return hBrush;
 }
 
-int findCursor () {
-	return hCursor;
+Cursor findCursor () {
+	return cursor;
+}
+
+Control findThemeControl () {
+	return null;
+}
+
+void fixShell (Shell newShell, Control control) {
+	if (this == newShell) return;
+	if (control == lastActive) setActiveControl (null);
+	if (toolTipHandle != 0) {
+		setToolTipText (control.handle, null);
+	}
+	newShell.setToolTipText (control.handle, control.toolTipText);
 }
 
 /**
- * Moves the receiver to the top of the drawing order for
- * the display on which it was created (so that all other
- * shells on that display, which are not the receiver's
- * children will be drawn behind it) and forces the window
- * manager to make the shell active.
+ * If the receiver is visible, moves it to the top of the 
+ * drawing order for the display on which it was created 
+ * (so that all other shells on that display, which are not 
+ * the receiver's children will be drawn behind it) and forces 
+ * the window manager to make the shell active.
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -524,6 +577,7 @@ int findCursor () {
  */
 public void forceActive () {
 	checkWidget ();
+	if(!isVisible()) return;
 	OS.SetForegroundWindow (handle);
 }
 
@@ -543,11 +597,6 @@ public Rectangle getBounds () {
 	return new Rectangle (rect.left, rect.top, width, height);
 }
 
-public Display getDisplay () {
-	if (display == null) error (SWT.ERROR_WIDGET_DISPOSED);
-	return display;
-}
-
 public boolean getEnabled () {
 	checkWidget ();
 	return (state & DISABLED) == 0;
@@ -599,11 +648,79 @@ public Point getLocation () {
 	return new Point (rect.left, rect.top);
 }
 
+/**
+ * Returns a point describing the minimum receiver's size. The
+ * x coordinate of the result is the minimum width of the receiver.
+ * The y coordinate of the result is the minimum height of the
+ * receiver.
+ *
+ * @return the receiver's size
+ *
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ * 
+ * @since 3.1
+ */
+public Point getMinimumSize () {
+	checkWidget ();
+	int width = Math.max (0, minWidth);
+	int trim = SWT.TITLE | SWT.CLOSE | SWT.MIN | SWT.MAX;
+	if ((style & SWT.NO_TRIM) == 0 && (style & trim) != 0) {
+		width = Math.max (width, OS.GetSystemMetrics (OS.SM_CXMINTRACK));
+	}
+	int height = Math.max (0, minHeight);
+	if ((style & SWT.NO_TRIM) == 0 && (style & trim) != 0) {
+		if ((style & SWT.RESIZE) != 0) {
+			height = Math.max (height, OS.GetSystemMetrics (OS.SM_CYMINTRACK));
+		} else {
+			RECT rect = new RECT ();
+			int bits1 = OS.GetWindowLong (handle, OS.GWL_STYLE);
+			int bits2 = OS.GetWindowLong (handle, OS.GWL_EXSTYLE);
+			OS.AdjustWindowRectEx (rect, bits1, false, bits2);
+			height = Math.max (height, rect.bottom - rect.top);
+		}
+	}
+	return new Point (width,  height);
+}
+
+/** 
+ * Returns the region that defines the shape of the shell,
+ * or null if the shell has the default shape.
+ *
+ * @return the region that defines the shape of the shell (or null)
+ *	
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ *
+ * @since 3.0
+ *
+ */
+public Region getRegion () {
+	checkWidget ();
+	return region;
+}
+
 public Shell getShell () {
 	checkWidget ();
 	return this;
 }
 
+public Point getSize () {
+	checkWidget ();
+	if (!OS.IsWinCE) {
+		if (OS.IsIconic (handle)) return super.getSize ();
+	}
+	RECT rect = new RECT ();
+	OS.GetWindowRect (handle, rect);
+	int width = rect.right - rect.left;
+	int height = rect.bottom - rect.top;
+	return new Point (width, height);
+}
+
 /**
  * Returns an array containing all shells which are 
  * descendents of the receiver.
@@ -640,18 +757,46 @@ public Shell [] getShells () {
 	return result;
 }
 
+public boolean isLayoutDeferred () {
+	checkWidget ();
+	return layoutCount > 0;
+}
+
 public boolean isEnabled () {
 	checkWidget ();
 	return getEnabled ();
 }
 
+public boolean isVisible () {
+	checkWidget ();
+	return getVisible ();
+}
+
+int hwndMDIClient () {
+	if (hwndMDIClient == 0) {
+		int widgetStyle = OS.MDIS_ALLCHILDSTYLES | OS.WS_CHILD | OS.WS_CLIPCHILDREN | OS.WS_CLIPSIBLINGS;
+		hwndMDIClient = OS.CreateWindowEx (
+			0,
+			new TCHAR (0, "MDICLIENT", true),
+			null,
+			widgetStyle, 
+			0, 0, 0, 0,
+			handle,
+			0,
+			OS.GetModuleHandle (null),
+			new CREATESTRUCT ());
+//		OS.ShowWindow (hwndMDIClient, OS.SW_SHOW);
+	}
+	return hwndMDIClient;
+}
+
 /**
  * Moves the receiver to the top of the drawing order for
  * the display on which it was created (so that all other
  * shells on that display, which are not the receiver's
  * children will be drawn behind it), marks it visible,
- * and sets focus to its default button (if it has one)
- * and asks the window manager to make the shell active.
+ * sets the focus and asks the window manager to make the
+ * shell active.
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -669,6 +814,7 @@ public boolean isEnabled () {
 public void open () {
 	checkWidget ();
 	bringToTop ();
+	if (isDisposed ()) return;
 	/*
 	* Feature on WinCE PPC.  A new application becomes
 	* the foreground application only if it has at least
@@ -679,6 +825,7 @@ public void open () {
 	if (OS.IsWinCE) OS.SetForegroundWindow (handle);
 	OS.SendMessage (handle, OS.WM_CHANGEUISTATE, OS.UIS_INITIALIZE, 0);
 	setVisible (true);
+	if (isDisposed ()) return;
 	/*
 	* Bug in Windows XP.  Despite the fact that an icon has been
 	* set for a window, the task bar displays the wrong icon the
@@ -686,14 +833,26 @@ public void open () {
 	* a call to BringToTop(), when a long time elapses between the
 	* ShowWindow() and the time the event queue is read.  The icon
 	* in the window trimming is correct but the one in the task
-	* bar does not get updated until a new icon is set into the
-	* window or the window text is changed.  The fix is to call
-	* PeekMessage() with the flag PM_NOREMOVE to touch the event
-	* queue but not dispatch events.
+	* bar does not get updated.  The fix is to call PeekMessage()
+	* with the flag PM_NOREMOVE and PM_QS_SENDMESSAGE to respond
+	* to a cross thread WM_GETICON.
+	* 
+	* NOTE: This allows other cross thread messages to be delivered,
+	* most notably WM_ACTIVATE.
 	*/
 	MSG msg = new MSG ();
-	OS.PeekMessage (msg, 0, 0, 0, OS.PM_NOREMOVE);
-	if (!restoreFocus ()) traverseGroup (true);
+	int flags = OS.PM_NOREMOVE | OS.PM_NOYIELD | OS.PM_QS_SENDMESSAGE;
+	OS.PeekMessage (msg, 0, 0, 0, flags);
+	if (!restoreFocus () && !traverseGroup (true)) setFocus ();
+}
+
+void releaseChild () {
+	/* Do nothing */
+}
+
+void releaseHandle () {
+	super.releaseHandle ();
+	hwndMDIClient = 0;
 }
 
 void releaseShells () {
@@ -709,7 +868,6 @@ void releaseWidget () {
 	super.releaseWidget ();
 	activeMenu = null;
 	display.clearModal (this);
-	display = null;
 	if (lpstrTip != 0) {
 		int hHeap = OS.GetProcessHeap ();
 		OS.HeapFree (hHeap, 0, lpstrTip);
@@ -727,10 +885,11 @@ void releaseWidget () {
 		if (hIMC != 0) OS.ImmDestroyContext (hIMC);
 	}
 	lastActive = null;
+	region = null;
 }
 
-void remove (Menu menu) {
-	super.remove (menu);
+void removeMenu (Menu menu) {
+	super.removeMenu (menu);
 	if (menu == activeMenu) activeMenu = null;
 }
 
@@ -777,11 +936,11 @@ LRESULT selectPalette (int hPalette) {
 }
 
 /**
- * Moves the receiver to the top of the drawing order for
- * the display on which it was created (so that all other
- * shells on that display, which are not the receiver's
- * children will be drawn behind it) and asks the window
- * manager to make the shell active.
+ * If the receiver is visible, moves it to the top of the 
+ * drawing order for the display on which it was created 
+ * (so that all other shells on that display, which are not 
+ * the receiver's children will be drawn behind it) and asks 
+ * the window manager to make the shell active 
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -799,7 +958,9 @@ LRESULT selectPalette (int hPalette) {
  */
 public void setActive () {
 	checkWidget ();
+	if(!isVisible()) return;
 	bringToTop ();
+	// widget could be disposed at this point
 }
 
 void setActiveControl (Control control) {
@@ -839,34 +1000,16 @@ void setActiveControl (Control control) {
 	}
 }
 
-void setBounds (int x, int y, int width, int height, int flags) {
-	if (OS.IsWinCE) {
-		swFlags = OS.SW_RESTORE;
-		if ((style & SWT.NO_TRIM) == 0) {
-			/* Set the WS_CAPTION bits when no longer maximized */
-			int bits = OS.GetWindowLong (handle, OS.GWL_STYLE);
-			if ((bits & OS.WS_CAPTION) != OS.WS_CAPTION) {
-				bits |= OS.WS_CAPTION;
-				OS.SetWindowLong (handle, OS.GWL_STYLE, bits);
-			}
-		}
-	} else {
-		if (OS.IsIconic (handle) || OS.IsZoomed (handle)) {
-			setPlacement (x, y, width, height, flags);
-			return;
-		}
-	}
-	OS.SetWindowPos (handle, 0, x, y, width, height, flags);
+void setBounds (int x, int y, int width, int height, int flags, boolean defer) {
+	super.setBounds (x, y, width, height, flags, false);
 }
 
 public void setEnabled (boolean enabled) {
 	checkWidget ();
-	state &= ~DISABLED;
-	if (!enabled) state |= DISABLED;
-	if (!Display.TrimEnabled) {
-		super.setEnabled (enabled);
-	} else {
-		if (isActive ()) setItemEnabled (OS.SC_CLOSE, enabled);
+	if (((state & DISABLED) == 0) == enabled) return;
+	super.setEnabled (enabled);
+	if (enabled && handle == OS.GetActiveWindow ()) {
+		if (!restoreFocus ()) traverseGroup (true);
 	}
 }
 
@@ -923,6 +1066,70 @@ public void setImeInputMode (int mode) {
 	OS.ImmReleaseContext (handle, hIMC);
 }
 
+/**
+ * Sets the receiver's minimum size to the size specified by the arguments.
+ * If the new minimum size is larger than the current size of the receiver,
+ * the receiver is resized to the new minimum size.
+ *
+ * @param width the new minimum width for the receiver
+ * @param height the new minimum height for the receiver
+ *
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ * 
+ * @since 3.1
+ */
+public void setMinimumSize (int width, int height) {
+	checkWidget ();
+	int widthLimit = 0, heightLimit = 0;
+	int trim = SWT.TITLE | SWT.CLOSE | SWT.MIN | SWT.MAX;
+	if ((style & SWT.NO_TRIM) == 0 && (style & trim) != 0) {
+		widthLimit = OS.GetSystemMetrics (OS.SM_CXMINTRACK);
+		if ((style & SWT.RESIZE) != 0) {
+			heightLimit = OS.GetSystemMetrics (OS.SM_CYMINTRACK);
+		} else {
+			RECT rect = new RECT ();
+			int bits1 = OS.GetWindowLong (handle, OS.GWL_STYLE);
+			int bits2 = OS.GetWindowLong (handle, OS.GWL_EXSTYLE);
+			OS.AdjustWindowRectEx (rect, bits1, false, bits2);
+			heightLimit = rect.bottom - rect.top;
+		}
+	} 
+	minWidth = Math.max (widthLimit, width);
+	minHeight = Math.max (heightLimit, height);
+	Point size = getSize ();
+	int newWidth = Math.max (size.x, minWidth);
+	int newHeight = Math.max (size.y, minHeight);
+	if (minWidth <= widthLimit) minWidth = SWT.DEFAULT;
+	if (minHeight <= heightLimit) minHeight = SWT.DEFAULT;
+	if (newWidth != size.x || newHeight != size.y) setSize (newWidth, newHeight);
+}
+
+/**
+ * Sets the receiver's minimum size to the size specified by the argument.
+ * If the new minimum size is larger than the current size of the receiver,
+ * the receiver is resized to the new minimum size.
+ *
+ * @param size the new minimum size for the receiver
+ *
+ * @exception IllegalArgumentException <ul>
+ *    <li>ERROR_NULL_ARGUMENT - if the point is null</li>
+ * </ul>
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ * 
+ * @since 3.1
+ */
+public void setMinimumSize (Point size) {
+	checkWidget ();
+	if (size == null) error (SWT.ERROR_NULL_ARGUMENT);
+	setMinimumSize (size.x, size.y);
+}
+
 void setItemEnabled (int cmd, boolean enabled) {
 	int hMenu = OS.GetSystemMenu (handle, false);
 	if (hMenu == 0) return;
@@ -935,6 +1142,39 @@ void setParent () {
 	/* Do nothing.  Not necessary for Shells */
 }
 
+/**
+ * Sets the shape of the shell to the region specified
+ * by the argument.  When the argument is null, the
+ * default shape of the shell is restored.  The shell
+ * must be created with the style SWT.NO_TRIM in order
+ * to specify a region.
+ *
+ * @param region the region that defines the shape of the shell (or null)
+ *
+ * @exception IllegalArgumentException <ul>
+ *    <li>ERROR_INVALID_ARGUMENT - if the region has been disposed</li>
+ * </ul>  
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ *
+ * @since 3.0
+ *
+ */
+public void setRegion (Region region) {
+	checkWidget ();
+	if ((style & SWT.NO_TRIM) == 0) return;
+	if (region != null && region.isDisposed()) error (SWT.ERROR_INVALID_ARGUMENT);
+	int hRegion = 0;
+	if (region != null) {
+		hRegion = OS.CreateRectRgn (0, 0, 0, 0);
+		OS.CombineRgn (hRegion, region.handle, hRegion, OS.RGN_OR);
+	}
+	OS.SetWindowRgn (handle, hRegion, true);
+	this.region = region;
+}
+
 void setToolTipText (int hwnd, String text) {
 	if (OS.IsWinCE) return;
 	if (toolTipHandle == 0) {
@@ -965,11 +1205,14 @@ void setToolTipText (int hwnd, String text) {
 	if (text == null) {
 		OS.SendMessage (toolTipHandle, OS.TTM_DELTOOL, 0, lpti);
 	} else {
-		lpti.uFlags = OS.TTF_IDISHWND | OS.TTF_SUBCLASS;
-		lpti.lpszText = OS.LPSTR_TEXTCALLBACK;
-		OS.SendMessage (toolTipHandle, OS.TTM_ADDTOOL, 0, lpti);
+		if (OS.SendMessage (toolTipHandle, OS.TTM_GETTOOLINFO, 0, lpti) != 0) {
+			OS.SendMessage (toolTipHandle, OS.TTM_UPDATE, 0, 0);
+		} else {
+			lpti.uFlags = OS.TTF_IDISHWND | OS.TTF_SUBCLASS;
+			lpti.lpszText = OS.LPSTR_TEXTCALLBACK;
+			OS.SendMessage (toolTipHandle, OS.TTM_ADDTOOL, 0, lpti);
+		}
 	}
-	OS.SendMessage (toolTipHandle, OS.TTM_UPDATE, 0, 0);
 }
 
 void setToolTipText (NMTTDISPINFO lpnmtdi, byte [] buffer) {
@@ -1006,28 +1249,30 @@ void setToolTipText (NMTTDISPINFO lpnmtdi, char [] buffer) {
 
 public void setVisible (boolean visible) {
 	checkWidget ();
+	if (drawCount != 0) {
+		if (((state & HIDDEN) == 0) == visible) return;
+	} else {
+		if (visible == OS.IsWindowVisible (handle)) return;
+	}
+	
 	/*
-	* Bug on Windows.  Calling ShowOnedPopups to
-	* hide the child windows of a hidden window causes the
-	* application to be deactivated.  The workaround is to
-	* call ShowOwnedPopups to hide child windows before
-	* hiding their parent window.
+	* Feature in Windows.  When ShowWindow() is called used to hide
+	* a window, Windows attempts to give focus to the parent. If the
+	* parent is disabled by EnableWindow(), focus is assigned to
+	* another windows on the desktop.  This means that if you hide
+	* a modal window before the parent is enabled, the parent will
+	* not come to the front.  The fix is to change the modal state
+	* before hiding or showing a window so that this does not occur.
 	*/
-	if (showWithParent && !visible) {
-		if (!OS.IsWinCE) OS.ShowOwnedPopups (handle, false);
-	}
-	super.setVisible (visible);
-	if (showWithParent == visible) return;
-	showWithParent = visible;
-	if (visible) {
-		if (!OS.IsWinCE) OS.ShowOwnedPopups (handle, true);
-	}
 	int mask = SWT.PRIMARY_MODAL | SWT.APPLICATION_MODAL | SWT.SYSTEM_MODAL;
 	if ((style & mask) != 0) {
 		if (visible) {
 			display.setModalShell (this);
-			Control control = display.getFocusControl ();
-			if (control != null && !control.isActive ()) bringToTop ();
+			Control control = display._getFocusControl ();
+			if (control != null && !control.isActive ()) {
+				bringToTop ();
+				if (isDisposed ()) return;
+			}
 			int hwndShell = OS.GetActiveWindow ();
 			if (hwndShell == 0) {
 				if (parent != null) hwndShell = parent.handle;
@@ -1042,6 +1287,29 @@ public void setVisible (boolean visible) {
 	} else {
 		updateModal ();
 	}
+	
+	/*
+	* Bug in Windows.  Calling ShowOwnedPopups() to hide the
+	* child windows of a hidden window causes the application
+	* to be deactivated.  The fix is to call ShowOwnedPopups()
+	* to hide children before hiding the parent.
+	*/
+	if (showWithParent && !visible) {
+		if (!OS.IsWinCE) OS.ShowOwnedPopups (handle, false);
+	}
+	super.setVisible (visible);
+	if (isDisposed ()) return;
+	if (showWithParent == visible) return;
+	showWithParent = visible;
+	if (visible) {
+		if (!OS.IsWinCE) OS.ShowOwnedPopups (handle, true);
+	}
+}
+
+boolean translateAccelerator (MSG msg) {
+	if (!isEnabled () || !isActive ()) return false;
+	if (menuBar != null && !menuBar.isEnabled ()) return false;
+	return translateMDIAccelerator (msg) || translateMenuAccelerator (msg);
 }
 
 boolean traverseEscape () {
@@ -1052,30 +1320,60 @@ boolean traverseEscape () {
 }
 
 void updateModal () {
-	if (!Display.TrimEnabled) {
-		super.setEnabled (isActive ());
-	} else {
+	if (Display.TrimEnabled) {
 		setItemEnabled (OS.SC_CLOSE, isActive ());
+	} else {
+		OS.EnableWindow (handle, isActive ());
 	}
 }
 
+CREATESTRUCT widgetCreateStruct () {
+	return null;
+}
+
+int widgetParent () {
+	if (handle != 0) return handle;
+	return parent != null ? parent.handle : 0;
+}
+
 int widgetExtStyle () {
-	int bits = super.widgetExtStyle ();
+	int bits = super.widgetExtStyle () & ~OS.WS_EX_MDICHILD;
+	if ((style & SWT.TOOL) != 0) bits |= OS.WS_EX_TOOLWINDOW;
+	
+	/*
+	* Feature in Windows.  When a window that does not have a parent
+	* is created, it is automatically added to the Windows Task Bar,
+	* even when it has no title.  The fix is to use WS_EX_TOOLWINDOW
+	* which does not cause the window to appear in the Task Bar.
+	*/
+	if (!OS.IsWinCE) {
+		if (parent == null) {
+			if ((style & SWT.ON_TOP) != 0) {
+				int trim = SWT.TITLE | SWT.CLOSE | SWT.MIN | SWT.MAX;
+				if ((style & SWT.NO_TRIM) != 0 || (style & trim) == 0) {
+					bits |= OS.WS_EX_TOOLWINDOW;
+				}
+			}
+		}
+	}
+	
 	/*
 	* Bug in Windows 98 and NT.  Creating a window with the
-	* WS_EX_TOPMOST extendes style can result in a dialog shell
+	* WS_EX_TOPMOST extended style can result in a dialog shell
 	* being moved behind its parent.  The exact case where this
 	* happens is a shell with two dialog shell children where
 	* each dialog child has another hidden dialog child with
 	* the WS_EX_TOPMOST extended style.  Clicking on either of
-	* the visible dialog causes them to become active but move
+	* the visible dialogs causes them to become active but move
 	* to the back, behind the parent shell.  The fix is to
 	* disallow the WS_EX_TOPMOST extended style on Windows 98
 	* and NT.
 	*/
-	if (OS.IsWin95) return bits;
-	if ((OS.WIN32_MAJOR << 16 | OS.WIN32_MINOR) < (4 << 16 | 10)) {
-		return bits;
+	if (parent != null) {
+		if (OS.IsWin95) return bits;
+		if (OS.WIN32_VERSION < OS.VERSION (4, 10)) {
+			return bits;
+		}
 	}
 	if ((style & SWT.ON_TOP) != 0) bits |= OS.WS_EX_TOPMOST;
 	return bits;
@@ -1083,16 +1381,24 @@ int widgetExtStyle () {
 
 TCHAR windowClass () {
 	if (OS.IsSP) return DialogClass;
+	if ((style & SWT.TOOL) != 0) {
+		int trim = SWT.TITLE | SWT.CLOSE | SWT.MIN | SWT.MAX | SWT.BORDER | SWT.RESIZE;
+		if ((style & trim) == 0) return display.windowShadowClass;
+	}
 	return parent != null ? DialogClass : super.windowClass ();
 }
 
 int windowProc () {
 	if (OS.IsSP) return DialogProc;
+	if ((style & SWT.TOOL) != 0) {
+		int trim = SWT.TITLE | SWT.CLOSE | SWT.MIN | SWT.MAX | SWT.BORDER | SWT.RESIZE;
+		if ((style & trim) == 0) super.windowProc ();
+	}
 	return parent != null ? DialogProc : super.windowProc ();
 }
 
 int widgetStyle () {
-	int bits = super.widgetStyle () & ~OS.WS_POPUP;
+	int bits = super.widgetStyle ();
 	if (handle != 0) return bits | OS.WS_CHILD;
 	bits &= ~OS.WS_CHILD;
 	/*
@@ -1148,18 +1454,22 @@ LRESULT WM_ACTIVATE (int wParam, int lParam) {
 			OS.SHSipPreference (handle, psai.fSipUp == 0 ? OS.SIP_DOWN : OS.SIP_UP);
 		} 
 	}
+
 	/*
 	* Bug in Windows XP.  When a Shell is deactivated, the
 	* IME composition window does not go away. This causes
-	* repaint issues.  The fix is to close the IME ourselves
-	* when the Shell is deactivated.
+	* repaint issues.  The fix is to close the IME to cause
+	* the composition string to be committed.
 	* 
-	* Note.  When the Shell is reactivated, the text in the
-	* composition window has been lost.
+	* Note. The IME needs to be reopened in order to preserve
+	* the input method status.
 	*/
-	if ((OS.WIN32_MAJOR << 16 | OS.WIN32_MINOR) >= (5 << 16 | 1)) {
+	if (OS.WIN32_VERSION >= OS.VERSION (5, 1)) {
 		if ((wParam & 0xFFFF) == 0 && OS.IsDBLocale && hIMC != 0) {
-			OS.ImmSetOpenStatus (hIMC, false);
+			if (OS.ImmGetOpenStatus(hIMC)) {
+				OS.ImmSetOpenStatus (hIMC, false);
+				OS.ImmSetOpenStatus (hIMC, true);
+			}
 		}
 	}
 	
@@ -1168,13 +1478,6 @@ LRESULT WM_ACTIVATE (int wParam, int lParam) {
 	return result;
 }
 
-LRESULT WM_CLOSE (int wParam, int lParam) {
-	if ((Display.TrimEnabled && !isEnabled ()) || !isActive ()) {
-		return LRESULT.ZERO;
-	}
-	return super.WM_CLOSE (wParam, lParam);
-}
-
 LRESULT WM_COMMAND (int wParam, int lParam) {
 	if (OS.IsPPC) {
 		/*
@@ -1188,14 +1491,26 @@ LRESULT WM_COMMAND (int wParam, int lParam) {
 			return LRESULT.ZERO;			
 		}
 	}
-	/* 
-	* Note in WinCE PPC.  Menu events originate from the command bar.
+	/*
+	* Feature in Windows.  On PPC, the menu is not actually an HMENU.
+	* By observation, it is a tool bar that is configured to look like
+	* a menu.  Therefore, when the PPC menu sends WM_COMMAND messages,
+	* lParam is not zero because the WM_COMMAND was not sent from a menu.
+	* Sub menu item events originate from the menu bar.  Top menu items
+	* events originate from a tool bar.  The fix is to detect the source
+	* of the WM_COMMAND and set lParam to zero to pretend that the message
+	* came from a real Windows menu, not a tool bar.
 	*/
 	if (OS.IsPPC || OS.IsSP) {
 		if (menuBar != null) {
 			int hwndCB = menuBar.hwndCB;
-			if (hwndCB != 0 && lParam == hwndCB) {
-				return super.WM_COMMAND (wParam, 0);
+			if (lParam != 0 && hwndCB != 0) {
+				if (lParam == hwndCB) {
+					return super.WM_COMMAND (wParam, 0);
+				} else {
+					int hwndChild = OS.GetWindow (hwndCB, OS.GW_CHILD);
+					if (lParam == hwndChild) return super.WM_COMMAND (wParam, 0);					
+				}
 			}
 		}
 	}
@@ -1218,11 +1533,66 @@ LRESULT WM_DESTROY (int wParam, int lParam) {
 	return result;
 }
 
+LRESULT WM_ENTERIDLE (int wParam, int lParam) {
+	LRESULT result = super.WM_ENTERIDLE (wParam, lParam);
+	if (result != null) return result;
+	if (OS.IsWinCE) {
+		if (display.runAsyncMessages (true)) display.wakeThread ();
+	}
+	return result;
+}
+
+LRESULT WM_GETMINMAXINFO (int wParam, int lParam) {
+	LRESULT result = super.WM_GETMINMAXINFO (wParam, lParam);
+	if (result != null) return result;
+	if (minWidth != SWT.DEFAULT || minHeight != SWT.DEFAULT) {
+		MINMAXINFO info = new MINMAXINFO ();
+		OS.MoveMemory (info, lParam, MINMAXINFO.sizeof);
+		if (minWidth != SWT.DEFAULT) info.ptMinTrackSize_x = minWidth;
+		if (minHeight != SWT.DEFAULT) info.ptMinTrackSize_y = minHeight;
+		OS.MoveMemory (lParam, info, MINMAXINFO.sizeof);
+		return LRESULT.ZERO;
+	}
+	return result;
+}
+
 LRESULT WM_MOUSEACTIVATE (int wParam, int lParam) {
 	LRESULT result = super.WM_MOUSEACTIVATE (wParam, lParam);
 	if (result != null) return result;
-	int hittest = lParam & 0xFFFF;
+	
+	/*
+	* Check for WM_MOUSEACTIVATE when an MDI shell is active
+	* and stop the normal shell activation but allow the mouse
+	* down to be delivered.
+	*/
+	int hittest = (short) (lParam & 0xFFFF);
+	switch (hittest) {
+		case OS.HTERROR:
+		case OS.HTTRANSPARENT:
+		case OS.HTNOWHERE:
+			break;
+		default: {
+			Control control = display._getFocusControl ();
+			if (control != null) {
+				Decorations decorations = control.menuShell ();
+				if (decorations.getShell () == this && decorations != this) {
+					display.ignoreRestoreFocus = true;
+					display.lastHittest = hittest;
+					display.lastHittestControl = null;
+					if (hittest == OS.HTMENU || hittest == OS.HTSYSMENU) {
+						display.lastHittestControl = control;
+						return null;
+					}
+					if (OS.IsWin95 && hittest == OS.HTCAPTION) {
+						display.lastHittestControl = control;
+					}
+					return new LRESULT (OS.MA_NOACTIVATE);
+				}
+			}
+		}
+	}
 	if (hittest == OS.HTMENU) return null;
+	
 	/*
 	* Get the current location of the cursor,
 	* not the location of the cursor when the
@@ -1244,20 +1614,25 @@ LRESULT WM_MOUSEACTIVATE (int wParam, int lParam) {
 	int hwnd = OS.WindowFromPoint (pt);
 	if (hwnd == 0) return null;
 	Control control = display.findControl (hwnd);
-	setActiveControl (control);
+	
 	/*
-	* This code is intentionally commented.  On some platforms,
-	* shells that are created with SWT.NO_TRIM won't take focus
-	* when the user clicks in the client area or on the border.
-	* This behavior is usedful when emulating tool tip shells
-	* Until this behavior is specified, this code will remain
-	* commented.
+	* When a shell is created with SWT.ON_TOP and SWT.NO_FOCUS,
+	* do not activate the shell when the user clicks on the
+	* the client area or on the border or a control within the
+	* shell that does not take focus.
 	*/
-//	if ((style & SWT.NO_TRIM) != 0) {
-//		if (hittest == OS.HTBORDER || hittest == OS.HTCLIENT) {
-//			return new LRESULT (OS.MA_NOACTIVATE);
-//		}
-//	}
+	if (control != null && (control.state & CANVAS) != 0) {
+		if ((control.style & SWT.NO_FOCUS) != 0) {
+			int bits = SWT.ON_TOP | SWT.NO_FOCUS;
+			if ((style & bits) == bits) {
+				if (hittest == OS.HTBORDER || hittest == OS.HTCLIENT) {
+					return new LRESULT (OS.MA_NOACTIVATE);
+				}
+			}
+		}
+	}
+	
+	setActiveControl (control);
 	return null;
 }
 
@@ -1265,18 +1640,46 @@ LRESULT WM_NCHITTEST (int wParam, int lParam) {
 	if (!OS.IsWindowEnabled (handle)) return null;
 	if (!isEnabled () || !isActive ()) {
 		if (!Display.TrimEnabled) return new LRESULT (OS.HTNOWHERE);
-		int hittest = callWindowProc (OS.WM_NCHITTEST, wParam, lParam);
+		int hittest = callWindowProc (handle, OS.WM_NCHITTEST, wParam, lParam);
 		if (hittest == OS.HTCLIENT || hittest == OS.HTMENU) hittest = OS.HTBORDER;
 		return new LRESULT (hittest);
 	}
 	if (menuBar != null && !menuBar.getEnabled ()) {
-		int hittest = callWindowProc (OS.WM_NCHITTEST, wParam, lParam);
+		int hittest = callWindowProc (handle, OS.WM_NCHITTEST, wParam, lParam);
 		if (hittest == OS.HTMENU) hittest = OS.HTBORDER;
 		return new LRESULT (hittest);
 	}
 	return null;
 }
 
+LRESULT WM_NCLBUTTONDOWN (int wParam, int lParam) {
+	LRESULT result = super.WM_NCLBUTTONDOWN (wParam, lParam);
+	if (result != null) return result;
+	/*
+	* When the normal activation was interruped in WM_MOUSEACTIVATE
+	* because the active shell was an MDI shell, set the active window
+	* to the top level shell but lock the active window and stop focus
+	* changes.  This allows the user to interact the top level shell
+	* in the normal manner.
+	*/
+	if (!display.ignoreRestoreFocus) return result;
+	Display display = this.display;
+	int hwndActive = 0;
+	boolean fixActive = OS.IsWin95 && display.lastHittest == OS.HTCAPTION;
+	if (fixActive) hwndActive = OS.SetActiveWindow (handle);
+	display.lockActiveWindow = true;
+	int code = callWindowProc (handle, OS.WM_NCLBUTTONDOWN, wParam, lParam);
+	display.lockActiveWindow = false;
+	if (fixActive) OS.SetActiveWindow (hwndActive);
+	Control focusControl = display.lastHittestControl;
+	if (focusControl != null && !focusControl.isDisposed ()) {
+		focusControl.setFocus ();
+	}
+	display.lastHittestControl = null;
+	display.ignoreRestoreFocus = false;
+	return new LRESULT (code);
+}
+
 LRESULT WM_PALETTECHANGED (int wParam, int lParam) {
 	if (wParam != handle) {
 		int hPalette = display.hPalette;
@@ -1300,7 +1703,7 @@ LRESULT WM_SETCURSOR (int wParam, int lParam) {
 	* fix is to detect this case and bring the shell
 	* forward.
 	*/
-	int msg = lParam >> 16;
+	int msg = (short) (lParam >> 16);
 	if (msg == OS.WM_LBUTTONDOWN) {
 		if (!Display.TrimEnabled) {
 			Shell modalShell = display.getModalShell ();
@@ -1315,7 +1718,7 @@ LRESULT WM_SETCURSOR (int wParam, int lParam) {
 			if (!OS.IsWinCE) {
 				int hwndPopup = OS.GetLastActivePopup (handle);
 				if (hwndPopup != 0 && hwndPopup != handle) {
-					if (WidgetTable.get (hwndPopup) == null) {
+					if (display.getControl (hwndPopup) == null) {
 						if (OS.IsWindowEnabled (hwndPopup)) {
 							OS.SetActiveWindow (hwndPopup);
 						}
@@ -1324,6 +1727,42 @@ LRESULT WM_SETCURSOR (int wParam, int lParam) {
 			}
 		}
 	}
+	/*
+	* When the shell that contains a cursor is disabled,
+	* WM_SETCURSOR is called with HTERROR.  Normally,
+	* when a control is disabled, the parent will get
+	* mouse and cursor events.  In the case of a disabled
+	* shell, there is no enabled parent.  In order to
+	* show the cursor when a shell is disabled, it is
+	* necessary to override WM_SETCURSOR when called
+	* with HTERROR to set the cursor but only when the
+	* mouse is in the client area of the shell.
+	*/
+	int hitTest = (short) (lParam & 0xFFFF);
+	if (hitTest == OS.HTERROR) {
+		if (!getEnabled ()) {
+			Control control = display.getControl (wParam);
+			if (control == this && cursor != null) {
+				POINT pt = new POINT ();
+				if (OS.GetCursorPos (pt)) {
+					OS.ScreenToClient (handle, pt);
+					RECT rect = new RECT ();
+					OS.GetClientRect (handle, rect);
+					if (OS.PtInRect (rect, pt)) {
+						OS.SetCursor (cursor.handle);
+						switch (msg) {
+							case OS.WM_LBUTTONDOWN:
+							case OS.WM_RBUTTONDOWN:
+							case OS.WM_MBUTTONDOWN:
+							case OS.WM_XBUTTONDOWN:
+								OS.MessageBeep (OS.MB_OK);
+						}
+						return LRESULT.ONE;
+					}
+				}
+			}
+		}
+	}
 	return super.WM_SETCURSOR (wParam, lParam);
 }
 
@@ -1374,4 +1813,65 @@ LRESULT WM_SHOWWINDOW (int wParam, int lParam) {
 	return result;
 }
 
+LRESULT WM_SYSCOMMAND (int wParam, int lParam) {
+	LRESULT result = super.WM_SYSCOMMAND (wParam, lParam);
+	if (result != null) return result;
+	/*
+	* Feature in Windows.  When the last visible window in
+	* a process minimized, Windows swaps out the memory for
+	* the process.  The assumption is that the user can no
+	* longer interact with the window, so the memory can be
+	* released to other applications.  However, for programs
+	* that use a lot of memory, swapping the memory back in
+	* can take a long time, sometimes minutes.  The fix is
+	* to intercept WM_SYSCOMMAND looking for SC_MINIMIZE
+	* and use ShowWindow() with SW_SHOWMINIMIZED to minimize
+	* the window, rather than running the default window proc.
+	* 
+	* NOTE:  The default window proc activates the next
+	* top-level window in the Z-order while ShowWindow()
+	* with SW_SHOWMINIMIZED does not.  There is no fix for
+	* this at this time.
+	*/
+	if (OS.IsWinNT) {
+		int cmd = wParam & 0xFFF0;
+		switch (cmd) {
+			case OS.SC_MINIMIZE:
+				long memory = Runtime.getRuntime ().totalMemory ();
+				if (memory >= 32 * 1024 * 1024) {
+					OS.ShowWindow (handle, OS.SW_SHOWMINIMIZED);
+					return LRESULT.ZERO;
+				}
+		}
+	}
+	return result;
+}
+
+LRESULT WM_WINDOWPOSCHANGING (int wParam, int lParam) {
+	LRESULT result = super.WM_WINDOWPOSCHANGING (wParam,lParam);
+	if (result != null) return result;
+	WINDOWPOS lpwp = new WINDOWPOS ();
+	OS.MoveMemory (lpwp, lParam, WINDOWPOS.sizeof);
+	if ((lpwp.flags & OS.SWP_NOSIZE) == 0) {
+		lpwp.cx = Math.max (lpwp.cx, minWidth);
+		int trim = SWT.TITLE | SWT.CLOSE | SWT.MIN | SWT.MAX;
+		if ((style & SWT.NO_TRIM) == 0 && (style & trim) != 0) {
+			lpwp.cx = Math.max (lpwp.cx, OS.GetSystemMetrics (OS.SM_CXMINTRACK));
+		}
+		lpwp.cy = Math.max (lpwp.cy, minHeight);
+		if ((style & SWT.NO_TRIM) == 0 && (style & trim) != 0) {
+			if ((style & SWT.RESIZE) != 0) {
+				lpwp.cy = Math.max (lpwp.cy, OS.GetSystemMetrics (OS.SM_CYMINTRACK));
+			} else {
+				RECT rect = new RECT ();
+				int bits1 = OS.GetWindowLong (handle, OS.GWL_STYLE);
+				int bits2 = OS.GetWindowLong (handle, OS.GWL_EXSTYLE);
+				OS.AdjustWindowRectEx (rect, bits1, false, bits2);
+				lpwp.cy = Math.max (lpwp.cy, rect.bottom - rect.top);
+			}
+		}
+		OS.MoveMemory (lParam, lpwp, WINDOWPOS.sizeof);
+	}
+	return result;
+}
 }
