@@ -1,9 +1,9 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
+ * http://www.eclipse.org/legal/epl-v10.html
  * 
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -16,74 +16,37 @@ import java.lang.reflect.InvocationTargetException;
 import java.util.ArrayList;
 import java.util.Iterator;
 
-import org.eclipse.core.resources.IFile;
-import org.eclipse.core.resources.IResource;
-import org.eclipse.core.resources.IWorkspaceRunnable;
-import org.eclipse.core.runtime.CoreException;
-import org.eclipse.core.runtime.IAdaptable;
-import org.eclipse.core.runtime.IProgressMonitor;
-import org.eclipse.core.runtime.SubProgressMonitor;
-import org.eclipse.jface.action.Action;
-import org.eclipse.jface.action.IAction;
-import org.eclipse.jface.action.IMenuListener;
-import org.eclipse.jface.action.IMenuManager;
-import org.eclipse.jface.action.IToolBarManager;
-import org.eclipse.jface.action.MenuManager;
-import org.eclipse.jface.action.Separator;
+import org.eclipse.core.resources.*;
+import org.eclipse.core.runtime.*;
+import org.eclipse.core.runtime.jobs.Job;
+import org.eclipse.jface.action.*;
 import org.eclipse.jface.dialogs.IDialogConstants;
 import org.eclipse.jface.dialogs.MessageDialog;
-import org.eclipse.jface.dialogs.ProgressMonitorDialog;
 import org.eclipse.jface.operation.IRunnableWithProgress;
 import org.eclipse.jface.preference.IPreferenceStore;
-import org.eclipse.jface.text.Document;
-import org.eclipse.jface.text.ITextOperationTarget;
-import org.eclipse.jface.text.TextViewer;
-import org.eclipse.jface.viewers.ColumnWeightData;
-import org.eclipse.jface.viewers.ISelection;
-import org.eclipse.jface.viewers.ISelectionChangedListener;
-import org.eclipse.jface.viewers.IStructuredContentProvider;
-import org.eclipse.jface.viewers.IStructuredSelection;
-import org.eclipse.jface.viewers.LabelProvider;
-import org.eclipse.jface.viewers.SelectionChangedEvent;
-import org.eclipse.jface.viewers.TableLayout;
-import org.eclipse.jface.viewers.TableViewer;
-import org.eclipse.jface.viewers.Viewer;
-import org.eclipse.jface.viewers.ViewerSorter;
+import org.eclipse.jface.text.*;
+import org.eclipse.jface.viewers.*;
+import org.eclipse.osgi.util.NLS;
 import org.eclipse.swt.SWT;
-import org.eclipse.swt.custom.BusyIndicator;
-import org.eclipse.swt.custom.SashForm;
-import org.eclipse.swt.custom.StyledText;
+import org.eclipse.swt.custom.*;
 import org.eclipse.swt.dnd.DND;
 import org.eclipse.swt.dnd.Transfer;
 import org.eclipse.swt.graphics.Image;
 import org.eclipse.swt.layout.GridData;
-import org.eclipse.swt.widgets.Composite;
-import org.eclipse.swt.widgets.Event;
-import org.eclipse.swt.widgets.Listener;
-import org.eclipse.swt.widgets.Menu;
-import org.eclipse.swt.widgets.Table;
+import org.eclipse.swt.widgets.*;
 import org.eclipse.team.core.RepositoryProvider;
 import org.eclipse.team.core.TeamException;
-import org.eclipse.team.internal.ccvs.core.CVSException;
-import org.eclipse.team.internal.ccvs.core.CVSProviderPlugin;
-import org.eclipse.team.internal.ccvs.core.CVSTag;
-import org.eclipse.team.internal.ccvs.core.CVSTeamProvider;
-import org.eclipse.team.internal.ccvs.core.ICVSFile;
-import org.eclipse.team.internal.ccvs.core.ICVSRemoteFile;
-import org.eclipse.team.internal.ccvs.core.ICVSResource;
-import org.eclipse.team.internal.ccvs.core.ILogEntry;
+import org.eclipse.team.core.synchronize.SyncInfo;
+import org.eclipse.team.internal.ccvs.core.*;
 import org.eclipse.team.internal.ccvs.core.client.Command;
 import org.eclipse.team.internal.ccvs.core.client.Update;
 import org.eclipse.team.internal.ccvs.core.resources.CVSWorkspaceRoot;
-import org.eclipse.team.internal.ccvs.ui.actions.CVSAction;
-import org.eclipse.team.internal.ccvs.ui.actions.MoveRemoteTagAction;
-import org.eclipse.team.internal.ccvs.ui.actions.OpenLogEntryAction;
-import org.eclipse.ui.IActionBars;
-import org.eclipse.ui.IActionDelegate;
-import org.eclipse.ui.IWorkbenchActionConstants;
-import org.eclipse.ui.PartInitException;
-import org.eclipse.ui.actions.WorkspaceModifyOperation;
-import org.eclipse.ui.help.WorkbenchHelp;
+import org.eclipse.team.internal.ccvs.ui.actions.*;
+import org.eclipse.team.internal.ccvs.ui.operations.*;
+import org.eclipse.team.internal.ui.Utils;
+import org.eclipse.team.ui.synchronize.SyncInfoCompareInput;
+import org.eclipse.ui.*;
+import org.eclipse.ui.ide.ResourceUtil;
 import org.eclipse.ui.part.ResourceTransfer;
 import org.eclipse.ui.part.ViewPart;
 import org.eclipse.ui.texteditor.ITextEditorActionConstants;
@@ -95,7 +58,6 @@ public class HistoryView extends ViewPart {
 	private IFile file;
 	// cached for efficiency
 	private ILogEntry[] entries;
-	private CVSTeamProvider provider;
 	
 	private HistoryTableProvider historyTableProvider;
 	
@@ -112,6 +74,7 @@ public class HistoryView extends ViewPart {
 	private Action getRevisionAction;
 	private Action refreshAction;
 	private Action tagWithExistingAction;
+	private Action linkWithEditorAction;
 	
 	private SashForm sashForm;
 	private SashForm innerSashForm;
@@ -120,24 +83,114 @@ public class HistoryView extends ViewPart {
 	private Image versionImage;
 	
 	private ILogEntry currentSelection;
+	private boolean linkingEnabled;
+	
+	private IPreferenceStore settings;
+	
+	private FetchLogEntriesJob fetchLogEntriesJob;
+	
+	private boolean shutdown = false;
 	
 	public static final String VIEW_ID = "org.eclipse.team.ccvs.ui.HistoryView"; //$NON-NLS-1$
 	
+	private IPartListener partListener = new IPartListener() {
+		public void partActivated(IWorkbenchPart part) {
+			if (part instanceof IEditorPart)
+				editorActivated((IEditorPart) part);
+		}
+		public void partBroughtToTop(IWorkbenchPart part) {
+			if(part == HistoryView.this)
+				editorActivated(getViewSite().getPage().getActiveEditor());
+		}
+		public void partOpened(IWorkbenchPart part) {
+			if(part == HistoryView.this)
+				editorActivated(getViewSite().getPage().getActiveEditor());
+		}
+		public void partClosed(IWorkbenchPart part) {
+		}
+		public void partDeactivated(IWorkbenchPart part) {
+		}
+	};
+	
+	private IPartListener2 partListener2 = new IPartListener2() {
+		public void partActivated(IWorkbenchPartReference ref) {
+		}
+		public void partBroughtToTop(IWorkbenchPartReference ref) {
+		}
+		public void partClosed(IWorkbenchPartReference ref) {
+		}
+		public void partDeactivated(IWorkbenchPartReference ref) {
+		}
+		public void partOpened(IWorkbenchPartReference ref) {
+		}
+		public void partHidden(IWorkbenchPartReference ref) {
+		}
+		public void partVisible(IWorkbenchPartReference ref) {
+			if(ref.getPart(true) == HistoryView.this)
+				editorActivated(getViewSite().getPage().getActiveEditor());
+		}
+		public void partInputChanged(IWorkbenchPartReference ref) {
+		}
+	};
+
+
+	private class FetchLogEntriesJob extends Job {
+		public ICVSRemoteFile remoteFile;
+		public FetchLogEntriesJob() {
+			super(CVSUIMessages.HistoryView_fetchHistoryJob);  //$NON-NLS-1$;
+		}
+		public void setRemoteFile(ICVSRemoteFile file) {
+			this.remoteFile = file;
+		}
+		public IStatus run(IProgressMonitor monitor) {
+			try {
+				if(remoteFile != null && !shutdown) {
+					entries = remoteFile.getLogEntries(monitor);
+					final String revisionId = remoteFile.getRevision();
+					getSite().getShell().getDisplay().asyncExec(new Runnable() {
+						public void run() {
+							if(entries != null && tableViewer != null && ! tableViewer.getTable().isDisposed()) {
+								tableViewer.refresh();
+								selectRevision(revisionId);
+							}
+						}
+					});
+				}
+				return Status.OK_STATUS;
+			} catch (TeamException e) {
+				return e.getStatus();
+			}
+		}
+		public ICVSRemoteFile getRemoteFile() {
+			return this.remoteFile;
+		}
+	};
+	
 	/**
 	 * Adds the action contributions for this view.
 	 */
 	protected void contributeActions() {
 		// Refresh (toolbar)
 		CVSUIPlugin plugin = CVSUIPlugin.getPlugin();
-		refreshAction = new Action(Policy.bind("HistoryView.refreshLabel"), plugin.getImageDescriptor(ICVSUIConstants.IMG_REFRESH_ENABLED)) { //$NON-NLS-1$
+		refreshAction = new Action(CVSUIMessages.HistoryView_refreshLabel, plugin.getImageDescriptor(ICVSUIConstants.IMG_REFRESH_ENABLED)) { //$NON-NLS-1$
 			public void run() {
 				refresh();
 			}
 		};
-		refreshAction.setToolTipText(Policy.bind("HistoryView.refresh")); //$NON-NLS-1$
+		refreshAction.setToolTipText(CVSUIMessages.HistoryView_refresh); //$NON-NLS-1$
 		refreshAction.setDisabledImageDescriptor(plugin.getImageDescriptor(ICVSUIConstants.IMG_REFRESH_DISABLED));
 		refreshAction.setHoverImageDescriptor(plugin.getImageDescriptor(ICVSUIConstants.IMG_REFRESH));
 		
+		//	Link with Editor (toolbar)
+		 linkWithEditorAction = new Action(CVSUIMessages.HistoryView_linkWithLabel, plugin.getImageDescriptor(ICVSUIConstants.IMG_LINK_WITH_EDITOR_ENABLED)) { //$NON-NLS-1$
+			 public void run() {
+				 setLinkingEnabled(isChecked());
+			 }
+		 };
+		linkWithEditorAction.setToolTipText(CVSUIMessages.HistoryView_linkWithLabel); //$NON-NLS-1$
+		linkWithEditorAction.setHoverImageDescriptor(plugin.getImageDescriptor(ICVSUIConstants.IMG_LINK_WITH_EDITOR));
+		linkWithEditorAction.setChecked(isLinkingEnabled());
+		
 		// Double click open action
 		openAction = new OpenLogEntryAction();
 		tableViewer.getTable().addListener(SWT.DefaultSelection, new Listener() {
@@ -147,7 +200,7 @@ public class HistoryView extends ViewPart {
 			}
 		});
 
-		getContentsAction = getContextMenuAction(Policy.bind("HistoryView.getContentsAction"), new IWorkspaceRunnable() { //$NON-NLS-1$
+		getContentsAction = getContextMenuAction(CVSUIMessages.HistoryView_getContentsAction, true /* needs progress */, new IWorkspaceRunnable() { //$NON-NLS-1$
 			public void run(IProgressMonitor monitor) throws CoreException {
 				ICVSRemoteFile remoteFile = currentSelection.getRemoteFile();
 				monitor.beginTask(null, 100);
@@ -163,29 +216,38 @@ public class HistoryView extends ViewPart {
 				}
 			}
 		});
-		WorkbenchHelp.setHelp(getContentsAction, IHelpContextIds.GET_FILE_CONTENTS_ACTION);	
+        PlatformUI.getWorkbench().getHelpSystem().setHelp(getContentsAction, IHelpContextIds.GET_FILE_CONTENTS_ACTION);	
 
-		getRevisionAction = getContextMenuAction(Policy.bind("HistoryView.getRevisionAction"), new IWorkspaceRunnable() { //$NON-NLS-1$
+		getRevisionAction = getContextMenuAction(CVSUIMessages.HistoryView_getRevisionAction, true /* needs progress */, new IWorkspaceRunnable() { //$NON-NLS-1$
 			public void run(IProgressMonitor monitor) throws CoreException {
 				ICVSRemoteFile remoteFile = currentSelection.getRemoteFile();
 				try {
 					if(confirmOverwrite()) {
-						CVSTeamProvider provider = (CVSTeamProvider)RepositoryProvider.getProvider(file.getProject());
 						CVSTag revisionTag = new CVSTag(remoteFile.getRevision(), CVSTag.VERSION);
 						
-						if(CVSAction.checkForMixingTags(getSite().getShell(), new IResource[] {file}, revisionTag)) {							
-							provider.update(new IResource[] {file}, new Command.LocalOption[] {Update.IGNORE_LOCAL_CHANGES}, 
-													   revisionTag, true /*create backups*/, monitor);
+						if(CVSAction.checkForMixingTags(getSite().getShell(), new IResource[] {file}, revisionTag)) {
+							new UpdateOperation(
+									null, 
+									new IResource[] {file},
+									new Command.LocalOption[] {Update.IGNORE_LOCAL_CHANGES}, 
+									revisionTag)
+										.run(monitor);
 							historyTableProvider.setFile(remoteFile);
-							tableViewer.refresh();
+							Display.getDefault().asyncExec(new Runnable() {
+								public void run() {
+									tableViewer.refresh();
+								}
+							});
 						}
 					}
-				} catch (TeamException e) {
-					throw new CoreException(e.getStatus());
+				} catch (InvocationTargetException e) {
+					CVSException.wrapException(e);
+				} catch (InterruptedException e) {
+					// Cancelled by user
 				}
 			}
 		});
-		WorkbenchHelp.setHelp(getRevisionAction, IHelpContextIds.GET_FILE_REVISION_ACTION);	
+        PlatformUI.getWorkbench().getHelpSystem().setHelp(getRevisionAction, IHelpContextIds.GET_FILE_REVISION_ACTION);	
 
 		// Override MoveRemoteTagAction to work for log entries
 		final IActionDelegate tagActionDelegate = new MoveRemoteTagAction() {
@@ -195,7 +257,7 @@ public class HistoryView extends ViewPart {
 					ArrayList logEntrieFiles = null;
 					if (!selection.isEmpty()) {
 						logEntrieFiles = new ArrayList();
-						Iterator elements = ((IStructuredSelection) selection).iterator();
+						Iterator elements = selection.iterator();
 						while (elements.hasNext()) {
 							Object next = elements.next();
 							if (next instanceof ILogEntry) {
@@ -218,35 +280,52 @@ public class HistoryView extends ViewPart {
 				}
 				return resources;
 			}
+            /*
+             * Override the creation of the tag operation in order to support
+             * the refresh of the view after the tag operation completes
+             */
+            protected ITagOperation createTagOperation() {
+                return new TagInRepositoryOperation(getTargetPart(), getSelectedRemoteResources()) {
+                    public void execute(IProgressMonitor monitor) throws CVSException, InterruptedException {
+                        super.execute(monitor);
+                        Display.getDefault().asyncExec(new Runnable() {
+                            public void run() {
+                                if( ! wasCancelled()) {
+                                    refresh();
+                                }
+                            }
+                        });
+                    };
+                };
+            }
 		};
-		tagWithExistingAction = getContextMenuAction(Policy.bind("HistoryView.tagWithExistingAction"), new IWorkspaceRunnable() { //$NON-NLS-1$
+		tagWithExistingAction = getContextMenuAction(CVSUIMessages.HistoryView_tagWithExistingAction, false /* no progress */, new IWorkspaceRunnable() { //$NON-NLS-1$
 			public void run(IProgressMonitor monitor) throws CoreException {
 				tagActionDelegate.selectionChanged(tagWithExistingAction, tableViewer.getSelection());
 				tagActionDelegate.run(tagWithExistingAction);
-				refresh();
 			}
 		});
-		WorkbenchHelp.setHelp(getRevisionAction, IHelpContextIds.TAG_WITH_EXISTING_ACTION);	
+        PlatformUI.getWorkbench().getHelpSystem().setHelp(getRevisionAction, IHelpContextIds.TAG_WITH_EXISTING_ACTION);	
 				
 		// Toggle text visible action
 		final IPreferenceStore store = CVSUIPlugin.getPlugin().getPreferenceStore();
-		toggleTextAction = new Action(Policy.bind("HistoryView.showComment")) { //$NON-NLS-1$
+		toggleTextAction = new Action(CVSUIMessages.HistoryView_showComment) { //$NON-NLS-1$
 			public void run() {
 				setViewerVisibility();
 				store.setValue(ICVSUIConstants.PREF_SHOW_COMMENTS, toggleTextAction.isChecked());
 			}
 		};
 		toggleTextAction.setChecked(store.getBoolean(ICVSUIConstants.PREF_SHOW_COMMENTS));
-		WorkbenchHelp.setHelp(toggleTextAction, IHelpContextIds.SHOW_COMMENT_IN_HISTORY_ACTION);	
+        PlatformUI.getWorkbench().getHelpSystem().setHelp(toggleTextAction, IHelpContextIds.SHOW_COMMENT_IN_HISTORY_ACTION);	
 		// Toggle list visible action
-		toggleListAction = new Action(Policy.bind("HistoryView.showTags")) { //$NON-NLS-1$
+		toggleListAction = new Action(CVSUIMessages.HistoryView_showTags) { //$NON-NLS-1$
 			public void run() {
 				setViewerVisibility();
 				store.setValue(ICVSUIConstants.PREF_SHOW_TAGS, toggleListAction.isChecked());
 			}
 		};
 		toggleListAction.setChecked(store.getBoolean(ICVSUIConstants.PREF_SHOW_TAGS));
-		WorkbenchHelp.setHelp(toggleListAction, IHelpContextIds.SHOW_TAGS_IN_HISTORY_ACTION);	
+        PlatformUI.getWorkbench().getHelpSystem().setHelp(toggleListAction, IHelpContextIds.SHOW_TAGS_IN_HISTORY_ACTION);	
 		
 		// Contribute actions to popup menu
 		MenuManager menuMgr = new MenuManager();
@@ -269,15 +348,16 @@ public class HistoryView extends ViewPart {
 		// Create the local tool bar
 		IToolBarManager tbm = getViewSite().getActionBars().getToolBarManager();
 		tbm.add(refreshAction);
+		tbm.add(linkWithEditorAction);
 		tbm.update(false);
 	
 		// Create actions for the text editor
 		copyAction = new TextViewerAction(textViewer, ITextOperationTarget.COPY);
-		copyAction.setText(Policy.bind("HistoryView.copy")); //$NON-NLS-1$
+		copyAction.setText(CVSUIMessages.HistoryView_copy); //$NON-NLS-1$
 		actionBars.setGlobalActionHandler(ITextEditorActionConstants.COPY, copyAction);
 		
 		selectAllAction = new TextViewerAction(textViewer, ITextOperationTarget.SELECT_ALL);
-		selectAllAction.setText(Policy.bind("HistoryView.selectAll")); //$NON-NLS-1$
+		selectAllAction.setText(CVSUIMessages.HistoryView_selectAll); //$NON-NLS-1$
 		actionBars.setGlobalActionHandler(ITextEditorActionConstants.SELECT_ALL, selectAllAction);
 
 		actionBars.updateActionBars();
@@ -313,20 +393,32 @@ public class HistoryView extends ViewPart {
 	 * Method declared on IWorkbenchPart
 	 */
 	public void createPartControl(Composite parent) {
+		settings = CVSUIPlugin.getPlugin().getPreferenceStore();
+		this.linkingEnabled = settings.getBoolean(ICVSUIConstants.PREF_HISTORY_VIEW_EDITOR_LINKING);
+
 		initializeImages();
+		
 		sashForm = new SashForm(parent, SWT.VERTICAL);
 		sashForm.setLayoutData(new GridData(GridData.FILL_BOTH));
+		
 		tableViewer = createTable(sashForm);
 		innerSashForm = new SashForm(sashForm, SWT.HORIZONTAL);
 		tagViewer = createTagTable(innerSashForm);
 		textViewer = createText(innerSashForm);
 		sashForm.setWeights(new int[] { 70, 30 });
 		innerSashForm.setWeights(new int[] { 50, 50 });
+		
 		contributeActions();
+		
 		setViewerVisibility();
+		
 		// set F1 help
-		WorkbenchHelp.setHelp(sashForm, IHelpContextIds.RESOURCE_HISTORY_VIEW);
+        PlatformUI.getWorkbench().getHelpSystem().setHelp(sashForm, IHelpContextIds.RESOURCE_HISTORY_VIEW);
 		initDragAndDrop();
+		 
+		// add listener for editor page activation - this is to support editor linking
+		getSite().getPage().addPartListener(partListener);	
+		getSite().getPage().addPartListener(partListener2);	
 	}
 	private void initializeImages() {
 		CVSUIPlugin plugin = CVSUIPlugin.getPlugin();
@@ -347,30 +439,32 @@ public class HistoryView extends ViewPart {
 		
 		viewer.setContentProvider(new IStructuredContentProvider() {
 			public Object[] getElements(Object inputElement) {
-				// Short-circuit to optimize
+				
+				// The entries of already been fetch so return them
 				if (entries != null) return entries;
 				
+				// The entries need to be fetch (or are being fetched)
 				if (!(inputElement instanceof ICVSRemoteFile)) return null;
 				final ICVSRemoteFile remoteFile = (ICVSRemoteFile)inputElement;
-				final Object[][] result = new Object[1][];
-				try {
-					new ProgressMonitorDialog(getViewer().getTable().getShell()).run(true, true, new IRunnableWithProgress() {
-						public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
-							try {
-								entries = remoteFile.getLogEntries(monitor);
-								result[0] = entries;
-							} catch (TeamException e) {
-								throw new InvocationTargetException(e);
-							}
+				if(fetchLogEntriesJob == null) {
+					fetchLogEntriesJob = new FetchLogEntriesJob();
+				} 
+				ICVSRemoteFile file = fetchLogEntriesJob.getRemoteFile();
+				if (file == null || !file.equals(remoteFile)) {
+					// The resource has changed so stop the currently running job
+					if(fetchLogEntriesJob.getState() != Job.NONE) {
+						fetchLogEntriesJob.cancel();
+						try {
+							fetchLogEntriesJob.join();
+						} catch (InterruptedException e) {
+							CVSUIPlugin.log(new CVSException(NLS.bind(CVSUIMessages.HistoryView_errorFetchingEntries, new String[] { remoteFile.getName() }), e)); //$NON-NLS-1$
 						}
-					});
-				} catch (InterruptedException e) { // ignore cancellation
-					result[0] = new Object[0];
-				} catch (InvocationTargetException e) {
-					CVSUIPlugin.openError(getViewSite().getShell(), null, null, e);
-					result[0] = new Object[0];
+					}
+					fetchLogEntriesJob.setRemoteFile(remoteFile);
 				}
-				return result[0];				
+				// Schedule the job even if it is already running
+				Utils.schedule(fetchLogEntriesJob, getViewSite());
+				return new Object[0];
 			}
 			public void dispose() {
 			}
@@ -457,6 +551,7 @@ public class HistoryView extends ViewPart {
 		return result;
 	}
 	public void dispose() {
+		shutdown = true;
 		if (branchImage != null) {
 			branchImage.dispose();
 			branchImage = null;
@@ -465,6 +560,19 @@ public class HistoryView extends ViewPart {
 			versionImage.dispose();
 			versionImage = null;
 		}
+		
+		if(fetchLogEntriesJob != null) {
+			if(fetchLogEntriesJob.getState() != Job.NONE) {
+				fetchLogEntriesJob.cancel();
+				try {
+					fetchLogEntriesJob.join();
+				} catch (InterruptedException e) {
+					CVSUIPlugin.log(new CVSException(NLS.bind(CVSUIMessages.HistoryView_errorFetchingEntries, new String[] { "" }), e)); //$NON-NLS-1$ //$NON-NLS-2$
+				}
+			}
+		}
+		getSite().getPage().removePartListener(partListener);
+		getSite().getPage().removePartListener(partListener2);
 	}	
 	/**
 	 * Returns the table viewer contained in this view.
@@ -477,7 +585,7 @@ public class HistoryView extends ViewPart {
 	 */
 	void initDragAndDrop() {
 		int ops = DND.DROP_MOVE | DND.DROP_COPY | DND.DROP_LINK;
-		Transfer[] transfers = new Transfer[] {ResourceTransfer.getInstance()};
+		Transfer[] transfers = new Transfer[] {ResourceTransfer.getInstance(), CVSResourceTransfer.getInstance()};
 		tableViewer.addDropSupport(ops, transfers, new HistoryDropAdapter(tableViewer, this));
 	}
 	private void fillTableMenu(IMenuManager manager) {
@@ -505,18 +613,7 @@ public class HistoryView extends ViewPart {
 		manager.add(copyAction);
 		manager.add(selectAllAction);
 	}
-	/**
-	 * Makes the history view visible in the active perspective. If there
-	 * isn't a history view registered <code>null</code> is returned.
-	 * Otherwise the opened view part is returned.
-	 */
-	public static HistoryView openInActivePerspective() {
-		try {
-			return (HistoryView)CVSUIPlugin.getActivePage().showView(VIEW_ID);
-		} catch (PartInitException pe) {
-			return null;
-		}
-	}
+
 	/** (Non-javadoc)
 	 * Method declared on IWorkbenchPart
 	 */
@@ -534,49 +631,119 @@ public class HistoryView extends ViewPart {
 	 * 
 	 * Only files are supported for now.
 	 */
-	public void showHistory(IResource resource) {
+	public void showHistory(IResource resource, boolean refetch) {
 		if (resource instanceof IFile) {
-			IFile file = (IFile)resource;
-			this.file = file;
+			IFile newfile = (IFile)resource;
+			if(!refetch && this.file != null && newfile.equals(this.file)) {
+				return;
+			} 
+			this.file = newfile;
 			RepositoryProvider teamProvider = RepositoryProvider.getProvider(file.getProject(), CVSProviderPlugin.getTypeId());
 			if (teamProvider != null) {
-				this.provider = (CVSTeamProvider)teamProvider;
 				try {
+					// for a file this will return the base
 					ICVSRemoteFile remoteFile = (ICVSRemoteFile)CVSWorkspaceRoot.getRemoteResourceFor(file);
-					historyTableProvider.setFile(remoteFile);
-					tableViewer.setInput(remoteFile);
-					setTitle(Policy.bind("HistoryView.titleWithArgument", remoteFile.getName())); //$NON-NLS-1$
+					if(remoteFile != null) {
+						historyTableProvider.setFile(remoteFile);
+						// input is set asynchronously so we can't assume that the view
+						// has been populated until the job that queries for the history
+						// has completed.
+						tableViewer.setInput(remoteFile);
+						setContentDescription(remoteFile.getName()); 
+						setTitleToolTip(resource.getFullPath().toString());
+					}
 				} catch (TeamException e) {
 					CVSUIPlugin.openError(getViewSite().getShell(), null, null, e);
 				}				
 			}
-			return;
+		} else {
+			this.file = null;
+			tableViewer.setInput(null);
+			setContentDescription(""); //$NON-NLS-1$
+			setTitleToolTip(""); //$NON-NLS-1$
 		}
-		this.file = null;
-		tableViewer.setInput(null);
-		setTitle(Policy.bind("HistoryView.title")); //$NON-NLS-1$
 	}
 	
 	/**
-	 * Shows the history for the given ICVSRemoteFile in the view.
+	 * An editor has been activated.  Fetch the history if it is shared with CVS and the history view
+	 * is visible in the current page.
+	 * 
+	 * @param editor the active editor
+	 * @since 3.0
 	 */
-	public void showHistory(ICVSRemoteFile remoteFile, String currentRevision) {
-		try {
-			if (remoteFile == null) {
-				tableViewer.setInput(null);
-				setTitle(Policy.bind("HistoryView.title")); //$NON-NLS-1$
-				return;
+	protected void editorActivated(IEditorPart editor) {
+		// Only fetch contents if the view is shown in the current page.
+		if (editor == null || !isLinkingEnabled() || !checkIfPageIsVisible()) {
+			return;
+		}		
+		IEditorInput input = editor.getEditorInput();
+		// Handle compare editors opened from the Synchronize View
+		if (input instanceof SyncInfoCompareInput) {
+			SyncInfoCompareInput syncInput = (SyncInfoCompareInput) input;
+			SyncInfo info = syncInput.getSyncInfo();
+			if(info instanceof CVSSyncInfo && info.getLocal().getType() == IResource.FILE) {
+                // Highlight the loaded versions unless there isn't one
+                ICVSRemoteFile loaded;
+                try {
+                    loaded = (ICVSRemoteFile)CVSWorkspaceRoot.getRemoteResourceFor(info.getLocal());
+                } catch (CVSException e) {
+                    CVSUIPlugin.log(e);
+                    loaded = null;
+                }
+                if (loaded != null) {
+                    showHistory(loaded, false);
+                } else {
+    				ICVSRemoteFile remote = (ICVSRemoteFile)info.getRemote();
+    				ICVSRemoteFile base = (ICVSRemoteFile)info.getBase();
+                    if(remote != null) {
+                        showHistory(remote, false);
+                    } else if(base != null) {
+                        showHistory(base, false);
+                    }
+                }
 			}
-			this.file = null;
-			historyTableProvider.setFile(remoteFile);
-			tableViewer.setInput(remoteFile);
-			setTitle(Policy.bind("HistoryView.titleWithArgument", remoteFile.getName())); //$NON-NLS-1$
-		} catch (CVSException e) {
-			CVSUIPlugin.openError(getViewSite().getShell(), null, null, e);
+		// Handle editors opened on remote files
+		} else if(input instanceof RemoteFileEditorInput) {
+			ICVSRemoteFile remote = ((RemoteFileEditorInput)input).getCVSRemoteFile();
+			if(remote != null) {
+				showHistory(remote, false);
+			}
+		// Handle regular file editors
+		} else {
+            IFile file = ResourceUtil.getFile(input);
+            if(file != null) {
+                showHistory(file, false /* don't fetch if already cached */);
+            }
 		}
 	}
 	
-	private Action getContextMenuAction(String title, final IWorkspaceRunnable action) {
+	private boolean checkIfPageIsVisible() {
+		return getViewSite().getPage().isPartVisible(this);
+	}
+	/**
+	 * Shows the history for the given ICVSRemoteFile in the view.
+	 */
+	public void showHistory(ICVSRemoteFile remoteFile, boolean refetch) {
+		try {
+			if (remoteFile == null) {
+				tableViewer.setInput(null);
+				setContentDescription(""); //$NON-NLS-1$
+				setTitleToolTip(""); //$NON-NLS-1$
+				return;
+			}
+			ICVSFile existingFile = historyTableProvider.getICVSFile(); 
+			if(!refetch && existingFile != null && existingFile.equals(remoteFile)) return;
+			this.file = null;
+			historyTableProvider.setFile(remoteFile);
+			tableViewer.setInput(remoteFile);
+			setContentDescription(remoteFile.getName());
+			setTitleToolTip(remoteFile.getRepositoryRelativePath());
+		} catch (TeamException e) {
+			CVSUIPlugin.openError(getViewSite().getShell(), null, null, e);
+		} 
+	}
+	
+	private Action getContextMenuAction(String title, final boolean needsProgressDialog, final IWorkspaceRunnable action) {
 			return new Action(title) {
 			public void run() {
 				try {
@@ -586,15 +753,23 @@ public class HistoryView extends ViewPart {
 					IStructuredSelection ss = (IStructuredSelection)selection;
 					Object o = ss.getFirstElement();
 					currentSelection = (ILogEntry)o;
-					new ProgressMonitorDialog(getViewSite().getShell()).run(false, true, new WorkspaceModifyOperation() {
-						protected void execute(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
-							try {				
-								action.run(monitor);
-							} catch (CoreException e) {
-								throw new InvocationTargetException(e);
+					if(needsProgressDialog) {
+						PlatformUI.getWorkbench().getProgressService().run(true, true, new IRunnableWithProgress() {
+							public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
+								try {				
+									action.run(monitor);
+								} catch (CoreException e) {
+									throw new InvocationTargetException(e);
+								}
 							}
+						});
+					} else {
+						try {				
+							action.run(null);
+						} catch (CoreException e) {
+							throw new InvocationTargetException(e);
 						}
-					});
+					}							
 				} catch (InvocationTargetException e) {
 					CVSUIPlugin.openError(getViewSite().getShell(), null, null, e, CVSUIPlugin.LOG_NONTEAM_EXCEPTIONS);
 				} catch (InterruptedException e) {
@@ -617,8 +792,8 @@ public class HistoryView extends ViewPart {
 			ICVSFile cvsFile = CVSWorkspaceRoot.getCVSFileFor(file);
 			try {
 				if(cvsFile.isModified(null)) {
-					String title = Policy.bind("HistoryView.overwriteTitle"); //$NON-NLS-1$
-					String msg = Policy.bind("HistoryView.overwriteMsg"); //$NON-NLS-1$
+					String title = CVSUIMessages.HistoryView_overwriteTitle; //$NON-NLS-1$
+					String msg = CVSUIMessages.HistoryView_overwriteMsg; //$NON-NLS-1$
 					final MessageDialog dialog = new MessageDialog(getViewSite().getShell(), title, null, msg, MessageDialog.QUESTION, new String[] { IDialogConstants.YES_LABEL, IDialogConstants.CANCEL_LABEL }, 0);
 					final int[] result = new int[1];
 					getViewSite().getShell().getDisplay().syncExec(new Runnable() {
@@ -631,7 +806,7 @@ public class HistoryView extends ViewPart {
 					}
 				}
 			} catch(CVSException e) {
-				CVSUIPlugin.log(e.getStatus());
+				CVSUIPlugin.log(e);
 			}
 		}
 		return true;
@@ -644,8 +819,86 @@ public class HistoryView extends ViewPart {
 		entries = null;
 		BusyIndicator.showWhile(tableViewer.getTable().getDisplay(), new Runnable() {
 			public void run() {
+				// if a local file was fed to the history view then we will have to refetch the handle
+				// to properly display the current revision marker. 
+				if(file != null) {
+					 ICVSRemoteFile remoteFile;
+					try {
+						remoteFile = (ICVSRemoteFile) CVSWorkspaceRoot.getRemoteResourceFor(file);
+						historyTableProvider.setFile(remoteFile);
+					} catch (CVSException e) {
+						// use previously fetched remote file, but log error
+						CVSUIPlugin.log(e);
+					}
+				}
 				tableViewer.refresh();
 			}
 		});
 	}
+	
+	/**
+	 * Select the revision in the receiver.
+	 */
+	public void selectRevision(String revision) {
+		if (entries == null) {
+			return;
+		}
+	
+		ILogEntry entry = null;
+		for (int i = 0; i < entries.length; i++) {
+			if (entries[i].getRevision().equals(revision)) {
+				entry = entries[i];
+				break;
+			}
+		}
+	
+		if (entry != null) {
+			IStructuredSelection selection = new StructuredSelection(entry);
+			tableViewer.setSelection(selection, true);
+		}
+	}
+	
+	/**
+	 * Enabled linking to the active editor
+	 * @since 3.0
+	 */
+	public void setLinkingEnabled(boolean enabled) {
+		this.linkingEnabled = enabled;
+
+		// remember the last setting in the dialog settings		
+		settings.setValue(ICVSUIConstants.PREF_HISTORY_VIEW_EDITOR_LINKING, enabled);
+	
+		// if turning linking on, update the selection to correspond to the active editor
+		if (enabled) {
+			editorActivated(getSite().getPage().getActiveEditor());
+		}
+	}
+	
+	/**
+	 * Returns if linking to the ative editor is enabled or disabled.
+	 * @return boolean indicating state of editor linking.
+	 */
+	private boolean isLinkingEnabled() {
+		return linkingEnabled;
+	}
+	
+	/*
+	 * Flatten the text in the multiline comment
+	 */
+	public static String flattenText(String string) {
+		StringBuffer buffer = new StringBuffer(string.length() + 20);
+		boolean skipAdjacentLineSeparator = true;
+		for (int i = 0; i < string.length(); i++) {
+			char c = string.charAt(i);
+			if (c == '\r' || c == '\n') {
+				if (!skipAdjacentLineSeparator)
+					buffer.append(CVSUIMessages.separator); //$NON-NLS-1$
+				skipAdjacentLineSeparator = true;
+			} else {
+				buffer.append(c);
+				skipAdjacentLineSeparator = false;
+			}
+		}
+		return buffer.toString();
+	}
 }
