@@ -1,9 +1,9 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2004 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
+ * http://www.eclipse.org/legal/epl-v10.html
  * 
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -13,38 +13,16 @@ package org.eclipse.team.internal.ccvs.ui.wizards;
 import java.io.BufferedInputStream;
 import java.io.InputStream;
 import java.lang.reflect.InvocationTargetException;
-import java.util.HashMap;
-import java.util.Iterator;
-import java.util.Map;
+import java.util.*;
 
-import org.eclipse.compare.CompareUI;
-import org.eclipse.compare.CompareViewerPane;
-import org.eclipse.compare.CompareViewerSwitchingPane;
-import org.eclipse.compare.IModificationDate;
-import org.eclipse.compare.IStreamContentAccessor;
-import org.eclipse.compare.ITypedElement;
-import org.eclipse.compare.Splitter;
-import org.eclipse.core.resources.IContainer;
-import org.eclipse.core.resources.IFile;
-import org.eclipse.core.resources.IFolder;
-import org.eclipse.core.resources.IResource;
-import org.eclipse.core.resources.ResourcesPlugin;
-import org.eclipse.core.runtime.CoreException;
-import org.eclipse.core.runtime.IProgressMonitor;
-import org.eclipse.core.runtime.Path;
+import org.eclipse.compare.*;
+import org.eclipse.core.resources.*;
+import org.eclipse.core.runtime.*;
+import org.eclipse.jface.dialogs.Dialog;
 import org.eclipse.jface.operation.IRunnableWithProgress;
 import org.eclipse.jface.resource.ImageDescriptor;
-import org.eclipse.jface.viewers.CheckStateChangedEvent;
-import org.eclipse.jface.viewers.CheckboxTableViewer;
-import org.eclipse.jface.viewers.DecoratingLabelProvider;
-import org.eclipse.jface.viewers.ICheckStateListener;
-import org.eclipse.jface.viewers.ISelection;
-import org.eclipse.jface.viewers.ISelectionChangedListener;
-import org.eclipse.jface.viewers.IStructuredContentProvider;
-import org.eclipse.jface.viewers.SelectionChangedEvent;
-import org.eclipse.jface.viewers.StructuredSelection;
-import org.eclipse.jface.viewers.TreeViewer;
-import org.eclipse.jface.viewers.Viewer;
+import org.eclipse.jface.viewers.*;
+import org.eclipse.osgi.util.NLS;
 import org.eclipse.swt.SWT;
 import org.eclipse.swt.events.SelectionAdapter;
 import org.eclipse.swt.events.SelectionEvent;
@@ -53,22 +31,14 @@ import org.eclipse.swt.layout.GridData;
 import org.eclipse.swt.widgets.Composite;
 import org.eclipse.swt.widgets.Widget;
 import org.eclipse.team.core.TeamException;
-import org.eclipse.team.internal.ccvs.core.CVSException;
-import org.eclipse.team.internal.ccvs.core.CVSProviderPlugin;
-import org.eclipse.team.internal.ccvs.core.CVSTag;
-import org.eclipse.team.internal.ccvs.core.ICVSFile;
-import org.eclipse.team.internal.ccvs.core.ICVSFolder;
-import org.eclipse.team.internal.ccvs.core.ICVSRemoteFile;
-import org.eclipse.team.internal.ccvs.core.ICVSRepositoryLocation;
-import org.eclipse.team.internal.ccvs.core.ILogEntry;
+import org.eclipse.team.core.variants.IResourceVariant;
+import org.eclipse.team.internal.ccvs.core.*;
 import org.eclipse.team.internal.ccvs.core.resources.CVSWorkspaceRoot;
 import org.eclipse.team.internal.ccvs.core.syncinfo.FolderSyncInfo;
-import org.eclipse.team.internal.ccvs.ui.AdaptableHierarchicalResourceList;
-import org.eclipse.team.internal.ccvs.ui.CVSUIPlugin;
-import org.eclipse.team.internal.ccvs.ui.HistoryTableProvider;
-import org.eclipse.team.internal.ccvs.ui.IHelpContextIds;
+import org.eclipse.team.internal.ccvs.core.util.KnownRepositories;
+import org.eclipse.team.internal.ccvs.ui.*;
 import org.eclipse.team.internal.ccvs.ui.Policy;
-import org.eclipse.ui.help.WorkbenchHelp;
+import org.eclipse.ui.PlatformUI;
 import org.eclipse.ui.model.WorkbenchLabelProvider;
 import org.eclipse.ui.views.navigator.ResourceSorter;
 
@@ -93,7 +63,7 @@ public class RestoreFromRepositoryFileSelectionPage extends CVSWizardPage {
 
 	private static final int WIZARD_WIDTH = 550;
 	
-	class HistoryInput implements ITypedElement, IStreamContentAccessor, IModificationDate {
+	class HistoryInput implements ITypedElement, IEncodedStreamContentAccessor, IModificationDate {
 		IFile file;
 		ILogEntry logEntry;
 		
@@ -102,7 +72,9 @@ public class RestoreFromRepositoryFileSelectionPage extends CVSWizardPage {
 			this.logEntry = logEntry;
 		}
 		public InputStream getContents() throws CoreException {
-			return getContentsFromLogEntry(logEntry);
+			IStorage s = getStorageFromLogEntry(logEntry);
+			if (s == null) return null;
+			return new BufferedInputStream(s.getContents());
 		}
 		public String getName() {
 			return file.getName();
@@ -116,6 +88,13 @@ public class RestoreFromRepositoryFileSelectionPage extends CVSWizardPage {
 		public long getModificationDate() {
 			return logEntry.getDate().getTime();
 		}
+		public String getCharset() throws CoreException {
+			IStorage s = getStorageFromLogEntry(logEntry);
+			if (s instanceof IEncodedStorage) {
+				return ((IEncodedStorage)s).getCharset();
+			}
+			return null;
+		}
 	}
 	
 	/**
@@ -137,10 +116,10 @@ public class RestoreFromRepositoryFileSelectionPage extends CVSWizardPage {
 	 * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
 	 */
 	public void createControl(Composite parent) {
-		Composite composite= createComposite(parent, 1);
+		Composite composite= createComposite(parent, 1, false);
 		setControl(composite);
 		
-		WorkbenchHelp.setHelp(composite, IHelpContextIds.RESTORE_FROM_REPOSITORY_FILE_SELECTION_PAGE);
+        PlatformUI.getWorkbench().getHelpSystem().setHelp(composite, IHelpContextIds.RESTORE_FROM_REPOSITORY_FILE_SELECTION_PAGE);
 		
 		// Top and bottom panes: top is the two selection panes, bottom is the file content viewer
 		Splitter vsplitter= new Splitter(composite,  SWT.VERTICAL);
@@ -161,11 +140,11 @@ public class RestoreFromRepositoryFileSelectionPage extends CVSWizardPage {
 		
 		// Top right: Revision selection pane
 		revisionSelectionPane = new CompareViewerPane(hsplitter, SWT.BORDER | SWT.FLAT);
-		data = new GridData(GridData.FILL_HORIZONTAL | GridData.FILL_VERTICAL);;
+		data = new GridData(GridData.FILL_HORIZONTAL | GridData.FILL_VERTICAL);
 		revisionSelectionPane.setLayoutData(data);
 		historyTableProvider = new HistoryTableProvider();
 		revisionsTable = createRevisionSelectionTable(revisionSelectionPane, historyTableProvider);
-		revisionSelectionPane.setText(Policy.bind("RestoreFromRepositoryFileSelectionPage.emptyRevisionPane")); //$NON-NLS-1$
+		revisionSelectionPane.setText(CVSUIMessages.RestoreFromRepositoryFileSelectionPage_emptyRevisionPane); //$NON-NLS-1$
 		
 		// Bottom: File content viewer
 		fileContentPane = new CompareViewerSwitchingPane(vsplitter, SWT.BORDER | SWT.FLAT) {
@@ -176,6 +155,7 @@ public class RestoreFromRepositoryFileSelectionPage extends CVSWizardPage {
 						
 		initializeValues();
 		updateWidgetEnablements();
+        Dialog.applyDialogFont(parent);
 	}
 
 	protected CheckboxTableViewer createRevisionSelectionTable(CompareViewerPane composite, HistoryTableProvider tableProvider) {
@@ -225,7 +205,7 @@ public class RestoreFromRepositoryFileSelectionPage extends CVSWizardPage {
 							ILogEntry entry = (ILogEntry)filesToRestore.get(element);
 							text = super.decorateText(input, element);
 							if (entry != null) {
-								text = Policy.bind("RestoreFromRepositoryFileSelectionPage.fileToRestore", text, entry.getRevision()); //$NON-NLS-1$
+								text = NLS.bind(CVSUIMessages.RestoreFromRepositoryFileSelectionPage_fileToRestore, new String[] { text, entry.getRevision() }); //$NON-NLS-1$
 							}
 						}
 						return text;
@@ -250,20 +230,30 @@ public class RestoreFromRepositoryFileSelectionPage extends CVSWizardPage {
 	 * Method updateWidgetEnablements.
 	 */
 	private void updateWidgetEnablements() {
-		setErrorMessage(null);
+		
 		if (filesToRestore.isEmpty()) {
 			setPageComplete(false);
+			setErrorMessage(null);
 			return;
 		}
+		
 		for (Iterator iter = filesToRestore.keySet().iterator(); iter.hasNext();) {
 			IFile file = (IFile) iter.next();
 			if (file.exists()) {
 				setPageComplete(false);
-				setErrorMessage(Policy.bind("RestoreFromRepositoryFileSelectionPage.fileExists", file.getName())); //$NON-NLS-1$
+				setErrorMessage(NLS.bind(CVSUIMessages.RestoreFromRepositoryFileSelectionPage_fileExists, new String[] { file.getName() })); //$NON-NLS-1$
+				return;
+			}
+			
+			ILogEntry entry = (ILogEntry) filesToRestore.get(file);
+			if (entry.isDeletion())  {
+				setPageComplete(false);
+				setErrorMessage(NLS.bind(CVSUIMessages.RestoreFromRepositoryFileSelectionPage_revisionIsDeletion, new String[] { entry.getRevision(), file.getName() })); //$NON-NLS-1$
 				return;
 			}
 		}
 		setPageComplete(true);
+		setErrorMessage(null);
 	}
 	
 	/**
@@ -326,13 +316,13 @@ public class RestoreFromRepositoryFileSelectionPage extends CVSWizardPage {
 		if (folder == null) return;
 		
 		if (fileSelectionPane != null && !fileSelectionPane.isDisposed()) {
-			fileSelectionPane.setText(Policy.bind("RestoreFromRepositoryFileSelectionPage.fileSelectionPaneTitle", folder.getProject().getName())); //$NON-NLS-1$
+			fileSelectionPane.setText(NLS.bind(CVSUIMessages.RestoreFromRepositoryFileSelectionPage_fileSelectionPaneTitle, new String[] { folder.getProject().getName() })); //$NON-NLS-1$
 			fileSelectionPane.setImage(CompareUI.getImage(folder.getProject()));
 		}
 		
 		if (revisionSelectionPane != null && !revisionSelectionPane.isDisposed()) {
 			if (selectedFile == null) {
-				revisionSelectionPane.setText(Policy.bind("RestoreFromRepositoryFileSelectionPage.emptyRevisionPane")); //$NON-NLS-1$
+				revisionSelectionPane.setText(CVSUIMessages.RestoreFromRepositoryFileSelectionPage_emptyRevisionPane); //$NON-NLS-1$
 				revisionSelectionPane.setImage(null);
 			}
 		}
@@ -341,7 +331,6 @@ public class RestoreFromRepositoryFileSelectionPage extends CVSWizardPage {
 		if (fileContentPane != null && !fileContentPane.isDisposed()) {
 			fileContentPane.setInput(null);
 		}
-		setErrorMessage(null);
 		
 		// refresh the tree
 		if (fileTree != null) {
@@ -373,12 +362,10 @@ public class RestoreFromRepositoryFileSelectionPage extends CVSWizardPage {
 			}
 		}
 		// Set the titlebar text for the revisions table
-		revisionSelectionPane.setText(Policy.bind("RestoreFromRepositoryFileSelectionPage.revisionSelectionPaneTitle", selectedFile.getName())); //$NON-NLS-1$
+		revisionSelectionPane.setText(NLS.bind(CVSUIMessages.RestoreFromRepositoryFileSelectionPage_revisionSelectionPaneTitle, new String[] { selectedFile.getName() })); //$NON-NLS-1$
 		revisionSelectionPane.setImage(CompareUI.getImage(selectedFile));
 		// Clear the file content pane
 		fileContentPane.setInput(null);
-		// Clear any previous error messages
-		setErrorMessage(null);
 	}
 	
 	private void handleFileSelection(SelectionChangedEvent event) {
@@ -411,8 +398,8 @@ public class RestoreFromRepositoryFileSelectionPage extends CVSWizardPage {
 				// First, we need to create a remote file handle so we can get the log entries
 				ICVSFolder parent = CVSWorkspaceRoot.getCVSFolderFor(file.getParent());
 				FolderSyncInfo info = parent.getFolderSyncInfo();
-				ICVSRepositoryLocation location = CVSProviderPlugin.getPlugin().getRepository(info.getRoot());
-				final ICVSRemoteFile remoteFile = location.getRemoteFile(new Path(info.getRepository()).append(file.getName()).toString(), CVSTag.DEFAULT);
+				ICVSRepositoryLocation location = KnownRepositories.getInstance().getRepository(info.getRoot());
+				final ICVSRemoteFile remoteFile = location.getRemoteFile(new Path(null, info.getRepository()).append(file.getName()).toString(), CVSTag.DEFAULT);
 				
 				// Then we need to fetch the log entries
 				getContainer().run(true, true, new IRunnableWithProgress() {
@@ -427,7 +414,6 @@ public class RestoreFromRepositoryFileSelectionPage extends CVSWizardPage {
 						}
 					}
 				});
-
 			} catch (CVSException e) {
 				setErrorMessage(
 					CVSUIPlugin.openError(getShell(), null, null, e, CVSUIPlugin.PERFORM_SYNC_EXEC)
@@ -451,19 +437,14 @@ public class RestoreFromRepositoryFileSelectionPage extends CVSWizardPage {
 		return (ILogEntry[])entriesCache.get(selectedFile);
 	}
 	
-	/**
-	 * Method getContents.
-	 * @param logEntry
-	 * @return InputStream
-	 */
-	private InputStream getContentsFromLogEntry(final ILogEntry logEntry) {
-		final InputStream[] is = new InputStream[] { null };
+	private IStorage getStorageFromLogEntry(final ILogEntry logEntry) {
+		final IStorage[] s = new IStorage[] { null };
 		try {
 			getContainer().run(true, true, new IRunnableWithProgress() {
 				public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
 					try {
 						ICVSRemoteFile remoteFile = logEntry.getRemoteFile();
-						is[0] = remoteFile.getContents(monitor);
+						s[0] = ((IResourceVariant)remoteFile).getStorage(monitor);
 					} catch (TeamException e) {
 						throw new InvocationTargetException(e);
 					}
@@ -477,19 +458,13 @@ public class RestoreFromRepositoryFileSelectionPage extends CVSWizardPage {
 		} catch (InterruptedException e) {
 			return null;
 		}
-		setErrorMessage(null);
-		return new BufferedInputStream(is[0]);
+		return s[0];
 	}
 
 	private void handleRevisionChecked(CheckStateChangedEvent event) {
-		// Only allow one element to be checked
 		if (event.getChecked()) {
-			if (((ILogEntry)event.getElement()).isDeletion()) {
-				revisionsTable.setChecked(event.getElement(), false);
-			} else {
-				revisionsTable.setCheckedElements(new Object[] {event.getElement()});
-				filesToRestore.put(selectedFile, event.getElement());
-			}
+			revisionsTable.setCheckedElements(new Object[] {event.getElement()});
+			filesToRestore.put(selectedFile, event.getElement());
 		}
 		if (revisionsTable.getCheckedElements().length == 0) {
 			filesToRestore.remove(selectedFile);
@@ -528,8 +503,7 @@ public class RestoreFromRepositoryFileSelectionPage extends CVSWizardPage {
 	 * @return String
 	 */
 	private String getEditionLabel(IFile selectedFile, ILogEntry selected) {
-		return Policy.bind("RestoreFromRepositoryFileSelectionPage.fileContentPaneTitle", //$NON-NLS-1$
-			new Object[] { selectedFile.getName(), selected.getRevision(), selectedFile.getFullPath().makeRelative().removeLastSegments(1).toString() });
+		return NLS.bind(CVSUIMessages.RestoreFromRepositoryFileSelectionPage_fileContentPaneTitle, (new Object[] { selectedFile.getName(), selected.getRevision(), selectedFile.getFullPath().makeRelative().removeLastSegments(1).toString() }));
 	}
 	
 	public boolean restoreSelectedFiles() {
