@@ -1,36 +1,71 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
 package org.eclipse.update.internal.core;
-import java.io.*;
+
+import java.io.File;
+import java.io.IOException;
 import java.net.MalformedURLException;
 import java.net.URL;
-import java.util.*;
+import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.Collection;
+import java.util.Date;
+import java.util.HashMap;
+import java.util.HashSet;
+import java.util.Hashtable;
+import java.util.Iterator;
+import java.util.Map;
+import java.util.Set;
 
-import org.eclipse.core.boot.BootLoader;
-import org.eclipse.core.boot.IPlatformConfiguration;
-import org.eclipse.core.runtime.*;
-import org.eclipse.core.runtime.model.PluginDescriptorModel;
-import org.eclipse.core.runtime.model.PluginFragmentModel;
-import org.eclipse.update.configuration.*;
-import org.eclipse.update.core.*;
+import org.eclipse.core.runtime.CoreException;
+import org.eclipse.core.runtime.IProgressMonitor;
+import org.eclipse.core.runtime.Platform;
+import org.eclipse.osgi.util.NLS;
+import org.eclipse.update.configuration.IActivity;
+import org.eclipse.update.configuration.IConfiguredSite;
+import org.eclipse.update.configuration.IInstallConfiguration;
+import org.eclipse.update.configuration.IInstallConfigurationChangedListener;
+import org.eclipse.update.configuration.IProblemHandler;
+import org.eclipse.update.configurator.ConfiguratorUtils;
+import org.eclipse.update.configurator.IPlatformConfiguration;
+import org.eclipse.update.core.FeatureContentProvider;
+import org.eclipse.update.core.IFeature;
+import org.eclipse.update.core.IFeatureReference;
+import org.eclipse.update.core.IPluginEntry;
+import org.eclipse.update.core.ISite;
+import org.eclipse.update.core.ISiteContentProvider;
+import org.eclipse.update.core.ISiteFeatureReference;
+import org.eclipse.update.core.Site;
+import org.eclipse.update.core.SiteManager;
+import org.eclipse.update.core.Utilities;
+import org.eclipse.update.core.VersionedIdentifier;
 import org.eclipse.update.core.model.SiteModel;
-import org.eclipse.update.internal.model.*;
+import org.eclipse.update.internal.configurator.ConfigurationActivator;
+import org.eclipse.update.internal.configurator.FeatureEntry;
+import org.eclipse.update.internal.configurator.IConfigurationConstants;
+import org.eclipse.update.internal.configurator.PlatformConfiguration;
+import org.eclipse.update.internal.configurator.PluginEntry;
+import org.eclipse.update.internal.configurator.SiteEntry;
+import org.eclipse.update.internal.model.ConfigurationActivityModel;
+import org.eclipse.update.internal.model.ConfiguredSiteModel;
+import org.eclipse.update.internal.model.InstallConfigurationModel;
+import org.osgi.framework.Bundle;
 
 /**
  * Manages ConfiguredSites
  *
  */
 
-public class InstallConfiguration extends InstallConfigurationModel implements IInstallConfiguration, IWritable {
-
+public class InstallConfiguration extends InstallConfigurationModel implements IInstallConfiguration {
+	private static boolean isWindows = System.getProperty("os.name").startsWith("Win"); //$NON-NLS-1$ //$NON-NLS-2$
 	private ListenersList listeners = new ListenersList();
 
 	/*
@@ -39,11 +74,32 @@ public class InstallConfiguration extends InstallConfigurationModel implements I
 	public InstallConfiguration() {
 	}
 
+	/*
+	 * Copy constructor 
+	 * @since 3.0
+	 */
+	public InstallConfiguration(IInstallConfiguration config) throws MalformedURLException, CoreException {
+		this(config, null, null);
+	}
+	
 	/*
 	 * copy constructor
 	 */
-	public InstallConfiguration(IInstallConfiguration config, URL newLocation, String label) throws MalformedURLException {
+	public InstallConfiguration(IInstallConfiguration config, URL newLocation, String label) throws CoreException, MalformedURLException {
+		// set current date and timeline as caller can call setDate if the
+		// date on the URL string has to be the same
+		Date now = new Date();
+		setCreationDate(now);
+		setCurrent(false);
+		
+		if (newLocation == null) {
+			String newFileName = UpdateManagerUtils.getLocalRandomIdentifier(SiteLocal.CONFIG_FILE, now);
+			newLocation = UpdateManagerUtils.getURL(((SiteLocal)SiteManager.getLocalSite()).getLocationURL(), newFileName, null);
+		}
 		setLocationURLString(newLocation.toExternalForm());
+		
+		if (label == null)
+			label = Utilities.format(now);
 		setLabel(label);
 
 		// do not copy list of listeners nor activities
@@ -58,16 +114,11 @@ public class InstallConfiguration extends InstallConfigurationModel implements I
 			}
 		}
 
-		// set dummy date and timeline as caller can call set date if the
-		// date on the URL string has to be the same
-		Date now = new Date();
-		setCreationDate(now);
-		setTimeline(now.getTime());
-		setCurrent(false);
 		resolve(newLocation, null);
 		// no need to parse file, all data are initialized
 		initialized = true;
 	}
+	
 
 	/*
 	 * Returns the list of configured sites or an empty array
@@ -84,7 +135,7 @@ public class InstallConfiguration extends InstallConfigurationModel implements I
 	 * Returns the default site policy
 	 */
 	private int getDefaultPolicy() {
-		return IPlatformConfiguration.ISitePolicy.USER_EXCLUDE;
+		return IConfigurationConstants.DEFAULT_POLICY_TYPE;
 	}
 
 	/**
@@ -93,6 +144,13 @@ public class InstallConfiguration extends InstallConfigurationModel implements I
 	 */
 	public IConfiguredSite createConfiguredSite(File file) throws CoreException {
 
+		if (!file.getName().equals("eclipse")) { //$NON-NLS-1$
+			file = new File(file, "eclipse"); //$NON-NLS-1$
+			file.mkdirs();
+		}
+		
+		if (isDuplicateSite(file))
+			throw Utilities.newCoreException(NLS.bind(Messages.InstallConfiguration_location_exists, (new String[] { file.getPath() })),null);
 		ISite site = InternalSiteManager.createSite(file);
 
 		//create a config site around the site
@@ -100,15 +158,23 @@ public class InstallConfiguration extends InstallConfigurationModel implements I
 		BaseSiteLocalFactory factory = new BaseSiteLocalFactory();
 		ConfiguredSite configSite = (ConfiguredSite) factory.createConfigurationSiteModel((SiteModel) site, getDefaultPolicy());
 
-		if (site != null && configSite.verifyUpdatableStatus().isOK()) {
+		if (configSite.isNativelyLinked()) {
+			throw Utilities.newCoreException(Messages.InstallConfiguration_AlreadyNativelyLinked, null); 
+		}
+		
+		if (configSite.isProductSite()) {
+			throw Utilities.newCoreException(Messages.InstallConfiguration_AlreadyProductSite, null); 
+		}
+		
+		if (site != null) {
 			configSite.setPlatformURLString(site.getURL().toExternalForm());
 
 			// obtain the list of plugins
-			IPlatformConfiguration runtimeConfiguration = BootLoader.getCurrentPlatformConfiguration();
-			ConfigurationPolicy configurationPolicy = (ConfigurationPolicy) configSite.getConfigurationPolicy();
+			IPlatformConfiguration runtimeConfiguration = ConfiguratorUtils.getCurrentPlatformConfiguration();
+			ConfigurationPolicy configurationPolicy = configSite.getConfigurationPolicy();
 			String[] pluginPath = new String[0];
 			if (configurationPolicy.getPolicy() == IPlatformConfiguration.ISitePolicy.USER_INCLUDE)
-				pluginPath = configurationPolicy.getPluginPath(site, null);
+				pluginPath = configurationPolicy.getPluginPath(site);
 
 			// create new Site in configuration
 			IPlatformConfiguration.ISitePolicy sitePolicy = runtimeConfiguration.createSitePolicy(configurationPolicy.getPolicy(), pluginPath);
@@ -119,8 +185,11 @@ public class InstallConfiguration extends InstallConfigurationModel implements I
 
 			// if the privatre marker doesn't already exist create it
 			configSite.createPrivateSiteMarker();
+			((SiteModel)site).setConfiguredSiteModel(configSite);
 		}
-
+		// configure all features as enable
+		configure(configSite);
+		
 		return configSite;
 	}
 
@@ -129,46 +198,49 @@ public class InstallConfiguration extends InstallConfigurationModel implements I
 	 * The policy is from <code> org.eclipse.core.boot.IPlatformConfiguration</code>
 	 */
 	public IConfiguredSite createLinkedConfiguredSite(File file) throws CoreException {
-
-		ISite site = InternalSiteManager.createSite(file);
-
-		//create a config site around the site
-		// even if the site == null
-		BaseSiteLocalFactory factory = new BaseSiteLocalFactory();
-		ConfiguredSite configSite = (ConfiguredSite) factory.createConfigurationSiteModel((SiteModel) site, getDefaultPolicy());
-
-		if (!configSite.isExtensionSite()) {
-			String msg = Policy.bind("InstallConfiguration.NotAnExtensionSite");
-			throw Utilities.newCoreException(msg, null);
-		}
-
-		if (configSite.isNativelyLinked()) {
-			throw Utilities.newCoreException("InstallConfiguration.AlreadyNativelyLinked", null);
-		}
-
-		if (site != null) {
-			configSite.setPlatformURLString(site.getURL().toExternalForm());
-
-			// obtain the list of plugins
-			IPlatformConfiguration runtimeConfiguration = BootLoader.getCurrentPlatformConfiguration();
-			ConfigurationPolicy configurationPolicy = (ConfigurationPolicy) configSite.getConfigurationPolicy();
-			String[] pluginPath = new String[0];
-			if (configurationPolicy.getPolicy() == IPlatformConfiguration.ISitePolicy.USER_INCLUDE)
-				pluginPath = configurationPolicy.getPluginPath(site, null);
-
-			// create new Site in configuration
-			IPlatformConfiguration.ISitePolicy sitePolicy = runtimeConfiguration.createSitePolicy(configurationPolicy.getPolicy(), pluginPath);
-
-			// change runtime
-			IPlatformConfiguration.ISiteEntry siteEntry = runtimeConfiguration.createSiteEntry(site.getURL(), sitePolicy);
-			runtimeConfiguration.configureSite(siteEntry);
-
-		}
-
-		// configure all features as enable
-		configure(configSite);
-
-		return configSite;
+		return createConfiguredSite(file);
+//		if (isDuplicateSite(file))
+//			throw Utilities.newCoreException(UpdateUtils.getFormattedMessage("InstallConfiguration.location.exists", file.getPath()),null);
+//		
+//		ISite site = InternalSiteManager.createSite(file);
+//
+//		//create a config site around the site
+//		// even if the site == null
+//		BaseSiteLocalFactory factory = new BaseSiteLocalFactory();
+//		ConfiguredSite configSite = (ConfiguredSite) factory.createConfigurationSiteModel((SiteModel) site, getDefaultPolicy());
+//
+//		if (!configSite.isExtensionSite()) {
+//			String msg = Policy.bind("InstallConfiguration.NotAnExtensionSite");
+//			throw Utilities.newCoreException(msg, null);
+//		}
+//
+//		if (configSite.isNativelyLinked()) {
+//			throw Utilities.newCoreException("InstallConfiguration.AlreadyNativelyLinked", null);
+//		}
+//
+//		if (site != null) {
+//			configSite.setPlatformURLString(site.getURL().toExternalForm());
+//
+//			// obtain the list of plugins
+//			IPlatformConfiguration runtimeConfiguration = ConfiguratorUtils.getCurrentPlatformConfiguration();
+//			ConfigurationPolicy configurationPolicy = configSite.getConfigurationPolicy();
+//			String[] pluginPath = new String[0];
+//			if (configurationPolicy.getPolicy() == IPlatformConfiguration.ISitePolicy.USER_INCLUDE)
+//				pluginPath = configurationPolicy.getPluginPath(site);
+//
+//			// create new Site in configuration
+//			IPlatformConfiguration.ISitePolicy sitePolicy = runtimeConfiguration.createSitePolicy(configurationPolicy.getPolicy(), pluginPath);
+//
+//			// change runtime
+//			IPlatformConfiguration.ISiteEntry siteEntry = runtimeConfiguration.createSiteEntry(site.getURL(), sitePolicy);
+//			runtimeConfiguration.configureSite(siteEntry);
+//
+//		}
+//
+//		// configure all features as enable
+//		configure(configSite);
+//
+//		return configSite;
 	}
 
 	/*
@@ -181,8 +253,8 @@ public class InstallConfiguration extends InstallConfigurationModel implements I
 		for (int i = 0; i < newFeaturesRef.length; i++) {
 			// TRACE
 			if (UpdateCore.DEBUG && UpdateCore.DEBUG_SHOW_RECONCILER) {
-				String reconciliationType = "enable (optimistic)";
-				UpdateCore.debug("New Linked Site:New Feature: " + newFeaturesRef[i].getURL() + " as " + reconciliationType);
+				String reconciliationType = "enable (optimistic)"; //$NON-NLS-1$
+				UpdateCore.debug("New Linked Site:New Feature: " + newFeaturesRef[i].getURL() + " as " + reconciliationType); //$NON-NLS-1$ //$NON-NLS-2$
 			}
 			ConfigurationPolicy policy = linkedSite.getConfigurationPolicy();
 			policy.configure(newFeaturesRef[i], true, false);
@@ -238,10 +310,7 @@ public class InstallConfiguration extends InstallConfigurationModel implements I
 				IInstallConfigurationChangedListener listener = ((IInstallConfigurationChangedListener) configurationListeners[i]);
 				listener.installSiteRemoved(site);
 			}
-
-			// remove marker and directory if we just created it
-			((ConfiguredSite)site).removePrivateSiteMarker();
-
+			
 			//activity
 			ConfigurationActivity activity = new ConfigurationActivity(IActivity.ACTION_SITE_REMOVE);
 			activity.setLabel(site.getSite().getURL().toExternalForm());
@@ -269,21 +338,6 @@ public class InstallConfiguration extends InstallConfigurationModel implements I
 		}
 	}
 
-	/*
-	 * write the Configuration.xml file
-	 */
-	private void export(File exportFile) throws CoreException {
-		try {
-			UpdateManagerUtils.Writer writer = UpdateManagerUtils.getWriter(exportFile, "UTF-8"); //$NON-NLS-1$
-			writer.write(this);
-		} catch (FileNotFoundException e) {
-			throw Utilities.newCoreException(Policy.bind("InstallConfiguration.UnableToSaveConfiguration", exportFile.getAbsolutePath()), e);
-			//$NON-NLS-1$
-		} catch (UnsupportedEncodingException e) {
-			throw Utilities.newCoreException(Policy.bind("InstallConfiguration.UnableToEncodeConfiguration", exportFile.getAbsolutePath()), e);
-			//$NON-NLS-1$
-		}
-	}
 
 	/*
 	 * Deletes the configuration from its URL/location
@@ -297,19 +351,17 @@ public class InstallConfiguration extends InstallConfigurationModel implements I
 		}
 	}
 
-	/*
+	/**
 	 * Saves the configuration into its URL/location
 	 * and changes the platform configuration.
-	 * The runtime site entries from platform.cfg are updated as required
+	 * The runtime site entries from platform.xml are updated as required
 	 * (cannot recreate these because must preserve other runtime state) [18520]
+	 * @return true if restart is needed
 	 */
-	public void save(boolean isTransient) throws CoreException {
-
-		// save the configuration.xml file
-		saveConfigurationFile(isTransient);
-
+	public boolean save() throws CoreException {
+		
 		// Write info  into platform for the next runtime
-		IPlatformConfiguration runtimeConfiguration = BootLoader.getCurrentPlatformConfiguration();
+		IPlatformConfiguration runtimeConfiguration = ConfiguratorUtils.getCurrentPlatformConfiguration();
 		ConfiguredSiteModel[] configurationSites = getConfigurationSitesModel();
 
 		// clean configured Entries from platform runtime
@@ -323,7 +375,7 @@ public class InstallConfiguration extends InstallConfigurationModel implements I
 		// correct lookup)
 		IPlatformConfiguration tempConfig = null;
 		try {
-			tempConfig = BootLoader.getPlatformConfiguration(null);
+			tempConfig = ConfiguratorUtils.getPlatformConfiguration(null);
 			IPlatformConfiguration.ISiteEntry[] tmpSites = runtimeConfiguration.getConfiguredSites();
 			for (int i = 0; i < tmpSites.length; i++) {
 				tempConfig.configureSite(tmpSites[i]);
@@ -335,8 +387,7 @@ public class InstallConfiguration extends InstallConfigurationModel implements I
 		//check sites
 		checkSites(configurationSites, runtimeConfiguration);
 
-		// Write the plugin path, primary feature and platform
-		// into platform.cfg
+		// Save the plugin path, primary feature and platform
 		for (int i = 0; i < configurationSites.length; i++) {
 			ConfiguredSite cSite = ((ConfiguredSite) configurationSites[i]);
 			ConfigurationPolicy configurationPolicy = cSite.getConfigurationPolicy();
@@ -368,12 +419,17 @@ public class InstallConfiguration extends InstallConfigurationModel implements I
 
 		try {
 			runtimeConfiguration.save();
+			// log configuration and activities
+			this.date = new Date(runtimeConfiguration.getChangeStamp());
+			if ("file".equalsIgnoreCase(getURL().getProtocol())) //$NON-NLS-1$
+				UpdateCore.log(this);
+			resetActivities();
+			return isRestartNeeded(runtimeConfiguration);
 		} catch (IOException e) {
-			CoreException exc = Utilities.newCoreException(Policy.bind("InstallConfiguration.UnableToSavePlatformConfiguration", runtimeConfiguration.getConfigurationLocation().toExternalForm()), e);
-			//$NON-NLS-1$
-			UpdateCore.warn("",exc);
+			CoreException exc = Utilities.newCoreException(NLS.bind(Messages.InstallConfiguration_UnableToSavePlatformConfiguration, (new String[] { runtimeConfiguration.getConfigurationLocation().toExternalForm() })), e);
+			UpdateCore.warn("",exc); //$NON-NLS-1$
 		}
-
+		return true;
 	}
 
 	/*
@@ -388,7 +444,7 @@ public class InstallConfiguration extends InstallConfigurationModel implements I
 
 		// create a ISitePolicy (policy, pluginPath)
 		// for the site
-		String[] pluginPath = configurationPolicy.getPluginPath(cSite.getSite(), cSite.getPreviousPluginPath());
+		String[] pluginPath = configurationPolicy.getPluginPath(cSite.getSite());
 		IPlatformConfiguration.ISitePolicy sitePolicy = runtimeConfiguration.createSitePolicy(configurationPolicy.getPolicy(), pluginPath);
 
 		// get the URL of the site that matches the one platform.cfg gave us
@@ -396,11 +452,9 @@ public class InstallConfiguration extends InstallConfigurationModel implements I
 		try {
 			urlToCheck = new URL(cSite.getPlatformURLString());
 		} catch (MalformedURLException e) {
-			throw Utilities.newCoreException(Policy.bind("InstallConfiguration.UnableToCreateURL", cSite.getPlatformURLString()), e);
-			//$NON-NLS-1$
+			throw Utilities.newCoreException(NLS.bind(Messages.InstallConfiguration_UnableToCreateURL, (new String[] { cSite.getPlatformURLString() })), e);
 		} catch (ClassCastException e) {
-			throw Utilities.newCoreException(Policy.bind("InstallConfiguration.UnableToCast"), e);
-			//$NON-NLS-1$
+			throw Utilities.newCoreException(Messages.InstallConfiguration_UnableToCast, e);	
 		}
 
 		// update runtime configuration [18520]
@@ -411,11 +465,12 @@ public class InstallConfiguration extends InstallConfigurationModel implements I
 			siteEntry = runtimeConfiguration.createSiteEntry(urlToCheck, sitePolicy);
 		else {
 			siteEntry.setSitePolicy(sitePolicy);
+			((SiteEntry)siteEntry).refreshPlugins();
 			if (tempConfig != null) // [19958] remove reused entries from list
 				tempConfig.unconfigureSite(siteEntry);
 		}
-		runtimeConfiguration.configureSite(siteEntry, true /*replace if exists*/
-		);
+		((SiteEntry)siteEntry).setEnabled(cSite.isEnabled());
+		runtimeConfiguration.configureSite(siteEntry, true /*replace if exists*/);
 	}
 
 	/*
@@ -435,6 +490,7 @@ public class InstallConfiguration extends InstallConfigurationModel implements I
 			if (id.equalsIgnoreCase(entries[k].getVersionedIdentifier().getIdentifier())) {
 				url = getRuntimeConfigurationURL(entries[k], cSite);
 				featurePlugin = entries[k];
+				break;
 			}
 		}
 		String pluginVersion = null;
@@ -457,14 +513,18 @@ public class InstallConfiguration extends InstallConfigurationModel implements I
 			// save information in runtime platform state
 			String version = feature.getVersionedIdentifier().getVersion().toString();
 			String application = feature.getApplication();
-			IPlatformConfiguration.IFeatureEntry featureEntry = runtimeConfiguration.createFeatureEntry(id, version, pluginIdentifier, pluginVersion, true, application, roots);
-			runtimeConfiguration.configureFeatureEntry(featureEntry);
+			FeatureEntry featureEntry = (FeatureEntry)runtimeConfiguration.createFeatureEntry(id, version, pluginIdentifier, pluginVersion, true, application, roots);
+			featureEntry.setURL(getFeatureRelativeURL(feature));
+			SiteEntry siteEntry = (SiteEntry)runtimeConfiguration.findConfiguredSite(cSite.getSite().getURL());
+			siteEntry.addFeatureEntry(featureEntry);
 		} else {
 			// write non-primary feature entries
 			String version = feature.getVersionedIdentifier().getVersion().toString();
 			String pluginIdentifier = feature.getPrimaryPluginID();
-			IPlatformConfiguration.IFeatureEntry featureEntry = runtimeConfiguration.createFeatureEntry(id, version, pluginIdentifier, pluginVersion, false, null, null);
-			runtimeConfiguration.configureFeatureEntry(featureEntry);
+			FeatureEntry featureEntry = (FeatureEntry)runtimeConfiguration.createFeatureEntry(id, version, pluginIdentifier, pluginVersion, false, null, null);
+			featureEntry.setURL(getFeatureRelativeURL(feature));
+			SiteEntry siteEntry = (SiteEntry)runtimeConfiguration.findConfiguredSite(cSite.getSite().getURL());
+			siteEntry.addFeatureEntry(featureEntry);
 		}
 
 		// write the platform features (features that contain special platform plugins)
@@ -493,82 +553,19 @@ public class InstallConfiguration extends InstallConfigurationModel implements I
 			try {
 				urlToCheck = new URL(configurationSites[i].getPlatformURLString());
 			} catch (MalformedURLException e) {
-				UpdateCore.warn(Policy.bind("InstallConfiguration.UnableToCreateURL", configurationSites[i].getPlatformURLString()), e);
-				//$NON-NLS-1$
+				UpdateCore.warn(NLS.bind(Messages.InstallConfiguration_UnableToCreateURL, (new String[] { configurationSites[i].getPlatformURLString() })), e);
 			} catch (ClassCastException e) {
-				UpdateCore.warn(Policy.bind("InstallConfiguration.UnableToCast"), e);
-				//$NON-NLS-1$
+				UpdateCore.warn(Messages.InstallConfiguration_UnableToCast, e);
 			}
 
 			// if the URL doesn't exits log it
 			IPlatformConfiguration.ISiteEntry siteEntry = runtimeConfiguration.findConfiguredSite(urlToCheck);
 			if (siteEntry == null) {
-				UpdateCore.warn(Policy.bind("Unable to find site {0} in platform configuration {1}.", urlToCheck.toExternalForm(), runtimeConfiguration.getConfigurationLocation().toExternalForm()));
-				//$NON-NLS-1$
+				UpdateCore.warn(NLS.bind(Messages.InstallConfiguration_unableToFindSite, (new String[] { urlToCheck.toExternalForm(), runtimeConfiguration.getConfigurationLocation().toExternalForm() }))); //$NON-NLS-1$
 			}
 		}
 	}
 
-	/*
-	 *
-	 */
-	public void saveConfigurationFile(boolean isTransient) throws CoreException {
-		// save the configuration
-		if ("file".equalsIgnoreCase(getURL().getProtocol())) { //$NON-NLS-1$
-			// the location points to a file
-			File file = new File(getURL().getFile());
-			if (!file.exists()) {
-				//log + 24642 [works for all activities]
-				UpdateCore.log(this);
-			}
-			if (isTransient)
-				file.deleteOnExit();
-			export(file);
-		}
-	}
-
-	/*
-	 * @see IWritable#write(int, PrintWriter)
-	 */
-	public void write(int indent, PrintWriter w) {
-		String gap = ""; //$NON-NLS-1$
-		for (int i = 0; i < indent; i++)
-			gap += " "; //$NON-NLS-1$
-		String increment = ""; //$NON-NLS-1$
-		for (int i = 0; i < IWritable.INDENT; i++)
-			increment += " "; //$NON-NLS-1$
-
-		//CONFIGURATION
-		w.print(gap + "<" + InstallConfigurationParser.CONFIGURATION + " ");
-		//$NON-NLS-1$ //$NON-NLS-2$
-		long time = (getCreationDate() != null) ? getCreationDate().getTime() : 0L;
-		w.print("date=\"" + time + "\" "); //$NON-NLS-1$ //$NON-NLS-2$
-		w.print("timeline=\"" + getTimeline() + "\" "); //$NON-NLS-1$ //$NON-NLS-2$
-		w.println(">"); //$NON-NLS-1$
-
-		// site configurations
-		if (getConfigurationSitesModel() != null) {
-			ConfiguredSiteModel[] sites = getConfigurationSitesModel();
-			for (int i = 0; i < sites.length; i++) {
-				ConfiguredSite element = (ConfiguredSite) sites[i];
-				((IWritable) element).write(indent + IWritable.INDENT, w);
-			}
-		}
-
-		// activities
-		if (getActivityModel() != null) {
-			ConfigurationActivityModel[] activities = getActivityModel();
-			for (int i = 0; i < activities.length; i++) {
-				ConfigurationActivity element = (ConfigurationActivity) activities[i];
-				((IWritable) element).write(indent + IWritable.INDENT, w);
-			}
-		}
-
-		// end
-		w.println(gap + "</" + InstallConfigurationParser.CONFIGURATION + ">");
-		//$NON-NLS-1$ //$NON-NLS-2$
-		w.println(""); //$NON-NLS-1$
-	}
 
 	/*
 	 * reverts this configuration to the match the new one
@@ -693,10 +690,10 @@ public class InstallConfiguration extends InstallConfigurationModel implements I
 			URL pluginEntryfullURL = siteContentProvider.getArchiveReference(pluginPathID);
 
 			//
-			if (!rootString.startsWith("platform")) {
+			if (!rootString.startsWith("platform")) { //$NON-NLS-1$
 				// DEBUG:
 				if (UpdateCore.DEBUG && UpdateCore.DEBUG_SHOW_CONFIGURATION)
-					UpdateCore.debug("getRuntimeConfiguration Plugin Entry Full URL:" + pluginEntryfullURL + " Platform String:" + rootString + " [NON PLATFORM URL].");
+					UpdateCore.debug("getRuntimeConfiguration Plugin Entry Full URL:" + pluginEntryfullURL + " Platform String:" + rootString + " [NON PLATFORM URL]."); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
 				return pluginEntryfullURL;
 			}
 
@@ -712,15 +709,15 @@ public class InstallConfiguration extends InstallConfigurationModel implements I
 
 			// DEBUG:
 			if (UpdateCore.DEBUG && UpdateCore.DEBUG_SHOW_CONFIGURATION)
-				UpdateCore.debug("getRuntimeConfiguration plugin Entry Full URL:" + pluginEntryfullURL + " Platform String:" + rootString + " Site URL:" + pluginEntryRootURL + " Relative:" + relativeString);
+				UpdateCore.debug("getRuntimeConfiguration plugin Entry Full URL:" + pluginEntryfullURL + " Platform String:" + rootString + " Site URL:" + pluginEntryRootURL + " Relative:" + relativeString); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
 
 			// verify we are about to write a valid file URL
 			// check with fullURL as it is not resolved to platform:base/
 			if (pluginEntryfullURL != null) {
-				if ("file".equals(pluginEntryfullURL.getProtocol())) {
+				if ("file".equals(pluginEntryfullURL.getProtocol())) { //$NON-NLS-1$
 					String fileString = pluginEntryfullURL.getFile();
 					if (!new File(fileString).exists()) {
-						UpdateCore.warn("The URL:" + result + " doesn't point to a valid platform plugin.The URL will not be written in the platform configuration", new Exception());
+						UpdateCore.warn("The URL:" + result + " doesn't point to a valid platform plugin.The URL will not be written in the platform configuration", new Exception()); //$NON-NLS-1$ //$NON-NLS-2$
 						return null;
 					}
 				}
@@ -728,8 +725,7 @@ public class InstallConfiguration extends InstallConfigurationModel implements I
 
 			return result;
 		} catch (IOException e) {
-			throw Utilities.newCoreException(Policy.bind("InstallConfiguration.UnableToCreateURL", rootString), e);
-			//$NON-NLS-1$
+			throw Utilities.newCoreException(NLS.bind(Messages.InstallConfiguration_UnableToCreateURL, (new String[] { rootString })), e);
 		}
 	}
 
@@ -742,28 +738,19 @@ public class InstallConfiguration extends InstallConfigurationModel implements I
 		VersionedIdentifier vid = entry.getVersionedIdentifier();
 
 		// get the plugin descriptor from the registry
-		IPluginRegistry reg = Platform.getPluginRegistry();
-		IPluginDescriptor desc = reg.getPluginDescriptor(vid.getIdentifier());
+		Bundle bundle = Platform.getBundle(vid.getIdentifier());
 		ArrayList list = new ArrayList();
-		if (desc != null) {
-			try {
-				// get all of the fragments
-				PluginDescriptorModel descModel = (PluginDescriptorModel) desc;
-				PluginFragmentModel[] frags = descModel.getFragments();
-
-				for (int i = 0; frags != null && i < frags.length; i++) {
-					String location = frags[i].getLocation();
-					try {
-						URL locationURL = new URL(location);
-						locationURL = Platform.resolve(locationURL);
-						list.add(asInstallRelativeURL(locationURL));
-					} catch (IOException e) {
-						// skip bad fragments
-					}
+		if (bundle != null && bundle.getState() != Bundle.UNINSTALLED && bundle.getState() != Bundle.INSTALLED) {
+			FragmentEntry[] fragments = UpdateManagerUtils.getFragments(bundle);
+			for (int i = 0; fragments != null && i < fragments.length; i++) {
+				String location = fragments[i].getLocation();
+				try {
+					URL locationURL = new URL(location);
+					locationURL = Platform.asLocalURL(Platform.resolve(locationURL));
+					list.add(asInstallRelativeURL(locationURL));
+				} catch (IOException e) {
+					// skip bad fragments
 				}
-
-			} catch (ClassCastException e) {
-				// cannot determine fragments
 			}
 		}
 		return (URL[]) list.toArray(new URL[0]);
@@ -783,23 +770,23 @@ public class InstallConfiguration extends InstallConfigurationModel implements I
 	 */
 	private URL asInstallRelativeURL(URL url) {
 		// get location of install
-		URL install = BootLoader.getInstallURL();
+		URL install = ConfiguratorUtils.getInstallURL();
 
 		// try to determine if supplied URL can be recast as install-relative
 		if (install.getProtocol().equals(url.getProtocol())) {
-			if (install.getProtocol().equals("file")) {
+			if (install.getProtocol().equals("file")) { //$NON-NLS-1$
 				String installS = new File(install.getFile()).getAbsolutePath().replace(File.separatorChar, '/');
-				if (!installS.endsWith("/"))
-					installS += "/";
+				if (!installS.endsWith("/")) //$NON-NLS-1$
+					installS += "/"; //$NON-NLS-1$
 				String urlS = new File(url.getFile()).getAbsolutePath().replace(File.separatorChar, '/');
-				if (!urlS.endsWith("/"))
-					urlS += "/";
-				int ix = installS.lastIndexOf("/");
+				if (!urlS.endsWith("/")) //$NON-NLS-1$
+					urlS += "/"; //$NON-NLS-1$
+				int ix = installS.lastIndexOf("/"); //$NON-NLS-1$
 				if (ix != -1) {
 					installS = installS.substring(0, ix + 1);
 					if (urlS.startsWith(installS)) {
 						try {
-							return new URL("platform:/base/" + urlS.substring(installS.length()));
+							return new URL("platform:/base/" + urlS.substring(installS.length())); //$NON-NLS-1$
 						} catch (MalformedURLException e) {
 						}
 					}
@@ -808,4 +795,162 @@ public class InstallConfiguration extends InstallConfigurationModel implements I
 		}
 		return url;
 	}
+	
+	private boolean isDuplicateSite(File siteDirectory) {
+		IConfiguredSite[] sites = getConfiguredSites();
+		URL fileURL;
+		try {
+			fileURL = siteDirectory.toURL();
+		} catch (MalformedURLException e) {
+			return false;
+		}
+		for (int i = 0; i < sites.length; i++) {
+			URL url = sites[i].getSite().getURL();
+			if (UpdateManagerUtils.sameURL(fileURL, url))
+				return true;
+		}
+		return false;
+	}
+	
+	/*
+	 * Returns the feature url relative to the site.
+	 */
+	private String getFeatureRelativeURL(IFeature feature) {
+		String url = feature.getURL().toExternalForm();
+		String siteURL = feature.getSite().getURL().toExternalForm();
+		// TODO fix this. toURL() returns file:/d:/eclipse/etc... wheareas the 
+		// platform.asLocalURL() returns file:d:/eclipse/etc... (no leading / )
+//		if (url.startsWith("file:/") && Platform.getOS().equals("win32"))
+//			url = "file:" + url.substring(6);
+		
+		if (url.startsWith(siteURL))
+			return url.substring(siteURL.length());
+		else
+			return url;
+	}
+	
+	/**
+	 * @return true if restart is needed
+	 */
+	private boolean isRestartNeeded(IPlatformConfiguration runtimeConfig) {
+
+		// First, create a map for faster lookups
+		Set newPluginsSet = null;
+		if (runtimeConfig instanceof PlatformConfiguration) {
+			newPluginsSet = ((PlatformConfiguration)runtimeConfig).getPluginPaths();
+			// On windows, we will be doing case insensitive search as well, so lower it now
+			if (isWindows) {
+				String[] newPluginsSetArray = (String[])newPluginsSet.toArray( new String[newPluginsSet.size()]);
+				for (int i = 0; i < newPluginsSetArray.length; i++) {
+					newPluginsSet.add(newPluginsSetArray[i].toLowerCase());
+				}
+			}
+		} else {
+			URL[] newBundlePaths = runtimeConfig.getPluginPath();
+			newPluginsSet = new HashSet(newBundlePaths.length);
+			for (int i=0; i<newBundlePaths.length; i++) {
+				
+				String pluginLocation = newBundlePaths[i].getFile();
+				newPluginsSet.add(pluginLocation);
+				// On windows, we will be doing case insensitive search as well, so lower it now
+				if (isWindows)
+					newPluginsSet.add(pluginLocation.toLowerCase());
+			}
+		}
+		
+		
+		
+		Bundle[] oldBundles = UpdateCore.getPlugin().getBundleContext().getBundles();
+
+		int offset = ConfigurationActivator.UPDATE_PREFIX.length();
+		for (int i=0; i<oldBundles.length; i++) {
+			if (oldBundles[i].getBundleId() == 0)
+				continue; // skip the system bundle
+			String oldBundleLocation = oldBundles[i].getLocation();
+			// Don't worry about bundles we did not install
+			if (!oldBundleLocation.startsWith(ConfigurationActivator.UPDATE_PREFIX))
+				continue;
+			oldBundleLocation = oldBundleLocation.substring(offset);
+			
+			if (newPluginsSet.contains(oldBundleLocation))
+				continue;
+			if (isWindows && newPluginsSet.contains(oldBundleLocation.toLowerCase()))
+				continue;
+			
+			if (UpdateCore.DEBUG && UpdateCore.DEBUG_SHOW_CONFIGURATION)
+				UpdateCore.debug("Bundle " + oldBundleLocation + " has been removed"); //$NON-NLS-1$ //$NON-NLS-2$
+			return true;
+		}
+		
+		if (runtimeConfig instanceof PlatformConfiguration) {
+			return areThereNewVersionOfOldPlugins( ((PlatformConfiguration)runtimeConfig).getPlugins(), oldBundles);
+		}
+
+		return false;
+	}
+	
+	/**
+	 * The method should only be called if all old plug-ins exist in the new
+	 * configuration. Determines if a new version was added to any plugin ID, so
+	 * more versions are enabled for any given plug-in id then in old
+	 * configuration.
+	 * 
+	 * @param newConfigurationPlugins
+	 * @param oldConfigurationBundles
+	 * @return
+	 */
+	private boolean areThereNewVersionOfOldPlugins(PluginEntry[] newConfigurationPlugins, Bundle[] oldConfigurationBundles) {
+
+		
+		for ( int i = 0; i < oldConfigurationBundles.length; i++) {
+			if (oldConfigurationBundles[i].getBundleId() == 0)
+				continue; // skip the system bundle
+			if ( getNumberOfPlugins(oldConfigurationBundles[i].getSymbolicName(), oldConfigurationBundles) != getNumberOfPlugins(oldConfigurationBundles[i].getSymbolicName(), newConfigurationPlugins)) {
+				return true;
+			}
+		}
+		return false;				
+	}
+
+	private int getNumberOfPlugins(String symbolicName, PluginEntry[] newConfigurationPlugins) {
+		
+		int numberOfPlugins = 0;
+		
+		for ( int i = 0; i < newConfigurationPlugins.length; i++) {
+			if ( symbolicName.equals(newConfigurationPlugins[i].getPluginIdentifier())) {
+				numberOfPlugins++;
+			}
+		}
+		
+		return numberOfPlugins;
+	}
+
+	private int getNumberOfPlugins(String symbolicName, Bundle[] oldConfigurationBundles) {
+
+		int numberOfPlugins = 0;
+		
+		for ( int i = 0; i < oldConfigurationBundles.length; i++) {
+			if ( symbolicName.equals(oldConfigurationBundles[i].getSymbolicName())) {
+				numberOfPlugins++;
+			}
+		}
+		
+		return numberOfPlugins;
+	}
+
+	/* (non-Javadoc)
+	 * @see java.lang.Object#equals(java.lang.Object)
+	 */
+	public boolean equals(Object obj) {
+		if (this == obj)
+			return true;
+		if (!(obj instanceof InstallConfiguration))
+			return false;
+		
+		InstallConfiguration config = (InstallConfiguration)obj;
+	
+		return getCreationDate().equals(config.getCreationDate()) && 
+				getLabel().equals(config.getLabel()) &&
+				getLocationURLString().equals(config.getLocationURLString());
+	}
 }
